#define BROWSEIO
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <limits.h>
#include <unistd.h>
#include <sys/stat.h>
#ifdef _ATFILE_SOURCE
#include <fcntl.h>
#endif
#include <dirent.h>
#include <libgen.h>
#include <time.h>
#include <zlib.h>
#include "../llib/loadimg.h"
#include "acolor.h"
#include "browseio.h"

#ifdef __MINGW32__
#include <windows.h>
#include <shlwapi.h>
#endif

#define MAX_IMAGE	65520
#define START_ICONS	1000 /* initial number of files in browser */
#define START_DIRS 	100  /* initial number of directories in a single directory */
#define MAX_PATH_FILE	500

static initConsumeProc initConsume;
static consumeBandProc consumeBand;
static endConsumeProc endConsume;
/* these functions are defined in load library; compiles, but crashes if they are defined static */
extern tryLoadProc tryLoad;
extern loadBandProc loadBand;
extern endLoadProc endLoad;

static int nowLoading = FALSE;	/* so we don't start loading again */

static int numAllocI=0; /* number of currently allocated icons */
static int includeSubDir = FALSE;
static int debugBrowse = FALSE;

/* load buffer parameters */
static unsigned char *data;	/* pointer to data */
static int bpl;			/* bytes per line */
static int bandh;		/* number of lines in one band */
static int progress;		/* advance in the data buffer or stay put */
static unsigned char cmap[256*3]; /* permanent cmap for colormap images */
static char loaderInfo[MAX_PATH_FILE];	/* tryload fills this in */

/* attributes of image being loaded */
static int loadType;		/* IMG_BW ... IMG_TRUE_COL */
static int xdpi;		/* horizontal dots per inch *1000 */
static int ydpi;		/* vertical dots per inch *1000 */
static int xpixels;
static int ypixels;

static int loadStatus;		/* the status of loading */
static int numLoaders;		/* number of loader functions */

static aColor *bimgC=NULL; /* browse image colors */
static unsigned char *bimageData;
static int bimageX, bimageY, bimageBPL; /* image sizes */
static int bimageType; /* 1bit: LOAD_BW 8bit: LOAD_CMAP 24 bits: LOAD_TRUE_COL */
static int breverseImage; /* is the image loaded in reverse */

static int iconSize;
static unsigned char useIconCache;
static int clearIconCacheDays; /* files older than how many days should be cleared from icon cache? */
static char iconPath[MAX_PATH_FILE];
static char browsePath[MAX_PATH_FILE];
static char fileBrowse[MAX_PATH_FILE];

static directoryInfo dirI; /* all directories in current directory */ 
static directoryInfo subDirI; /* all subdirectories in current directory */

#ifdef __MINGW32__
static char slash = '\\';
#else
static char slash = '/';
#endif

#ifdef _ATFILE_SOURCE
static int iconDirFD = -1; /* open icon directroy at init time */ 
static DIR *iconDir = NULL;
#endif


static int loadIcon(char *filen, imageInfo *iI);
/* uses the currently loaded image to generate the icon, and put it into a given imageInfo structure */
static int generateIcon(imageInfo *iI);

static displayReadProgressProc displayReadProgress;

static void freePtr(void **ptr)
{
 free(*ptr);
 *ptr=NULL;
}

static void loadIOerror(int code)
{
 printf("iolib load error %d\n", code);
}

static void processBand(int ypos)
{
 if ((loadStatus!=LOAD_OK)||(ypos>=ypixels))
  {
   endLoad();
   endConsume(loadStatus);
   if (loadStatus>LOAD_USER_ABORT)
    loadIOerror(loadStatus);
   nowLoading = FALSE;
   return;
  }

 loadStatus = loadBand(data, min(bandh, ypixels-ypos), bpl);
 if (loadStatus==LOAD_OK)
  loadStatus = consumeBand(data, min(bandh, ypixels-ypos), bpl, ypos);

 ypos+=min(bandh, ypixels-ypos);
 if (progress)
  data = &data[bpl*bandh];

 /* try to finish in one pass without unnecessary timeouts, if we have read everything in one pass */
 if ((loadStatus!=LOAD_OK)||(ypos>=ypixels))
  {
   endLoad();
   endConsume(loadStatus);
   if (loadStatus>LOAD_USER_ABORT)
    loadIOerror(loadStatus);
   nowLoading = FALSE;
   return;
  }
 processBand(ypos);
}

/* loads the image from filename, using all available loaders, and consuming */
/* it with the passed functions; can specify desired image type, since sometimes */
/* already the loader can convert the image */
/* if you know the type of the loader from a previous getImageInfo, you can pass it in loaderID */
/* if you do not know it pass UNKNOWN_LOADER and program will try all available loaders */
static int loadImagePrivate(char *filename, initConsumeProc iConsume,
 consumeBandProc cBand, endConsumeProc eConsume, int desiredLoadType, unsigned char *loaderID)
{
 int i, numInterlace, dummy;
 char reverse;

 /* only one load job at a time */
 if (nowLoading)
  return(LOAD_INTERNAL_ERROR);

 /* don't try to load files, which are too small to be an image */
 {
  struct stat statBuf;
  if (stat(filename, &statBuf))
   return(LOAD_IO_ERROR);
  else if (statBuf.st_size<MIN_IMAGE_FILSIZE)
   return(LOAD_NOT_IMAGE);
 }

 initConsume = iConsume;
 consumeBand = cBand;
 endConsume = eConsume;
 loadStatus = LOAD_OK;

 if (debugBrowse)
  printf("image browse try %s\n", filename);

//printf("loadImagePrivate: %s loader ID %s\n", filename, returnLoaderString(returnLoaderFunctions(0, filename)));

 if (access(filename, R_OK)!=0)
  loadStatus = LOAD_IO_ERROR;
 else
  for (i=1; i<numLoaders; i++)
   {
    nowLoading = TRUE;
    if ((i==1)&&(*loaderID!=UNKNOWN_LOADER))
     i= *loaderID;
//printf("loadImagePrivate: using loader %s size %dx%d\n", returnLoaderString(i), xpixels, ypixels);
    loadType = desiredLoadType;
    returnLoaderFunctions(i, filename);
    loadStatus = tryLoad(filename, &loadType, &xpixels, &ypixels, &xdpi, &ydpi, cmap, loaderInfo, &reverse, &numInterlace, &dummy, TRUE);
    if (loadStatus==LOAD_OK)
     {
      *loaderID = (unsigned char) i;
      i=numLoaders; /* to exit loop */
      loadStatus = initConsume(loadType, xpixels, ypixels, xdpi, ydpi, reverse, 1, cmap, &data, &bpl, &bandh, &progress);
      if (loadStatus==LOAD_OK)
       processBand(0);
      else
       endLoad();
     }
    else
     if (loadStatus==LOAD_NOT_IMAGE)
      i=numLoaders; /* bailout */
   }
 if (loadStatus!=LOAD_OK)
  {
   loadIOerror(loadStatus);
   nowLoading = FALSE;
  }
 return(loadStatus);
}

/* tries to get image information before loading it; returns "" on fail */
static char *getImageInfoPrivate(char *filename, int *x, int *y, unsigned char *loaderID, int *coreFile)
{
 int i, numInterlace, status;
 char reverse;

 if (nowLoading)
  return("");
//printf("getImageInfoPrivate:%s\n", filename);
 if (access(filename, R_OK)==0)
  if (filename[strlen(filename)-1]!=slash)
   {
    *loaderID = (unsigned char) returnLoaderFunctions(0, filename);
    if (*loaderID!=UNKNOWN_LOADER) /* first try with the loader suggested on filename heuristics */
     {
//printf("getImageInfoPrivate: default loader ID %s\n", returnLoaderString(*loaderID));
      loadType = LOAD_DEFAULT;
      status = tryLoad(filename, &loadType, x, y, &xdpi, &ydpi, cmap, loaderInfo, &reverse, &numInterlace, coreFile, FALSE);
      if (status == LOAD_OK)
       { endLoad(); return(loaderInfo); }
      else
       if (status==LOAD_NOT_IMAGE)
        return(""); 
     }
    /* then try all the other loaders */
    for (i=1; i<numLoaders; i++)
     if (i!=*loaderID) /* don't try with the same loader again */
      {
//printf("getImageInfoPrivate: try loader ID %s\n", returnLoaderString(i));
       returnLoaderFunctions(i, filename);
       loadType = LOAD_DEFAULT;
       status = tryLoad(filename, &loadType, x, y, &xdpi, &ydpi, cmap, loaderInfo, &reverse, &numInterlace, coreFile, FALSE);
       if (status == LOAD_OK)
	{
         endLoad(); 
	 *loaderID = (unsigned char) i;
	 return(loaderInfo);
	}
       else
	if (status==LOAD_NOT_IMAGE)
         return(""); 
      }
   }
//printf("getImageInfoPrivate FAIL:%s\n", filename);
 return("");
}

/* registers the function to display the progress during reading image directories */
DBROWSEIO void registerBrowseIconProgress(displayReadProgressProc readProgress)
{
 displayReadProgress = readProgress;
}

/* set subdirectory browsing on or off */
DBROWSEIO void setImageBrowseIncludeSubDir(int set)
{
 includeSubDir = set;
}

/* enable printout of filename before it is loaded on or off */
DBROWSEIO void setImageBrowseDebug(int set)
{
 debugBrowse = set;
}

/* gets the variable which keeps the number of days that unused icons should be kept in cache */
/* should be called and variable set before widget initialisation */
DBROWSEIO int *getIconCacheDays(void)
{
 return(&clearIconCacheDays);
}

/* return string with current browse path */
DBROWSEIO char *getImageBrowsePath(void)
{
 return(browsePath);
}

/* return string with current browse filename (with path) */
DBROWSEIO char *getImageBrowseFile(void)
{
 return(fileBrowse);
}

/* return the maximum size of image icons */
DBROWSEIO int getImageIconSize(void)
{
 return(iconSize);
}

/* returns the pointer to data of image used of icon creation and full screen image display */
DBROWSEIO unsigned char *getBrowseImageData(void)
{
 return(bimageData);
}

/* frees the datao data of image used of icon creation and full screen image display */
DBROWSEIO void freeBrowseImageData(void)
{
 freePtr((void *) &bimageData);
}

/* returns the pointer to the directory which contains image icons */
DBROWSEIO char *getImageIconPath(void)
{
 return(iconPath);
}

/* delete all icons which were not accessed since clearIconCacheDays */
DBROWSEIO void clearIconCache(int clearIconCacheDays)
{
 if (useIconCache)
  {
   DIR *d;
   struct dirent *di;
   int dirNamelen;
   struct stat statBuf;
   char filen[MAX_PATH_FILE];
   char iconPathSub[MAX_PATH_FILE];
   time_t currentTime = time(NULL);

   printf("Please wait, clearing image icon cache...\n");
   sprintf(iconPathSub, "%s%c%02x", iconPath, slash, rand()&255);
   d = opendir(iconPathSub);
   if (d)
    {
     dirNamelen = strlen(iconPathSub);
     strcpy(filen, iconPathSub);
     filen[dirNamelen]=slash;
     while ((di=readdir(d)))
      {
       filen[dirNamelen+1]=0;
       strcat(&filen[dirNamelen+1], di->d_name);
       if (stat(filen, &statBuf))
	break;
       if (S_ISREG(statBuf.st_mode))
	if ((currentTime-statBuf.st_atime)/86400 >= clearIconCacheDays)
	 if (access(filen, W_OK)==0)
          remove(filen);
      }
     closedir(d);
    }  
   printf("Clearing image icon cache finished.\n");
  }
}

static int myMkdir(char *path, mode_t mode)
{
#ifdef __MINGW32__
 return(mkdir(path));
#else
 return(mkdir(path, mode));
#endif
}

/* checks or creates icon cache directory, if it fails then icons are then still created but not saved */
static void checkAndSetupIconCache()
{
 mode_t mode = S_IRUSR | S_IWUSR | S_IXUSR;
 struct stat statBuf;
 int i;
 char iconSubPath[MAX_PATH_FILE];

#ifdef __MINGW32__
    if (!SUCCEEDED(SHGetFolderPathA(NULL, CSIDL_LOCAL_APPDATA, NULL, 0, iconPath)))
     { perror("iolib failed to get app data dir"); useIconCache=FALSE; i=0xFFFF; }
    sprintf(iconPath, "%s%c.aiconsI", iconPath, slash);
#else

 /* don't use or create icon cache, if running as root */
 if(geteuid() == 0)
  {
   printf("Running as root, image icon cache is not used\n");
   useIconCache = FALSE;
   iconPath[0]=0;
   return;
  }

 sprintf(iconPath, "%s%c.aiconsI", getenv("HOME"), slash);
#endif
 if (stat(iconPath, &statBuf))
  { /* it does not exist; create it */
   if(myMkdir(iconPath, mode))
    { perror("iolib image icon dir creation"); useIconCache=FALSE; }
   for (i=0; i<=0xFF; i++)
    {
     sprintf(iconSubPath, "%s%c%02x", iconPath, slash, i);
     if (myMkdir(iconSubPath, mode))
      { perror("iolib image icon subdir creation"); useIconCache=FALSE; i=0xFFFF; }
    }
  }
 else 
  {
   if (S_ISDIR(statBuf.st_mode))
    {
     if (!(access(iconPath, W_OK)==0))
      {
       printf("Error: %s exists, but we do not have write permission!\n", iconPath);
       useIconCache=FALSE;
      }
     else
      /* check if subdirectories exists and create them if not */
      for (i=0; i<=0xFF; i++)
       {
	sprintf(iconSubPath, "%s%c%02x", iconPath, slash, i);
        if (stat(iconSubPath, &statBuf))
	 {
	  if (myMkdir(iconSubPath, mode))
	   { perror("iolib image icon subdir creation"); useIconCache=FALSE; i=0xFFFF; }
	 }
	else
	 i=0xFFFF; /* if one subdirectory exists assume all of them exist; otherwise there is too much checking on startup */
       }     
    }
   else
    {
     printf("Error: %s exists, but is not a directory!\n", iconPath);
     useIconCache=FALSE;
    }
  }
}

/* initialises image browsing library */
DBROWSEIO void initBrowseIO(void)
{
 static char first = TRUE;

 if (!first)
  return;

#ifdef __MINGW32__
#ifdef DEBUG
 /* so we can use printf in windows DLL */
 AllocConsole();
 freopen("CONIN$", "r", stdin);
 freopen("CONOUT$", "w", stdout);
 freopen("CONOUT$", "w", stderr);
#endif
#endif

 initImageLoadLibrary();
 first = FALSE;
 displayReadProgress = NULL;
 numLoaders = returnLoaderFunctions(-1, NULL);
 useIconCache = TRUE;
 iconSize = IMAGE_ICON_SIZE;
 bimageData = NULL;
 memset(&dirI, 0, sizeof(directoryInfo));
 memset(&subDirI, 0, sizeof(directoryInfo));
 checkAndSetupIconCache();
#ifdef _ATFILE_SOURCE
 if (useIconCache)
  {
   iconDir = opendir(iconPath);
   iconDirFD = dirfd(iconDir);
  }
#endif
}

/* check can remove old entries from icon cache on average every 255nd  run */
DBROWSEIO void clearImageIconCacheSometimes(void)
{
 if (useIconCache)
  if ((rand()&15)==0) /* we now clear it on every sixteenth time on average, since we only need to check 1/256 of all cache at one time */
   clearIconCache(clearIconCacheDays);
}

/* you must set the browse path prior to mapping the browse window */
/* if you pass the full path with filename, then this filename can be used to position the browser to the location */
/* of that file and to select it, if it exists */
DBROWSEIO void setBrowsePath(char *path)
{
 int i;
 char tmpstr[250], defaultFile[250]="/bla";
 struct stat statBuf;
 char *dirName;

 strcpy(tmpstr, path);
 /* go up to 10 entries back in path until we find something readable */
 for (i=0; i<10; i++)
  {
   if (access(tmpstr, R_OK)==0)
    {
     if (!stat(tmpstr, &statBuf))
      {
       if (S_ISDIR(statBuf.st_mode))
	break;
       else
        sprintf(defaultFile, "%c%s", slash, basename(tmpstr));
      }
    }
   /* ensure tmpstr does not overlap during strcpy */
   dirName = strdup(dirname(tmpstr));
   strcpy(tmpstr, dirName);
   free(dirName);
  }
 strcpy(browsePath, tmpstr);
 strcpy(fileBrowse, tmpstr);
 strcat(fileBrowse, defaultFile);
}

/* starts loading the image */
static int initImageLoad(int loadType, int xpixels, int ypixels,
 int xdpi, int ydpi, char reverse, int numInterlace, unsigned char *cmap,
 unsigned char **data, int *bpl, int *bandh, int *progress)
{
 freePtr((void *) &bimageData);
 bimageX = xpixels;
 bimageY = ypixels;
 *bandh = ypixels; /* all in one go */

 *data=bimageData; 
 *progress = TRUE;

 bimageType=loadType;
 
 switch (bimageType)
  {
   case LOAD_BW:
    bimageBPL = (xpixels+7)/8;
    break;
   case LOAD_CMAP:
    {
     int i;
     aColor *col;

     bimageBPL = xpixels;
     for (i=0; i<256; i++)
      {
       col = &bimgC[i];
       col->red = col->pred = cmap[i*3];
       col->green = col->pgreen = cmap[i*3+1];
       col->blue = col->pblue = cmap[i*3+2];
      }
    }
    break;
   case LOAD_TRUE_COL:
    bimageBPL = xpixels*3;
    break;
   case LOAD_RGBA:
    bimageBPL = xpixels*4;
    break;
  }
 breverseImage = reverse;
 *bpl = bimageBPL;
 *data = bimageData = malloc(bimageY*bimageBPL);
 if (!bimageData)
  return(LOAD_NO_MEMORY);

 return(LOAD_OK);
}

static int bandImageLoad(unsigned char *data, int lines, int bpl, int ypos)
{
 return(LOAD_OK);
}

/* generic function for reversing images in Y direction */
DBROWSEIO void reverseImageY(unsigned char *data, int bpl, int sizeY)
{
 int y;
 unsigned char line[bpl], *data1, *data2;

 if (!data)
  return;
 for (y=0; y<(sizeY>>1); y++)
  {
   data1 = &data[y*bpl];
   data2 = &data[(sizeY-y-1)*bpl];
   memcpy(line, data1, bpl);
   memcpy(data1, data2, bpl);
   memcpy(data2, line, bpl);
  }
}

static int endImageLoad(int status)
{
 if (status!=LOAD_OK)
  freePtr((void *) &bimageData);
 if (breverseImage && (status==LOAD_OK))
  reverseImageY(bimageData, bimageBPL, bimageY);
 return(status);  
}

/* return the directory name for the given ID */
DBROWSEIO char *getImageDir(int numDir)
{
 if (numDir<dirI.numDir)
  return(dirI.dirB[numDir].name);
 else
  {
   printf("browseio: dir out of bounds %d\n", numDir);
   return(""); /* just to prevent the crash in client */
  }
}

/* return the number of directories in the current directory */
DBROWSEIO int getImageDirNum(void)
{
 return(dirI.numDir);
}

/* return the subdirectory path for the given ID */
DBROWSEIO char *getImageSubDir(int numDir)
{
 if (numDir<subDirI.numDir)
  return(subDirI.dirB[numDir].name);
 else
  {
   printf("browseio: subdir out of bounds %d\n", numDir);
   return(""); /* just to prevent the crash in client */
  }
}

/* return the number of currently loaded subirectories */
DBROWSEIO int getImageSubDirNum(void)
{
 return(subDirI.numDir);
}

/* loads the image in the browser's internal image and creates the corresponding icon */
DBROWSEIO int loadImageMakeIcon(imageInfo *iI, aColor *browseImageC, int *browseImageBPL)
{
 char strbuf[MAX_PATH_FILE];

 if (iI->subDirID)
  sprintf(strbuf, "%s%c%s%c%s", browsePath, slash, subDirI.dirB[iI->subDirID].name, slash, iI->Fname);
 else
  sprintf(strbuf, "%s%c%s", browsePath, slash, iI->Fname);
 bimgC = browseImageC;
 if (LOAD_OK==loadImagePrivate(strbuf, initImageLoad, bandImageLoad, endImageLoad, LOAD_RGBA, &iI->loaderID))
  {
   *browseImageBPL = bimageBPL;
   iI->imageType = (unsigned char) bimageType;
   iI->imageX = (unsigned short) bimageX;
   iI->imageY = (unsigned short) bimageY;
   return(generateIcon(iI));
  }
 else
  {
   iI->iconType = LOAD_BLACKLIST;
   return(FALSE);
  }
}

/* loads the image icon, for the cases when we are sure that the icon already exists in the cache */
DBROWSEIO int loadImageIcon(imageInfo *iI)
{
 char filen[MAX_PATH_FILE];

 if (iI->subDirID)
  sprintf(filen, "%s%c%s%c%s", browsePath, slash, subDirI.dirB[iI->subDirID].name, slash, iI->Fname);
 else
  sprintf(filen, "%s%c%s", browsePath, slash, iI->Fname);

 freePtr((void *) &iI->iconData);
 freePtr((void *) &iI->iconCmap);
 return(loadIcon(filen, iI));
}

/* swaps two image icon structurs */
DBROWSEIO void swapImageIcon(imageInfo *i1, imageInfo *i2)
{
 imageInfo tmpI;
 
 memcpy(&tmpI, i1, sizeof(imageInfo));
 memcpy(i1, i2, sizeof(imageInfo));
 memcpy(i2, &tmpI, sizeof(imageInfo));
}

/* free the image info data */
DBROWSEIO void freeImageI(imageInfo *iI)
{
 freePtr((void *) &iI->iconData);
 freePtr((void *) &iI->iconCmap);
 freePtr((void *) &iI->Fname);
 iI->subDirID = 0; /* just a precaution */
}

/* for loading of icons which involves loading of images we must provide colormap before calling loadImageI() */
DBROWSEIO void setImageBrowseCmap(aColor *browseImageC)
{
 bimgC = browseImageC;
}

/* load the image into image info structure (if loadable) and create the icon, or load it from icon cache */ 
DBROWSEIO int loadImageI(char *filen, imageInfo *iI)
{
 int x,y;
 char *info;
 int coreFile=FALSE;

 freePtr((void *) &(iI->Fname));
 freePtr((void *) &iI->iconData);
 freePtr((void *) &iI->iconCmap);
 /* note: at this point, fileSize and accessTime fields of iI have already been filled in by the caller of loadImageI() */
 if (loadIcon(filen, iI))
  return(TRUE);
 info=getImageInfoPrivate(filen, &x, &y, &iI->loaderID, &coreFile);
//printf("got info %s %dx%d id %d core %d\n", info, x, y, iI->loaderID, coreFile);
 if (coreFile && strlen(info) && (x<=MAX_IMAGE) && (y<=MAX_IMAGE) && (x>0) && (y>0))
  {
   iI->imageX = (unsigned short) x;
   iI->imageY = (unsigned short) y;
   iI->xdpi = (unsigned short) (xdpi/1000.0 + 0.5); /* must scale it down to fit into unsigned short */
   iI->ydpi = (unsigned short) (ydpi/1000.0 + 0.5);

   if ((x<iconSize)&&(y<iconSize)) /* if a small image load it right away and make the darn icon */
    {
     if (LOAD_OK==loadImagePrivate(filen, initImageLoad, bandImageLoad, endImageLoad, LOAD_RGBA, &iI->loaderID))
      {
       iI->imageType = (unsigned char) bimageType;
       return(generateIcon(iI));
      }
     else
      return(FALSE);
    }
   return(TRUE);
  }
 else
  return(FALSE);
}

static void getMoreIcons(imageInfo **imageI)
{
 imageInfo *iArray;

//printf("before I had  %d icons (%d)\n", numAllocI, sizeof(imageInfo)*numAllocI);
 *imageI = realloc(*imageI, sizeof(imageInfo)*numAllocI*2);
 iArray = *imageI;
 memset(&iArray[numAllocI], 0, sizeof(imageInfo)*numAllocI);
 numAllocI*=2;
//printf("now have %d icons (%d)\n", numAllocI, sizeof(imageInfo)*numAllocI);
}

static void getMoreDirs(directoryInfo *dI)
{
//printf("before I had %d directories\n", dI->numAllocDir);
 dI->dirB = realloc(dI->dirB, sizeof(dirBrowse)*dI->numAllocDir*2);
 memset(&dI->dirB[dI->numAllocDir], 0, sizeof(dirBrowse)*dI->numAllocDir);
 dI->numAllocDir*=2;
//printf("now have %d directories\n", dI->numAllocDir);
}

/* return true if file .nomedia does not exist in given directory */
DBROWSEIO int mediaDir(char *dir)
{
 char filen[MAX_PATH_FILE];

 sprintf(filen, "%s%c.nomedia", dir, slash);
 return(access(filen, F_OK)!=0);
}

static void readSubDir(imageInfo **imageI, char *loadPath, int *filesSofar, int dirID)
{
 DIR *d=NULL;
 struct dirent *di;
 int dirNamelen;
 struct stat statBuf;
 char filen[MAX_PATH_FILE];
 imageInfo *iArray = *imageI;
 imageInfo *iI;
#ifdef _ATFILE_SOURCE
 int pathFD = -1;
#endif

 sprintf(filen, "%s%c%s", loadPath, slash, subDirI.dirB[dirID].name);
 if (mediaDir(filen))
  d = opendir(filen);
 if (d)
  {
#ifdef _ATFILE_SOURCE
   pathFD = dirfd(d);
#endif
   dirNamelen = strlen(filen);
   filen[dirNamelen]=slash;
   while ((di=readdir(d)))
    {
     /* allocate some more and clear the newly allocated, if needed */
     if (*filesSofar==numAllocI)
      {
       getMoreIcons(imageI);
       iArray = *imageI;
      }
#ifdef _ATFILE_SOURCE
     if (!fstatat(pathFD, di->d_name, &statBuf, 0))
#else
     filen[dirNamelen+1]=0;
     strcat(&filen[dirNamelen+1], di->d_name);
     if (!stat(filen, &statBuf))
#endif
      {
       // filter out the current directory and the ones starting with "." (hidden) 
       if (S_ISDIR(statBuf.st_mode))
	if (di->d_name[0]!='.')
	 {
#ifdef _ATFILE_SOURCE
	  if (faccessat(pathFD, di->d_name, R_OK, 0)==0)
#else
	  filen[dirNamelen+1]=0;
	  strcat(&filen[dirNamelen+1], di->d_name);
	  if (access(filen, R_OK)==0)
#endif
	   {
            char totalSubDir[300];

	    if (subDirI.numDir==subDirI.numAllocDir)
	     getMoreDirs(&subDirI);
	    sprintf(totalSubDir, "%s%c%s", subDirI.dirB[dirID].name, slash, di->d_name);
	    subDirI.dirB[subDirI.numDir++].name=strdup(totalSubDir);
	    readSubDir(imageI, loadPath, filesSofar, subDirI.numDir-1);
	    iArray = *imageI; /* because pointer to imageI could be modified */
	   }
	 }

       if (S_ISREG(statBuf.st_mode))
        {
#ifdef _ATFILE_SOURCE
	 if (faccessat(pathFD, di->d_name, R_OK, 0)==0)
#else
	 filen[dirNamelen+1]=0;
	 strcat(&filen[dirNamelen+1], di->d_name);
	 if (access(filen, R_OK)==0)
#endif
	  {
	   iI = &iArray[*filesSofar];
           // get some info already here, so we do not call stat two times 
	   iI->modifiedTime = statBuf.st_mtime;
	   iI->fileSize = statBuf.st_size;
	   iI->subDirID = (unsigned short) dirID;
           iI->FnameLen = (unsigned char) strlen(di->d_name);
	   memcpy(&filen[dirNamelen+1], di->d_name, iI->FnameLen+1);
           if (loadImageI(filen, iI))
	    {
             iI->Fname = strdup(di->d_name);
	     *filesSofar += 1;
	     if (displayReadProgress)
	      displayReadProgress(*filesSofar);
	    }
	  }
	}
      }
    }
   closedir(d);
  }
}

/* load all the images in readable formats from specified directory to the browser */
/* it allocates and sets the imageI icon stucture array, fills in dirB array of directory names, */
/* sets returns number of loaded images and number of directories */
/* browseImageC is a pre-allocated 256 array of aColor needed for loading of colormapped images */
DBROWSEIO void loadImageIcons(char *loadPath, imageInfo **imageI, int *numIconImages, directoryInfo **dirInfo, aColor *browseImageC, int iconDisplayBits)
{
 DIR *d=NULL;
 struct dirent *di;
 int i, j, dirNamelen;
 struct stat statBuf;
 char filen[MAX_PATH_FILE];
 imageInfo *iArray;
 imageInfo *iI;
#ifdef _ATFILE_SOURCE
 int pathFD = -1;
#endif
 char root[2]="/";

 root[0]=slash;
 bimgC = browseImageC;

 /* first allocate space for directory names, if we did not do it so far */ 
 if (!dirI.dirB)
  {
   dirI.dirB = (dirBrowse *) calloc(START_DIRS, sizeof(dirBrowse));
   dirI.numDir = 0;
   dirI.numAllocDir = START_DIRS;
  }
 if (!dirI.dirB)
  return;

 /* then free old directory names */
 for (i=0; i<dirI.numDir; i++)
  freePtr((void *) &dirI.dirB[i].name);
 dirI.numDir=0;

 /* also free the old subdirectory names */
 if (includeSubDir)
  {
   if (!subDirI.dirB)
    {
     subDirI.dirB = (dirBrowse *) calloc(START_DIRS, sizeof(dirBrowse));
     subDirI.numAllocDir = START_DIRS;
     subDirI.numDir = 0;
    }
   if (!subDirI.dirB)
    return;

   for (i=0; i<subDirI.numDir; i++)
    freePtr((void *) &subDirI.dirB[i].name);
   subDirI.dirB[0].name = strdup("");
   subDirI.numDir=1;
  }

 /* free the old image icons */
 iArray = *imageI;
 for (j=0; j<*numIconImages; j++)
  freeImageI(&iArray[j]); /* &imageI[j] would advance by ptr, but we need to hop by imageInfo structure */
 freePtr((void *) imageI);

 *imageI = calloc(sizeof(imageInfo), START_ICONS);
 numAllocI = START_ICONS;
 if (!*imageI)
  return;
 iArray = *imageI;

 i=0;
 if (mediaDir(strlen(loadPath) ? loadPath : root))
  d = opendir(strlen(loadPath) ? loadPath : root); /* handles root directory, too */
 if (d)
  {
#ifdef _ATFILE_SOURCE
   pathFD = dirfd(d);
#endif
   dirNamelen = strlen(loadPath);
   strcpy(filen, loadPath);
   filen[dirNamelen]=slash; /* faster than strcat of "/" */

   while ((di=readdir(d)))
    {
     /* allocate some more and clear the newly allocated, if needed */
     if (i==numAllocI)
      {
       getMoreIcons(imageI);
       iArray = *imageI;
      }
#ifdef _ATFILE_SOURCE
     if (!fstatat(pathFD, di->d_name, &statBuf, 0))
#else
     filen[dirNamelen+1]=0;
     strcat(&filen[dirNamelen+1], di->d_name);
     if (!stat(filen, &statBuf))
#endif
      {
       /* filter out the current directory and the ones starting with "." (hidden) */
       if (S_ISDIR(statBuf.st_mode))
	if (di->d_name[0]!='.')
	 {
#ifdef _ATFILE_SOURCE
	  if (faccessat(pathFD, di->d_name, R_OK, 0)==0)
#else
	  filen[dirNamelen+1]=0;
	  strcat(&filen[dirNamelen+1], di->d_name);
	  if (access(filen, R_OK)==0)
#endif
	   {
	    if (includeSubDir)
	     {
	      if (subDirI.numDir==subDirI.numAllocDir)
	       getMoreDirs(&subDirI);
	      subDirI.dirB[subDirI.numDir++].name=strdup(di->d_name);
	      readSubDir(imageI, loadPath, &i, subDirI.numDir-1); 
              iArray = *imageI; /* because pointer to imageI could be modified */
	     }
	    else
	     {
	      if (dirI.numDir==dirI.numAllocDir)
	       getMoreDirs(&dirI);
	      dirI.dirB[dirI.numDir].name=strdup(di->d_name);
	      if (iconDisplayBits & IMAGE_ICON_DIR_INFO)
	       {
		char fn[MAX_PATH_FILE];

		strcpy(fn, loadPath);
		fn[dirNamelen]='/';
		fn[dirNamelen+1]=0;
		strcat(&fn[dirNamelen+1], di->d_name);
		countDirFiles(fn, &dirI.dirB[dirI.numDir].numDirs, &dirI.dirB[dirI.numDir].numFiles);
	       }
              dirI.numDir++;
	     }
	   }
	 }

       if (S_ISREG(statBuf.st_mode))
        if (di->d_name[0]!='.') /* do now show hidden files */
	 {
#ifdef _ATFILE_SOURCE
	  if (faccessat(pathFD, di->d_name, R_OK, 0)==0)
#else
	  filen[dirNamelen+1]=0;
	  strcat(&filen[dirNamelen+1], di->d_name);
	  if (access(filen, R_OK)==0)
#endif
	   {
	    iI = &iArray[i];
            /* get some info already here, so we do not call stat two times */
	    iI->modifiedTime = statBuf.st_mtime;
	    iI->fileSize = statBuf.st_size;
	    iI->subDirID = 0;
	    iI->FnameLen = (unsigned char) strlen(di->d_name);
	    memcpy(&filen[dirNamelen+1], di->d_name, iI->FnameLen+1);
            if (loadImageI(filen, iI))
	     {
	      iI->Fname = strdup(di->d_name);
	      i++;
	      if (displayReadProgress)
	       displayReadProgress(i);
	     }
	   }
	 }
      }
     else
      perror("iolib stat() failed\n");
    }
   closedir(d);
  }

 /* now shove off the extra space that you do not need */
 if (i)
  *imageI = realloc(*imageI, sizeof(imageInfo)*i);
 *numIconImages = i;

 if (dirI.numDir>1)
  qsort(dirI.dirB, dirI.numDir, sizeof(dirBrowse), (int (*)()) dirBrowseCmp);

 *dirInfo = &dirI;
}

/* take 32 bytes hex string and insert a "/" or "\\" (Windows case) after second byte, so that we divide */
/* icon cache in 256 directories and icon name is 2 bytes shorter, as subdir is the first part of the name */
DBROWSEIO void insertHexDir(char *md5)
{
 int i;
 
 for (i=32; i>2; i--)
  md5[i+1]=md5[i];
 md5[2]=slash;
}

#define ICON_HEADER_LENGTH	24
#define ICON_VERSION		3

static int saveIcon(imageInfo *iI)
{
 char iconBaseName[MAX_PATH_FILE];
 char iconSubName[35];
 unsigned char header[ICON_HEADER_LENGTH];
#ifdef _ATFILE_SOURCE
 int iconFD;
#else
 FILE *iconF;
 char iconFullName[600];
#endif
 int writeIconData, statusOK = TRUE;

 if (!useIconCache)
  return(FALSE);

 if (iI->subDirID)
  sprintf(iconBaseName, "%s%c%s%c%s", browsePath, slash, subDirI.dirB[iI->subDirID].name, slash, iI->Fname);
 else
  sprintf(iconBaseName, "%s%c%s", browsePath, slash, iI->Fname);

 strncpy(iconSubName, md5string((const unsigned char *) iconBaseName, strlen(iconBaseName)), 33);
 insertHexDir(iconSubName);
#ifdef _ATFILE_SOURCE
 if ((iconFD = openat(iconDirFD, iconSubName, O_WRONLY | O_CREAT, S_IRUSR | S_IWUSR)) >=0)
#else
 sprintf(iconFullName, "%s%c%s", iconPath, slash, iconSubName);
 if ((iconF = fopen(iconFullName, "wb" )) != NULL)
#endif
  {
   writeIconData = (iI->iconData && ( (iI->iconType==LOAD_CMAP) || (iI->iconType==LOAD_GREY) )
    && ((iI->imageX*iI->imageY)>((iconSize*iconSize)>>2))) ? 1 : 0;
   header[0]='A'; /* "Aicon" is the file identifer */ 
   header[1]='i';
   header[2]='c';
   header[3]='o';
   header[4]='n';
   header[5]=ICON_VERSION; /* icon version; must match on loading, or it will be discarded */
   header[6]= (unsigned char)(iI->imageX>>8);
   header[7]= (unsigned char)(iI->imageX & 0xFF);
   header[8]= (unsigned char)(iI->imageY>>8);
   header[9]= (unsigned char)(iI->imageY & 0xFF);
   header[10]=(unsigned char)(iI->xdpi>>8);
   header[11]=(unsigned char)(iI->xdpi & 0xFF);
   header[12]=(unsigned char)(iI->ydpi>>8);
   header[13]=(unsigned char)(iI->ydpi & 0xFF);
   header[14]=iI->usedColors;
   header[15]=iI->loaderID;
   header[16]=iI->iconX;
   header[17]=iI->iconY;
   header[18]=iI->imageType;
   header[19]=iI->iconType;
   header[20]=iI->cmapSize;
   header[21]=(unsigned char)(iI->compressedIconLen>>8);
   header[22]=(unsigned char)(iI->compressedIconLen & 0xFF);
   header[23]= (unsigned char) writeIconData; /* support saving of image info without icon */
#ifdef _ATFILE_SOURCE
   write(iconFD, header, ICON_HEADER_LENGTH);
   /* all icons which we write are of LOAD_CMAP or LOAD_GREY type, so iI->iconX equals bpl */
   if (writeIconData)
    {
     if (iI->iconType==LOAD_CMAP)
      write(iconFD, iI->iconCmap, iI->cmapSize*3);
     write(iconFD, iI->iconData, iI->compressedIconLen);  
    }
   close(iconFD);
#else
   fwrite(header, 1, ICON_HEADER_LENGTH, iconF);
   /* all icons which we write are of LOAD_CMAP or LOAD_GREY type, so iI->iconX equals bpl */
   if (writeIconData)
    {
     if (iI->iconType==LOAD_CMAP)
      fwrite(iI->iconCmap, 3, iI->cmapSize, iconF);
     fwrite(iI->iconData, 1, iI->compressedIconLen, iconF);  
    }
   fflush(iconF);
   fclose(iconF);
#endif
  }
 else
  statusOK=FALSE;
 return(statusOK);
}

static int loadIcon(char *filen, imageInfo *iI)
{
 char iconName[35];
 unsigned char header[ICON_HEADER_LENGTH];
 int readIconData, statusOK = TRUE;
 struct stat statBuf;
#ifdef _ATFILE_SOURCE
 int iconFD;
#else
 FILE *iconF;
 char iconFullName[600];
#endif

 if (!useIconCache)
  return(FALSE);
 memcpy(iconName, md5string((const unsigned char *) filen, strlen(filen)), 33);
 insertHexDir(iconName);

#ifdef _ATFILE_SOURCE
 if (fstatat(iconDirFD, iconName, &statBuf, 0))
  return(FALSE);
#else
 sprintf(iconFullName, "%s%c%s", iconPath, slash, iconName);
 if (stat(iconFullName, &statBuf))
  return(FALSE);
#endif

 /* files from the future will confuse comparison between icon time and file time */
 /* this occurs on network file sharing with unsynchronized clocks */ 
 if (iI->modifiedTime<time(NULL)) 
  if (iI->modifiedTime >= statBuf.st_mtime) /* check if the icon file is newer than the image file */
   return(FALSE); /* icon is old; discard it */

#ifdef _ATFILE_SOURCE
 if ((iconFD = openat(iconDirFD, iconName, O_RDONLY)) < 0)
  return(FALSE);
 statusOK = read(iconFD, header, ICON_HEADER_LENGTH);
#else
 if ((iconF = fopen(iconFullName, "rb" )) == NULL)
  return(FALSE);
 statusOK = fread(header, 1, ICON_HEADER_LENGTH, iconF);
#endif

 /* check the "Aicon" identifier, icon version and length of header which was read */
 if ((header[0]!='A') || (header[1]!='i') || (header[2]!='c') || (header[3]!='o') || (header[4]!='n') ||
  (header[5]!=ICON_VERSION) || (statusOK!=ICON_HEADER_LENGTH) )
#ifdef _ATFILE_SOURCE
  { close(iconFD); return(FALSE); }
#else
  { fclose(iconF); return(FALSE); }
#endif

 iI->imageX = (unsigned short) ((header[6]<<8)  | header[7]);
 iI->imageY = (unsigned short) ((header[8]<<8)  | header[9]);
 iI->xdpi =   (unsigned short) ((header[10]<<8) | header[11]);
 iI->ydpi =   (unsigned short) ((header[12]<<8) | header[13]);

 iI->usedColors = header[14];
 iI->loaderID = header[15];
 iI->iconX = header[16];
 iI->iconY = header[17];
 iI->imageType = header[18];
 iI->iconType = header[19];
 iI->cmapSize = header[20];
 iI->compressedIconLen = (unsigned short) ((header[21]<<8) | header[22]);
 readIconData = header[23]; /* icon is also present in the file */

#ifdef _ATFILE_SOURCE
 if (readIconData)
  {
   iI->iconData = malloc(iI->compressedIconLen);
   if (!iI->iconData)
    { close(iconFD); return(FALSE); }
   if (iI->iconType==LOAD_CMAP)
    {
     iI->iconCmap=malloc(iI->cmapSize*3);
     if (!iI->iconCmap)
      { freePtr((void *) iI->iconData); close(iconFD); return(FALSE); }
     statusOK = read(iconFD, iI->iconCmap, iI->cmapSize*3);
     if (statusOK!=iI->cmapSize*3)
      {
       freePtr((void *) iI->iconData);
       freePtr((void *) iI->iconCmap);
       close(iconFD);
       return(FALSE);
      }
    }
   statusOK = read(iconFD, iI->iconData, iI->compressedIconLen);
  }
 close(iconFD);
#else
 if (readIconData)
  {
   iI->iconData = malloc(iI->compressedIconLen);
   if (!iI->iconData)
    { fclose(iconF); return(FALSE); }
   if (iI->iconType==LOAD_CMAP)
    {
     iI->iconCmap=malloc(iI->cmapSize*3);
     if (!iI->iconCmap)
      { freePtr((void *) iI->iconData); fclose(iconF); return(FALSE); }
     statusOK = fread(iI->iconCmap, 3, iI->cmapSize, iconF);
     if (statusOK!=iI->cmapSize)
      {
       freePtr((void *) iI->iconData);
       freePtr((void *) iI->iconCmap);
       fclose(iconF);
       return(FALSE);
      }
    }
   statusOK = fread(iI->iconData, 1, iI->compressedIconLen, iconF);
  }
 fclose(iconF);
#endif
 return(statusOK);
}

/* provide a way to replace and make a better icon, in cases when image loader is not able make good icon */
/* this is useful mainly for ArahDrape, where drape project is rendered, and image browser is unable to do it on its own */
DBROWSEIO void updateAndSaveImageIcon(imageInfo *iI, char *path)
{
 char fname[MAX_PATH_FILE];
 char oldBrowsePath[MAX_PATH_FILE];
 struct stat statBuf;

 sprintf(fname, "%s%c%s", path, slash, iI->Fname);
 /* need to set the modified time and file size */
 if (stat(fname, &statBuf)==0)
  {
   iI->modifiedTime = statBuf.st_mtime;
   iI->fileSize = statBuf.st_size;
   strcpy(oldBrowsePath, browsePath);
   strcpy(browsePath, path);
   saveIcon(iI);
   strcpy(browsePath, oldBrowsePath);
  }
}

/* function reduces given image to a smaller icon. All data must be allocated. If source image is of type LOAD_BW */
/* program generates grayscale 8-bit icon; otherwise the icon is always 24 bits per pixel */
/* one exception: if source is RGBA, then icon is also RGBA */
/* usedColors returns number of used colors, if cmap image */
DBROWSEIO int reduceImageToIcon(unsigned char *srcData, int srcBPL, int srcX, int srcY, int srcType, aColor *srcCol,
                                unsigned char *dstData, int dstBPL, int dstX, int dstY, unsigned char *usedColors)
{
 int i, x, y, val;
 unsigned int *rgbSum, *rgbSumPtr, *rgbSumPtrY;
 unsigned short mapY[max(srcY, dstY)], scanX, scanY;
 unsigned int mapX[max(srcX, dstX)], *mapXptr;
 unsigned short mapXsrc[dstX], *mapXsrcPtr; /* need a separate table for upscaling colormap images, since mapX is multipled by 3 or 4 */
 unsigned int sumX[dstX], sumY[dstY], sumy;
 int divideSum;
 unsigned char *sdata, *ddata, *sval;
 char usedColor[256];
 int dstMultiplier=1;
 
 switch (srcType)
  {
   case LOAD_BW: dstMultiplier = 1; break;
   case LOAD_RGBA: dstMultiplier = 4; break;
   default: dstMultiplier = 3; break;
  }

 if ((srcX==0) || (srcY==0) || (dstX==0) || (dstY==0) || (!srcData) || (!dstData))
  return(FALSE);
 rgbSum = (unsigned int *) calloc(dstX*dstY*dstMultiplier, sizeof(unsigned int));
 if (!rgbSum)
  return(FALSE);

 memset(sumX, 0, dstX*sizeof(unsigned int));
 memset(sumY, 0, dstY*sizeof(unsigned int));

 if (srcX>dstX)
  {
   scanX = (unsigned short) srcX;
   for (i=0; i<srcX; i++)
    {
     val = (dstX*i)/srcX;
     mapX[i] = val*dstMultiplier;
     sumX[val]++;
    }
   }
 else
  {
   scanX = (unsigned short) dstX;
   for (i=0; i<dstX; i++)
    {
     val = (srcX*i)/dstX;
     mapX[i] = val*dstMultiplier;
     mapXsrc[i] = (unsigned short) val; /* needed for LOAD_CMAP upscaling */
     sumX[i]=1;
    }
  }

 if (srcY>dstY)
  {
   scanY = (unsigned short) srcY;
   for (i=0; i<srcY; i++)
    {
     val = (dstY*i)/srcY;
     mapY[i] = (unsigned short) val;
     sumY[val]++;
    }
  }
 else
  {
   scanY = (unsigned short) dstY;
   for (i=0; i<dstY; i++)
    {
     val = (srcY*i)/dstY;
     mapY[i] = (unsigned short) val;
     sumY[i] = 1;
    }
  }

 if (srcType==LOAD_CMAP)
  memset(usedColor, 0, 256);

 for (y=0; y<scanY; y++)
  {
   if (scanY==dstY)
    {
     sdata = &srcData[mapY[y]*srcBPL];
     rgbSumPtrY = &rgbSum[y*dstX*dstMultiplier];
    }
   else
    {
     sdata = &srcData[y*srcBPL];
     rgbSumPtrY = &rgbSum[mapY[y]*dstX*dstMultiplier];
    }
   mapXptr = mapX;
   mapXsrcPtr = mapXsrc;

   switch (srcType)
    {
     case LOAD_RGBA:
      if (scanX==dstX)
       for (x=0; x<scanX; x++)
	{
	 sval = sdata + *mapXptr++;
	 *rgbSumPtrY++ += sval[0]*sval[3];
	 *rgbSumPtrY++ += sval[1]*sval[3];
	 *rgbSumPtrY++ += sval[2]*sval[3];
	 *rgbSumPtrY++ += sval[3];
	}
      else
       for (x=0; x<srcX; x++)
	{
	 rgbSumPtr = rgbSumPtrY + *mapXptr++;
	 *rgbSumPtr++ += sdata[0]*sdata[3];
	 *rgbSumPtr++ += sdata[1]*sdata[3];
	 *rgbSumPtr++ += sdata[2]*sdata[3];
	 *rgbSumPtr   += sdata[3];
	 sdata+=4;
	}
      break;
     case LOAD_TRUE_COL:
      if (scanX==dstX)
       for (x=0; x<scanX; x++)
	{
	 sval = sdata + *mapXptr++;
	 *rgbSumPtrY++ += *sval++;
	 *rgbSumPtrY++ += *sval++;
	 *rgbSumPtrY++ += *sval;
	}
      else
       for (x=0; x<srcX; x++)
	{
	 rgbSumPtr = rgbSumPtrY + *mapXptr++;
	 *rgbSumPtr++ += *sdata++;
	 *rgbSumPtr++ += *sdata++;
	 *rgbSumPtr   += *sdata++;
	}
      break;
     case LOAD_CMAP:
      if (scanX==dstX)
       for (x=0; x<scanX; x++)
	{
	 sval = sdata + *mapXsrcPtr++;
	 usedColor[*sval]=TRUE;
	 *rgbSumPtrY++ += srcCol[*sval].red;
	 *rgbSumPtrY++ += srcCol[*sval].green;
	 *rgbSumPtrY++ += srcCol[*sval].blue;
	}
      else
       for (x=0; x<srcX; x++)
	{
	 rgbSumPtr = rgbSumPtrY + *mapXptr++;
	 usedColor[*sdata]=TRUE;
	 *rgbSumPtr++ += srcCol[*sdata].red;
	 *rgbSumPtr++ += srcCol[*sdata].green;
	 *rgbSumPtr   += srcCol[*sdata++].blue;
	}
      break;
     case LOAD_BW:
      /* for black and white, we just count the black an white points instead of going for the full cmap approach */
      if (scanX==dstX)
       {
	rgbSumPtrY = &rgbSum[((scanY==dstY) ? y : mapY[y])*dstX];
	for (x=0; x<scanX; x++)
	 {
	  if (!(sdata[(*mapXptr)>>3]&(128>>((*mapXptr)&7))))
	   rgbSumPtrY[x]++;
	  mapXptr++;
	 }
       }
      else
       {
	rgbSumPtrY = &rgbSum[mapY[y]*dstX];
	for (x=0; x<srcX; x++)
	 if (sdata[x>>3]&(128>>(x&7)))
	  mapXptr++;
	 else
	  rgbSumPtrY[*mapXptr++]++;
       }
      break;
    }
  }

 /* count the used colors */
 *usedColors=0;
 if (srcType==LOAD_BW)
  *usedColors=2;
 if (srcType==LOAD_CMAP)
  for (i=0; i<256; i++)
   if (usedColor[i])
    (*usedColors)++;

 rgbSumPtr = rgbSum;
 for (y=0; y<dstY; y++)
  {
   sumy = sumY[y];
   ddata = &dstData[y*dstBPL];
   switch (srcType)
    {
     case LOAD_BW:
      for (x=0; x<dstX; x++)
       {
	divideSum = sumX[x]*sumy;
	switch (divideSum)
	 {
          case 0:
          case 1:   *ddata++= (unsigned char) (*rgbSumPtr++  *255);      break;
          case 2:   *ddata++= (unsigned char) ((*rgbSumPtr++ *255)>>1); break;
          case 3:   *ddata++= (unsigned char) ((*rgbSumPtr++ *255*85)>>8); break;
          case 4:   *ddata++= (unsigned char) ((*rgbSumPtr++ *255)>>2); break;
          case 5:   *ddata++= (unsigned char) ((*rgbSumPtr++ *255*51)>>8); break;
          case 6:   *ddata++= (unsigned char) ((*rgbSumPtr++ *255*85)>>9); break;
          case 7:   *ddata++= (unsigned char) ((*rgbSumPtr++ *255*73)>>9); break;
          case 8:   *ddata++= (unsigned char) ((*rgbSumPtr++ *255)>>3); break;
          case 16:  *ddata++= (unsigned char) ((*rgbSumPtr++ *255)>>4); break;
          case 32:  *ddata++= (unsigned char) ((*rgbSumPtr++ *255)>>5); break;
          case 64:  *ddata++= (unsigned char) ((*rgbSumPtr++ *255)>>6); break;
          case 128: *ddata++= (unsigned char) ((*rgbSumPtr++ *255)>>7); break;
          default:  *ddata++= (unsigned char) ((*rgbSumPtr++ *255)/divideSum); break;
	}
       }
      break;

     case LOAD_RGBA:
      for (x=0; x<dstX; x++)
       {
        if (rgbSumPtr[3] && sumX[x])
	 {
	  *ddata++= (unsigned char) (rgbSumPtr[0]/(rgbSumPtr[3]));
	  *ddata++= (unsigned char) (rgbSumPtr[1]/(rgbSumPtr[3]));
	  *ddata++= (unsigned char) (rgbSumPtr[2]/(rgbSumPtr[3]));
	  *ddata++= (unsigned char) (rgbSumPtr[3]/(sumX[x]*sumy));
	 }
	else
	 {
	  *ddata++ = 0;
	  *ddata++ = 0;
	  *ddata++ = 0;
	  *ddata++ = 0;
	 }
	rgbSumPtr+=4;
       }
      break;

     default:
      for (x=0; x<dstX; x++)
       {
	divideSum = sumX[x]*sumy;
	switch (divideSum)
	 {
          case 0:
          case 1:
	   *ddata++= (unsigned char) (*rgbSumPtr++);
	   *ddata++= (unsigned char) (*rgbSumPtr++);
	   *ddata++= (unsigned char) (*rgbSumPtr++);
	   break;
          case 2:
	   *ddata++= (unsigned char) ((*rgbSumPtr++)>>1);
	   *ddata++= (unsigned char) ((*rgbSumPtr++)>>1);
	   *ddata++= (unsigned char) ((*rgbSumPtr++)>>1);
	   break;
          case 3:
	   *ddata++= (unsigned char) ((*rgbSumPtr++ *85)>>8);
	   *ddata++= (unsigned char) ((*rgbSumPtr++ *85)>>8);
	   *ddata++= (unsigned char) ((*rgbSumPtr++ *85)>>8);
	   break;
          case 4:
	   *ddata++= (unsigned char) ((*rgbSumPtr++)>>2);
	   *ddata++= (unsigned char) ((*rgbSumPtr++)>>2);
	   *ddata++= (unsigned char) ((*rgbSumPtr++)>>2);
	   break;
          case 5:
	   *ddata++= (unsigned char) ((*rgbSumPtr++ *51)>>8);
	   *ddata++= (unsigned char) ((*rgbSumPtr++ *51)>>8);
	   *ddata++= (unsigned char) ((*rgbSumPtr++ *51)>>8);
	   break;
	  case 6:
	   *ddata++= (unsigned char) ((*rgbSumPtr++ *85)>>9);
	   *ddata++= (unsigned char) ((*rgbSumPtr++ *85)>>9);
	   *ddata++= (unsigned char) ((*rgbSumPtr++ *85)>>9);
           break;
	  case 7:
	   *ddata++= (unsigned char) ((*rgbSumPtr++ *73)>>9);
	   *ddata++= (unsigned char) ((*rgbSumPtr++ *73)>>9);
	   *ddata++= (unsigned char) ((*rgbSumPtr++ *73)>>9);
           break;
          case 8:
	   *ddata++= (unsigned char) ((*rgbSumPtr++)>>3);
	   *ddata++= (unsigned char) ((*rgbSumPtr++)>>3);
	   *ddata++= (unsigned char) ((*rgbSumPtr++)>>3);
	   break;
          case 16:
	   *ddata++= (unsigned char) ((*rgbSumPtr++)>>4);
	   *ddata++= (unsigned char) ((*rgbSumPtr++)>>4);
	   *ddata++= (unsigned char) ((*rgbSumPtr++)>>4);
	   break;
          case 32:
	   *ddata++= (unsigned char) ((*rgbSumPtr++)>>5);
	   *ddata++= (unsigned char) ((*rgbSumPtr++)>>5);
	   *ddata++= (unsigned char) ((*rgbSumPtr++)>>5);
	   break;
          case 64:
	   *ddata++= (unsigned char) ((*rgbSumPtr++)>>6);
	   *ddata++= (unsigned char) ((*rgbSumPtr++)>>6);
	   *ddata++= (unsigned char) ((*rgbSumPtr++)>>6);
	   break;
          case 128:
	   *ddata++= (unsigned char) ((*rgbSumPtr++)>>7);
	   *ddata++= (unsigned char) ((*rgbSumPtr++)>>7);
	   *ddata++= (unsigned char) ((*rgbSumPtr++)>>7);
	   break;
          default:
	   *ddata++= (unsigned char) (*rgbSumPtr++/divideSum);
	   *ddata++= (unsigned char) (*rgbSumPtr++/divideSum);
	   *ddata++= (unsigned char) (*rgbSumPtr++/divideSum);       
	 }
       }
      break;
    }
  }
 free(rgbSum);
 return(TRUE);
}

/* compress the icon data */
DBROWSEIO void compressIconData(unsigned char **iconData, int iconLen, unsigned short *compressedIconLen)
{
 uLongf destLen = (int)(iconLen*1.02)+12;
 unsigned char iData[destLen];

 if (Z_OK==compress(iData, &destLen, *iconData, iconLen))
  {
   freePtr((void *) iconData);
   *compressedIconLen = (unsigned short) destLen;
   *iconData = malloc(destLen);
   if (*iconData)
    memcpy(*iconData, iData, destLen);
  }
 else
  freePtr((void *) iconData);
}

/* uses the currently loaded image to generate the icon, and put it into a given imageInfo structure */
static int generateIcon(imageInfo *iI)
{
 int i,j;
 float factor;
 char distortIcon = FALSE;

 freePtr((void *) &iI->iconData);
 freePtr((void *) &iI->iconCmap);
 if (!bimageData)
  return(FALSE);
 /* first calculate the dimensions of our icon */
 if ((xdpi==ydpi) || (xdpi==0) || (ydpi==0))
  {
   factor = (float) min((1.0*iconSize)/iI->imageX, (1.0*iconSize)/iI->imageY);
   iI->iconX = (unsigned char) (factor*iI->imageX+0.5);
   iI->iconY = (unsigned char) (factor*iI->imageY+0.5);
  }
 else
  {
   float dpiRatio = (float) ((1.0*xdpi)/ydpi);

   distortIcon = TRUE;
   if  ((dpiRatio>100.0)||(dpiRatio<0.01))
    dpiRatio=1.0;
   if (dpiRatio>1.0)
    {
     factor = (float) min((1.0*iconSize)/iI->imageX, (1.0*iconSize)/(iI->imageY*dpiRatio));
     iI->iconX = (unsigned char) (factor*iI->imageX+0.5);
     iI->iconY = (unsigned char) (factor*iI->imageY*dpiRatio+0.5);
    }
   else
    {
     factor = (float) min((1.0*iconSize)/(iI->imageX/dpiRatio), (1.0*iconSize)/iI->imageY);
     iI->iconX = (unsigned char) (factor*iI->imageX/dpiRatio+0.5);
     iI->iconY = (unsigned char) (factor*iI->imageY+0.5);
    }
  }
 iI->iconX = (unsigned char) min(max(1, iI->iconX), iconSize);
 iI->iconY = (unsigned char) min(max(1, iI->iconY), iconSize);

 if ((factor<1.0) || distortIcon)
  {
   unsigned char *iconData24=NULL; /* the 24 bits per pixel icon data */
   int requestColors = 128; /* icons look good enough on 128 colors */
   unsigned char cmap[256*3];

   iI->iconData=malloc(iI->iconX*iI->iconY);
   if (!iI->iconData)
    return(FALSE);
   switch (iI->imageType)
    {
     case LOAD_BW:    
      iI->iconType=LOAD_GREY;
      if (!reduceImageToIcon(bimageData, bimageBPL, iI->imageX, iI->imageY, iI->imageType, bimgC, iI->iconData, iI->iconX, iI->iconX, iI->iconY, &iI->usedColors))
       { freePtr((void *) &iI->iconData); return(FALSE); }
      break;
     case LOAD_CMAP:
     case LOAD_TRUE_COL:
      iconData24 = malloc(3*iI->iconX*iI->iconY);
      if (!iconData24)
       { freePtr((void *) &iI->iconData); return(FALSE); }
      if (!reduceImageToIcon(bimageData, bimageBPL, iI->imageX, iI->imageY, iI->imageType, bimgC, iconData24, iI->iconX*3, iI->iconX, iI->iconY, &iI->usedColors))
       { freePtr((void *) &iI->iconData); free(iconData24); return(FALSE); }
      colorQuantize(iconData24, iI->iconX*3, iI->iconData, iI->iconX, iI->iconX, iI->iconY, cmap, 0, &requestColors);
      iI->iconType=LOAD_CMAP;
      iI->cmapSize= (unsigned char) requestColors;
      iI->iconCmap = malloc(requestColors*3);
      if (!iI->iconCmap)
       { free(iconData24); freePtr((void *) &iI->iconData); return(FALSE); }
      memcpy(iI->iconCmap, cmap, requestColors*3);
      free(iconData24);
      break;
     case LOAD_RGBA:
      iconData24 = malloc(4*iI->iconX*iI->iconY); /* allocate 4 bytes instead of 3, so we have enough space */
      if (!iconData24)
       { freePtr((void *) &iI->iconData); return(FALSE); }
      if (!reduceImageToIcon(bimageData, bimageBPL, iI->imageX, iI->imageY, iI->imageType, bimgC, iconData24, iI->iconX*4, iI->iconX, iI->iconY, &iI->usedColors))
       { freePtr((void *) &iI->iconData); free(iconData24); return(FALSE); }
      /* in memory collapse the icon from RGBA to RGB and interpret alpha channel */
      {
       int dummyBPL = iI->iconX*4;

       iconData24 = collapseRGBA(iconData24, iI->iconX, iI->iconY, &dummyBPL, TRUE);
      }
      colorQuantize(iconData24, iI->iconX*3, iI->iconData, iI->iconX, iI->iconX, iI->iconY, cmap, 0, &requestColors);
      iI->iconType=LOAD_CMAP;
      iI->cmapSize= (unsigned char) requestColors;
      iI->iconCmap = malloc(requestColors*3);
      if (!iI->iconCmap)
       { free(iconData24); freePtr((void *) &iI->iconData); return(FALSE); }
      memcpy(iI->iconCmap, cmap, requestColors*3);
      free(iconData24);
      break;
    }
   compressIconData(&iI->iconData, iI->iconX*iI->iconY, &iI->compressedIconLen);
   if (useIconCache)
    saveIcon(iI);
  }
 else /* just copy the whole image and use it as an icon */
  {
   int iconBPL = bimageBPL;

   iI->iconType = iI->imageType;
   iI->iconX = (unsigned char) iI->imageX;
   iI->iconY = (unsigned char) iI->imageY;
   iI->iconData=malloc(iconBPL*bimageY);
   if (!iI->iconData)
    return(FALSE);
   memcpy(iI->iconData, bimageData, iconBPL*bimageY);

   iI->usedColors = (unsigned char) ((iI->iconType==LOAD_BW) ? 2 :0);
   if (iI->iconType==LOAD_CMAP)
    {
     unsigned char usedColor[256];
     unsigned char *dataPtr;
     int cmapSize; /* must not be unsigned char, so it can take 256 */

     /* calculate the actually used colors */
     memset(usedColor, 0, 256);
     for (i=0; i<bimageY; i++)
      {
       dataPtr = &bimageData[iconBPL*i];
       for (j=0; j<bimageX; j++)
        usedColor[*dataPtr++]=TRUE;
      }
     iI->usedColors = 0;
     for (i=0; i<256; i++)
      if (usedColor[i])
       iI->usedColors++;
     /* allocate palette only up to last used color, not the unnecessary ones */
     cmapSize=255;
     while (!usedColor[cmapSize])
      cmapSize--;
     cmapSize++;
     iI->iconCmap=malloc(cmapSize*3);
     if (!iI->iconCmap)
      { free(iI->iconData); return(FALSE); }
     iI->cmapSize = (unsigned char) cmapSize;
     for (i=0; i<cmapSize; i++)
      {
       iI->iconCmap[i*3]=bimgC[i].red;
       iI->iconCmap[i*3+1]=bimgC[i].green;
       iI->iconCmap[i*3+2]=bimgC[i].blue;
      }
    }
   /* if RGBA convert the icon to RGB */
   if (iI->iconType==LOAD_RGBA)
    {
     iI->iconData = collapseRGBA(iI->iconData, iI->iconX, iI->iconY, &iconBPL, TRUE);
     iI->iconType=LOAD_TRUE_COL;
    }
   compressIconData(&iI->iconData, iconBPL*iI->iconY, &iI->compressedIconLen);
  }
 return(TRUE);
}

/* renders the icon image into the passed buffer in the specified image depth, repeat mode and zoom */
/* warning! dstData must have space for iconSize+2 vertically, or rendering will crash */
DBROWSEIO int renderBrowseIconImage(unsigned char *iconData, unsigned short compressedIconLen,
 unsigned char iconType, unsigned char iconX, unsigned char iconY, unsigned char *iconCmap,
 int iconSize, int magnifyIcon, int dispXY, int noRepeatImage, int bytesPerPixel, unsigned char *dstData, int dstbpl)
{
 int x, y;
 unsigned char *dst8=NULL;
 unsigned short *dst16=NULL;
 unsigned int *dst32=NULL;
 unsigned char *iconLine;
 int xoff, yoff, iconZoom, iconZoomX, iconZoomY;
 int xicon[dispXY];
 int iconBPL=1;
 int invertAlpha;
 int iconStartX, iconEndX;
 unsigned char *icmapPtr, *srcPtr;
 uLongf uncompressedIconLen = iconY;

 switch (iconType)
  {
   case LOAD_TRUE_COL: uncompressedIconLen *= iconX*3; break;
   case LOAD_RGBA: uncompressedIconLen *= iconX*4; break;
   case LOAD_BW: uncompressedIconLen *= (iconX+7)/8; break;
   default: uncompressedIconLen *= iconX; break;
  } 

 unsigned char iData[uncompressedIconLen];

 if (Z_OK==uncompress(iData, &uncompressedIconLen, iconData, compressedIconLen))
  {
   if (magnifyIcon)
    iconZoom = max(1, min(iconSize/iconX, iconSize/iconY));
   else
    iconZoom = 1;
   iconZoomX = iconX*iconZoom;
   iconZoomY = iconY*iconZoom;
   xoff = (iconZoomX<<6) - (dispXY-iconZoomX)/2;
   yoff = (iconZoomY<<6) - (iconSize-iconZoomY)/2-2;

   for (x=0; x<dispXY; x++)
    xicon[x] = ((x+xoff)/iconZoom)%iconX;
   if (noRepeatImage && (iconSize-iconZoomX))
    {
     iconStartX = (iconSize-iconZoomX)/2 + 1;
     iconEndX = iconStartX + iconZoomX;
    }
   else
    {
     iconStartX = 0;
     iconEndX = dispXY;
    }
   switch (iconType)
    {
     case LOAD_RGBA: iconBPL = iconX*4; break;
     case LOAD_TRUE_COL: iconBPL = iconX*3; break;
     case LOAD_CMAP:
     case LOAD_GREY: iconBPL = iconX; break;
     case LOAD_BW: iconBPL = (iconX+7)/8; break;
    }
   switch (bytesPerPixel)
    {
     case 4:
      for (y=0; y<(iconSize+2); y++)
       {
	dst32 = (unsigned int *) &dstData[y*dstbpl];
        iconLine=&iData[iconBPL*(((y+yoff)/iconZoom)%iconY)];
	if ( noRepeatImage && ( (y < ((iconSize-iconZoomY)/2 + 2)) || (y >= ((iconSize-iconZoomY)/2 + iconZoomY + 2)) ) )
	 for (x=dispXY; x; x--)
	  *dst32++ =  0xFFBFBFBF;
	else
	 {
	  x=0;
	  while (x<iconStartX)
	   { *dst32++ = 0xFFBFBFBF; x++; }
	  switch (iconType)
	   {   
	    case LOAD_RGBA:
	     while (x<iconEndX)
              {
	       srcPtr = &iconLine[xicon[x]<<2];
	       invertAlpha = ((int)255-srcPtr[3])*(((((x++)>>3)+(y>>3))&1) ? DARK_CHECK : LIGHT_CHECK);
               *dst32++ = (255<<24) | 
	        (((invertAlpha + (int)srcPtr[0]*srcPtr[3])>>8)<<16) |
		(((invertAlpha + (int)srcPtr[1]*srcPtr[3])>>8)<<8) |
		 ((invertAlpha + (int)srcPtr[2]*srcPtr[3])>>8);
	      }
	     break;
	    case LOAD_TRUE_COL:
	     while (x<iconEndX)
              { *dst32++ = (255<<24) | (iconLine[xicon[x]*3]<<16) | (iconLine[xicon[x]*3+1]<<8) | iconLine[xicon[x]*3+2]; x++; }
	     break;
	    case LOAD_CMAP:
	     while (x<iconEndX)
              {
	       icmapPtr = &iconCmap[iconLine[xicon[x++]]*3];
               *dst32++ = (255<<24) | (*icmapPtr<<16) | (icmapPtr[1]<<8) | icmapPtr[2];
	      }
	     break;
	    case LOAD_GREY:
	     while (x<iconEndX)
              { *dst32++ = (255<<24) | (iconLine[xicon[x]]<<16) | (iconLine[xicon[x]]<<8) | iconLine[xicon[x]]; x++; }
	     break;
	    case LOAD_BW:
	     while (x<iconEndX)
              { *dst32++ = ((iconLine[xicon[x]>>3]&(128>>(xicon[x]&7))) ? 0xFF000000 : 0xFFFFFFFF); x++; }
 	     break;
           }
	  while (x<dispXY)
	   { *dst32++ = 0xFFBFBFBF; x++; }
	 }
       }
     break;

     case 3:
      for (y=0; y<(iconSize+2); y++)
       {
	dst8 = (unsigned char *) &dstData[y*dstbpl];
        iconLine=&iData[iconBPL*(((y+yoff)/iconZoom)%iconY)];
	if ( noRepeatImage && ( (y < ((iconSize-iconZoomY)/2 + 2)) || (y >= ((iconSize-iconZoomY)/2 + iconZoomY + 2)) ) )
	 memset(dst8, 0xBF, 3*dispXY);
	else
         {
	  x=0;
	  while (x<iconStartX)
	   { *dst8++ = 0xBF; *dst8++ = 0xBF; *dst8++ = 0xBF; x++; }
	  switch (iconType)
	   {   
	    case LOAD_RGBA:
	     while (x<iconEndX)
              {
	       srcPtr = &iconLine[xicon[x]<<2];
	       invertAlpha = ((int)255-srcPtr[3])*(((((x++)>>3)+(y>>3))&1) ? DARK_CHECK : LIGHT_CHECK);
	       *dst8++ = (unsigned char) ((invertAlpha + (int)srcPtr[0]*srcPtr[3])>>8);
	       *dst8++ = (unsigned char) ((invertAlpha + (int)srcPtr[1]*srcPtr[3])>>8);
	       *dst8++ = (unsigned char) ((invertAlpha + (int)srcPtr[2]*srcPtr[3])>>8);
	      }
	     break;
	    case LOAD_TRUE_COL:
	     while (x<iconEndX)
	      {
	       icmapPtr = &iconLine[xicon[x++]*3];
               *dst8++ = *icmapPtr++;
	       *dst8++ = *icmapPtr++;
	       *dst8++ = *icmapPtr;
              }
	     break;
	    case LOAD_CMAP:
	     while (x<iconEndX)
              {
  	       icmapPtr = &iconCmap[iconLine[xicon[x++]]*3];
	       *dst8++ = *icmapPtr++; *dst8++ = *icmapPtr++; *dst8++ = *icmapPtr;
	      }
	     break;
	    case LOAD_GREY:
	     while (x<iconEndX)
	      { *dst8++ = iconLine[xicon[x]]; *dst8++ = iconLine[xicon[x]]; *dst8++ = iconLine[xicon[x++]]; }
	     break;
	    case LOAD_BW:
	     while (x<iconEndX)
	      {
	       if (iconLine[xicon[x]>>3]&(128>>(xicon[x]&7)))
		{ *dst8++ = 0; *dst8++ = 0; *dst8++ = 0; }
	       else
		{ *dst8++ = 0xFF; *dst8++ = 0xFF; *dst8++ = 0xFF; }
               x++;
	      }
	     break;
           }
	  while (x<dispXY)
	   { *dst8++ = 0xBF; *dst8++ = 0xBF; *dst8++ = 0xBF; x++; }
	 }
       }
     break;

     case 2:
      for (y=0; y<(iconSize+2); y++)
       {
	dst16 = (unsigned short *)&dstData[y*dstbpl];
        iconLine=&iData[iconBPL*(((y+yoff)/iconZoom)%iconY)];
	if ( noRepeatImage && ( (y < ((iconSize-iconZoomY)/2 + 2)) || (y >= ((iconSize-iconZoomY)/2 + iconZoomY + 2)) ) )
	 for (x=0; x<dispXY; x++)
          *dst16++ = (unsigned short) (((0xBF&248)<<8) | ((0xBF&252)<<3) | (0xBF>>3));
	else
         {
	  x=0;
	  while (x<iconStartX)
	   { *dst16++ = (unsigned short) (((0xBF&248)<<8) | ((0xBF&252)<<3) | (0xBF>>3)); x++; }
	  switch (iconType)
	   {   
	    case LOAD_RGBA:
	     while (x<iconEndX)
              {
	       srcPtr = &iconLine[xicon[x]<<2];
	       invertAlpha = ((int)255-srcPtr[3])*(((((x++)>>3)+(y>>3))&1) ? DARK_CHECK : LIGHT_CHECK);
               *dst16++ = (unsigned short) (
		((((invertAlpha + (int)srcPtr[0]*srcPtr[3])>>8)&248)<<8) |
		((((invertAlpha + (int)srcPtr[1]*srcPtr[3])>>8)&252)<<3) |
		 (((invertAlpha + (int)srcPtr[2]*srcPtr[3])>>8)>>3));
	      }
	     break;
	    case LOAD_TRUE_COL:
	     while (x<iconEndX)
              { *dst16++ = (unsigned short) (((iconLine[xicon[x]*3]&248)<<8) | ((iconLine[xicon[x]*3+1]&252)<<3) | (iconLine[xicon[x]*3+2]>>3)); x++; }
	     break;
	    case LOAD_CMAP:
	     while (x<iconEndX)
              {
	       icmapPtr = &iconCmap[iconLine[xicon[x++]]*3];
               *dst16++ = (unsigned short) ((*icmapPtr&248)<<8) | ((icmapPtr[1]&252)<<3) | (icmapPtr[2]>>3);
	      }
	     break;
	    case LOAD_GREY:
	     while (x<iconEndX)
              { *dst16++ = (unsigned short) ((iconLine[xicon[x]]&248)<<8) | ((iconLine[xicon[x]]&252)<<3) | (iconLine[xicon[x]]>>3); x++; }
	     break;
	    case LOAD_BW:
	     while (x<iconEndX)
              { *dst16++ = (unsigned short) ((iconLine[xicon[x]>>3]&(128>>(xicon[x]&7))) ? 0 : (((0xFF&248)<<8) | ((0xFF&252)<<3) | (0xFF>>3))); x++; }
 	     break;
           }
	  while (x<dispXY)
	   { *dst16++ = (unsigned short) (((0xBF&248)<<8) | ((0xBF&252)<<3) | (0xBF>>3)); x++; }	 
	 }
       }
     break;
    }
   return(TRUE);
  }
 else
  return(FALSE);
}

/* directory comparison function */
DBROWSEIO int dirBrowseCmp(const void *arg1, const void *arg2)
{
 const dirBrowse *d1=arg1, *d2=arg2;

 return(strcmp(d1->name, d2->name));
}

/* count the number of directories and files in the given path */
DBROWSEIO void countDirFiles(const char *loadPath, int *numDir, int *numFiles)
{
 DIR *d;
 struct dirent *di;
 int dirNamelen;
 struct stat statBuf;
 char filen[MAX_PATH_FILE];
 char root[2];

 root[1]=0;
 root[0]=slash;
 *numDir = 0;
 *numFiles = 0;
 d = opendir(strlen(loadPath) ? loadPath : root); /* handles root directory, too */
 if (d)
  {
   dirNamelen = strlen(loadPath);
   strcpy(filen, loadPath);
   filen[dirNamelen]=slash;
   while ((di=readdir(d)))
    if (di->d_name[0]!='.')
     {
#if (!defined(__MINGW32__)) && (!defined(__CYGWIN__)) && (!defined(__APPLE__))
      /* only Btrfs, ext2, ext3, and ext4 support d_type, so we have to read status by stat as well */
      if (di->d_type == DT_REG)
       *numFiles+=1;
      else if (di->d_type == DT_DIR)
       *numDir+=1;
      else if (di->d_type == DT_UNKNOWN)
#endif
       {
	filen[dirNamelen+1]=0;
	strcat(&filen[dirNamelen+1], di->d_name);
	if (!stat(filen, &statBuf))
	 {
	  /* filter out the current directory and the ones starting with "." (hidden) */
	  if (S_ISDIR(statBuf.st_mode))
	    if (access(filen, R_OK)==0)
	     *numDir+=1;

	  if (S_ISREG(statBuf.st_mode))
	   if (access(filen, R_OK)==0)
	    *numFiles+=1;
	 }
       }
     }
   closedir(d);
  }
}
/* return true if image browser is currently loading the image, so we don't start another loading from the main program */
DBROWSEIO int imageBrowserNowLoading(void)
{
 return(nowLoading);
}

