/* lpbm.c load pbm plus coded images */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "loadimg.h"

static FILE *fp;
static int usefulbpl;
static int bitsPerPixel;
static int imageX, imageY;
static int readSofar;
static char invertBits=FALSE;

#define HEADER_SIZE	54

static int tryLoad(char *filename, int *loadType, int *xpixels,
 int *ypixels, int *xdpi, int *ydpi, unsigned char *cmap, char *info,
 char *reverse, int *numInterlace, int *coreFile, int realLoad)
{
 unsigned char header[HEADER_SIZE];
 int i, compression;
 int cmapColors;

 if ((fp = fopen(filename, "rb" )) == NULL)
  return(LOAD_IO_ERROR);

 if (fread(header, 1, HEADER_SIZE, fp)!=HEADER_SIZE)
  { fclose(fp); return(LOAD_FILETYPE_UNKNOWN); }
 
  /* check BMP header ID */
 if ((header[0]!='B')||(header[1]!='M'))
  { fclose(fp); return(LOAD_FILETYPE_UNKNOWN); }

 /* check if the length of second level header is 40 */
 if ((header[17]!=0)||(header[16]!=0)||(header[15]!=0)||(header[14]!=40))
  { fclose(fp); return(LOAD_FILETYPE_UNKNOWN); }
 *xpixels = (header[21]<<24) + (header[20]<<16) + (header[19]<<8) + header[18];
 *ypixels = (header[25]<<24) + (header[24]<<16) + (header[23]<<8) + header[22];

 /* check mandatory number of image planes = 1 */
 if ((header[27]!=0)||(header[26]!=1))
  { fclose(fp); return(LOAD_FILETYPE_UNKNOWN); }
 bitsPerPixel =(header[29]<<8) + header[28];

 compression = (header[33]<<24) + (header[32]<<16) + (header[31]<<8) + header[30];

 /* check type of compression */
 if (compression!=0)
  { fclose(fp); return(LOAD_FILETYPE_UNKNOWN); }

 /* get the pixels per meter and convert them to dpi */
 *xdpi = (header[41]<<24) + (header[40]<<16) + (header[39]<<8) + header[38];
 *xdpi = (int)(*xdpi*25.4);
 *ydpi = (header[45]<<24) + (header[44]<<16) + (header[43]<<8) + header[42];
 *ydpi = (int)(*ydpi*25.4);

 cmapColors = (header[49]<<24) + (header[48]<<16) + (header[47]<<8) + header[46];
 if ((cmapColors==0)&&(bitsPerPixel!=24))
  cmapColors = 1<<bitsPerPixel;

 switch (bitsPerPixel)
  {
   case 1:
    *loadType = LOAD_BW;
    usefulbpl = (*xpixels+7)/8;
    break;
   case 4:
    *loadType = LOAD_CMAP;
    usefulbpl = (*xpixels+1)/2;
    break;
   case 8:
    *loadType = LOAD_CMAP;
    usefulbpl = *xpixels;
    break;
   case 24:
    *loadType = LOAD_TRUE_COL;
    usefulbpl = *xpixels*3;
    break;
   default:
    fclose(fp);
    return(LOAD_FILETYPE_UNKNOWN);
    break;
  }

 if ((*xpixels<=0)||(*ypixels<=0)||(*xpixels>MAX_LOAD_IMAGE)||(*ypixels>MAX_LOAD_IMAGE))
  { fclose(fp); return(LOAD_FILE_CORRUPT); }

 if (realLoad)
  {
   for (i=0; i<cmapColors; i++)
    {
     cmap[i*3+2] = (unsigned char) fgetc(fp);
     cmap[i*3+1] = (unsigned char) fgetc(fp);
     cmap[i*3]   = (unsigned char) fgetc(fp);
     fgetc(fp);
    }
   invertBits = (char) ((bitsPerPixel==1) && (((int)cmap[0]+cmap[1]+cmap[2])<((int)cmap[3]+cmap[4]+cmap[5])));
  }
 imageX = *xpixels;
 imageY = *ypixels;
 readSofar =0;
 strcpy(info, "PBM plus");
 *reverse = FALSE;
 *numInterlace = 1; /* no interlace */
 *coreFile = TRUE;
 return(LOAD_OK);
}

/* loader reads a band */
static int loadBand(unsigned char *data, int  lines, int  bpl)
{
 int i,j, dsty;
 int srcbpl = ((usefulbpl + 3)>>2)<<2;
 unsigned char buff[srcbpl], *bufPtr, *dataPtr;

 for (i=0; i<lines; i++)
  {
   dsty = imageY-readSofar-i-1;
   if (fread(buff, 1, srcbpl, fp)!=srcbpl)
    return(LOAD_FILE_CORRUPT);
   bufPtr = buff;
   switch (bitsPerPixel)
    {
     case 24:
      dataPtr = &data[dsty*bpl];
      for (j=usefulbpl/3; j; j--)
       {
	dataPtr[2] = *bufPtr++;
	dataPtr[1] = *bufPtr++;
	dataPtr[0] = *bufPtr++;
	dataPtr+=3;
       }
      break;
     case 4:
      dataPtr = &data[dsty*bpl];
      for (j=0; j<imageX; j++)
       *dataPtr++ = (j&1) ? *bufPtr++&15 : *bufPtr>>4;
      break;
     default:
      if (invertBits)
       for (j=0; j<usefulbpl; j++)
        { *bufPtr = (unsigned char) (~*bufPtr); bufPtr++; }
      memcpy(&data[dsty*bpl], buff, usefulbpl);
      break;
    }
  }
 readSofar+=lines;
 return(LOAD_OK);
}

/* closes the input file and releases any temporary storage */
static int endLoad()
{
 fclose(fp);
 return(LOAD_OK);
}

/* register bmp image loader */
void registerLoadBMP()
{
 registerLoader(*tryLoad, *loadBand, *endLoad);
}
