/* lpbm.c load pbm plus coded images */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "loadimg.h"

static FILE *fp;
static int srcbpl;

#define LINE_LENGTH	300
static int tryLoad(char *filename, int *loadType, int *xpixels,
 int *ypixels, int *xdpi, int *ydpi, unsigned char *cmap, char *info,
 char *reverse, int *numInterlace, int *coreFile, int realLoad)
{
 char dummy[LINE_LENGTH];
 int i, range;
 char ptype;

 if ((fp = fopen(filename, "rb" )) == NULL)
  return(LOAD_IO_ERROR);

 fgets(dummy, LINE_LENGTH, fp);
 dummy[LINE_LENGTH-1]=0;
 ptype = dummy[1];
 if ((dummy[0]!='P')||(!(isdigit((int) ptype)))||(strlen(dummy)!=3))
  { fclose(fp); return(LOAD_FILETYPE_UNKNOWN); }

 if ((ptype!='4')&&(ptype!='5')&&(ptype!='6'))
  { fclose(fp); return(LOAD_FILETYPE_UNKNOWN); }
 fgets(dummy, LINE_LENGTH, fp);
 dummy[LINE_LENGTH-1]=0;
 sprintf(info, "Pbm Plus type P%c\n%s", ptype, ((dummy[0]=='#') ? &dummy[1]:""));
 while (dummy[0]=='#')
  fgets(dummy, LINE_LENGTH, fp);
 dummy[LINE_LENGTH-1]=0;

 sscanf(dummy, "%5d %5d\n", xpixels, ypixels);
 if ((*xpixels<=0)||(*ypixels<=0)||(*xpixels>MAX_LOAD_IMAGE)||(*ypixels>MAX_LOAD_IMAGE))
  { fclose(fp); return(LOAD_FILE_CORRUPT); }

 if (ptype=='4')
  {
   srcbpl = (*xpixels+7)/8;
   *loadType = LOAD_BW;
  }
 if (ptype=='5')
  {
   fgets(dummy, LINE_LENGTH, fp);
   dummy[LINE_LENGTH-1]=0;
   range = atoi(dummy);
   if ((range<2)||(range>255))
    { fclose(fp); return(LOAD_FILE_CORRUPT); }

   for (i=0; i<=range; i++)
    cmap[i*3]=cmap[i*3+1]=cmap[i*3+2]= (unsigned char) ((i*range)/255);
   srcbpl = *xpixels;
   *loadType = LOAD_CMAP;
  }
 if (ptype=='6')
  {
   fgets(dummy, LINE_LENGTH, fp);
   dummy[LINE_LENGTH-1]=0;
   range = atoi(dummy);
   if (range!=255)
    { fclose(fp); return(LOAD_FILE_CORRUPT); }
   srcbpl = *xpixels*3;
   *loadType = LOAD_TRUE_COL;
  }

 *xdpi = *ydpi = 0;
 *reverse = FALSE;
 *numInterlace = 1; /* no interlace */
 *coreFile = TRUE;
 return(LOAD_OK);
}

/* loader reads a band */
static int loadBand(unsigned char *data, int  lines, int  bpl)
{
 int i;

 if (bpl==srcbpl) /* read in one block step, if destination bpl is equal to the source */
  {
   if (fread((char *)data, bpl, lines, fp)!=lines)
    return(LOAD_FILE_CORRUPT);
  }
 else
  for (i=0;i<lines; i++)
   if (fread((char *)&data[i*bpl], 1, srcbpl, fp)!=srcbpl)
    return(LOAD_FILE_CORRUPT);

 return(LOAD_OK);
}

/* closes the input file and releases any temporary storage */
static int endLoad()
{
 fclose(fp);
 return(LOAD_OK);
}

/* register pbm plus image loader */
void registerLoadPBM()
{
 registerLoader(*tryLoad, *loadBand, *endLoad);
}
