/* lpng.c */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <png.h>
#include "loadimg.h"

static FILE *fp=NULL;
static png_structp   png_ptr;
static png_infop     info_ptr;
static int npasses, ytotal, readSoFar;
static int color_type;
static png_uint_32 width;
static unsigned char *myCmap;
static unsigned char alpha[256];
static unsigned char haveAlpha = FALSE;

static int tryLoad(char *filename, int *loadType, int *xpixels,
 int *ypixels, int *xdpi, int *ydpi, unsigned char *cmap, char *info,
 char *reverse, int *numInterlace, int *coreFile, int realLoad)
{
 unsigned char header[8];
 int i;
 png_uint_32 height;
 int bit_depth, interlace_type, num_palette;
 png_colorp palette;
 png_bytep trans_alpha;
 int num_trans;
 png_color_16p trans_color;

 if ((fp = fopen(filename, "rb" )) == NULL)
  return(LOAD_IO_ERROR);
 fread(header, 1, 8, fp);
// if (!png_check_sig(header, 8))
 if (png_sig_cmp(header, 0, 8))
  { fclose(fp); return(LOAD_FILETYPE_UNKNOWN); }

 png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
 if (!png_ptr)
  { fclose(fp); return(LOAD_NO_MEMORY); }

 info_ptr = png_create_info_struct(png_ptr);
 if (!info_ptr)
  {
   png_destroy_read_struct(&png_ptr, (png_infopp) NULL, (png_infopp) NULL);
   fclose(fp); 
   return(LOAD_NO_MEMORY);
  }

// if (setjmp(png_ptr->jmpbuf))
 if (setjmp(png_jmpbuf(png_ptr)))
  {
   png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp) NULL);
   fclose(fp);
   return(LOAD_IO_ERROR);
  }

 png_init_io(png_ptr, fp);
 /* tell the library how many bytes of signature have we already read */
 png_set_sig_bytes(png_ptr, 8);
 png_read_info(png_ptr, info_ptr);
 png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth, &color_type, &interlace_type, NULL, NULL);

 if (png_get_valid(png_ptr, info_ptr, PNG_INFO_PLTE))
  {
   png_get_PLTE(png_ptr, info_ptr, &palette, &num_palette);
   for (i = 0;  i < num_palette;  i++)
    {
     cmap[i*3]   = palette[i].red;
     cmap[i*3+1] = palette[i].green;
     cmap[i*3+2] = palette[i].blue;
    }
  }

 if (png_get_valid(png_ptr, info_ptr, PNG_INFO_pHYs))
  {
   png_uint_32 res_x, res_y;
   int unit_type;

   png_get_pHYs(png_ptr, info_ptr, &res_x, &res_y, &unit_type);
   *xdpi = (int) (res_x*25.4 + 0.5);
   *ydpi = (int) (res_y*25.4 + 0.5);
  }
 else
  *xdpi = *ydpi = 0;

 haveAlpha = FALSE;
 if (png_get_tRNS(png_ptr, info_ptr, &trans_alpha, &num_trans, &trans_color))
  if ( (color_type==PNG_COLOR_TYPE_PALETTE) || (color_type==PNG_COLOR_TYPE_GRAY))
   {
    haveAlpha = TRUE;
    memset(alpha, 255, 255);
    if (color_type==PNG_COLOR_TYPE_GRAY) /* only one transparent grey */
     alpha[trans_color->gray&255] = 0;
    else
     for (i=0; i<num_trans; i++)
      alpha[i] = trans_alpha[i];
   }
 
 /* our treatment of black and white is inverted with respect to PNG so tell it */
 png_set_invert_mono(png_ptr);

 /* documentation says it will return 1 if it is not interlaced */
 npasses = png_set_interlace_handling(png_ptr);
 switch (color_type)
  {
   case PNG_COLOR_TYPE_PALETTE:
    *loadType=LOAD_CMAP;
    if (bit_depth < 8)
     png_set_packing(png_ptr);
    /* expand colormap alpha pictures into RGBA */
    if (haveAlpha)
     *loadType=LOAD_RGBA;
    break;

   case PNG_COLOR_TYPE_RGB:
    if (bit_depth == 16)
     png_set_strip_16(png_ptr);
    *loadType=LOAD_TRUE_COL;
    break;

   case PNG_COLOR_TYPE_GRAY_ALPHA:
   case PNG_COLOR_TYPE_GRAY:
    if ((bit_depth < 8)&&(bit_depth>1))
     png_set_packing(png_ptr);
    *loadType=LOAD_CMAP;
    if (bit_depth == 16)
     { png_set_strip_16(png_ptr); bit_depth = 8; }
    switch (bit_depth)
     {
      case 1:
       cmap[0]=cmap[1]=cmap[2]=0;
       cmap[3]=cmap[4]=cmap[5]=255;
       *loadType=LOAD_BW;
       break;
      case 2:
       cmap[0]=cmap[1]=cmap[2]=255;
       cmap[3]=cmap[4]=cmap[5]=170;
       cmap[6]=cmap[7]=cmap[8]=85; /* 255/3 */
       cmap[9]=cmap[10]=cmap[11]=0;
       break;
      case 4:
       for (i = 0;  i < 16;  ++i)
	cmap[i*3]=cmap[i*3+1]=cmap[i*3+2]= (unsigned char) ((15-i)*17);
       break;
      case 8:
       for (i = 0;  i < 256;  ++i)
	cmap[i*3]=cmap[i*3+1]=cmap[i*3+2]= (unsigned char) (255-i);
       break;
     }
    /* expand grey alpha ang gray transparent colormap pictures into RGBA */
    if ( (color_type==PNG_COLOR_TYPE_GRAY_ALPHA) || haveAlpha)
     *loadType=LOAD_RGBA;
    break;

   case PNG_COLOR_TYPE_RGB_ALPHA:
    if (bit_depth == 16)
     png_set_strip_16(png_ptr);
    *loadType=LOAD_RGBA;
    break;
  }
 myCmap = cmap; /* the colormap pointer must stay valid until endload, or we will crash in loadBand() */
 png_read_update_info(png_ptr, info_ptr);

 strcpy(info, "PNG");
 *xpixels = width;
 *ypixels = ytotal = height;
 *reverse = FALSE;
 *numInterlace = interlace_type;
 *coreFile = TRUE;
 readSoFar=0;
 return(LOAD_OK);
}

/* loader reads a band */
static int loadBand(unsigned char *data, int  lines, int  bpl)
{
 png_bytep row_pointers[lines+1000];
 int i,j;
 unsigned char *linePtr;

 if ((!png_ptr) || (!info_ptr)) /* can't load if we lost a pointers to png structure */
  return(LOAD_IO_ERROR);

 /* need to set it for every function which calls routines from PNG library */
// if (setjmp(png_ptr->jmpbuf))
 if (setjmp(png_jmpbuf(png_ptr)))
  return(LOAD_IO_ERROR); /* the acutal destroy code will be called in endLoad */

 for (i=0;  i < lines;  i++)
  row_pointers[i] = (png_bytep) &data[i*bpl];

 for (i=0; i<npasses; i++)
  png_read_rows(png_ptr, row_pointers, NULL, lines);

 /* in memory backwards expansion to RGBA for formats which would otherwise be unsupported */
 if (color_type==PNG_COLOR_TYPE_GRAY_ALPHA)
  for (i=0;  i < lines;  i++)
   {
    linePtr = &data[i*bpl];
    for (j=width-1; j>=0; j--)
     {
      linePtr[(j<<2)+3]=linePtr[(j<<1)+1]; 
      linePtr[(j<<2)+2]=myCmap[linePtr[j<<1]*3];
      linePtr[(j<<2)+1]=myCmap[linePtr[j<<1]*3];
      linePtr[(j<<2)]  =myCmap[linePtr[j<<1]*3];
     }
   }
 if (haveAlpha)
  {
   if (color_type==PNG_COLOR_TYPE_GRAY)
    for (i=0;  i < lines;  i++)
     {
      linePtr = &data[i*bpl];
      for (j=width-1; j>=0; j--)
       {
	linePtr[(j<<2)+3]= alpha[myCmap[linePtr[j]*3]]; 
	linePtr[(j<<2)+2]=myCmap[linePtr[j]*3];
	linePtr[(j<<2)+1]=myCmap[linePtr[j]*3];
	linePtr[(j<<2)]  =myCmap[linePtr[j]*3];
       }
     }
   if (color_type==PNG_COLOR_TYPE_PALETTE)
    for (i=0;  i < lines;  i++)
     {
      linePtr = &data[i*bpl];
      for (j=width-1; j>=0; j--)
       {
	linePtr[(j<<2)+3]=alpha[linePtr[j]]; 
	linePtr[(j<<2)+2]=myCmap[linePtr[j]*3+2];
	linePtr[(j<<2)+1]=myCmap[linePtr[j]*3+1];
	linePtr[(j<<2)]  =myCmap[linePtr[j]*3];
       }
     }
  }
 readSoFar+=lines;
 /* cannot call this at endLoad, since sometimes endLoad did not read the whole image */
 if (readSoFar==ytotal)
  png_read_end(png_ptr, info_ptr);
 return(LOAD_OK);
}

/* closes the input file and releases any temporary storage */
static int endLoad()
{
 if (png_ptr && info_ptr) /* need to have pointers in order to free them */
  png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
 if (fp)
  fclose(fp);
 fp=NULL;
 return(LOAD_OK);
}

/* register PNG image loader */
void registerLoadPNG()
{
 registerLoader(*tryLoad, *loadBand, *endLoad);
}
