/***************************************************************************
                          apaintconfig.cpp  -  description
                             -------------------
    begin                : Mon Aug 25 2003
    copyright            : (C) 2003-2014 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "apaintconfig.h"
#include <iostream>
#include <qfile.h>
#include <qtextstream.h>
#include <qdir.h>
#include <qtextcodec.h>
#include <qstringlist.h>
#include <stdlib.h>
#include <qfontmetrics.h>
#include <qfont.h>
#include <qpaintdevice.h>
#include "apaint.h"
extern "C"{
#include "../../iolib/acolor.h"
#include "../../iolib/browseio.h"
#include "../../llib/loadimg.h"
}

#define equal(str1,str2)        !strcasecmp((str1),(str2))

#define PATH_IMAGES_TAG		0
#define LANG_TAG		1
#define FONT_SIZE_TAG		2
#define FONT_TAG		3
#define CLEAR_ICON_CACHE_TAG	4
#define FONT_SMALL_TAG		5
#define RULER_ON_TAG		6
#define HIDE_SAVE_MENU		7
#define SCREEN_X_DPI		8
#define SCREEN_Y_DPI		9
//#define SCREEN_WIDTH_PIXELS	10
//#define SCREEN_HEIGHT_PIXELS	11
#define METRIC_SYSTEM		12
#define T_PRINT_COMMAND		13
#define PRINTER_NAME_PRINTCAP	14
#define FONT_PRINT_TAG		15
#define PRINTER_TYPE		16
#define BROWSE_WITH_FILTER	17
#define OPEN_WITH_BROWSE	18
#define FONT_XFT		19
#define XFT_STATUS		20
#define DEF_SIZE_MAIN		21
#define DEF_SIZE_IMAGE_BROWSE	22
#define USE_SHARED_MEMORY	23
#define PRINTER_XDPI		24
#define PRINTER_YDPI		25
#define PRINT_LEFT_BORDER	26
#define PRINT_UPPER_BORDER	27
#define PRINT_WIDTH		28
#define PRINT_HEIGHT		29
#define NUM_RECENT_FILES	30
#define DIVIDE_RECENT_FILES_ICON 31
#define RECENT_FILE_WITH_PATH	32
#define SHOW_RECENTLY_READ	33
#define SHOW_RECENT_FILES_DETAIL 34
#define SHOW_RECENT_FILES_DATE	35
#define T_GZIP_COMMAND		36
#define UNZIP_COMMAND		37
#define T_TIFF_LZW		38
#define HIDE_LOAD_RECENT	39
#define HIDE_LOAD_MENU		40
#define KEEP_COLORMAP		41
#define T_ACROBAT_COMMAND	42
#define T_WWW_BROWSER_COMMAND	43
#define BROWSE_IMAGE_DISPLAY	44
#define T_FIRST_PALETTE_COLOR	45
#define T_FILENAME_PATH_IN_WINDOW_TITLE	46
#define T_IMAGE_FORMAT		47
/* don't forget to increase MAX_TAGS, when adding entries */


// ZZZZZZZ  print dpi and print size done, but still need to put it in action...

#define MAX_TAGS    		48
#define MAX_MSGS    		1000

#define C_DIR1    "../common/"
#define C_DIR2    "../../common/"
#define MSG_DIR   "./msg/"

#define FONT_FAMILY    1
#define FONT_WEIGHT    2
#define FONT_WIDTH     4

#define FONT_LIGHT      25
#define FONT_NORMAL     50
#define FONT_BOLD       75
#define FONT_DEMIBOLD   63
#define FONT_BLACK      87

static const char *tags[] = {
 "#PATH_IMAGES=",
 "#MESSAGES_FILE=",
 "#FONT_NORMAL_PT=",
 "#FONT_SCALE=",
 "#CLEAR_ICON_CACHE_DAYS=",
 "#FONT_SMALL_PT=",
 "#RULER_ON=",
 "#HIDE_SAVE_MENU=",
 "#SCREEN_X_DPI=",
 "#SCREEN_Y_DPI=",
 "",
 "",
 "#METRIC_SYSTEM=",
 "#PRINT_COMMAND=",
 "#PRINTER_NAME_PRINTCAP=",
 "#FONT_SCALE_PRINT=",
 "#PRINTER_TYPE=",
 "#BROWSE_WITH_FILTER=",
 "#OPEN_WITH_BROWSE=",
 "#FONT_XFT=",
 "#XFT_STATUS=",
 "#DEF_SIZE_MAIN=",
 "#DEF_SIZE_IMAGE_BROWSE=",
 "#USE_SHARED_MEMORY=",
 "#PRINTER_XDPI=",
 "#PRINTER_YDPI=",
 "#PRINT_LEFT_BORDER=",
 "#PRINT_UPPER_BORDER=",
 "#PRINT_WIDTH=",
 "#PRINT_HEIGHT=",
 "#NUM_RECENT_FILES=",
 "#DIVIDE_RECENT_FILES_ICON=",
 "#RECENT_FILE_WITH_PATH=",
 "#SHOW_RECENTLY_READ=",
 "#SHOW_RECENT_FILES_DETAIL=",
 "#SHOW_RECENT_FILES_DATE=",
 "#GZIP_COMMAND=",
 "#UNZIP_COMMAND=",
 "#TIFF_LZW=",
 "#HIDE_LOAD_RECENT=",
 "#HIDE_LOAD_MENU=",
 "#KEEP_COLORMAP=",
 "#ACROBAT_COMMAND=",
 "#WWW_BROWSER_COMMAND=",
 "#BROWSE_IMAGE_DISPLAY=",
 "#FIRST_PALETTE_COLOR=",
 "#FILENAME_PATH_IN_WINDOW_TITLE=",
 "#IMAGE_FORMAT=",
};

#define ICON_PATH   "../../common/icons/"
#define ICON_PATH2  "../common/icons/"

#define TMPDIR "/tmp/"
#define MAX_DAYS_OLD  7

static int lenTags[MAX_TAGS];
static QString *msgList[MAX_MSGS];
static QString getData(QString line, int tagNum);
static int getWeightFromStr(QString weight);

static QFile *cfgFile;
static QString cfgFilename;
static QString imagesPath, langData, fontName, tmp, printerType,
               iconPath, printCommand, printCap, fontNamePrint,
	       fontXFT, acrobatCommand, wwwBrowserCommand, imageFormat;
static int fontSize, smallFontSize;
static int screenDpiX = 100000, screenDpiY = 1000000; /* default 100 dpi * 1000 */
static int weight;
static bool italic;
static int cIconCacheDays=100;
static int rulerOn=TRUE;
static bool useMetric=TRUE;
static bool browseWithFilter=TRUE;
static bool openWithBrowse=FALSE;
static bool useSharedMemory=TRUE;
static bool filenamePathInWindowTitle=FALSE;
static bool hideSaveMenu=FALSE;
static bool hideLoadMenu=TRUE;
static bool hideLoadRecent=FALSE;
static bool keepColormap=FALSE;
static bool useXFT=TRUE;
static bool boldXFT=FALSE;
static bool italicXFT=FALSE;
static bool recentFilesWithPath=TRUE;
static bool showRecentlyRead = TRUE;
static bool showRecentFilesDetail = TRUE;
static bool showRecentFilesDate = TRUE;
static int firstPaletteColor = 0;
static int initialWindowSizeX=800;
static int initialWindowSizeY=750;
static int initialBrowseSizeX=800;
static int initialBrowseSizeY=750;
static int printerXdpi=720;
static int printerYdpi=720;
static int numRecentFiles=10;
static int divideRecentFilesIcon=4;
static int browseImageDisplay=IMAGE_ICON_FILENAME | IMAGE_ICON_IMAGESIZE | IMAGE_ICON_REPEAT | IMAGE_ICON_SUB_DIR | IMAGE_ICON_NO_SUFFIX;
float leftBorderMM=3;
float upperBorderMM=3;
float printWidthMM=203;
float printHeightMM=279;

#ifdef __APPLE__
#define PRINT_COMMAND	"/usr/bin/lp -o raw -dlp -n%d "
#define ACROBAT_COMMAND "open -a Preview"
#define BROWSER_COMMAND "open -a Safari"
#define GZIP_COMMAND	"/usr/bin/gzip"
#else
#define PRINT_COMMAND	"/usr/bin/lpr -o raw -Plp -#%d "
#define ACROBAT_COMMAND "/usr/bin/acroread"
#define BROWSER_COMMAND "/usr/bin/firefox"
#define GZIP_COMMAND	"/bin/gzip"
#endif

// fileDate should not be newer than now
static bool isOld(QDate now, QDate fileDate)
{
  if (now.year()> fileDate.year()) return true;
  if (now.month() > fileDate.month() ) return true;
  if((now.month() == fileDate.month()) && ((now.day() - fileDate.day())>MAX_DAYS_OLD))
   return true;
  return false;
}

void ApaintConfig::initConfig()
{
 cfgFile = new QFile();
 for (int i=0; i<MAX_TAGS; i++)
  lenTags[i] = strlen(tags[i]);

 QDir dir;
 dir.setPath(ICON_PATH);
 if (dir.exists())
  iconPath = dir.path()+"/";
 else
  {
   dir.setPath(ICON_PATH2);
   if (dir.exists())
    iconPath = dir.path()+"/";
  }
 fontSize = 12;
 smallFontSize = 10;
 rulerOn = 1;
 cIconCacheDays = 100;
 weight = FONT_NORMAL;
 imagesPath = "../data/img";
 langData = "eng.txt";
 fontName = "Helvetica";
 fontNamePrint = "Helvetica";
 iconPath = ICON_PATH;
 italic = FALSE;
 printCommand = PRINT_COMMAND;
 acrobatCommand = ACROBAT_COMMAND;
 wwwBrowserCommand = BROWSER_COMMAND;
 imageFormat = "png";
 llibSetGzipCommand(GZIP_COMMAND);
 llibSetTmpDir(TMPDIR);

 screenDpiX = 100;
 screenDpiY = 100;
 useMetric = true;
 printCap = "";
 printerType = "";
}

void ApaintConfig::clearTmpDir()
{
 QDateTime fdate;
 QDate now  = QDate::currentDate();

 QDir dir = QDir(TMPDIR);

 if (!dir.exists()) return;
 const QFileInfoList *dirList = dir.entryInfoList(QString("%1%2").arg(PFX_APAINT).arg("*"));
 if (dirList)
  {
   QFileInfoListIterator it(*dirList);
   QFileInfo *fi;
   while ((fi = it.current()) != 0)
    {
     ++it;
     if (!fi->isDir()) continue;
     fdate = fi->lastModified();
     if (isOld(now, fdate.date()))
      if (clearDir(fi->filePath()))
       printf("%s deleted...\n", fi->filePath().local8Bit().data()) ;
    }
  }
}

bool ApaintConfig::clearDir(QString dirName)
{
 QDir dir, dir2;
 uint i;

 dir=QDir(dirName);
 for (i=0; i<dir.count(); i++)
  {
   if (dir[i]!="." && dir[i]!="..")
    dir.remove(dir[i], true);
  }
 return (dir.rmdir(dirName, true));
}

int ApaintConfig::getNum(QString line)
{
 if (line[0]=='#')
  {
   QString numstr = line.section(' ', 0,0);
   numstr.remove(0,1);
   int num = numstr.toInt();
   if (num>=0 && num<MAX_MSGS)
    return num;
   else
    return 0;
  }
  return(0);
}

bool ApaintConfig::loadMsgFile(QString msg)
{
 int lineNum;
 QFile mfile;
 QString msgFilename = QString("apaint.%1.msg").arg(msg);
 
 mfile.setName(QString("%1%2").arg(MSG_DIR).arg(msgFilename));
 if (!mfile.open(IO_ReadOnly))
  return false;
 
 QTextStream mstream(&mfile);
 QString line, buf;
 lineNum = 0;
 while (!mstream.eof())
  {
   line = mstream.readLine();
   if (line[0]=='#')
    {
     if (buf.length()!=0)
      if (lineNum)
       msgList[lineNum]= new QString(buf);
      /* we ignore message 0, all encodings are in UTF8, regardless of the first message with encoding */
     lineNum = getNum(line);
     buf.truncate(0);
    }
   else
    buf+=line;   
  }
 return true;
}

QString ApaintConfig::getText(int num)
{
 if (num<0 || num>MAX_MSGS)
  return QString();

 if (!msgList[num])
   return QString();
 else
   return (*msgList[num]);
}

QString getData(QString line, int tagNum)
{
 return (line.right(line.length() - lenTags[tagNum]));
}

int getWeightFromStr(QString wstr)
{
 if (wstr == "bold")
  return(FONT_BOLD);
 else 
  return(FONT_NORMAL);
}

#define MAX_LINE 1000
void ApaintConfig::saveCfgData(char *language)
{
 char tmpFilename[500];
 char line[MAX_LINE];
 FILE *inF, *outF;
 int haveInput=TRUE;

 sprintf(tmpFilename, "%s.tmp", cfgFilename.local8Bit().data());

 if ((inF = fopen(cfgFilename.local8Bit().data(), "r" )) == NULL)
  haveInput=FALSE;

 if ((outF = fopen(tmpFilename, "w" )) == NULL)
  {
   if (haveInput)
    fclose(inF);
   return;
  }

 if (haveInput)
  {
   while (fgets(line, MAX_LINE, inF)!=NULL)
    {
     if (strncmp(line, tags[LANG_TAG], strlen(tags[LANG_TAG]))==0)
      fprintf(outF, "%s%s.txt\n", tags[LANG_TAG], language);
     else if (strncmp(line, tags[PATH_IMAGES_TAG], strlen(tags[PATH_IMAGES_TAG]))==0)
      fprintf(outF, "%s%s\n", tags[PATH_IMAGES_TAG], apaint->getImagePath().local8Bit().data());
     else if (strncmp(line, tags[KEEP_COLORMAP], strlen(tags[KEEP_COLORMAP]))==0)
      fprintf(outF, "%s%d\n", tags[KEEP_COLORMAP], apaint->keepOriginalColormap());
     else if (strncmp(line, tags[T_FIRST_PALETTE_COLOR], strlen(tags[T_FIRST_PALETTE_COLOR]))==0)
      fprintf(outF, "%s%d\n", tags[T_FIRST_PALETTE_COLOR], firstPaletteColor);
     else if (strncmp(line, tags[RULER_ON_TAG], strlen(tags[RULER_ON_TAG]))==0)
      fprintf(outF, "%s%d\n", tags[RULER_ON_TAG], apaint->rulerOn());
     else if (strncmp(line, tags[BROWSE_WITH_FILTER], strlen(tags[BROWSE_WITH_FILTER]))==0)
      fprintf(outF, "%s%d\n", tags[BROWSE_WITH_FILTER], browseWithFilter);
     else if (strncmp(line, tags[BROWSE_IMAGE_DISPLAY], strlen(tags[BROWSE_IMAGE_DISPLAY]))==0)
      fprintf(outF, "%s%d\n", tags[BROWSE_IMAGE_DISPLAY], browseImageDisplay);
     else if (strncmp(line, tags[DEF_SIZE_MAIN], strlen(tags[DEF_SIZE_MAIN]))==0)
      fprintf(outF, "%s%d %d\n", tags[DEF_SIZE_MAIN], apaint->getWindowSize().width(), apaint->getWindowSize().height());
     else
      fprintf(outF, "%s", line);
    }
   fclose(inF);
  }
 else
  {
   fprintf(outF, "%s%s.txt\n", tags[LANG_TAG], language);
   fprintf(outF, "%s%s\n", tags[PATH_IMAGES_TAG], apaint->getImagePath().local8Bit().data());
   fprintf(outF, "%s%d\n", tags[KEEP_COLORMAP], apaint->keepOriginalColormap());
   fprintf(outF, "%s%d\n", tags[T_FIRST_PALETTE_COLOR], firstPaletteColor);
   fprintf(outF, "%s%d\n", tags[RULER_ON_TAG], apaint->rulerOn());
   fprintf(outF, "%s%d\n", tags[BROWSE_IMAGE_DISPLAY], browseImageDisplay);
   fprintf(outF, "%s%d %d\n", tags[DEF_SIZE_MAIN], apaint->getWindowSize().width(), apaint->getWindowSize().height());
  }
 fclose(outF);
 rename(tmpFilename, cfgFilename.local8Bit().data());
}

bool ApaintConfig::loadCfgData(QString path)
{
/* should figure out one fo the way to get default dpi form the system */
//printf("dpi %d %d\n", QApplication::desktop()->physicalDpiX(), QApplication::desktop()->physicalDpiY());
//printf("dpi %d %d\n", QPaintDevice::physicalDpiX(), QPaintDevice::physicalDpiY());

  cfgFilename = path;
  cfgFile->setName(path);
  if (!cfgFile->open(IO_ReadOnly))  
   return false;

  QTextStream fstr(cfgFile);
  QString line, allName;
  while (!fstr.eof())
  {
    line = fstr.readLine();

    if (line.find(tags[PATH_IMAGES_TAG])!=-1)
     imagesPath = getData(line, PATH_IMAGES_TAG);
    else if (line.find(tags[LANG_TAG])!=-1)
     langData =  getData(line, LANG_TAG);
    else if (line.find(tags[FONT_TAG])!=-1)
    {
      allName =  getData(line, FONT_TAG);
      QStringList nameList = QStringList::split("-", allName);
      fontName = nameList[FONT_FAMILY];
      weight = getWeightFromStr(nameList[FONT_WEIGHT]);             
    }
    else if (line.find(tags[FONT_PRINT_TAG])!=-1)
    {
      allName =  getData(line, FONT_PRINT_TAG);
      QStringList nameList = QStringList::split("-", allName);
      fontNamePrint = nameList[FONT_FAMILY];
    }    
    else if (line.find(tags[FONT_SMALL_TAG])!=-1)
    {
      tmp = getData(line, FONT_SMALL_TAG);
      smallFontSize = tmp.toInt()/10;
    }
    else if (line.find(tags[FONT_SIZE_TAG])!=-1)
    {
      tmp =  getData(line, FONT_SIZE_TAG);
      fontSize = tmp.toInt()/10;
    }
    else if (line.find(tags[CLEAR_ICON_CACHE_TAG])!=-1)
    {
      tmp =  getData(line, CLEAR_ICON_CACHE_TAG);
      cIconCacheDays = tmp.toInt();
    }
    else if (line.find(tags[T_FIRST_PALETTE_COLOR])!=-1)
    {
      tmp = getData(line, T_FIRST_PALETTE_COLOR);
      firstPaletteColor = tmp.toInt();
    }
    else if (line.find(tags[RULER_ON_TAG])!=-1)
    {
      tmp = getData(line, RULER_ON_TAG);
      rulerOn = tmp.toInt();
    }
    else if (line.find(tags[SCREEN_X_DPI])!=-1)
    {
      tmp = getData(line, SCREEN_X_DPI);
      screenDpiX = tmp.toInt();
    }
    else if (line.find(tags[SCREEN_Y_DPI])!=-1)
    {
      tmp = getData(line, SCREEN_Y_DPI);
      screenDpiY = tmp.toInt();
    }
    else if (line.find(tags[T_IMAGE_FORMAT])!=-1)
     imageFormat =  getData(line, T_IMAGE_FORMAT);
    else if (line.find(tags[METRIC_SYSTEM])!=-1)
    {
      tmp = getData(line, METRIC_SYSTEM);
      useMetric = (bool)tmp.toInt();
    }
    else if (line.find(tags[T_PRINT_COMMAND])!=-1)
     printCommand = getData(line, T_PRINT_COMMAND);
    else if (line.find(tags[T_ACROBAT_COMMAND])!=-1)
     acrobatCommand = getData(line, T_ACROBAT_COMMAND);
    else if (line.find(tags[T_WWW_BROWSER_COMMAND])!=-1)
     wwwBrowserCommand = getData(line, T_WWW_BROWSER_COMMAND);
    else if (line.find(tags[T_GZIP_COMMAND])!=-1)
    {
      tmp = getData(line, T_GZIP_COMMAND);
      llibSetGzipCommand((char *) tmp.ascii());
    }
    else if (line.find(tags[UNZIP_COMMAND])!=-1)
    {
      tmp = getData(line, UNZIP_COMMAND);
      llibSetUnzipCommand((char *) tmp.ascii());
    }
    else if (line.find(tags[PRINTER_XDPI])!=-1)
     {
      tmp = getData(line, PRINTER_XDPI);
      printerXdpi = tmp.toInt();
     }
    else if (line.find(tags[PRINTER_YDPI])!=-1)
     {
      tmp = getData(line, PRINTER_YDPI);
      printerYdpi = tmp.toInt();
     }
    else if (line.find(tags[PRINTER_TYPE])!=-1)
    {
      printerType = getData(line, PRINTER_TYPE);
    }
    else if (line.find(tags[PRINT_LEFT_BORDER])!=-1)
     {
      tmp = getData(line, PRINT_LEFT_BORDER);
      leftBorderMM = tmp.toFloat();
     }
    else if (line.find(tags[PRINT_UPPER_BORDER])!=-1)
     {
      tmp = getData(line, PRINT_UPPER_BORDER);
      upperBorderMM = tmp.toFloat();
     }
    else if (line.find(tags[PRINT_WIDTH])!=-1)
     {
      tmp = getData(line, PRINT_WIDTH);
      printWidthMM = tmp.toFloat();
     }
    else if (line.find(tags[PRINT_HEIGHT])!=-1)
     {
      tmp = getData(line, PRINT_HEIGHT);
      printHeightMM = tmp.toFloat();
     }
    else if (line.find(tags[PRINTER_NAME_PRINTCAP])!=-1)
    {
     printCap = getData(line, PRINTER_NAME_PRINTCAP);
    }
    else if (line.find(tags[BROWSE_WITH_FILTER])!=-1)
    {
     tmp = getData(line, BROWSE_WITH_FILTER);
     browseWithFilter = (bool)tmp.toInt();
    }
    else if (line.find(tags[OPEN_WITH_BROWSE])!=-1)
    {
     tmp = getData(line, OPEN_WITH_BROWSE);
     openWithBrowse = (bool)tmp.toInt();
    }
    else if (line.find(tags[BROWSE_IMAGE_DISPLAY])!=-1)
    {
     tmp = getData(line, BROWSE_IMAGE_DISPLAY);
     browseImageDisplay = tmp.toInt();
    }
    else if (line.find(tags[FONT_XFT])!=-1)
    {
     fontXFT =  getData(line, FONT_XFT);
    }
    else if (line.find(tags[XFT_STATUS])!=-1)
    {
      int i1, i2, i3; /* all because of bool typecasting */
      tmp =  getData(line, XFT_STATUS);
      sscanf(tmp.ascii(), "%1d %1d %1d", &i1, &i2, &i3);
      useXFT = TRUE;
      italicXFT = i2;
      boldXFT = i3;
      if (useXFT)
       {
        weight = boldXFT ? FONT_BOLD : FONT_NORMAL;
        italic = italicXFT;
       }            
    }
    else if (line.find(tags[DEF_SIZE_MAIN])!=-1)
    {
     tmp =  getData(line, DEF_SIZE_MAIN);
     sscanf(tmp.ascii(), "%5d %5d", &initialWindowSizeX, &initialWindowSizeY);
    }
    else if (line.find(tags[DEF_SIZE_IMAGE_BROWSE])!=-1)
    {
     tmp =  getData(line, DEF_SIZE_IMAGE_BROWSE);
     sscanf(tmp.ascii(), "%5d %5d", &initialBrowseSizeX, &initialBrowseSizeY);
    }
    else if (line.find(tags[USE_SHARED_MEMORY])!=-1)
    {
     tmp = getData(line, USE_SHARED_MEMORY);
     useSharedMemory = (bool)tmp.toInt();
    }
    else if (line.find(tags[T_TIFF_LZW])!=-1)
    {
     tmp = getData(line, T_TIFF_LZW);
     setTiffLZW(tmp.toInt());
    }
    else if (line.find(tags[T_FILENAME_PATH_IN_WINDOW_TITLE])!=-1)
    {
     tmp = getData(line, T_FILENAME_PATH_IN_WINDOW_TITLE);
     filenamePathInWindowTitle = (bool)tmp.toInt();
    }
    else if (line.find(tags[HIDE_SAVE_MENU])!=-1)
    {
     tmp = getData(line, HIDE_SAVE_MENU);
     hideSaveMenu = (bool)tmp.toInt();
    }
    else if (line.find(tags[HIDE_LOAD_MENU])!=-1)
    {
     tmp = getData(line, HIDE_LOAD_MENU);
     hideLoadMenu = (bool)tmp.toInt();
    }
    else if (line.find(tags[HIDE_LOAD_RECENT])!=-1)
    {
     tmp = getData(line, HIDE_LOAD_RECENT);
     hideLoadRecent = (bool)tmp.toInt();
    }
    else if (line.find(tags[NUM_RECENT_FILES])!=-1)
    {
     tmp = getData(line, NUM_RECENT_FILES);
     numRecentFiles = tmp.toInt();
    }
    else if (line.find(tags[DIVIDE_RECENT_FILES_ICON])!=-1)
    {
     tmp = getData(line, DIVIDE_RECENT_FILES_ICON);
     divideRecentFilesIcon = tmp.toInt();
    }
    else if (line.find(tags[RECENT_FILE_WITH_PATH])!=-1)
    {
     tmp = getData(line, RECENT_FILE_WITH_PATH);
     recentFilesWithPath = (bool)tmp.toInt();
    }
    else if (line.find(tags[SHOW_RECENTLY_READ])!=-1)
    {
     tmp = getData(line, SHOW_RECENTLY_READ);
     showRecentlyRead = (bool)tmp.toInt();
    }
    else if (line.find(tags[SHOW_RECENT_FILES_DETAIL])!=-1)
    {
     tmp = getData(line, SHOW_RECENT_FILES_DETAIL);
     showRecentFilesDetail = (bool)tmp.toInt();
    }
    else if (line.find(tags[SHOW_RECENT_FILES_DATE])!=-1)
    {
     tmp = getData(line, SHOW_RECENT_FILES_DATE);
     showRecentFilesDate = (bool)tmp.toInt();
    }
    else if (line.find(tags[KEEP_COLORMAP])!=-1)
    {
     tmp = getData(line, KEEP_COLORMAP);
     keepColormap = (bool)tmp.toInt();
    }
  }

  cfgFile->close();
  if (useXFT)
   fontName = fontXFT;

  return true;
}

int ApaintConfig::getScreenDpi()
{
 int screenDPI = (int)((screenDpiX + screenDpiY)/2000);

 if (abs(100-screenDPI)<abs(75-screenDPI))
   screenDPI = 100;
 else
   screenDPI = 75;

 return screenDPI; 
}

int ApaintConfig::getScreenDpiX()
{
  return (screenDpiX/1000);
}

int ApaintConfig::getScreenDpiY()
{
  return (screenDpiY/1000);
}

int ApaintConfig::getPixelSize(int pointSize)
{
  int sdpi = getScreenDpi();
  if (sdpi == 75)
   return pointSize;

  return ((int)((pointSize*10)*sdpi/720));
  
}

int ApaintConfig::getBestPointSize(QFont &font, int pixelSize)
{
  QFontMetrics fm(font);

  int pointSize = font.pointSize();
  int hei = fm.height()-fm.descent();

  //just in case
  if (pixelSize < 8)
   pixelSize = 10;

  if (hei == pixelSize)
  {
   return pointSize;
  }
  else if (hei>pixelSize)
  {
    while (hei>pixelSize)
    {
      pointSize--;
      font.setPointSize(pointSize);
      fm = QFontMetrics(font);
      hei =  fm.height()-fm.descent();
    }
    if (hei<pixelSize) pointSize++;
  }
  else if (hei<pixelSize)
  {
    while (hei<pixelSize)
    {
      pointSize++;
      font.setPointSize(pointSize);
      fm = QFontMetrics(font);
      hei =  fm.height()-fm.descent();
    }
    if (hei>pixelSize) pointSize--;
  }
  return pointSize; 
}

int ApaintConfig::getBestPointSizeMainWin(QFont &font, int pixelSize)
{
  QFont tmpFont;
  
  tmpFont = QFont(font.family());
  tmpFont.setPixelSize(pixelSize);
  
  return tmpFont.pointSize();
}

/*
QString &ApaintConfig::getIconPath()
{
  QDir dir;
  iconPath = "./";
  
  if (dir.exists(ICON_PATH))
   iconPath = ICON_PATH;
  else if (dir.exists(ICON_PATH2))
   iconPath = ICON_PATH2;
   
  return(iconPath);
}
*/

QString ApaintConfig::getAcrobatCommand()
{
 return(acrobatCommand);
}

QString ApaintConfig::getWwwBrowserCommand()
{
 return(wwwBrowserCommand);
}

QString ApaintConfig::getPrintCommand(QString which)
{
 if ( which!= "" )
 {
   QStringList sl = QStringList::split(" ", printCommand);
   for (int i=0; i<(int)sl.count(); i++)
    if (sl[i].find("-P") != -1)
     sl[i] = "-P"+which;
   printCommand = sl.join(" ");
 }
 return printCommand;
}

bool ApaintConfig::makeTmpDir(const char *tmpdir, const char *pfx,  QString &dirCreated)
{
  char *dirName;
  QDir dir;
  QString path;
  bool ret;

  path=QString(tmpdir);
  if (!path.endsWith("/"))
   path.append("/");
  
  dirName = (char *) malloc(300);
  if (!dirName)
   return(FALSE);
  sprintf(dirName, "%s%sXXXXXX", path.local8Bit().data(), pfx);
  if (mkstemp(dirName)<0)
   { free(dirName); return(FALSE); }
  /* mkstemp creates file, but we need a directoy, so remove file and create
   a directory */
  remove(dirName);
 
  dirCreated=QString(dirName);
  dirCreated.append("/");
  ret=dir.mkdir(dirCreated, true);
  free(dirName);
  return(ret); 
}

void ApaintConfig::clearTmpDir(QString dirName)
{
  QDir dir, dir2;
  uint i;

  dir=QDir(dirName);
  dir.setFilter(QDir::Files | QDir::Hidden);
  for (i=0; i<dir.count(); i++)
  {
   if (dir[i]!="." && dir[i]!="..")
    dir.remove(dir[i], true);
  }
  dir.rmdir(dirName, true);
}

char *ApaintConfig::getManualLanguage()
{
 if (langData.startsWith("ita"))
  return((char *) "IT");
 if (langData.startsWith("tur"))
  return((char *) "TR");

 return((char *) "EN");
}

QString &ApaintConfig::getImgPath() {return imagesPath;}
QString &ApaintConfig::getLang() {return langData;}

int ApaintConfig::getFontSize() {return fontSize;}
int ApaintConfig::getSmallFontSize() { return smallFontSize;}
QString ApaintConfig::getFontName() {return fontName;}
int ApaintConfig::getFontWeight() {return weight;}
bool ApaintConfig::getFontItalic() {return italic;}
int ApaintConfig::getClearIconCacheDays() {return cIconCacheDays;}
int ApaintConfig::isRulerOn() { return rulerOn; }
//int ApaintConfig::getPrinterXdpi() {return printerXdpi;}
//int ApaintConfig::getPrinterYdpi() {return printerYdpi;}
int ApaintConfig::getNumRecentFiles() {return numRecentFiles;}
int ApaintConfig::getDivideRecentFilesIcon() {return divideRecentFilesIcon;}
int ApaintConfig::getFirstPaletteColor() {return firstPaletteColor;}
bool ApaintConfig::useMetricSystem(){return useMetric;}
bool ApaintConfig::startBrowseWithFilter(){return browseWithFilter;}
QString ApaintConfig::getPrintCap() {return printCap;}
QString ApaintConfig::getPrintFontName() {return fontNamePrint;}
QString ApaintConfig::getPrinterType() { return printerType;}
float ApaintConfig::getLeftBorderMM() { return leftBorderMM; }
float ApaintConfig::getUpperBorderMM() { return upperBorderMM; }
float ApaintConfig::getPrintWidthMM() { return printWidthMM; }
float ApaintConfig::getPrintHeightMM() { return printHeightMM; }
bool ApaintConfig::getOpenWithBrowse() { return openWithBrowse;}
bool ApaintConfig::getUseSharedMemory() { return useSharedMemory;}
bool ApaintConfig::getHideSaveMenu() { return hideSaveMenu;}
bool ApaintConfig::getHideLoadMenu() { return hideLoadMenu;}
bool ApaintConfig::getHideLoadRecent() { return hideLoadRecent;}
bool ApaintConfig::getRecentFilesWithPath() { return recentFilesWithPath;}
bool ApaintConfig::getShowRecentFilesDetail() { return showRecentFilesDetail;}
bool ApaintConfig::getShowRecentFilesDate() { return showRecentFilesDate;}
bool ApaintConfig::getUseXFT() { return useXFT;}
bool ApaintConfig::getKeepColormap() { return keepColormap;}
bool ApaintConfig::getFilenamePathInWindowTitle() { return filenamePathInWindowTitle;}

int ApaintConfig::getBrowseImageDisplay() { return browseImageDisplay; };
void ApaintConfig::setBrowseImageDisplayBit(int flag, bool enable)
{
 if (enable)
  browseImageDisplay |=  flag;
 else
  browseImageDisplay &= ~flag;
};

void ApaintConfig::setBrowseWithFilter(int set) { browseWithFilter = set; };

int ApaintConfig::initialWindowSizeXpix() { return initialWindowSizeX; }
int ApaintConfig::initialWindowSizeYpix() { return initialWindowSizeY; }
int ApaintConfig::initialBrowseSizeXpix() { return initialBrowseSizeX; }
int ApaintConfig::initialBrowseSizeYpix() { return initialBrowseSizeY; }
QString ApaintConfig::getImageFormat() { return imageFormat; }
