/***************************************************************************
                          drawinrepeat.cpp  -  description
                             -------------------
    begin                : Thu Sep 25 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "apaintdrawrepeat.h"
#include "apaintdoc.h"
#include "apaintdraw.h"
#include "apaintUtil.h"
#include <qpointarray.h>
#include <cmath>
#include <cstdlib>

using namespace std;

void DrawOp::draw(QPainter *p)
{
}

void DrawOp::draw(QPainter *p, QRect imgRect, QRect mrect, QRect srect, bool repeatHor)
{
}

void DrawOp::refresh(QRect nrect, QRect drawRect, bool useFg, bool final)
{
}

void DrawOp::mirrorPoint(int x, int y, int *xo1, int *yo1)
{
  if (mirrorx)
   *xo1 = drawArea.right()-x+drawArea.x();  
  else
   *xo1 = x;

  if (mirrory)
    *yo1 = drawArea.bottom()-y+drawArea.y();
  else
    *yo1 = y;
}

/*
void DrawOp::mirrorPoint(int x, int y, int &xo1, int &yo1, QRect mRect)
{
  if (mirrorx)
   xo1 = mRect.right()-x+mRect.x();
  else
   xo1 = x;

  if (mirrory)
   yo1 = mRect.bottom()-y+mRect.y();
  else
   yo1 = y;
}
*/

void DrawOp::mirrorPoint(int x, int y, int *xo1, int *yo1, QRect mRect)
{
  if (mirrorx)
   *xo1 = mRect.right()-x+mRect.x();
  else
   *xo1 = x;

  if (mirrory)
   *yo1 = mRect.bottom()-y+mRect.y();
  else
   *yo1 = y;
}

bool DrawOp::lineInRect(int x1, int y1, int x2, int y2, QRect clip)
{
  if (x1<clip.left() && x2<clip.left())
   return false;
  if (x1>clip.right() && x2>clip.right())
   return false;
  if (y1<clip.top() && y2<clip.top())
   return false;
  if (y1>clip.bottom() && y2>clip.bottom())
   return false;

  return true; 
}

void EllipseOp::draw(QPainter *p)
{
  p->drawEllipse(drawArea);
}
void EllipseOp::refresh(QRect nrect, QRect drawRect, bool useFg, bool final)
{
 d->refreshRBand(nrect, drawRect, useFg, final);
}

void RectOp::draw(QPainter *p)
{
   p->drawRect(drawArea);
}

void RectOp::refresh(QRect nrect, QRect drawRect, bool useFg, bool final)
{
 d->refreshRBand(nrect, drawRect, useFg, final);
}

void LineOp::setDrawLine(int xi1, int yi1, int xi2, int yi2)
{
  x1 = xi1;
  y1 = yi1;
  x2 = xi2;
  y2 = yi2; 
}

void LineOp::draw(QPainter *p)
{
 int xi1, yi1, xi2, yi2;

 mirrorPoint(x1, y1, &xi1, &yi1);
 mirrorPoint(x2, y2, &xi2, &yi2);
 /* on SUSE 12.3 and later, drawing line on same point draws on pixel to the right, so we have to draw pixel instead */
 if ((xi1==xi2)&&(yi1==yi2) && (!p->pen().width()))
   p->drawPoint(xi1, yi1);
 else
  p->drawLine(xi1, yi1, xi2, yi2);
}

void LineOp::refresh(QRect nrect, QRect drawRect, bool useFg, bool final)
{
  d->refreshRBand(nrect, drawRect, useFg, final);
}

void BezierOp::setDrawLine(const QPointArray &points, const QValueList<int> &segments)
{
 this->points = &points;
 this->segments = &segments;
}

void BezierOp::draw(QPainter *p)
{
 int size = points->size();
 QPointArray pts(size);

 for (int i=0; i<size; ++i)
  {
   int x, y;

   mirrorPoint((*points)[i].x(), (*points)[i].y(), &x, &y);
   pts[i] = QPoint(x,y);
  }

 if (size == 2) p->drawLine(pts[0], pts[1]);
 else if (size > 2)
 {
  for (uint i=0; i<segments->size(); i++)
   if ((*segments)[i]+3 < pts.size())
    p->drawCubicBezier(pts, (*segments)[i]);
   else
    printf("%d\n", (*segments)[i]);
 }
}

void BezierOp::refresh(QRect nrect, QRect drawRect, bool useFg, bool final)
{
  d->refreshDirectDraw(nrect, useFg, true, final);
}

void BrushOp::refresh(QRect nrect, QRect drawRect, bool useFg, bool final)
{
  d->refreshDirectDraw(nrect, useFg, true, final);
}

void EllipArcOp::setDrawPts(QPointArray &pts, int nPts)
{
  dpts = pts;
  numPts = nPts;
}

void EllipArcOp::draw(QPainter *p)
{
  int x, y, x1, y1;
  QPointArray *tpts = new QPointArray(numPts);
  for (int i=0; i<(int)numPts; i++)
   {
    dpts.point(i, &x, &y);
    mirrorPoint(x, y, &x1, &y1);
    tpts->setPoint(i, x1, y1);
   }
  if (isClosed)
   {
	  if (isPreview) {
		  QPen pen = p->pen();
		  pen.setWidth(0);
		  p->setPen(pen);
	  }
    p->drawPolygon(*tpts);
   }
  else
   p->drawPolyline(*tpts);
  delete tpts;
}

void EllipArcOp::refresh(QRect nrect, QRect drawRect, bool useFg, bool final)
{
 d->refreshRBand(nrect, drawRect, useFg, final);
}

DrawInRepeat::~DrawInRepeat()
{

}

void ScreenLineOp::setLineArea(int xi1, int yi1, int xi2, int yi2)
{
  x1 = xi1;
  y1 = yi1;
  x2 = xi2;
  y2 = yi2;
}

void ScreenLineOp::draw(QPainter *p, QRect imgRect, QRect mrect, QRect srect, bool repeatHor)
{ 
  QRect normMain, normSec;
  int sx, sy;
  double k;
  bool drawSecond = false;
  int xr1=0, yr1=0, xr2=0, yr2=0;
  int xr3, yr3, xr4, yr4;
  
  normMain = QRect(0, 0, mrect.width(), mrect.height());
  if (repeatHor)
   {
    normSec = QRect(mrect.width(), 0, srect.width(), srect.height());  
    sx = mrect.width();
    k = (double)(y2-y1)/(double)(x2-x1);
    sy = (int)(k*(sx-x1)+y1);
   }
  else
   {
    normSec = QRect(0, mrect.height(), srect.width(), srect.height());
    sy = mrect.height();
    k = (double)(y2-y1)/(double)(x2-x1);
    sx = (int)((double)(sy-y1)/k+x1);
   }
  
  if (normMain.contains(x1, y1) && normMain.contains(x2, y2))
   {
    mirrorPoint(x1+mrect.x(), y1+mrect.y(), &xr1, &yr1, mrect);
    mirrorPoint(x2+mrect.x(), y2+mrect.y(), &xr2, &yr2, mrect);
   }
  else if (normSec.contains(x1, y1) && normSec.contains(x2, y2))
   {
    if (repeatHor)
     {
      mirrorPoint(x1-normSec.x(), y1+srect.y(), &xr1, &yr1, srect);
      mirrorPoint(x2-normSec.x(), y2+srect.y(), &xr2, &yr2, srect);
     }
    else
     {
      mirrorPoint(x1+srect.x(), y1-normSec.y(), &xr1, &yr1, srect);
      mirrorPoint(x2+srect.x(), y2-normSec.y(), &xr2, &yr2, srect);
     }
   }
  else if (normMain.contains(x1, y1) && normSec.contains(x2, y2))
   {
    drawSecond = true;
    mirrorPoint(x1+mrect.x(), y1+mrect.y(), &xr1, &yr1, mrect);
    mirrorPoint(sx+mrect.x(), sy+mrect.y(), &xr2, &yr2, mrect);
                  
    if (repeatHor)
     {
      mirrorPoint(sx-normSec.x(), sy+srect.y(), &xr3, &yr3, srect);
      mirrorPoint(x2-normSec.x(), y2+srect.y(), &xr4, &yr4, srect);
     }
    else
     {
      mirrorPoint(sx+srect.x(), sy-normSec.y(), &xr3, &yr3, srect);
      mirrorPoint(x2+srect.x(), y2-normSec.y(), &xr4, &yr4, srect);
     }
   }
  else if (normSec.contains(x1, y1) && normMain.contains(x2, y2))
   {
    drawSecond = true;
    mirrorPoint(x2+mrect.x(), y2+mrect.y(), &xr1, &yr1, mrect);
    mirrorPoint(sx+mrect.x(), sy+mrect.y(), &xr2, &yr2, mrect);
    if (repeatHor)
     {
      mirrorPoint(x1-normSec.x(), y1+srect.y(), &xr3, &yr3, srect);
      mirrorPoint(sx-normSec.x(), sy+srect.y(), &xr4, &yr4, srect);        
     }
    else
     {
      mirrorPoint(x1+srect.x(), y1-normSec.y(), &xr3, &yr3, srect);
      mirrorPoint(sx+srect.x(), sy-normSec.y(), &xr4, &yr4, srect);
     }
   }
  if (lineInRect(xr1, yr1, xr2, yr2, area))
    p->drawLine(  (int) ((xr1) * scalex)+(int)(scalex/2),
                  (int) ((yr1) * scaley)+(int)(scaley/2),
                  (int) ((xr2) * scalex)+(int)(scalex/2),
                  (int) ((yr2) * scaley)+(int)(scaley/2));

  if (drawSecond && lineInRect(xr3, yr3, xr4, yr4, area))
    p->drawLine(  (int) ((xr3) * scalex)+(int)(scalex/2),
                  (int) ((yr3) * scaley)+(int)(scaley/2),
                  (int) ((xr4) * scalex)+(int)(scalex/2),
                  (int) ((yr4) * scaley)+(int)(scaley/2));
}

DrawInRepeat::DrawInRepeat(ApaintDoc *docp, ApaintDraw *drawp)
{
  Q_CHECK_PTR(docp);
  Q_CHECK_PTR(drawp);  
  doc = docp;
  draw = drawp;
  ellipOp = new EllipseOp(draw);
  rectOp = new RectOp(draw);
  lineOp = new LineOp(draw);
  brushOp = new BrushOp(draw);
  eArcOp = new EllipArcOp(draw);
  srcLineOp = new ScreenLineOp();
  bezierOp = new BezierOp(draw);
}

void DrawInRepeat::adjustWithPenSize(QPainter *p, QRect &area, bool isDrawArea)
{
  int refAdd = (p->pen().width()+4)*2;

  Layer *layer = doc->getLayer();
  
  QRect r = QRect(area.x()-refAdd, area.y()-refAdd, area.width()+refAdd*2+1, area.height()+refAdd*2+1);
  if (isDrawArea)
    r = r.intersect(layer->drawAreaPos());
  else
    r = r.intersect(layer->layerPos());
  area = r;

}

QRect DrawInRepeat::getLineArea(int xi1, int yi1, int xi2, int yi2)
{
  int xm, ym, w, h;

  xm = QMIN(xi1,xi2);
  ym = QMIN(yi1, yi2);
  w =  QMAX(1, QMAX(xi1,xi2)-xm);
  h =  QMAX(1, QMAX(yi1,yi2)-ym);

  return (QRect(xm, ym, w, h));
}


void DrawInRepeat::drawScreenLine(QPainter *p, int x1, int y1, int x2, int y2, double scalex, double scaley)
{
 srcLineOp->setScale(scalex, scaley);
 srcLineOp->setLineArea(x1, y1, x2, y2);
 srcLineOp->setArea(draw->getDrawArea());
 drawItOnScreen(p, srcLineOp);
}

void DrawInRepeat::drawRectBase(DrawOp *op, QPainter *p, QRect rect, bool doRefresh, bool useFg, bool final)
{
  double ax = doc->getAspectX();
  double ay = doc->getAspectY();

  QRect ra = QRect((int)(rect.x()*ax), (int)(rect.y()*ay), (int)(rect.width()*ax), (int)(rect.height()*ay));
  op->setDrawArea(ra);
  adjustWithPenSize(p, ra, true);
  op->setAspectArea(ra);
  op->setArea(QRect((int)(ra.x()/ax), (int)(ra.y()/ay), (int)(ra.width()/ax), (int)(ra.height()/ay)));
  drawIt(p, op, doRefresh, useFg, final);  
 
}
void DrawInRepeat::drawRect(QPainter *p, QRect rect, bool doRefresh, bool useFg, bool final)
{
  drawRectBase(rectOp, p, rect, doRefresh, useFg, final);
}

void DrawInRepeat::drawRectWithoutAspect(QPainter *p, QRect rect, bool doRefresh, bool useFg, bool final)
{
  rectOp->setDrawArea(rect);
  adjustWithPenSize(p, rect, false);
  rectOp->setArea(rect);
  rectOp->setAspectArea(rect);
  drawIt(p, rectOp, doRefresh, useFg, final); 
}

void DrawInRepeat::drawEllipse(QPainter *p, QRect ellipse, bool doRefresh, bool useFg, bool final)
{
  drawRectBase(ellipOp, p, ellipse, doRefresh, useFg, final);
}

void DrawInRepeat::clearRect(int lineWid, int xr1, int yr1, int xr2, int yr2)
{
  QRect ra, rd;
  double ax = doc->getAspectX();
  double ay = doc->getAspectY();
  int x1 = (int)(xr1*ax);
  int y1 = (int)(yr1*ay);
  int x2 = (int)(xr2*ax);
  int y2 = (int)(yr2*ay);
  
  rd = getLineArea(x1, y1, x2, y2);
  rd = QRect(rd.x()-lineWid, rd.y()-lineWid, rd.width()+lineWid*2+1, rd.height()+lineWid*2+1);
  ra = QRect((int)(rd.x()/doc->getAspectX()), (int)(rd.y()/doc->getAspectY()),
             (int)(rd.width()/doc->getAspectX()), (int)(rd.height()/doc->getAspectY()));
  draw->clearRectSlot(ra, rd);
}

void DrawInRepeat::drawLineBase(DrawOp *op, QPainter *p, int xr1, int yr1, int xr2, int yr2,
                bool doRefresh, bool useFg, bool final)
{
  LineOp *lop = (LineOp*)op;  
  QRect rect;
  double ax = doc->getAspectX();
  double ay = doc->getAspectY();
  int x1 = (int)(xr1*ax);
  int y1 = (int)(yr1*ay);
  int x2 = (int)(xr2*ax);
  int y2 = (int)(yr2*ay);

  rect = getLineArea(x1, y1, x2, y2);
  lop->setDrawArea(rect);
  adjustWithPenSize(p, rect, true);
  lop->setAspectArea(rect);
  rect = QRect((int)(rect.x()/ax), (int)(rect.y()/ay), (int)(rect.width()/ax), (int)(rect.height()/ay));
  lop->setArea(rect);
  lop->setDrawLine(x1, y1, x2, y2);
  drawIt(p, op, doRefresh,  useFg, final); 
}                

void DrawInRepeat::drawLine(QPainter *p, int xr1, int yr1, int xr2, int yr2, bool doRefresh, bool useFg, bool final)
{
  drawLineBase(lineOp, p, xr1, yr1, xr2, yr2, doRefresh, useFg, final);
}

namespace
{
#define UPDATE_MIN_MAX(t0)\
{\
 double t = t0, t1 = 1 - t;\
 double a1 = a*t1+b*t, b1 = b*t1+c*t, c1 = c*t1+d*t;\
 double a2 = a1*t1+b1*t, b2 = b1*t1+c1*t;\
 double v = a2*t1+b2*t;\
 if (v < minimum) minimum = v;\
 if (v > maximum) maximum = v;\
}

 void minmax(double a, double b, double c, double d, int &min, int &max)
 {
  double minimum, maximum;

  if (a < d)
  {
   minimum = a;
   maximum = d;
  }
  else
  {
   minimum = d;
   maximum = a;
  }

  double alpha = a - 3*(b - c) - d, beta = a - 2*b + c, gamma = a - b, dis = beta*beta - alpha*gamma;

  if (dis > 0)
  {
   double delta = beta > 0 ? beta + sqrt(dis) : beta - sqrt(dis);

   if (alpha*delta > 0 && abs(delta) < abs(alpha)) UPDATE_MIN_MAX(delta/alpha)
   if (gamma*delta > 0 && abs(gamma) < abs(delta)) UPDATE_MIN_MAX(gamma/delta)
  }

  min = floor(minimum);
  max = ceil(maximum);
 }

#undef UPDATE_MIN_MAX

 QRect getBezierArea(float xa, float ya, float xb, float yb, float xc, float yc, float xd, float yd)
 {
  int xmin, xmax, ymin, ymax;

  minmax(xa, xb, xc, xd, xmin, xmax);
  minmax(ya, yb, yc, yd, ymin, ymax);

  return QRect(xmin, ymin, xmax-xmin, ymax-ymin);
 }
}

QRect getBezierRect(const QPointArray &points, int segment, int lineWidth)
{
  if (points.size() == 0)
	  return QRect();

  int refAdd = (lineWidth+4)*2;
  QRect area;

  if (points.size() == 2)
  {
   int left = QMIN(points[0].x(), points[1].x());
   int right = points[0].x() + points[1].x() - left;
   int top = QMIN(points[0].y(), points[1].y());
   int bottom = points[0].y() + points[1].y() - top;

   area = QRect(left, top, right-left+1, bottom-top+1);
  }
  else area = getBezierArea(points[segment].x(), points[segment].y(),
		  points[segment+1].x(), points[segment+1].y(),
		  points[segment+2].x(), points[segment+2].y(),
		  points[segment+3].x(), points[segment+3].y());

  QRect r = QRect(area.x()-refAdd, area.y()-refAdd, area.width()+refAdd*2+1, area.height()+refAdd*2+1);

  return r;
}

void DrawInRepeat::drawBezier(QPainter *p,
		const QPointArray &oldPoints, const QPointArray &points,
		const QValueList<int> &erasingSegments, QValueList<int> &drawingSegments,
		int oldLineWidth, int lineWidth,
		bool final)
{
 QRect rect;
 double ax = doc->getAspectX();
 double ay = doc->getAspectY();
 int oldSize = oldPoints.size();
 QPointArray oldPts(oldSize);
 int size = points.size();
 QPointArray pts(size);
 QValueList<QRect> drawingRects;
 QValueList<QRect> erasingRects;
 QRect area;

 for (int i=0; i<oldSize; ++i) oldPts[i] = QPoint(oldPoints[i].x()*ax, oldPoints[i].y()*ay);
 for (int i=0; i<size; ++i) pts[i] = QPoint(points[i].x()*ax, points[i].y()*ay);

 if (erasingSegments.size() < 3)
 {
  if (erasingSegments.size() > 0)
  {
   area = getBezierRect(oldPts, erasingSegments[0], oldLineWidth);
   erasingRects.append(area);

   for (uint i=1; i<erasingSegments.size(); i++)
   {
    QRect r = getBezierRect(oldPts, erasingSegments[i], oldLineWidth);
    area = area.unite(r);
    erasingRects.append(r);
   }
  }

  int firstDrawingSegment = -1, secondDrawingSegment = -1;

  if (drawingSegments.size() > 0) firstDrawingSegment = drawingSegments[0];
  if (drawingSegments.size() > 1) secondDrawingSegment = drawingSegments[1];

  for (uint i=0; (i+2 < pts.size()) || (i == 0 && pts.size() > 0); i += 3)
  {
   if (i == firstDrawingSegment || i == secondDrawingSegment)
   {
    drawingRects.append(getBezierRect(pts, i, lineWidth));

    continue;
   }

   QRect r = getBezierRect(pts, i, lineWidth);

   for (uint j = 0; j < erasingRects.size(); j++)
    if (r.intersects(erasingRects[j]))
    {
	 drawingSegments.append(i);
	 drawingRects.append(r);

     break;
    }
  }
 }

 if (drawingSegments.size() > 0)
  for (uint i=0; i<drawingRects.size(); i++)
   area = area.unite(drawingRects[i]);

  QPen pen = p->pen();
  pen.setWidth(oldLineWidth);
  p->setPen(pen);
  p->setRasterOp(Qt::NorROP);
  bezierOp->setDrawArea(area);
  bezierOp->setAspectArea(area);
  int xmin = floor(area.left()/ax)-2;
  int ymin = floor(area.top()/ay)-2;
  int xmax = ceil(area.right()/ax)+2;
  int ymax = ceil(area.bottom()/ay)+2;
  bezierOp->setArea(QRect(xmin, ymin, xmax-xmin+1, ymax-ymin+1));
  bezierOp->setDrawLine(oldPts, erasingSegments);
  drawIt(p, bezierOp, false, false, false);
  pen = p->pen();
  pen.setWidth(lineWidth);
  p->setPen(pen);
  p->setRasterOp(Qt::CopyROP);
  bezierOp->setDrawArea(area);
  bezierOp->setAspectArea(area);
  xmin = floor(area.left()/ax)-2;
  ymin = floor(area.top()/ay)-2;
  xmax = ceil(area.right()/ax)+2;
  ymax = ceil(area.bottom()/ay)+2;
  bezierOp->setArea(QRect(xmin, ymin, xmax-xmin+1, ymax-ymin+1));
  bezierOp->setDrawLine(pts, drawingSegments);
  drawIt(p, bezierOp, true, true, final);
}

void DrawInRepeat::drawBrush(QPainter *p, int xr1, int yr1, int xr2, int yr2, bool doRefresh, bool useFg, bool final)
{
  drawLineBase(brushOp, p, xr1, yr1, xr2, yr2, doRefresh, useFg, final);
}

void DrawInRepeat::drawClosedPolygon(QPainter *p, QPointArray &rpts, int nRPts, bool doRefresh, bool useFg, bool final)
{
  eArcOp->setClosed(true);

  if (!final)
	  eArcOp->setPreview(true);

  drawPolygon(p, rpts, nRPts, doRefresh, useFg, final);
  eArcOp->setPreview(false);
  eArcOp->setClosed(false);
}

void DrawInRepeat::drawPolygon(QPainter *p, QPointArray &rpts, bool doRefresh, bool useFg, bool final)
{
  drawPolygon(p, rpts, rpts.count(), doRefresh, useFg, final);
}

void DrawInRepeat::adjustPolygon(QPointArray &ptsIn, QPointArray &ptsOut, double ax, double ay)
{
 int i;
 int x, y;
 
 for (i=0; i<(int)ptsOut.count(); i++)
 {
   ptsIn.point(i, &x, &y);
   ptsOut.setPoint(i, (int)(x*ax), (int)(y*ay));
 }
}

void DrawInRepeat::drawPolygon(QPainter *p, QPointArray &rpts, int nRPts, bool doRefresh, bool useFg, bool final)
{
  QRect rect;
  QPointArray pts = QPointArray(nRPts);
  double ax = doc->getAspectX();
  double ay = doc->getAspectY();
  
  adjustPolygon(rpts, pts, ax, ay);
  eArcOp->setDrawPts(pts, nRPts);
  rect = pts.boundingRect().normalize();
  eArcOp->setDrawArea(rect);
  adjustWithPenSize(p, rect, true);
  eArcOp->setAspectArea(rect);
  rect = QRect((int)(rect.x()/ax), (int)(rect.y()/ay), (int)(rect.width()/ax), (int)(rect.height()/ay));
  eArcOp->setArea(rect);
  drawIt(p, eArcOp, doRefresh,  useFg, final);
}

void DrawInRepeat::drawIt(QPainter *p, DrawOp *op, bool doRefresh, bool useFg, bool final)
{
  Layer *layer = doc->getLayer();
  QSize imgs = QSize(layer->imageBuf.width(), layer->imageBuf.height());  
  QRect area = op->getArea();

  double ax = doc->getAspectX();
  double ay = doc->getAspectY();
  
  if (!area.isValid())
  {
    if (doRefresh && final)
      op->refresh(area, op->getAspectArea(), useFg, final);
    return;
  }

  if (!layer->isUsingTiles())
  {
    op->setMirror(false, false);
    op->draw(p);
  }
  else
  {
    QPtrList<Tile>& tList = layer->getTileList();
    QWMatrix mx;
    Tile *tile;
    QRect mrect, srect;
    int sx, sy;
    for (int i=0; i<(int)tList.count(); i++)
    {
      tile = tList.at(i);
      mx.reset();
      mrect = tile->getPosMain();
      srect = tile->getPosSecond();
      if (mrect.intersects(area))
      {
        sx = area.x() - mrect.x();
        sy = area.y() - mrect.y();
        if (tile->mirrorHor())
          sx = imgs.width()-sx-area.width();
        if (tile->mirrorVert())
          sy = imgs.height()-sy-area.height();
        sx = (int)((area.x() - sx)*ax);
        sy = (int)((area.y() - sy)*ay);
        MatrixUtil::translatemx(&mx, -sx, -sy);
        p->setWorldMatrix(mx);
        op->setMirror(tile->mirrorHor(), tile->mirrorVert());
        op->draw(p);
      }
      mx.reset();
      if (srect.intersects(area))
      {
        sx = area.x() - srect.x();
        sy = area.y() - srect.y();
        if (tile->getSecondHor())
          sx+=(imgs.width()-srect.width());
        else
          sy+=(imgs.height()-srect.height());
        if (tile->mirrorHor())
          sx = imgs.width()-sx-area.width();
        if (tile->mirrorVert())
          sy = imgs.height()-sy-area.height();
        sx = (int)((area.x() - sx)*ax);
        sy = (int)((area.y() - sy)*ay);
        MatrixUtil::translatemx(&mx, -sx, -sy);
        p->setWorldMatrix(mx);
        op->setMirror(tile->mirrorHor(), tile->mirrorVert());
        op->draw(p);
      }
    }
  }
  if (doRefresh)
      op->refresh(area, op->getAspectArea(), useFg, final);
}

void DrawInRepeat::drawItOnScreen(QPainter *p, DrawOp *op)
{
  Layer *layer = doc->getLayer();
  QRect imgRect = QRect(0, 0, layer->imageBuf.width(), layer->imageBuf.height());

  if (!layer->isUsingTiles())
  {    
    op->draw(p, imgRect, layer->layerPos(), QRect(), false);
  }
  else
  {
    QPtrList<Tile>& tList = layer->getTileList();
    Tile *tile;
    for (int i=0; i<(int)tList.count(); i++)
    {
      tile = tList.at(i);
      op->setMirror(tile->mirrorHor(), tile->mirrorVert());      
      op->draw(p, imgRect, tile->getPosMain(), tile->getPosSecond(), tile->getSecondHor());
    } 
  }
}

void DrawInRepeat::transPoint(int xin, int yin, int *xout, int *yout)
{
  Layer *layer = doc->getLayer();    
  int sx, sy;
    
  sx = xin;
  sy = yin;
  if (layer->isUsingTiles())  
  {
    QPtrList<Tile>& tList = layer->getTileList();
    Tile *tile;
    QSize imgs = QSize(layer->imageBuf.width()-1, layer->imageBuf.height()-1);
    QRect mrect, srect;    
    bool found = false;
    for (int i=0; i<(int)tList.count() && !found; i++)    
    {
      tile = tList.at(i);
      mrect = tile->getPosMain();
      srect = tile->getPosSecond();
      if (mrect.contains(xin, yin))
      {
        sx = xin - mrect.x();
        sy = yin - mrect.y();
        if (tile->mirrorHor())
          sx = imgs.width()-sx;
        if (tile->mirrorVert())
          sy = imgs.height()-sy;
        found = true;
      }
      else if (srect.contains(xin, yin))
      {
        sx = xin - srect.x();
        sy = yin - srect.y();
        if (tile->getSecondHor())
          sx+=(imgs.width()-srect.width());
        else
          sy+=(imgs.height()-srect.height());
        if (tile->mirrorHor())
          sx = imgs.width()-sx;
        if (tile->mirrorVert())
          sy = imgs.height()-sy;
        found = true;
      }
    }
  }
  *xout = sx;
  *yout = sy;
}

void DrawInRepeat::refreshDirect(int x, int y, int w, int h, bool useFg, bool doDraw, bool final)
{
  QRect rd(x, y, w, h);
  
  draw->refreshDirectDraw(rd, useFg, doDraw, final);
}

void DrawInRepeat::refreshImageDraw(int x, int y, int w, int h, bool final)
{
  draw->refreshImageDraw(x, y, w, h, final);
}

void DrawInRepeat::refreshIntersectedArea(const QRect &oldRec, const QRect &newRec, int border)
{
  draw->screenRefresh(oldRec, newRec, border); 
}

void DrawInRepeat::refreshArea(int x1, int y1, int x2, int y2, int border)
{
  draw->screenRefresh(x1, y1, x2, y2, border);
}
