/***************************************************************************
                          apaintimageUtil.cpp  -  description
                             -------------------
    begin                : Mon Mar 22 2004
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <math.h>
#include <qimage.h>
#include <qfontmetrics.h>
#include <qpainter.h>

#include "apaintimageUtil.h"

#define SQR(a) ((a)*(a))

void ApaintImageUtil::filterApply(ApaintImage& img, bool *protColors, bool doRepeat, int what, int fgCol, int bgCol, int penX, int penY, int value, bool doVarColors, bool doVarSize, bool doGradient)
{
 filterApply(img, 0, 0, img.width(), img.height(), protColors, doRepeat, what, fgCol, bgCol, penX, penY, value, doVarColors, doVarSize, doGradient);
}

void ApaintImageUtil::averageContour(ApaintImage& img, bool *protColors, bool doRepeat, int radius, double aspectX, double aspectY)
{
 averageContour(img, 0, 0, img.width(), img.height(), protColors, doRepeat, radius, aspectX, aspectY);
}

void ApaintImageUtil::averageContourThickness(ApaintImage& img, bool *protColors, bool doRepeat, int thickX, int thickY)
{
 averageContourThickness(img, 0, 0, img.width(), img.height(), protColors, doRepeat, thickX, thickY);
}

void ApaintImageUtil::averageContour24thickness(ApaintImage& img, bool doRepeat, int thickX, int thickY)
{
 averageContour24thickness(img, 0, 0, img.width(), img.height(), doRepeat, thickX, thickY);
}

void ApaintImageUtil::despecleContour(ApaintImage& img, bool *protColors, bool doRepeat, int what, int neighbours)
{
 despecleContour(img, 0, 0, img.width(), img.height(), protColors, doRepeat, what, neighbours);
}

#define MOTIF_RECTANGLE			0
#define MOTIF_DIAMOND			1
#define MOTIF_CIRCLE			2
#define MOTIF_EQULATERAL_TRIANGLE	3
#define MOTIF_SQUARE_TRIANGLE		4
#define MOTIF_FOREWARD_SLASH		5
#define MOTIF_BACKWARD_SLASH		6
#define MOTIF_CROSS_X			7
#define MOTIF_CROSS_PLUS		8

#define DRAW_POINT(x, y) {\
 if (drawIfPlace) \
  { \
   if (protColors[img.scanLine(y)[x]]) \
    return(FALSE); \
   if (img.scanLine(y)[x]!=centerColor) \
    return(FALSE); \
  } \
 else \
  if (!protColors[img.scanLine(y)[x]]) \
   img.scanLine(y)[x]=drawColor; }

#define DRAW_POINT_RGB(x, y) {\
 img.scanLine(y)[(x)*3]=rgb[0];\
 img.scanLine(y)[(x)*3+1]=rgb[1];\
 img.scanLine(y)[(x)*3+2]=rgb[2];}

bool ApaintImageUtil::drawMotif(ApaintImage& img, int w, int h, bool *protColors, int what, int drawColor, int dropX, int dropY, int sizeX, int sizeY, bool drawIfPlace)
{
 int j, k, x, y;
 unsigned char centerColor = img.scanLine((dropY + sizeY/2)%h)[(dropX + sizeX/2)%w];
 unsigned char rgb[3] = {0, 0, 0};

 if (img.type() != LOAD_CMAP)
  {
   rgb[0]=qRed(drawColor);
   rgb[1]=qGreen(drawColor);
   rgb[2]=qBlue(drawColor);
  }

 switch (what)
  {
   case MOTIF_RECTANGLE:
    for (j=0; j<sizeY; j++)
     {
      y = (dropY + j)%h;
      if (img.type() == LOAD_CMAP)
       for (k=0; k<sizeX; k++)
	{
	 x = (dropX + k)%w;
         DRAW_POINT(x, y)
	}
      else
       for (k=0; k<sizeX; k++)
	{
	 x = (dropX + k)%w;
         DRAW_POINT_RGB(x, y)
	}
     }
    break;

   case MOTIF_DIAMOND:
    {
     int wid2, hei2, maxwh, d1, d2;

     wid2 = max(1, sizeX/2);
     hei2 = max(1, sizeY/2);
     maxwh = max(wid2, hei2);
     d1 = d2 = maxwh;

     for (j=0; j<sizeY; j++)
      {
       y = (dropY + j)%h;
       if (img.type() == LOAD_CMAP)
	for (k=0; k<sizeX; k++)
	 {
	  x = (dropX + k)%w;
          if (sizeX==sizeY)
	   d1 = abs(k-wid2) + abs(j-hei2);
	  else
	   if (sizeX>sizeY)
	    d1 = abs(k-wid2) + abs(((j-hei2)*wid2)/hei2);
	   else /* sizeX < sizeY */
	    d1 = abs(((k-wid2)*hei2)/wid2) + abs(j-hei2);
          if (d1<d2)
	   DRAW_POINT(x, y)
	 }
       else
	for (k=0; k<sizeX; k++)
	 {
	  x = (dropX + k)%w;
          if (sizeX==sizeY)
	   d1 = abs(k-wid2) + abs(j-hei2);
	  else
	   if (sizeX>sizeY)
	    d1 = abs(k-wid2) + abs(((j-hei2)*wid2)/hei2);
	   else /* sizeX < sizeY */
	    d1 = abs(((k-wid2)*hei2)/wid2) + abs(j-hei2);
          if (d1<d2)
	   DRAW_POINT_RGB(x, y)
	 }
      }
    }
    break;

   case MOTIF_CIRCLE:
    {
     int wid2, hei2, maxwh, d1, d2, scaledSize;

     wid2 = max(1, sizeX/2);
     hei2 = max(1, sizeY/2);
     maxwh = max(wid2, hei2);
     d1 = d2 = maxwh*maxwh;
     for (j=0; j<sizeY; j++)
      {
       y = (dropY + j)%h;
       for (k=0; k<sizeX; k++)
	{
	 x = (dropX + k)%w;
         if (sizeX==sizeY)
	  d1 = (k-wid2)*(k-wid2) + (j-hei2)*(j-hei2);
	 else
	  if (sizeX>sizeY)
	   {
	    scaledSize = ((j-hei2)*wid2)/hei2;
	    d1 = (k-wid2)*(k-wid2) + scaledSize*scaledSize;
	   }
	  else /* wid < hei */
	   {
	    scaledSize = ((k-wid2)*hei2)/wid2;
	    d1 = scaledSize*scaledSize + (j-hei2)*(j-hei2);
	   }
         if (d1<d2)
	  {
           if (img.type() == LOAD_CMAP)
	    DRAW_POINT(x, y)
	   else
	    DRAW_POINT_RGB(x, y)
	  }
	}
      }
    }
    break;

   case MOTIF_EQULATERAL_TRIANGLE:
    {
     int xoff;

     for (j=0; j<sizeY; j++)
      {
       xoff = (sizeX*(sizeY - j - 1))/(sizeY*2);
       y = (dropY + j)%h;
       if (img.type() == LOAD_CMAP)
	for (k=(sizeX*j)/sizeY; k>=0; k--)
	 {
	  x = (dropX + k + xoff)%w;
	  DRAW_POINT(x, y)
	 }
       else
 	for (k=(sizeX*j)/sizeY; k>=0; k--)
	 {
	  x = (dropX + k + xoff)%w;
	  DRAW_POINT_RGB(x, y)
	 }
     }
    }
    break;

   case MOTIF_SQUARE_TRIANGLE:
    for (j=0; j<sizeY; j++)
     {
      y = (dropY + j)%h;
      if (img.type() == LOAD_CMAP)
       for (k=(sizeX*j)/sizeY; k>=0; k--)
	{
	 x = (dropX + k)%w;
	 DRAW_POINT(x, y)
	}
      else
       for (k=(sizeX*j)/sizeY; k>=0; k--)
	{
	 x = (dropX + k)%w;
	 DRAW_POINT_RGB(x, y)
	}
     }
    break;

   case MOTIF_FOREWARD_SLASH:
   case MOTIF_BACKWARD_SLASH:
    {
     int max;

     max = (sizeX>=sizeY) ? sizeX : sizeY;

     for (j=0; j<max; j++)
      {
       if (sizeX>=sizeY)
	{
         x = j;
         y = (int)((x*sizeY+0.5)/sizeX);
	}
       else
	{
         y = j;
         x = (int)((y*sizeX+0.5)/sizeY);
	}
       if (what==MOTIF_FOREWARD_SLASH)
	x = (w - x - 1 + dropX)%w;
       else
        x = (x + dropX)%w;
       y = (y + dropY)%h;
       if (img.type() == LOAD_CMAP)
        DRAW_POINT(x, y)
       else
        DRAW_POINT_RGB(x, y)
      }
    }
    break;

   case MOTIF_CROSS_X:
    {
     int max;

     max = (sizeX>=sizeY) ? sizeX : sizeY;

     for (j=0; j<max; j++)
      {
       if (sizeX>=sizeY)
	{
         x = j;
         y = (int)((x*sizeX+0.5)/sizeX);
	}
       else
	{
         y = j;
         x = (int)((y*sizeY+0.5)/sizeY);
	}
       x = sizeX - x - 1 + dropX;
       if (x<0)
	{
	 while (x<0)
	  x+=w;
	}
       else
	x%=w;
       y = (y + dropY)%h;
       if (img.type() == LOAD_CMAP)
        DRAW_POINT(x, y)
       else
        DRAW_POINT_RGB(x, y)

       if (sizeX>=sizeY)
	{
         x = j;
         y = (int)((x*sizeX+0.5)/sizeX);
	}
       else
	{
         y = j;
         x = (int)((y*sizeY+0.5)/sizeY);
	}
       x = (x + dropX)%w;
       y = (y + dropY)%h;
       if (img.type() == LOAD_CMAP)
        DRAW_POINT(x, y)
       else
        DRAW_POINT_RGB(x, y)
      }
    }
    break;

   case MOTIF_CROSS_PLUS:
    {
     int midX, midY;

     midX = dropX + sizeX/2;
     while (midX>=w)
      midX-=w;
     midY = dropY + sizeY/2;
     while (midY>=h)
      midY-=h;

     if (img.type() == LOAD_CMAP)
      {
       if ((dropX + sizeX)<w)
	for (j=0; j<sizeX; j++)
	 DRAW_POINT(dropX+j, midY)
       else
	for (j=0; j<sizeX; j++)
	 DRAW_POINT((dropX+j)%w, midY)

       if ((dropY+sizeY)<h)
	for (j=0; j<sizeY; j++)
	 DRAW_POINT(midX, dropY+j)
       else
	for (j=0; j<sizeY; j++)
	 DRAW_POINT(midX, (dropY+j)%h)
      }
     else
      {
       if ((dropX + sizeX)<w)
	for (j=0; j<sizeX; j++)
	 DRAW_POINT_RGB(dropX+j, midY)
       else
	for (j=0; j<sizeX; j++)
	 DRAW_POINT_RGB((dropX+j)%w, midY)

       if ((dropY+sizeY)<h)
	for (j=0; j<sizeY; j++)
	 DRAW_POINT_RGB(midX, dropY+j)
       else
	for (j=0; j<sizeY; j++)
	 DRAW_POINT_RGB(midX, (dropY+j)%h)
      }
    }
    break;
  }
 return(TRUE);
}

/* create table of movements for ikat, zizgag or wave */
void ApaintImageUtil::makeWaveTable(int *offsetTable, int size, int valueOffset, int maxOffset, int numWaves, bool doGradient, bool doIkat, bool doMarble, bool doZigzag, bool doWave)
{
 int thisOffset;
 int offset=0, i;
 int marbleDir = 1;

 if (valueOffset==0)
  valueOffset = maxOffset;
 if (numWaves==0)
  numWaves = 1;
 valueOffset = min(valueOffset, maxOffset);

 if (doMarble)
  offset = ((rand()>>2)%(valueOffset+1)) - (valueOffset>>1);

 /* prepare the offset table */
 for (i = 0; i < size; i++)
  {
   thisOffset = valueOffset;
   if (doGradient)
    thisOffset = (valueOffset*i)/size;

   if (doMarble)
    {
     if ((i%numWaves)==0)
      marbleDir = (rand()&1) ? 1 : -1;
 
     offset += marbleDir;

     if (thisOffset)
      {
       if (offset<(- (thisOffset>>1)))
	{ offset+=2; marbleDir = -marbleDir; }
       if (offset>(thisOffset>>1))
	{ offset-=2; marbleDir = -marbleDir; } 
      }
     else
      offset = 0; 
    }
   if (doIkat)
    {
     offset = ((rand()>>2)%(thisOffset+1)) - (thisOffset>>1);
     if (i) /* if previous offset is equal to this one, calcualte a new one */
      if (offsetTable[i-1]==offset)
       offset = ((rand()>>2)%(thisOffset+1)) - (thisOffset>>1);
    }
   if (doZigzag)
    {
     if (thisOffset)
      offset = abs((((i*thisOffset*numWaves)/(size*2))%(thisOffset*2))-thisOffset) - (thisOffset>>1);
     else
      offset = 0;
    }
   if (doWave)
    offset = (int)(cos((i*numWaves*M_PI/2)/size)*thisOffset+0.5);// here we want to make quarter, half, etc of wave
   offsetTable[i]=offset;
  }

 /* continue random movements to make offset ends match at the end of repeat */
 i = 0;
 if (doMarble && (!doGradient))
  while ((abs(offsetTable[i%size]-offset)>1) && (i<(size*10)))
   {
    thisOffset = valueOffset;

    if ((i%numWaves)==0)
     marbleDir = (rand()&1) ? 1 : -1;

    offset += marbleDir;

    if (thisOffset)
     {
      if (offset<(- (thisOffset>>1)))
       { offset+=2; marbleDir = -marbleDir; } 
      if (offset>(thisOffset>>1))
       { offset-=2; marbleDir = -marbleDir; } 
     }
    else
     offset = 0; 
    offsetTable[i%size]=offset;
    i++;
   }
}

#define SWAP(a,b) {a^=b; b^=a; a^=b;}

void ApaintImageUtil::filterApply(ApaintImage& img, int xin, int yin, int w, int h, bool *protColors, bool doRepeat, int what, int fgCol, int bgCol, int penX, int penY, int value, bool doVarColors, bool doVarSize, bool doGradient)
{
 switch (what)
  {
   case FILTER_CORAL_1:
   case FILTER_CORAL_2:
   case FILTER_CORAL_3:
   case FILTER_CORAL_4:
   case FILTER_CORAL_5:
   case FILTER_CORAL_6:
    {
     int numPoints = max(10, min((int)(((float)value*w*h)/1000), 5000000));
     int i, j, doRandomWalk;
     int startX, startY, startX1, startX2, startY1, startY2;
     int normalCoral = (what<FILTER_CORAL_4);
     int attackType;

     if (img.type() != LOAD_CMAP)
      return;

     if (normalCoral)
      attackType = what-FILTER_CORAL_1;
     else
      attackType = what-FILTER_CORAL_4;

     for (j=0; j<numPoints; j++)
      {
       /* two random points, must be uncorrelated */
       startX = rand()%w;
       startY = rand()%h;

       doRandomWalk = TRUE;
       if (normalCoral)
        {
	 startX1 = startX+1;
	 if (startX1>=w)
          startX1=0;
	 startX2 = startX-1;
	 if (startX2<0)
	  startX2=w-1;
	 startY1 = startY+1;
	 if (startY1>=h)
	  startY1=0;
	 startY2 = startY-1;
	 if (startY2<0)
	  startY2=h-1;

	 /* don't put starting point on existing point or on 1 distance to existing point */
	 if ((img.scanLine(startY )[startX ]==fgCol) || (img.scanLine(startY )[startX ]==bgCol) ||
             (img.scanLine(startY )[startX1]==fgCol) || (img.scanLine(startY )[startX1]==bgCol) ||
             (img.scanLine(startY )[startX2]==fgCol) || (img.scanLine(startY )[startX2]==bgCol) ||
             (img.scanLine(startY1)[startX ]==fgCol) || (img.scanLine(startY1)[startX ]==bgCol) ||
             (img.scanLine(startY1)[startX1]==fgCol) || (img.scanLine(startY1)[startX1]==bgCol) ||
             (img.scanLine(startY1)[startX2]==fgCol) || (img.scanLine(startY1)[startX2]==bgCol) ||
             (img.scanLine(startY2)[startX ]==fgCol) || (img.scanLine(startY2)[startX ]==bgCol) ||
             (img.scanLine(startY2)[startX1]==fgCol) || (img.scanLine(startY2)[startX1]==bgCol) ||
             (img.scanLine(startY2)[startX2]==fgCol) || (img.scanLine(startY2)[startX2]==bgCol))
          doRandomWalk = FALSE;
	} /* cloud type coral */
       else
	/* don't put starting point on existing point */
	if ((img.scanLine(startY )[startX ]==fgCol) || (img.scanLine(startY )[startX ]==bgCol))
         doRandomWalk = FALSE;

       if (doRandomWalk)
        {
	 /* calculate border points if we did not do it yet */
         if (!normalCoral)
	  {
	   startX1 = startX+1;
	   if (startX1>=w)
            startX1=0;
	   startX2 = startX-1;
	   if (startX2<0)
	    startX2=w-1;
	   startY1 = startY+1;
	   if (startY1>=h)
	    startY1=0;
	   startY2 = startY-1;
	   if (startY2<0)
	    startY2=h-1;
	  }

	 for (i=10; i; i--)
	  {
	   /* randomly walk the point and choose one of 8 directions */
	   /* make the new neighboring points depending on changes in random walk */
	   switch (rand()&7)
	    {
	     case 0:
              startY--;
	      if (startY<0)
	       startY=h-1;
	      startX--;
	      if (startX<0)
	       startX=w-1;
              /* new neighbors X Y*/
	      startX1 = startX+1;
	      if (startX1>=w)
	       startX1=0;
	      startX2 = startX-1;
	      if (startX2<0)
	       startX2=w-1;
	      startY1 = startY+1;
	      if (startY1>=h)
	       startY1=0;
	      startY2 = startY-1;
	      if (startY2<0)
	       startY2=h-1;
	      break;
	     case 1:
              startY--;
	      if (startY<0)
	       startY=h-1;
	      /* new neighbors Y */
	      startY1 = startY+1;
	      if (startY1>=h)
	       startY1=0;
	      startY2 = startY-1;
	      if (startY2<0)
	       startY2=h-1;
	      break;
	     case 2:
              startY--;
	      if (startY<0)
	       startY=h-1;
	      startX++;
	      if (startX>=w)
	       startX=0;
              /* new neighbors X Y */
	      startX1 = startX+1;
	      if (startX1>=w)
	       startX1=0;
	      startX2 = startX-1;
	      if (startX2<0)
	       startX2=w-1;
	      startY1 = startY+1;
	      if (startY1>=h)
	       startY1=0;
	      startY2 = startY-1;
	      if (startY2<0)
	       startY2=h-1;
	      break;
	     case 3:
	      startX--;
	      if (startX<0)
	       startX=w-1;
              /* new neighbors X */
	      startX1 = startX+1;
	      if (startX1>=w)
	       startX1=0;
	      startX2 = startX-1;
	      if (startX2<0)
	       startX2=w-1;
	      break;
	     case 4:
	      startX++;
	      if (startX>=w)
	       startX=0;
              /* new neighbors X */
	      startX1 = startX+1;
	      if (startX1>=w)
	       startX1=0;
	      startX2 = startX-1;
	      if (startX2<0)
	       startX2=w-1;
	      break;
	     case 5:
              startY++;
	      if (startY>=h)
	       startY=0;
	      startX--;
	      if (startX<0)
	       startX=w-1;
              /* new neighbors X Y */
	      startX1 = startX+1;
	      if (startX1>=w)
	       startX1=0;
	      startX2 = startX-1;
	      if (startX2<0)
	       startX2=w-1;
	      startY1 = startY+1;
	      if (startY1>=h)
	       startY1=0;
	      startY2 = startY-1;
	      if (startY2<0)
	       startY2=h-1;
	      break;
	     case 6:
              startY++;
	      if (startY>=h)
	       startY=0;
	      /* new neighbors Y */
	      startY1 = startY+1;
	      if (startY1>=h)
	       startY1=0;
	      startY2 = startY-1;
	      if (startY2<0)
	       startY2=h-1;
	      break;
	     case 7:
              startY++;
	      if (startY>=h)
	       startY=0;
	      startX++;
	      if (startX>=w)
	       startX=0;
              /* new neighbors X Y */
	      startX1 = startX+1;
	      if (startX1>=w)
	       startX1=0;
	      startX2 = startX-1;
	      if (startX2<0)
	       startX2=w-1;
	      startY1 = startY+1;
	      if (startY1>=h)
	       startY1=0;
	      startY2 = startY-1;
	      if (startY2<0)
	       startY2=h-1;
	      break;
	    }

	   /* draw the point if we are close to some other point */
	   if (attackType==0)
	    {
	     if ((img.scanLine(startY )[startX1]==fgCol) || (img.scanLine(startY )[startX1]==bgCol) ||
                 (img.scanLine(startY )[startX2]==fgCol) || (img.scanLine(startY )[startX2]==bgCol) ||
                 (img.scanLine(startY1)[startX ]==fgCol) || (img.scanLine(startY1)[startX ]==bgCol) ||
                 (img.scanLine(startY1)[startX1]==fgCol) || (img.scanLine(startY1)[startX1]==bgCol) ||
                 (img.scanLine(startY1)[startX2]==fgCol) || (img.scanLine(startY1)[startX2]==bgCol) ||
                 (img.scanLine(startY2)[startX ]==fgCol) || (img.scanLine(startY2)[startX ]==bgCol) ||
                 (img.scanLine(startY2)[startX1]==fgCol) || (img.scanLine(startY2)[startX1]==bgCol) ||
                 (img.scanLine(startY2)[startX2]==fgCol) || (img.scanLine(startY2)[startX2]==bgCol))
	      {
	       if (!protColors[img.scanLine(startY)[startX]])
        	img.scanLine(startY)[startX]=fgCol;
	       break;
	      }
	    }
	   else
	    {
	     if (attackType==1)
	      {
	       if ((img.scanLine(startY )[startX1]==fgCol) || (img.scanLine(startY )[startX1]==bgCol) ||
                   (img.scanLine(startY )[startX2]==fgCol) || (img.scanLine(startY )[startX2]==bgCol) ||
                   (img.scanLine(startY1)[startX ]==fgCol) || (img.scanLine(startY1)[startX ]==bgCol) ||
                   (img.scanLine(startY2)[startX ]==fgCol) || (img.scanLine(startY2)[startX ]==bgCol))
		{
		 if (!protColors[img.scanLine(startY)[startX]])
        	  img.scanLine(startY)[startX]=fgCol;
		 break;
		}
	      }
	     else
	      if (attackType==2)
	       {
		if ((img.scanLine(startY1)[startX1]==fgCol) || (img.scanLine(startY1)[startX1]==bgCol) ||
                    (img.scanLine(startY1)[startX2]==fgCol) || (img.scanLine(startY1)[startX2]==bgCol) ||
                    (img.scanLine(startY2)[startX1]==fgCol) || (img.scanLine(startY2)[startX1]==bgCol) ||
                    (img.scanLine(startY2)[startX2]==fgCol) || (img.scanLine(startY2)[startX2]==bgCol))
		 {
		  if (!protColors[img.scanLine(startY)[startX]])
        	   img.scanLine(startY)[startX]=fgCol;
		  break;
		 }
	       }
	    }
          }
	}
      }
     break; 
    }

   case FILTER_SELF_AVOID_MOTIFS1:
   case FILTER_SELF_AVOID_MOTIFS2:
   case FILTER_SELF_AVOID_MOTIFS3:
   case FILTER_SELF_AVOID_MOTIFS4:
   case FILTER_SELF_AVOID_MOTIFS5:
   case FILTER_SELF_AVOID_MOTIFS6:
   case FILTER_SELF_AVOID_MOTIFS7:
   case FILTER_SELF_AVOID_MOTIFS8:
   case FILTER_SELF_AVOID_MOTIFS9:
   case FILTER_RANDOM_MOTIFS1:
   case FILTER_RANDOM_MOTIFS2:
   case FILTER_RANDOM_MOTIFS3:
   case FILTER_RANDOM_MOTIFS4:
   case FILTER_RANDOM_MOTIFS5:
   case FILTER_RANDOM_MOTIFS6:
   case FILTER_RANDOM_MOTIFS7:
   case FILTER_RANDOM_MOTIFS8:
   case FILTER_RANDOM_MOTIFS9:
   case FILTER_PLACE_MOTIFS1:
   case FILTER_PLACE_MOTIFS2:
   case FILTER_PLACE_MOTIFS3:
   case FILTER_PLACE_MOTIFS4:
   case FILTER_PLACE_MOTIFS5:
   case FILTER_PLACE_MOTIFS6:
   case FILTER_PLACE_MOTIFS7:
   case FILTER_PLACE_MOTIFS8:
   case FILTER_PLACE_MOTIFS9:
    {
     int i;
     int numPoints = max(10, min(((float)value*w*h)/100000, 50000));
     unsigned short *dropXY;
     int drawColor = fgCol;
     int numColors, numNoiseColors;
     unsigned char mapNoiseColors[256];
     int sizeX = penX;
     int sizeY = penY;
     int sizeX2, sizeY2, dropX, dropY;
     int motifType=0;
     int randOff;
     bool drawIfPlace = FALSE;

     numNoiseColors = 0;
     if (img.type() == LOAD_CMAP)
      {
       numColors = img.numColors();
       if (doVarColors)
	for (i=0; i<numColors; i++)
	 if ((!protColors[i]) && (i!=bgCol))
	  mapNoiseColors[numNoiseColors++]=i;
      }

     if ((what>=FILTER_RANDOM_MOTIFS1) && (what<=FILTER_RANDOM_MOTIFS9))
      motifType=what-FILTER_RANDOM_MOTIFS1;

     if ((what>=FILTER_PLACE_MOTIFS1) && (what<=FILTER_PLACE_MOTIFS9))
      {
       motifType = what-FILTER_PLACE_MOTIFS1;
       drawIfPlace = TRUE;
      }

     if ((what>=FILTER_SELF_AVOID_MOTIFS1) && (what<=FILTER_SELF_AVOID_MOTIFS9))
      {
       dropXY = selfAvoidTable(numPoints, w, h);
       motifType = what-FILTER_SELF_AVOID_MOTIFS1;
      }
     else
      {
       dropXY = (unsigned short *) malloc(numPoints*2*sizeof(unsigned short));
       if (!dropXY)
        return;
       for (i=0; i<numPoints; i++)
        {
         dropXY[i<<1]     = rand()%w;
         dropXY[(i<<1)+1] = rand()%h;
	}
      }
     if ((penX<=1) && (penY<=1))
      {
       for (i=0; i<numPoints; i++)
        if (!protColors[img.scanLine(dropXY[(i<<1)+1])[dropXY[i<<1]]])
	 img.scanLine(dropXY[(i<<1)+1])[dropXY[i<<1]]=fgCol;
      }
     else
      for (i=0; i<numPoints; i++)
       {
        sizeX = penX;
        sizeY = penY;
	if (doVarSize && (penX!=penY))
	 sizeX = sizeY = min(penX, penY) + rand()%abs(penX-penY);

	if (doGradient)
	 {
	  if (doVarSize && (penX!=penY))
	   {
            randOff = rand()%(h/max(1, max(penX, penY)) + 1);
	    sizeX = min(penX, penY) + (abs(penX-penY)*(dropXY[(i<<1)+1]+randOff))/h;
	    sizeY = min(penX, penY) + (abs(penX-penY)*(dropXY[(i<<1)+1]+randOff))/h;
            dropXY[(i<<1)+1] += max(0, h-(min(penX, penY) + abs(penX-penY))/2);
	   }
	  else
	   {
            randOff = rand()%(h/max(1, penX) + 1);
	    sizeX = max(1, (sizeX*(dropXY[(i<<1)+1]+randOff))/h);
	    sizeY = max(1, (sizeY*(dropXY[(i<<1)+1]+randOff))/h);
            dropXY[(i<<1)+1] += max(0, h-sizeY);
	   }
	 }
        if ((numNoiseColors>=2) && (img.type()== LOAD_CMAP))
         drawColor = mapNoiseColors[rand()%numNoiseColors];

        if (drawIfPlace)
	 {
          dropX = dropXY[i<<1];
	  dropY = dropXY[(i<<1)+1];
	  sizeX2 = sizeX;
	  sizeY2 = sizeY;
          
	  dropX = (dropX-1+w)%w;
	  dropY = (dropY-1+h)%h;
	  if (motifType<=MOTIF_SQUARE_TRIANGLE)
	   { /* enlarge the tested motif by one pixel while checking for space */
	    sizeX2+=2;
	    sizeY2+=2;
	    if (drawMotif(img, w, h, protColors, motifType, drawColor, dropX, dropY, sizeX2, sizeY2, TRUE))
	     drawMotif(img, w, h, protColors, motifType, drawColor, dropXY[i<<1], dropXY[(i<<1)+1], sizeX, sizeY, FALSE);
	   }
	  else
	   { /* move the tested motif by one pixel while checking for space */
            unsigned char centerColor = img.scanLine((dropY + 1 + sizeY/2)%h)[(dropX + 1 + sizeX/2)%w];
	    bool drawOK=TRUE;

            /* first check one point in all directions if it is same as center color */
	    if (img.scanLine((dropY + 0 + sizeY/2)%h)[(dropX + 1 + sizeX/2)%w]!=centerColor)
	     drawOK=FALSE;
	    if (img.scanLine((dropY + 1 + sizeY/2)%h)[(dropX + 0 + sizeX/2)%w]!=centerColor)
	     drawOK=FALSE;
	    if (img.scanLine((dropY + 2 + sizeY/2)%h)[(dropX + 1 + sizeX/2)%w]!=centerColor)
	     drawOK=FALSE;
	    if (img.scanLine((dropY + 1 + sizeY/2)%h)[(dropX + 2 + sizeX/2)%w]!=centerColor)
	     drawOK=FALSE;

            /* then try drawing complete motif moved by one in all directions */
            if (drawOK)
             drawOK = drawMotif(img, w, h, protColors, motifType, drawColor, dropX, dropY, sizeX, sizeY, TRUE);
            if (drawOK)
             drawOK = drawMotif(img, w, h, protColors, motifType, drawColor, dropX+1, dropY, sizeX, sizeY, TRUE);
            if (drawOK)
             drawOK = drawMotif(img, w, h, protColors, motifType, drawColor, dropX, dropY+1, sizeX, sizeY, TRUE);
            if (drawOK)
             drawOK = drawMotif(img, w, h, protColors, motifType, drawColor, dropX+1, dropY+1, sizeX, sizeY, TRUE);
            if (drawOK)
             drawOK = drawMotif(img, w, h, protColors, motifType, drawColor, dropX+2, dropY+1, sizeX, sizeY, TRUE);
            if (drawOK)
             drawOK = drawMotif(img, w, h, protColors, motifType, drawColor, dropX+1, dropY+2, sizeX, sizeY, TRUE);
            if (drawOK)
             drawMotif(img, w, h, protColors, motifType, drawColor, dropX+1, dropY+1, sizeX, sizeY, FALSE);
	   }
	 }
	else
         drawMotif(img, w, h, protColors, motifType, drawColor, dropXY[i<<1], dropXY[(i<<1)+1], sizeX, sizeY, FALSE);
       }
     free(dropXY);
     break;
    }

   case FILTER_NOISE_XY:
   case FILTER_NOISE_X:
   case FILTER_NOISE_Y:
    {
     int i, j, len, numColors, pixel, length, offset, pos, bpl, numNoiseColors, lastNoisePixel;
     uchar *selPtr;
     unsigned char mapNoiseColors[256];
     unsigned char lineCache[w];
     int minLen = max(1, min(penX, penY));
     int maxLen = max(1, max(penX, penY));
     int modLen = maxLen-minLen;

     if (img.type() != LOAD_CMAP)
      return;

     bpl = img.data->imageBpl;
     numColors = img.numColors();
     numNoiseColors = 0;
     for (i=0; i<numColors; i++)
      if (!protColors[i])
       mapNoiseColors[numNoiseColors++]=i;

     if (numNoiseColors<2)
      return;

     lastNoisePixel = mapNoiseColors[0];

     if (value==0)
      {
       /* make one pixel dots */
       if (what==FILTER_NOISE_XY)
	for (int y = 0; y < h; y++)
	 {
	  selPtr = img.scanLine(y);
	  for (int x = 0; x<w; x++)
	   if (protColors[*selPtr])
	    selPtr++;
	   else
            *selPtr++ = mapNoiseColors[rand()%numNoiseColors];
	 }
       /* draw horizontal lines */
       if (what==FILTER_NOISE_X)
        {
	 len = 1;
	 pixel = mapNoiseColors[rand()%numNoiseColors];
	 for (int y = 0; y < h;)
	  {
           if ((penX==0)&&(penY==0))
	    pixel = mapNoiseColors[rand()%numNoiseColors];
	   else
	    {
             while (pixel==lastNoisePixel)
	      pixel = mapNoiseColors[rand()%numNoiseColors];
	     if (modLen)
	      len = minLen + (rand()%modLen);
	     else
	      len = minLen;
	    }
           for (j=0; j<len; j++)
	    if ((y+j)<h)
	     {
	      selPtr = img.scanLine(y+j);
	      for (int x = 0; x<w; x++)
	       if (protColors[*selPtr])
		selPtr++;
	       else
        	*selPtr++ = pixel;
	     }
	   y+=len;
	   lastNoisePixel = pixel;
	  }
	}
       /* draw vertical lines */
       if (what==FILTER_NOISE_Y)
        {
	 pixel = mapNoiseColors[rand()%numNoiseColors];
         if ((penX==0)&&(penY==0))
	  for (int x = 0; x<w; x++)
           lineCache[x] = mapNoiseColors[rand()%numNoiseColors];
	 else
	  for (int x = 0; x<w; )
	   {
            while (pixel==lastNoisePixel)
	     pixel = mapNoiseColors[rand()%numNoiseColors];
	    if (modLen)
	     len = minLen + (rand()%modLen);
	    else
	     len = minLen;
           for (j=0; j<len; j++)
	     if ((x+j)<w)
              lineCache[x+j] = pixel;
	    x+=len;
	    lastNoisePixel = pixel;
	   }
	 for (int y = 0; y < h; y++)
	  {
	   selPtr = img.scanLine(y);
	   for (int x = 0; x<w; x++)
	    if (protColors[*selPtr])
	     selPtr++;
	    else
             *selPtr++ = lineCache[x];
	  }
	}
      }
     else
      {
       penX = max(1, penX);
       penY = max(1, penY);
       /* draw squares */
       if (what==FILTER_NOISE_XY)
	for (int y = 0; y < h; y+=penY)
	 {
	  for (int x = 0; x<w; x+=penX)
           lineCache[x] = mapNoiseColors[rand()%numNoiseColors];

	  for (int yy=0; yy<penY; yy++)
	   if (y+yy<h)
	    {
	     selPtr = img.scanLine(y+yy);
	     for (int x =0; x<w; x+=penX)
	      {
	       for (int xx=0; xx<penX; xx++)
		if (x+xx<w)
	         if (!protColors[selPtr[x+xx]])
        	  selPtr[x+xx] = lineCache[x];
	      }
	    }
	 }
       /* draw horizontal line sections */
       if (what==FILTER_NOISE_X)
	for (int y = 0; y < h; y++)
	 {
	  pixel = mapNoiseColors[rand()%numNoiseColors];
	  length = 0;
	  offset = rand()%w;
	  selPtr = img.scanLine(y);
	  for (int x = 0; x<w; x++)
	   {
	    if (!length)
	     {
              while (pixel==lastNoisePixel)
	       pixel = mapNoiseColors[rand()%numNoiseColors];
	      if (modLen)
	       length = minLen + rand()%modLen;
	      else
	       length = minLen;
	      lastNoisePixel = pixel;
	     }
	    pos = x+offset;
	    if (pos>=w)
	     pos-=w;
	    if (!protColors[selPtr[pos]])
             selPtr[pos] = pixel;
	    length--;
	   }
	 }
       /* draw vertical line sections */
       if (what==FILTER_NOISE_Y)
	for (int x = 0; x < w; x++)
	 {
	  pixel = mapNoiseColors[rand()%numNoiseColors];
	  length = 0;
	  offset = rand()%h;
	  selPtr = img.scanLine(0)+x;
	  for (int y = 0; y<h; y++)
	   {
	    if (!length)
	     {
              while (pixel==lastNoisePixel)
	       pixel = mapNoiseColors[rand()%numNoiseColors];
	      if (modLen)
	       length = minLen + rand()%modLen;
	      else
	       length = minLen;
	      lastNoisePixel = pixel;
	     }
	    pos = y+offset;
	    if (pos>=h)
	     pos-=h;
	    if (!protColors[selPtr[pos*bpl]])
             selPtr[pos*bpl] = pixel;
	    length--;
	   }
	 }
      }
    }
    break;

   case FILTER_GRADIENT_X:
   case FILTER_GRADIENT_Y:
    {
     uchar *selPtr;
     int bidirectional = doGradient;
     int h2 = bidirectional ? h/2 : h;
     int w2 = bidirectional ? w/2 : w;
     int cmapImage = (img.type() == LOAD_CMAP);
     int horizontal = (what==FILTER_GRADIENT_X);
     int color1 = fgCol;
     int color2 = bgCol;
     int rgb1[3], rgb2[3], rgbDiff[3], dstCol[3];
     int numVarColors = abs(color1-color2);
     int dirUp = (color1<color2) ? 1 : -1;

     /* prevent crash on division or modulo */
     if (bidirectional && ((horizontal & !w2) || (!horizontal && !h2)))
      return;

     if (cmapImage && doVarColors)
      if ((horizontal && (w2<numVarColors)) || (!horizontal && (h2<numVarColors)))
       return;

     rgb1[0]=qRed(fgCol);
     rgb1[1]=qGreen(fgCol);
     rgb1[2]=qBlue(fgCol);
     rgb2[0]=qRed(bgCol);
     rgb2[1]=qGreen(bgCol);
     rgb2[2]=qBlue(bgCol);
     rgbDiff[0] = rgb2[0]-rgb1[0];
     rgbDiff[1] = rgb2[1]-rgb1[1];
     rgbDiff[2] = rgb2[2]-rgb1[2];

     for (int y = 0; y < h; y++)
      {
       selPtr = img.scanLine(y);
       if (horizontal)
        {
	 if (cmapImage)
	  {
	   if (doVarColors)
	    {
	     if (bidirectional)
	      for (int x = 0; x<w; x++)
	       {
		if (protColors[*selPtr])
		 selPtr++;
		else
		 {
 	          if (x<w2)      
        	   *selPtr++ = color1 + ((x*numVarColors)/w2 + (((rand()%(w2/numVarColors))>(x-(((x*numVarColors)/w2)*w2)/numVarColors )) ? 0 : 1))*dirUp;
		  else
        	   *selPtr++ = color2 - (((x-w2)*numVarColors)/w2 + (((rand()%(w2/numVarColors))>((x-w2)-((((x-w2)*numVarColors)/w2)*w2)/numVarColors )) ? 0 : 1))*dirUp;
		 }
	       }
	     else
	      for (int x = 0; x<w; x++)
	       {
		if (protColors[*selPtr])
		 selPtr++;
		else
        	 *selPtr++ = color1 + ((x*numVarColors)/w + (((rand()%(w/numVarColors))>(x-(((x*numVarColors)/w)*w)/numVarColors )) ? 0 : 1))*dirUp;
	       }
	    }
           else
	    {
	     if (bidirectional)
	      for (int x = 0; x<w; x++)
	       {
		if (protColors[*selPtr])
		 selPtr++;
		else
		 {
	          if (x<w2)
                   *selPtr++ = ((rand()%w2)>x) ? color1 : color2;
		  else
                   *selPtr++ = ((rand()%w2)>(x-w2)) ? color2 : color1;
		 }
	       }
	     else
	      for (int x = 0; x<w; x++)
	       {
		if (protColors[*selPtr])
		 selPtr++;
		else
        	 *selPtr++ = ((rand()%w)>x) ? color1 : color2;
	       }
	    }
	  }
	 else
	  {
	   if (bidirectional)
	    for (int x = 0; x<w; x++)
	     {
	      if (x<w2)
	       {
		*selPtr++ = rgb1[0] + (x*rgbDiff[0])/w2;
		*selPtr++ = rgb1[1] + (x*rgbDiff[1])/w2;
		*selPtr++ = rgb1[2] + (x*rgbDiff[2])/w2;
	       }
	      else
	       {
		*selPtr++ = rgb2[0] - ((x-w2)*rgbDiff[0])/w2;
		*selPtr++ = rgb2[1] - ((x-w2)*rgbDiff[1])/w2;
		*selPtr++ = rgb2[2] - ((x-w2)*rgbDiff[2])/w2;
	       }
	     }
	   else
	    for (int x = 0; x<w; x++)
	     {
	      *selPtr++ = rgb1[0] + (x*rgbDiff[0])/w;
	      *selPtr++ = rgb1[1] + (x*rgbDiff[1])/w;
	      *selPtr++ = rgb1[2] + (x*rgbDiff[2])/w;
	     }
	  }
	}
       else
        {
	 if (cmapImage)
	  {
	   if (doVarColors)
	    {
	     if (bidirectional)
	      for (int x = 0; x<w; x++)
	       {
		if (protColors[*selPtr])
		 selPtr++;
		else
		 {
 	          if (y<h2)      
        	   *selPtr++ = color1 + ((y*numVarColors)/h2 + (((rand()%(h2/numVarColors))>(y-(((y*numVarColors)/h2)*h2)/numVarColors )) ? 0 : 1))*dirUp;
		  else
        	   *selPtr++ = color2 - (((y-h2)*numVarColors)/h2 + (((rand()%(h2/numVarColors))>((y-h2)-((((y-h2)*numVarColors)/h2)*h2)/numVarColors )) ? 0 : 1))*dirUp;
		 }
	       }
	     else
	      for (int x = 0; x<w; x++)
	       {
		if (protColors[*selPtr])
		 selPtr++;
		else
        	 *selPtr++ = color1 + ((y*numVarColors)/h + (((rand()%(h/numVarColors))>(y-(((y*numVarColors)/h)*h)/numVarColors )) ? 0 : 1))*dirUp;
	       }
	    }
	   else
	    {
	     if (bidirectional)
	      for (int x = 0; x<w; x++)
	       {
		if (protColors[*selPtr])
		 selPtr++;
		else
		 {
	          if (y<h2)
                   *selPtr++ = ((rand()%h2)>y) ? color1 : color2;
		  else
                   *selPtr++ = ((rand()%h2)>(y-h2)) ? color2 : color1;
        	}
	       }
	     else
	      for (int x = 0; x<w; x++)
	       {
		if (protColors[*selPtr])
		 selPtr++;
		else
        	 *selPtr++ = ((rand()%h)>y) ? color1 : color2;
	       }
	    }
	  }
	 else
	  {
	   if (bidirectional)
	    {
	     if (y<h2)
	      {
	       dstCol[0] = rgb1[0] + (y*rgbDiff[0])/h2;
	       dstCol[1] = rgb1[1] + (y*rgbDiff[1])/h2;
	       dstCol[2] = rgb1[2] + (y*rgbDiff[2])/h2;
	      }
	     else
	      {
	       dstCol[0] = rgb2[0] - ((y-h2)*rgbDiff[0])/h2;
	       dstCol[1] = rgb2[1] - ((y-h2)*rgbDiff[1])/h2;
	       dstCol[2] = rgb2[2] - ((y-h2)*rgbDiff[2])/h2;
	      }
	    }
	   else
	    {
	     dstCol[0] = rgb1[0] + (y*rgbDiff[0])/h;
	     dstCol[1] = rgb1[1] + (y*rgbDiff[1])/h;
	     dstCol[2] = rgb1[2] + (y*rgbDiff[2])/h;
	    }
	   for (int x = 0; x<w; x++)
	    {
	     *selPtr++ = dstCol[0];
	     *selPtr++ = dstCol[1];
	     *selPtr++ = dstCol[2];
	    }
	  }
	}
      }
     }   
    break;

   case FILTER_IKAT_X:
   case FILTER_IKAT_Y:
   case FILTER_MARBLE_X:
   case FILTER_MARBLE_Y:
   case FILTER_ZIGZAG_X:
   case FILTER_ZIGZAG_Y:
   case FILTER_WAVE_X:
   case FILTER_WAVE_Y:
    {
     uchar *selPtr, *srcPtr;
     int horizontal = (what==FILTER_IKAT_X) || (what==FILTER_MARBLE_X) || (what==FILTER_ZIGZAG_X) || (what==FILTER_WAVE_X);
     int offset=0, bpl, pos;
     unsigned char line[MAX_IMAGE*3];
     int cmapImage = (img.type() == LOAD_CMAP);
     int offsetTable[horizontal ? h : w];

     bpl = img.data->imageBpl;

     makeWaveTable(offsetTable, horizontal ? h : w, value, horizontal ? w : h, horizontal ? penY : penX, doGradient,
      (what==FILTER_IKAT_X) || (what==FILTER_IKAT_Y), (what==FILTER_MARBLE_X) || (what==FILTER_MARBLE_Y), (what==FILTER_ZIGZAG_X) || (what==FILTER_ZIGZAG_Y), (what==FILTER_WAVE_X) || (what==FILTER_WAVE_Y));

     if (horizontal)
      for (int y = 0; y < h; y++)
       {
	selPtr = img.scanLine(y);
	offset = offsetTable[y];
	if (offset)
	 {
	  memcpy(line, selPtr, bpl);
	  if (cmapImage)
	   for (int x = 0; x<w; x++)
	    {
	     pos = x+offset;
	     if (pos<0)
	      pos+=w;
	     else
	      if (pos>=w)
	       pos-=w;
	     if (protColors[*selPtr] || protColors[line[pos]])
	      selPtr++;
	     else
              *selPtr++ = line[pos];
	    }
	  else
	   for (int x = 0; x<w; x++)
	    {
	     pos = x+offset;
	     if (pos<0)
	      pos+=w;
	     else
	      if (pos>=w)
	       pos-=w;
	     srcPtr = &line[pos*3];
             *selPtr++ = *srcPtr++;
             *selPtr++ = *srcPtr++;
             *selPtr++ = *srcPtr;
	    }
	 }
       }
     else
      for (int x=0; x<w; x++)
       if (offsetTable[x])
        {
	 offset = offsetTable[x];
	 if (cmapImage)
	  {
           selPtr = img.scanLine(0)+x;
           for (int y=0; y<h; y++)
	    {
	     pos = y+offset;
	     if (pos<0)
	      pos+=h;
	     else
	      if (pos>=h)
	       pos-=h;

	     if (protColors[selPtr[pos*bpl]])
	      line[y] = selPtr[y*bpl];
	     else
	      line[y] = selPtr[pos*bpl];
	    }
           for (int y=0; y<h; y++)
	    if (!protColors[selPtr[y*bpl]])
	     selPtr[y*bpl]=line[y];
	  }
	 else
	  {
           selPtr = img.scanLine(0)+x*3;
	   srcPtr = line;
           for (int y=0; y<h; y++)
	    {
	     pos = y+offset;
	     if (pos<0)
	      pos+=h;
	     else
	      if (pos>=h)
	       pos-=h;
             pos*=bpl;
	     *srcPtr++ = selPtr[pos++];
	     *srcPtr++ = selPtr[pos++];
	     *srcPtr++ = selPtr[pos];
	    }
	   srcPtr = line;
           for (int y=0; y<h; y++)
	    {
	     selPtr[y*bpl] = *srcPtr++;
	     selPtr[y*bpl+1] = *srcPtr++;
	     selPtr[y*bpl+2] = *srcPtr++;
	    }
	  }
       }
     }
    break;

   case FILTER_IKAT_XY:
   case FILTER_MARBLE_XY:
   case FILTER_ZIGZAG_XY:
   case FILTER_WAVE_XY:
    {
     unsigned char *original, *originalPtr;
     unsigned char *dst;
     int x, y, bpl, offsetX[w], offsetY[h], posX, posY;

     bpl = img.data->imageBpl;
     original = (unsigned char *) malloc(bpl*h);
     if (!original)
      return;
     memcpy(original, img.scanLine(0), bpl*h);
     makeWaveTable(offsetY, h, value, w, penY, doGradient, what==FILTER_IKAT_XY, what==FILTER_MARBLE_XY, what==FILTER_ZIGZAG_XY, what==FILTER_WAVE_XY);
     makeWaveTable(offsetX, w, value, h, penX, doGradient, what==FILTER_IKAT_XY, what==FILTER_MARBLE_XY, what==FILTER_ZIGZAG_XY, what==FILTER_WAVE_XY);

     if (img.type() == LOAD_CMAP)
      for (y = 0; y < h; y++)
       {
	dst = img.scanLine(y);
	for (x = 0; x < w; x++)
	 {
          posX = x+offsetY[y];
	  while (posX<0)
	   posX+=w;
	  while (posX>=w)
	   posX-=w;

          posY = y+offsetX[posX];
	  while (posY<0)
	   posY+=h;
	  while (posY>=h)
	   posY-=h;

          posX = x+offsetY[posY];
	  while (posX<0)
	   posX+=w;
	  while (posX>=w)
	   posX-=w;

          *dst++ = original[posY*bpl+posX];
	 }
	}
     else /* true color */
      for (y = 0; y < h; y++)
       {
	dst = img.scanLine(y);
	for (x = 0; x < w; x++)
	 {
          posX = x+offsetY[y];
	  while(posX<0)
	   posX+=w;
	  while (posX>=w)
	   posX-=w;

          posY = y+offsetX[posX];
	  while (posY<0)
	   posY+=h;
	  while (posY>=h)
	   posY-=h;

          posX = x+offsetY[posY];
	  while (posX<0)
	   posX+=w;
	  while (posX>=w)
	   posX-=w;

          originalPtr = &original[posY*bpl+posX*3];
          *dst++ = *originalPtr++;
          *dst++ = *originalPtr++;
          *dst++ = *originalPtr;
	 }
	}

     free(original);
    }
   break;

   case FILTER_CONTOUR:
    {
     int numColors, numNoiseColors;
     unsigned char mapNoiseColors[256];
     unsigned char drawColor = fgCol;
     int value2, numCheckPoints=0;

     unsigned char *data, *dataxy;
     int buffHalf;
     int i, j, x, y, xx, yy, ybpl;
     int bpl;
     int p1, p2;

     int thick = max(2, min(30, max(penX, penY)));
     int thick1 = max(2, min(30, max(penX, penY)));
     int thick2 = max(2, min(30, min(penX, penY)));

     char checkX[(thick*2+1)*(thick*2+1)], checkY[(thick*2+1)*(thick*2+1)];
     int checkYbpl[(thick*2+1)*(thick*2+1)]; /* int is correct, so it works both on 64 and 32 bits */
     int checkXYbpl[(thick*2+1)*(thick*2+1)];
     int checkDistance[(thick*2+1)*(thick*2+1)];
     unsigned short xmod[w+2*thick], *xmodPtr; /* can be unsigned short, since image is always smaller than this */
     unsigned long ymod[h+2*thick], *ymodPtr; /* must be unsigned long, since we multiply "y" by bytes per line */

     if (img.type() != LOAD_CMAP)
      return;

     if (doVarSize && (thick1>thick2))
      thick = thick2 + rand()%(thick1-thick2+1);

     bpl = img.data->imageBpl;
     numColors = img.numColors();
     numNoiseColors = 0;
     if (doVarColors)
      for (i=0; i<numColors; i++)
       if (!protColors[i])
	mapNoiseColors[numNoiseColors++]=i;

     if (numNoiseColors>=2)
      drawColor = mapNoiseColors[rand()%numNoiseColors];
     bpl = img.data->imageBpl;

     p1 = max(2, min(30, min(penX, penY)));
     p2 = max(2, min(30, max(penX, penY)));
     if (p1!=p2)
      thick = p1 + rand()%(p2-p1);

     /* make a table of overruns to avoid expensive modulo */
     /* access the array by pointer with offset to allow negative values in array access */
     xmodPtr = &xmod[thick];
     ymodPtr = &ymod[thick];
     for (xx=-thick; xx<w+thick; xx++)
      xmodPtr[xx] = (xx+thick*w)%w;
     for (yy=-thick; yy<h+thick; yy++)
      ymodPtr[yy] = ((yy+thick*h)%h)*bpl;
     /* if we do not work on the whole image, do not wrap around to the other side */
     if ((w!=img.width()) || (!doRepeat))
      for (xx=1; xx<=thick; xx++)
       {
	xmodPtr[-xx] =  xmodPtr[0];
	xmodPtr[w+xx-1] =  xmodPtr[w-1];
       }
     if (!doRepeat)
      for (yy=1; yy<=thick; yy++)
       {
	ymodPtr[-yy] =  ymodPtr[0];
	ymodPtr[h+yy-1] =  ymodPtr[h-1];
       }

     /* make a list of points around the center averaging point, use circle, not sqaure checking */
     for (xx=-thick; xx<=thick; xx++)
      for (yy=-thick; yy<=thick; yy++)
       if ((xx*xx + yy*yy)<=(thick*thick))
	{
	 checkX[numCheckPoints]=xx;
	 checkY[numCheckPoints]=yy;
	 checkDistance[numCheckPoints] = xx*xx + yy*yy;
	 checkYbpl[numCheckPoints]=yy*bpl;
	 checkXYbpl[numCheckPoints++]=yy*bpl+xx;
	}
     /* sort the check points so we check them in order */
     for (i=0; i<numCheckPoints; i++)
      for (j=i; j<numCheckPoints; j++)
       if (checkDistance[i]>checkDistance[j])
        {
	 SWAP(checkX[i], checkX[j])
	 SWAP(checkY[i], checkY[j])
	 SWAP(checkDistance[i], checkDistance[j])
	 SWAP(checkYbpl[i], checkYbpl[j])
	 SWAP(checkXYbpl[i], checkXYbpl[j])
	}

     if (value==0)
      value = 1;
     value2 = max(1, min(thick-value, thick-1));
     if (doVarSize)
      value2 = max(1, min(thick-(rand()%value+1), thick-1));

     value2 *= value2;

     buffHalf = w * thick*8;	/* allocate thick*8 rows of data */
     if (!img.allocDrawBuff(buffHalf))
      return;
     data = img.data->imageData;

     for (y = 0; y < h; y++)
      {
       ybpl = y*bpl;
       if (img.data->changecount > (buffHalf/2))
	img.flushChanges(protColors, FALSE);
       for (x = 0; x < w; x++)
	{
	 dataxy = &data[ybpl + x];
	 if ((*dataxy!=drawColor) && (!protColors[*dataxy]))
	  {
	   /* checking all the points without overrun check is faster */
	   i = 0;
	   if ((x>thick) && (x<(w-thick)) && (y>thick) && (y<(h-thick)))
	    while ( (i<numCheckPoints) && (dataxy[checkXYbpl[i]] == *dataxy) )
	     i++;
	   else
	    while ( (i<numCheckPoints) && (data[ymodPtr[y+checkY[i]] + xmodPtr[x+checkX[i]] ] == *dataxy) )
	     i++;

           if (i<numCheckPoints)
	    {
	     if (checkDistance[i]>value2)
	      img.setChangePosVal(img.data->changecount++, ybpl + x, drawColor);
	     else
	      {
	       if ((drawColor!=bgCol) && (!doVarColors))
		img.setChangePosVal(img.data->changecount++, ybpl + x, bgCol);
	      }
	    }
	  }
	}
      }
     img.flushChanges(protColors, TRUE);
     img.freeDrawBuff();
    }
    break;

   case FILTER_MIRROR_X:
    {
     uchar *srcPtr;
     int cmapImage = (img.type() == LOAD_CMAP);
     uchar tmpPix[3];
     int segment, numSegment;
     int haveProtColors = FALSE;

     if (cmapImage)
      for (int i=0; i<img.numColors(); i++)
       if (!protColors[i])
	haveProtColors = TRUE;
     
     for (int y = 0; y < h; y++)
      {
       srcPtr = img.scanLine(y);
       numSegment = 0;
       if (cmapImage)
        for (int x = 0; x < w; )
         {
	  if (value)
	   {
	    segment = (w-x)/(penX-numSegment);
	    numSegment++;
	   }
	  else
	   segment = max(2, penX);
	  if (haveProtColors)
	   {
	    if ((x+segment)<=w)
	     for (int xx = 0; xx < (segment>>1); xx++)
	      if (!(protColors[srcPtr[x+xx]] || protColors[srcPtr[x+segment-xx-1]]))
	       {
		tmpPix[0] = srcPtr[x+xx];
		srcPtr[x+xx] = srcPtr[x+segment-xx-1];
		srcPtr[x+segment-xx-1] = tmpPix[0];
	       }
	   }
	  else
	   {
	    if ((x+segment)<=w)
	     for (int xx = 0; xx < (segment>>1); xx++)
	      {
	       tmpPix[0] = srcPtr[x+xx];
	       srcPtr[x+xx] = srcPtr[x+segment-xx-1];
	       srcPtr[x+segment-xx-1] = tmpPix[0];
	      }
	   }
          x+=segment;
	 }
       else
        for (int x = 0; x < w; )
         {
	  if (value)
	   {
	    segment = (w-x)/(penX-numSegment);
	    numSegment++;
	   }
	  else
	   segment = max(2, penX);
	  if ((x+segment)<=w)
	   for (int xx = 0; xx < (segment>>1); xx++)
	    {
	     memcpy(tmpPix, &srcPtr[(x+xx)*3], 3);
	     memcpy(&srcPtr[(x+xx)*3], &srcPtr[(x+segment-xx-1)*3] , 3);
	     memcpy(&srcPtr[(x+segment-xx-1)*3], tmpPix, 3);
	    }
          x+=segment;
	 }
      }
    }
    break;

   case FILTER_MIRROR_Y:
    {
     unsigned char line[MAX_IMAGE*3];
     int segment, numSegment;
     int haveProtColors = FALSE;
     uchar *dst1, *dst2;

     if (img.type() == LOAD_CMAP)
      for (int i=0; i<img.numColors(); i++)
       if (!protColors[i])
	haveProtColors = TRUE;

     numSegment = 0;
     for (int y = 0; y < h; )
      {
       if (value)
	{
	 segment = (h-y)/(penY-numSegment);
	 numSegment++;
	}
       else
	segment = max(2, penY);

       if ((y+segment)<=h)
	for (int yy = 0; yy < (segment>>1); yy++)
         {
          memcpy(line, img.scanLine(y+yy), img.data->imageBpl);
	  if (haveProtColors) /* also means we have colormap image */
	   {
	    dst1 = img.scanLine(y+yy);
	    dst2 = img.scanLine(y+segment-yy-1);
	    for (int x=0; x<w; x++)
	     if (!(protColors[dst1[x]] || protColors[dst2[x]]))
	      {
	       dst1[x]= dst2[x];
	       dst2[x]= line[x];
	      }
	   }
	  else
	   {
            memcpy(img.scanLine(y+yy), img.scanLine(y+segment-yy-1), img.data->imageBpl);
            memcpy(img.scanLine(y+segment-yy-1), line, img.data->imageBpl);
	   }
	 }
       y+=segment;
      }
    }
    break;

   case FILTER_MIX_X:
   case FILTER_MIX_Y:
    {
     uchar *selPtr, *srcPtr;
     int mixOffset = value;
     int thisOffset, bpl, pos;
     unsigned char line[MAX_IMAGE*3];
     int cmapImage = (img.type() == LOAD_CMAP);
     int rgb[3];

     if (!mixOffset)
      mixOffset = (what==FILTER_MIX_X) ? penX : penY;
     mixOffset = min(mixOffset, (what==FILTER_MIX_X) ? w : h);
     bpl = img.data->imageBpl;
     if (what==FILTER_MIX_X)
      for (int y = 0; y < h; y++)
       {
        thisOffset = mixOffset;
	if (doGradient)
	 thisOffset = (mixOffset*y)/h;
	selPtr = img.scanLine(y);
	memcpy(line, selPtr, bpl);
	if (cmapImage)
	 for (int x = 0; x<w; x++)
	  {
	   pos = x+((rand()>>2)%(thisOffset+1)) - (thisOffset>>1);
	   if (pos<0)
	    pos+=w;
	   else
	    if (pos>=w)
	     pos-=w;
	   if (protColors[*selPtr] || protColors[line[pos]])
	    selPtr++;
	   else
            *selPtr++ = line[pos];
	  }
	else
	 for (int x = 0; x<w; x++)
	  {
           rgb[0]=rgb[1]=rgb[2]=0;

           for (int i=0; i<32; i++)
	    {
	     pos = x+((rand()>>2)%(thisOffset+1)) - (thisOffset>>1);
	     if (pos<0)
	      pos+=w;
	     else
	      if (pos>=w)
	       pos-=w;
	     srcPtr = &line[pos*3];
             rgb[0]+= *srcPtr++;
             rgb[1]+= *srcPtr++;
             rgb[2]+= *srcPtr;
	    }
           *selPtr++ = rgb[0]>>5;
           *selPtr++ = rgb[1]>>5;
           *selPtr++ = rgb[2]>>5;
	  }
       }
     else /* (what==FILTER_MIX_Y) */
      for (int x=0; x<w; x++)
       {
        thisOffset = mixOffset;
	if (doGradient)
	 thisOffset = (mixOffset*x)/w;
	if (cmapImage)
	 {
          selPtr = img.scanLine(0)+x;
          for (int y=0; y<h; y++)
	   {
	    pos = y+((rand()>>2)%(thisOffset+1)) - (thisOffset>>1);
	    if (pos<0)
	     pos+=h;
	    else
	     if (pos>=h)
	      pos-=h;
	   if (protColors[selPtr[pos*bpl]])
	    line[y] = selPtr[y*bpl];
	   else
	    line[y] = selPtr[pos*bpl];
	   }
          for (int y=0; y<h; y++)
	   if (!protColors[selPtr[y*bpl]])
	    selPtr[y*bpl]=line[y];
	 }
	else
	 {
          selPtr = img.scanLine(0)+x*3;
	  srcPtr = line;
          for (int y=0; y<h; y++)
	   {
            rgb[0]=rgb[1]=rgb[2]=0;
            for (int i=0; i<32; i++)
	     {
	      pos = y+((rand()>>2)%(thisOffset+1)) - (thisOffset>>1);
	      if (pos<0)
	       pos+=h;
	      else
	       if (pos>=h)
		pos-=h;
              pos*=bpl;
	      rgb[0] += selPtr[pos++];
	      rgb[1] += selPtr[pos++];
	      rgb[2] += selPtr[pos];
	     }
	    *srcPtr++ = rgb[0]>>5;
	    *srcPtr++ = rgb[1]>>5;
	    *srcPtr++ = rgb[2]>>5;
	   }
	  srcPtr = line;
          for (int y=0; y<h; y++)
	   {
	    selPtr[y*bpl] = *srcPtr++;
	    selPtr[y*bpl+1] = *srcPtr++;
	    selPtr[y*bpl+2] = *srcPtr++;
	   }
	 }
       }
     }
    break;
#define MIX_RAND_TABLE_SHIFT	13
   case FILTER_MIX_XY:
    {
     unsigned char *data, *rgbdata=NULL, *rgbline;
     int cmapImage = (img.type() == LOAD_CMAP);
     int buffHalf;
     int i, x, y, xdim, ydim;
     int bpl, pos;
     int  mixpos;
     int numCheckPoints=0;
     unsigned short randList[1<<MIX_RAND_TABLE_SHIFT];
     int randAnd = (1<<MIX_RAND_TABLE_SHIFT)-1;
     int randPos;
     int rgb[3];

     if ( w < 0 ) w = img.width();
     if ( h < 0 ) h = img.height();
     if ( xin < 0 ) { w += xin; xin = 0; }
     if ( yin < 0 ) { h += xin; xin = 0; }
     if ( xin + w > img.width() ) w = img.width() - xin;
     if ( yin + h > img.height() ) h = img.height() - yin;
     if ( w <= 0 || h <= 0 ) return;

     bpl = img.data->imageBpl;
     xdim = w;
     ydim = h;
     {
      char checkX[(penX*2+1)*(penY*2+1)], checkY[(penX*2+1)*(penY*2+1)];
      unsigned short xmod[xdim+2*penX], *xmodPtr; /* can be unsigned short, since image is always smaller than this */
      unsigned long ymod[ydim+2*penY], *ymodPtr; /* must be unsigned long, since we multiply "y" by bytes per line */

      /* make a table of overruns to avoid expensive modulo */
      /* access the array by pointer with offset to allow negative values in array access */
      xmodPtr = &xmod[penX];
      ymodPtr = &ymod[penY];
      for (x=-penX; x<xdim+penX; x++)
       xmodPtr[x] = (x+penX*xdim)%xdim;
      for (y=-penY; y<ydim+penY; y++)
       ymodPtr[y] = ((y+penY*ydim)%ydim)*bpl;
      /* if we do not work on the whole image, do not wrap around to the other side */
      if ((xdim!=img.width()) || (!doRepeat))
       for (x=1; x<=penX; x++)
	{
	 xmodPtr[-x] =  xmodPtr[0];
	 xmodPtr[xdim+x-1] =  xmodPtr[xdim-1];
	}
      if ((ydim!=img.height()) || (!doRepeat))
       for (y=1; y<=penY; y++)
	{
	 ymodPtr[-y] =  ymodPtr[0];
	 ymodPtr[ydim+y-1] =  ymodPtr[ydim-1];
	}

      /* make a list of points around the center averaging point, use circle, not sqaure checking */
      for (x=-penX; x<=penX; x++)
       for (y=-penY; y<=penY; y++)
	if ((x*x + y*y)<=(penX*penY))
	 {
	  checkX[numCheckPoints]=x;
	  checkY[numCheckPoints++]=y;
	 }

      for (i=(1<<MIX_RAND_TABLE_SHIFT)-1; i>=0; i--)
       randList[i] = (unsigned short)(rand()%numCheckPoints);

      buffHalf = xdim * penY*2;	/* allocate penY*2 rows of data */
      if (cmapImage)
       {
        if (!img.allocDrawBuff(buffHalf))
         return;
       }
      else
       {/*  must have the whole image, or we get error at the end, because buffer is flushed from beginning, and date is read from the other side ... */
        rgbdata = (unsigned char *) malloc(ydim*bpl);
	if (!rgbdata)
	 return;
       }
      data = img.data->imageData;
      for (y = 0; y < ydim; y++)
       {
	if (doGradient)
	 {
	  int thisPenX, thisPenY;
	  int xx, yy;

          numCheckPoints=0;
	  thisPenX = (penX*y)/ydim;
	  thisPenY = (penY*y)/ydim;
	  /* make a list of points around the center averaging point, use circle, not sqaure checking */
	  for (xx=-thisPenX; xx<=thisPenX; xx++)
	   for (yy=-thisPenY; yy<=thisPenY; yy++)
	    if ((xx*xx + yy*yy)<=(thisPenX*thisPenY))
	     {
	      checkX[numCheckPoints]=xx;
	      checkY[numCheckPoints++]=yy;
	     }

	  for (i=(1<<MIX_RAND_TABLE_SHIFT); i>=0; i--)
	   if (numCheckPoints)
	    randList[i] = (unsigned short)(rand()%numCheckPoints);
	   else randList[i] = 0;
         }

        randPos = rand();
        if (cmapImage)
	 {
	  if (img.data->changecount > (buffHalf/2))
	   img.flushChanges(protColors, FALSE);
	  for (x = 0; x < xdim; x++)
	   {
	    pos = y*bpl + x;
	    mixpos = data[ymodPtr[y+checkY[randList[randPos&randAnd]]] + xmodPtr[x+checkX[randList[randPos&randAnd]]]];
	    if (!protColors[mixpos])
	     img.setChangePosVal(img.data->changecount++, pos, mixpos);
	    randPos++;
	   }
	 }
	else
         {
	  rgbline = &rgbdata[y*bpl];
	  for (x = 0; x < xdim; x++)
	   {
            rgb[0]=rgb[1]=rgb[2]=0;
            for (int i=0; i<32; i++)
             {
	      mixpos = ymodPtr[y+checkY[randList[randPos&randAnd]]] + xmodPtr[x+checkX[randList[randPos&randAnd]]]*3;
	      rgb[0]+=data[mixpos];
	      rgb[1]+=data[mixpos+1];
	      rgb[2]+=data[mixpos+2];
	      randPos++;
             }
	    *rgbline++ = rgb[0]>>5;
	    *rgbline++ = rgb[1]>>5;
	    *rgbline++ = rgb[2]>>5;
	   }
	 }
       }
     }
     if (cmapImage)
      {
       img.flushChanges(protColors, TRUE);
       img.freeDrawBuff();
      }
     else
      {
       for (y = 0; y < ydim; y++)
        memcpy(&data[y*bpl], &rgbdata[y*bpl], bpl);
       free(rgbdata);
      }
    }

   case FILTER_COLOR_SHORTER_INSIDE_X:
   case FILTER_COLOR_SHORTER_INSIDE_Y:
   case FILTER_COLOR_SHORTER_OUTSIDE_X:
   case FILTER_COLOR_SHORTER_OUTSIDE_Y:
    {
     uchar *selPtr;
     int shorter = ((what==FILTER_COLOR_SHORTER_INSIDE_X) || (what==FILTER_COLOR_SHORTER_OUTSIDE_X)) ? penX : penY;
     int x, y, z, bpl, len, changePos, numColors;
     unsigned char line[max(w,h)];
     unsigned char bgColList[256];

     if ((img.type() != LOAD_CMAP) || (bgCol==fgCol) || protColors[fgCol])
      return;

     shorter = min(shorter, ((what==FILTER_COLOR_SHORTER_INSIDE_X) || (what==FILTER_COLOR_SHORTER_OUTSIDE_X)) ? w : h);
     bpl = img.data->imageBpl;

     numColors = img.numColors();
     for (x=0; x<numColors; x++)
      bgColList[x]=protColors[x];
     bgColList[bgCol]=TRUE;

     if (what==FILTER_COLOR_SHORTER_INSIDE_X)
      for (y = 0; y < h; y++)
       {
	selPtr = img.scanLine(y);
	memcpy(line, selPtr, bpl);
        /* search for start */
	if (bgColList[line[0]] && (!bgColList[line[w-1]]))
	 changePos = 0;
	else
	 {
          changePos = 1;
          while ( (changePos<w) && !( bgColList[line[changePos]] && (!bgColList[line[changePos-1]]) ) )
	   changePos++;
         }
        if (changePos<w)
	 {
	  x = changePos;
	  while ((x-changePos)<w)
	   {
	    len = 0;
	    while (((x-changePos)<w) && bgColList[line[(x<w) ? x : x-w]])
	     { len++; x++; }
	    if (len<=shorter)
	     for (z=1; z<=len; z++)
	      selPtr[((x-z)<w) ? x-z : x-z-w]=fgCol;
	    while (((x-changePos)<w) && (!bgColList[line[(x<w) ? x : x-w]]))
	     x++;
	   }
	 }
       }

     if (what==FILTER_COLOR_SHORTER_INSIDE_Y)
      for (x = 0; x < w; x++)
       {
	selPtr = img.scanLine(0)+x;
        for (y = 0; y < h; y++)
	 {
	  line[y] = *selPtr;
	  selPtr+=bpl;
	 }
	selPtr = img.scanLine(0)+x;
        /* search for start */
	if (bgColList[line[0]] && (!bgColList[line[h-1]]))
	 changePos = 0;
	else
	 {
          changePos = 1;
          while ( (changePos<h) && !( bgColList[line[changePos]] && (!bgColList[line[changePos-1]]) ) )
	   changePos++;
         }
        if (changePos<h)
	 {
	  y = changePos;
	  while ((y-changePos)<h)
	   {
	    len = 0;
	    while (((y-changePos)<h) && bgColList[line[(y<h) ? y : y-h]])
	     { len++; y++; }
	    if (len<=shorter)
	     for (z=1; z<=len; z++)
	      selPtr[(((y-z)<h) ? y-z : y-z-h)*bpl]=fgCol;

	    while (((y-changePos)<h) && (!bgColList[line[(y<h) ? y : y-h]]))
	     y++;
	   }
	 }
       }

     if (what==FILTER_COLOR_SHORTER_OUTSIDE_X)
      for (y = 0; y < h; y++)
       {
	selPtr = img.scanLine(y);
	memcpy(line, selPtr, bpl);
        /* search for start */
	if ((!bgColList[line[0]]) && bgColList[line[w-1]])
	 changePos = 0;
	else
	 {
          changePos = 1;
          while ( (changePos<w) && !( (!bgColList[line[changePos]]) && bgColList[line[changePos-1]] ) )
	   changePos++;
         }
        if (changePos<w)
	 {
	  x = changePos;
	  while ((x-changePos)<w)
	   {
	    len = 0;
	    while (((x-changePos)<w) && (!bgColList[line[(x<w) ? x : x-w]]) )
	     { len++; x++; }
	    if (len<=shorter)
	     for (z=1; z<=len; z++)
	      selPtr[((x-z)<w) ? x-z : x-z-w]=fgCol;
	    while (((x-changePos)<w) && bgColList[line[(x<w) ? x : x-w]])
	     x++;
	   }
	 }
       }
     if (what==FILTER_COLOR_SHORTER_OUTSIDE_Y)
      for (x = 0; x < w; x++)
       {
	selPtr = img.scanLine(0)+x;
        for (y = 0; y < h; y++)
	 {
	  line[y] = *selPtr;
	  selPtr+=bpl;
	 }
	selPtr = img.scanLine(0)+x;
        /* search for start */
	if ((!bgColList[line[0]]) && bgColList[line[h-1]])
	 changePos = 0;
	else
	 {
          changePos = 1;
          while ( (changePos<h) && !( (!bgColList[line[changePos]]) && bgColList[line[changePos-1]] ) )
	   changePos++;
         }
        if (changePos<h)
	 {
	  y = changePos;
	  while ((y-changePos)<h)
	   {
	    len = 0;
	    while (((y-changePos)<h) && (!bgColList[line[(y<h) ? y : y-h]]))
	     { len++; y++; }
	    if (len<=shorter)
	     for (z=1; z<=len; z++)
	      selPtr[(((y-z)<h) ? y-z : y-z-h)*bpl]=fgCol;

	    while (((y-changePos)<h) && bgColList[line[(y<h) ? y : y-h]])
	     y++;
	   }
	 }
       }
     }
    break;
   case FILTER_TILT_RIGHT:
   case FILTER_TILT_LEFT:
   case FILTER_TILT_DOWN:
   case FILTER_TILT_UP:
    {
     uchar *selPtr;
     int cmapImage = (img.type()== LOAD_CMAP);
     int bpp = cmapImage ? 1 : 3;
     unsigned char line[max(w,h)*bpp], *linePtr;
     int x, y, pos, bpl, offset;
 
     bpl = img.data->imageBpl;

     if ((what==FILTER_TILT_LEFT) || (what==FILTER_TILT_RIGHT))
      for (y = 0; y < h; y++)
       {
	selPtr = img.scanLine(y);
	memcpy(line, selPtr, bpl);

        /* use h-y-1 instead of y, since we want to tilt from the buttom */
	if (penX==0)
	 offset = (int)(((h-y-1)*w*1.0)/h);
	else
	 offset = (h-y-1)/penX;

        /* move by absolute value of penY */
	if (penY && (penX==0) && (value==0))
	 offset = (int)(((h-y-1)*penY*1.0)/h);
 
	if (what==FILTER_TILT_LEFT)
         offset = offset*w - offset;
        if (value)
	 offset*=value;
	offset*=bpp;
	if (cmapImage)
	 for (x = 0; x < w; x++)
          {
	   if (!protColors[*selPtr])
	    {
	     pos = (x+offset)%w;
	     if (!protColors[line[pos]])
              *selPtr = line[pos];
	    }
	   selPtr++;
	  }
	else
	 for (x = 0; x < bpl; x++)
          *selPtr++ = line[(x+offset)%bpl];
       }
     else /* (what==FILTER_TILT_UP) || (what==FILTER_TILT_DOWN) */
      for (x = 0; x < w; x++)
       {
	selPtr = img.scanLine(0) + x*bpp;
	linePtr = line;
	if (cmapImage)
	 for (y = 0; y < h; y++)
	  {
	   *linePtr++ = *selPtr;
	   selPtr+=bpl;
	  }
	else
	 for (y = 0; y < h; y++)
	  {
	   *linePtr++ = *selPtr++;
	   *linePtr++ = *selPtr++;
	   *linePtr++ = *selPtr;
	   selPtr+=bpl-2;
	  }
	selPtr = img.scanLine(0)+x*bpp;
	if (penY==0)
	 offset = (int)((x*h*1.0)/w);
	else
	 offset = x/penY;

        /* move by absolute value of penX */
	if (penX && (penY==0) && (value==0))
	 offset = (int)((x*penX*1.0)/w);

	if (what==FILTER_TILT_UP)
         offset = offset*h - offset;
        if (value)
	 offset*=value;
	if (cmapImage)
	 {
	  for (y = 0; y < h; y++)
	   if (!protColors[selPtr[bpl*y]])
	    {
	     pos = (y+offset)%h;
	     if (!protColors[line[pos]])
              selPtr[bpl*y] = line[pos];
	    }
	 }
	else
	 for (y = 0; y < h; y++)
	  {
	   linePtr = &line[((y+offset)%h)*3];
           *selPtr++ = *linePtr++;
           *selPtr++ = *linePtr++;
           *selPtr   = *linePtr++;
	   selPtr += bpl-2;
	  }
       }
    }
    break;
   case FILTER_CLEAR_IMAGE:
    {
     uchar *selPtr;
     int x, y;

     if (img.type() == LOAD_CMAP)
      for (y = 0; y < h; y++)
       {
	selPtr = img.scanLine(y);
	for (x = 0; x < w; x++)
         {
	  if (protColors[*selPtr])
	   selPtr++;
	  else
	   *selPtr++ = fgCol;
         }
       }
     else
      for (y = 0; y < h; y++)
       {
	selPtr = img.scanLine(y);
	for (x = 0; x < w; x++)
	 {
	  *selPtr++ = fgCol>>16;
	  *selPtr++ = (fgCol>>8)&255;
	  *selPtr++ = fgCol&255;
	 }
       }
    }
    break;
   case FILTER_SPLIT_X:
   case FILTER_JOIN_X:
    {
     unsigned char *original;
     int y, bpl, blockSize;

     if (penY<2)
      return;
     /* could be implemented with in memory swapping, without allocating another image */
     bpl = img.data->imageBpl;
     original = (unsigned char *) malloc(bpl*h);
     if (!original)
      return;
     memcpy(original, img.scanLine(0), bpl*h);

     blockSize = h/penY;
     if (what==FILTER_SPLIT_X)
      for (y = 0; y < h; y++)
       memcpy(img.scanLine(blockSize*(y%penY) + y/penY), &original[y*bpl], bpl);
     else
      for (y = 0; y < h; y++)
       memcpy(img.scanLine(y), &original[(blockSize*(y%penY) + y/penY)*bpl], bpl);

     free(original);
    }
   break;
   case FILTER_SPLIT_Y:
   case FILTER_JOIN_Y:
    {
     uchar *selPtr;
     unsigned char line[w];
     unsigned short lace[w];
     int x, y, bpl, blockSize;

     if ((img.type() != LOAD_CMAP) || (penX<2))
      return;
     bpl = img.data->imageBpl;
     blockSize = w/penX;
     for (x = 0; x < w; x++)
      lace[x] = blockSize*(x%penX) + x/penX;
     for (y = 0; y < h; y++)
      {
       selPtr = img.scanLine(y);
       memcpy(line, selPtr, bpl);
       if (what==FILTER_SPLIT_Y)
        for (x = 0; x < w; x++)
         selPtr[lace[x]] = line[x];
       else
        for (x = 0; x < w; x++)
         *selPtr++ = line[lace[x]];
      }
    }
    break;
   case FILTER_MOVE_RIGHT:
   case FILTER_MOVE_LEFT:
    {
     uchar *selPtr;
     int cmapImage = (img.type() == LOAD_CMAP);
     unsigned char line[w*(cmapImage ? 1 : 3)];
     unsigned short move[w*(cmapImage ? 1 : 3)];
     int x, y, pos, limit, bpl = img.data->imageBpl;

     penX = max(1, penX);
     penY = max(1, penY);
     for (x = 0; x < w; x++)
      {
       pos = ((what==FILTER_MOVE_LEFT) ? (x+penX) : (x+w*penX-penX))%w;
       if (cmapImage)
        move[x] = pos;
       else
        {
	 pos*=3;
	 move[x*3] = pos++;
	 move[x*3+1] = pos++;
	 move[x*3+2] = pos;
	}
      }
     limit = cmapImage ? w : w*3;
     for (y = 0; y < h; y++)
      {
       selPtr = img.scanLine(y);
       memcpy(line, selPtr, bpl);
       for (x = 0; x < limit; x++)
        *selPtr++ = line[move[x]];
      }
    }
    break;
   case FILTER_MOVE_UP:
   case FILTER_MOVE_DOWN:
    {
     int y, bpl = img.data->imageBpl;
     unsigned char *original;

     penX = max(1, penX);
     penY = max(1, penY);
     /* could be implemented with in memory swapping, without allocating another image */
     original = (unsigned char *) malloc(bpl*h);
     if (!original)
      return;
     memcpy(original, img.scanLine(0), bpl*h);

     for (y = 0; y < h; y++)
      memcpy(img.scanLine(y), &original[(((what==FILTER_MOVE_UP) ? (y+penY) : (y+h*penY-penY))%h)*bpl], bpl);
     free(original);
    }
    break;
  }
}

void ApaintImageUtil::averageContour24thickness(ApaintImage& img, int xin, int yin, int w, int h, bool doRepeat, int thickX, int thickY)
{
 unsigned char *data, *dataPtr, *dataxy;
 int i, x, y, ybpl, xdim, ydim;
 int bpl;
 int numCheckPoints=0;

 if (img.type() !=LOAD_TRUE_COL) return;

 if ( w < 0 ) w = img.width();
 if ( h < 0 ) h = img.height();
 if ( xin < 0 ) { w += xin; xin = 0; }
 if ( yin < 0 ) { h += xin; xin = 0; }
 if ( xin + w > img.width() ) w = img.width() - xin;
 if ( yin + h > img.height() ) h = img.height() - yin;
 if ( w <= 0 || h <= 0 ) return;

 bpl = img.data->imageBpl;
 xdim = w;
 ydim = h;

 thickX = min(30, thickX);
 thickY = min(30, thickY);
 {
  char checkX[(thickX*2+1)*(thickY*2+1)], checkY[(thickX*2+1)*(thickY*2+1)];
  int checkXYbpl[(thickX*2+1)*(thickY*2+1)];
  unsigned long xmod[xdim+2*thickX], *xmodPtr; /* must be unsigned long, since we multiply "x" by bytes per per pixel (3) */
  unsigned long ymod[ydim+2*thickY], *ymodPtr; /* must be unsigned long, since we multiply "y" by bytes per line */
  uint rgb[3];
  unsigned char *stripeData, *stripePtr;
  /* it makes horizontal stripe in repeat, if we draw interactively */
  /* it is not logical and probably there is some bug in the code. But by unsing full image as draw buffer, the result is */
  /* visually correct */
  int stripeY = min(ydim, doRepeat ? ydim : thickY+2);

  /* make a table of overrruns to avoid expensive modulo */
  /* access the array by pointer with offset to allow negative values in array access */
  xmodPtr = &xmod[thickX];
  ymodPtr = &ymod[thickY];
  for (x=-thickX; x<(xdim+thickX); x++)
   xmodPtr[x] = ((x+thickX*xdim)%xdim)*3;
  for (y=-thickY; y<(ydim+thickY); y++)
   ymodPtr[y] = ((y+thickY*ydim)%ydim)*bpl;
  /* if we do not work on the whole image, do not wrap around to the other side */
  if ((xdim!=img.width()) || (!doRepeat))
   for (x=1; x<=thickX; x++)
    {
     xmodPtr[-x] =  xmodPtr[0];
     xmodPtr[xdim+x-1] =  xmodPtr[xdim-1];
    }
  if ((ydim!=img.height()) || (!doRepeat))
   for (y=1; y<=thickY; y++)
    {
     ymodPtr[-y] =  ymodPtr[0];
     ymodPtr[ydim+y-1] =  ymodPtr[ydim-1];
    }

  /* make a list of points around the center averaging point, use circle, not square checking */
  for (x=-thickX; x<=thickX; x++)
   for (y=-thickY; y<=thickY; y++)
    if ((x*x + y*y)<=(thickX*thickY))
     {
      checkX[numCheckPoints]=x;
      checkY[numCheckPoints]=y;
      checkXYbpl[numCheckPoints++]=y*bpl+x*3;
     }

  data = img.data->imageData;
  stripeData = (unsigned char *) malloc(xdim*3*stripeY);
  if (stripeData)
   {
    for (y = 0; y < ydim; y++)
     {
      if (y>=stripeY)
       memcpy(&data[(y-stripeY)*bpl], &stripeData[((y-stripeY)%stripeY)*xdim*3], xdim*3);
      ybpl = y*bpl;
      stripePtr = &stripeData[(y%stripeY)*xdim*3];
      for (x = 0; x < xdim; x++)
       {
	rgb[0] = rgb[1] = rgb[2] = 0;

        dataxy = &data[ybpl + x*3];
	/* summing up all the points without overrun check is faster */
	if ((x>thickX) && (x<(xdim-thickX)) && (y>thickY) && (y<(ydim-thickY)))
	 for (i=0; i<numCheckPoints; i++)
          {
           dataPtr = &dataxy[checkXYbpl[i]];
	   rgb[0] += *dataPtr++;
	   rgb[1] += *dataPtr++;
	   rgb[2] += *dataPtr;
          }
	else
	 for (i=0; i<numCheckPoints; i++)
          {
           dataPtr = &data[ymodPtr[y+checkY[i]] + xmodPtr[x+checkX[i]]];
	   rgb[0] += *dataPtr++;
	   rgb[1] += *dataPtr++;
	   rgb[2] += *dataPtr;
          }
	*stripePtr++ = rgb[0]/numCheckPoints;
	*stripePtr++ = rgb[1]/numCheckPoints;
	*stripePtr++ = rgb[2]/numCheckPoints;
       }
     }
    for (y = ydim-stripeY; y < ydim; y++)
     memcpy(&data[y*bpl], &stripeData[(y%stripeY)*xdim*3], xdim*3);
    free(stripeData);
   }
 }
}

void ApaintImageUtil::averageContour(ApaintImage& img, int xin, int yin, int w, int h, bool *protColors, bool doRepeat,
                      int radius, double aspectX, double aspectY)
{
 int i, j, x, y, xdim, ydim;
 int xlimit, ylimit;
 int bpl, pos, bpp;
 uint  max, posval, maxval;
 unsigned char value[10];
 unsigned char count[10];
 int left, right, up, down, midx, midy, buffHalf;
 bool cmpMaxVal;
 int *xtab, *ytab;
 int rad = radius*radius;
 int xc, yc;
 int isLineAngle; /* check if we have horizontal, vertical or diagonal line, or perfect angle, and do not average in those cases */
 unsigned char *data;
 
 if (img.type() !=LOAD_CMAP) return;

 if ( w < 0 ) w = img.width();
 if ( h < 0 ) h = img.height();
 if ( xin < 0 ) { w += xin; xin = 0; }
 if ( yin < 0 ) { h += xin; xin = 0; }
 if ( xin + w > img.width() ) w = img.width() - xin;
 if ( yin + h > img.height() ) h = img.height() - yin;
 if ( w <= 0 || h <= 0 ) return;

 posval = 0;
 bpl = img.data->imageBpl;
 bpp = img.depth()/8;
 xdim = w;
 ydim = h;
 xlimit = xdim;
 ylimit = ydim;

 /* center of area */
 xc = xin+xdim/2;
 yc = yin+ydim/2;

 xtab = new int[xdim+2];
 ytab = new int[ydim+2];
 if (!xtab || !ytab)
  {
   qDebug("averageContour: NULL pointer!");
   return;
  }
 for (i=0; i<xdim+2; i++)
  {
   if (i==0 && doRepeat)
     xtab[i] = xin+xdim-1;
   else if (i == 0)
     xtab[i] = xin;
   else if (i == xdim+1 && doRepeat)
     xtab[i] = xin;
   else if (i == xdim+1)
     xtab[i] = xin+xdim-1;
   else
     xtab[i] = xin+i-1;
  }
 for (i=0; i<ydim+2; i++)
  {
   if (i==0 && doRepeat)
     ytab[i] = yin+ydim-1;
   else if (i==0)
     ytab[i] = yin;
   else if (i == ydim+1 && doRepeat)
     ytab[i] = yin;
   else if (i == ydim+1)
     ytab[i] = yin+ydim-1;
   else
     ytab[i] = yin+i-1;
  }

 buffHalf = xdim * 6;	/* allocate 6 rows of data */
 if (!img.allocDrawBuff(buffHalf))
  {
   delete[] xtab;
   delete[] ytab;
   return;
  }
 data = img.data->imageData;
 for (y = 1 ; y <= ylimit; y++)
  {
   if (img.data->changecount > buffHalf)
    img.flushChanges(protColors, FALSE);
   midy = (ytab[y]) * bpl;
   up = (ytab[y - 1]) * bpl;
   down = (ytab[y + 1]) * bpl;

   for (x = 1; x <= xlimit; x++)
    {
     if (rad>0 && (SQR((xtab[x]-xc)*aspectX)+SQR((ytab[y]-yc)*aspectY)>rad))
      continue;
     isLineAngle = FALSE;
     midx = xtab[x];
     pos = midx*bpp + midy;
     left = xtab[x - 1];
     right = xtab[x + 1];

     posval = data[pos];
     /* check if we have horizontal, vertical or diagonal line, or perfect angle, and do not average in those cases */
     if ((posval==data[midy + left]) && ( (posval==data[midy + right]) || (posval==data[up + right]) || (posval==data[down + right]) )) /* horizontal line + diagonal */
      isLineAngle=TRUE;
     else
      if ((posval==data[up + midx]) && ( (posval==data[down + midx]) || (posval==data[down + left]) || (posval==data[down + right]) )) /* vertical line + diagonal */
       isLineAngle=TRUE;
      else
       if ((posval==data[up + right]) && ( (posval==data[down + left]) )) /* right diagonal */
	isLineAngle=TRUE;
       else
	if ((posval==data[up + left]) && ( (posval==data[down + right]) )) /* left diagonal */
	 isLineAngle=TRUE;
	else
	 if ((posval==data[up + left]) && (posval==data[up + midx]) && (posval==data[midy + left])) /* upper left angle */
	  isLineAngle=TRUE;
	 else
	  if ((posval==data[up + right]) && (posval==data[up + midx]) && (posval==data[midy + right])) /* upper right angle */
	   isLineAngle=TRUE;
	  else
	   if ((posval==data[down + left]) && (posval==data[down + midx]) && (posval==data[midy + left])) /* lower left angle */
	    isLineAngle=TRUE;
	   else
	    if ((posval==data[down + right]) && (posval==data[down + midx]) && (posval==data[midy + right])) /* lower left angle */
	     isLineAngle=TRUE;
     if (!isLineAngle)
      {
       memset(count, 0, sizeof(unsigned char)*10);
       memset(value, 0, sizeof(unsigned char)*10);
       for (j = 0; j < 9; j++)
	{
	 switch (j)
   	  {
   	   case 0: posval = data[up + left]; break;
   	   case 1: posval = data[up + midx]; break;
   	   case 2: posval = data[up + right]; break;
   	   case 3: posval = data[midy + left]; break;
   	   case 4: posval = data[pos]; break;
   	   case 5: posval = data[midy + right]; break;
   	   case 6: posval = data[down + left]; break;
   	   case 7: posval = data[down + midx]; break;
   	   case 8: posval = data[down + right]; break;
   	  }

	 i = 0;
	 while ((value[i] != posval) && (count[i] != 0))
  	   i++;
	 value[i] = posval;
	 count[i]++;
	}
       maxval = value[0];
       max = count[0];
       i = 1;
       while (count[i] > 0)
	{
	 if (count[i] > max)
  	  { max = count[i]; maxval = value[i]; }
	 i++;
	}
       cmpMaxVal = maxval != img.data->imageData[pos];

       if (cmpMaxVal && (max > 2))
        img.setChangePosVal(img.data->changecount++, pos, maxval);
      }
    }
  }
 img.flushChanges(protColors, TRUE);
 img.freeDrawBuff();
 delete[] xtab;
 delete[] ytab;
}

void ApaintImageUtil::averageContourThickness(ApaintImage& img, int xin, int yin, int w, int h, bool *protColors, bool doRepeat, int thickX, int thickY)
{
 unsigned char *data, *dataxy;
 int buffHalf;
 int i, x, y, ybpl, xdim, ydim;
 int bpl;
 int maxpos;
 int numColors = img.numColors();
 int numCheckPoints=0;

 if (img.type() !=LOAD_CMAP) return;

 if ( w < 0 ) w = img.width();
 if ( h < 0 ) h = img.height();
 if ( xin < 0 ) { w += xin; xin = 0; }
 if ( yin < 0 ) { h += xin; xin = 0; }
 if ( xin + w > img.width() ) w = img.width() - xin;
 if ( yin + h > img.height() ) h = img.height() - yin;
 if ( w <= 0 || h <= 0 ) return;

 bpl = img.data->imageBpl;
 xdim = w;
 ydim = h;

 thickX = min(30, thickX);
 thickY = min(30, thickY);

 {
  unsigned short numPoints[numColors];
  char checkX[(thickX*2+1)*(thickY*2+1)], checkY[(thickX*2+1)*(thickY*2+1)];
  int checkXYbpl[(thickX*2+1)*(thickY*2+1)];
  unsigned short xmod[xdim+2*thickX], *xmodPtr; /* can be unsigned short, since image is always smaller than this */
  unsigned long ymod[ydim+2*thickY], *ymodPtr; /* must be unsigned long, since we multiply "y" by bytes per line */

  /* make a table of overruns to avoid expensive modulo */
  /* access the array by pointer with offset to allow negative values in array access */
  xmodPtr = &xmod[thickX];
  ymodPtr = &ymod[thickY];
  for (x=-thickX; x<xdim+thickX; x++)
   xmodPtr[x] = (x+thickX*xdim)%xdim;
  for (y=-thickY; y<ydim+thickY; y++)
   ymodPtr[y] = ((y+thickY*ydim)%ydim)*bpl;
  /* if we do not work on the whole image, do not wrap around to the other side */
  if ((xdim!=img.width()) || (!doRepeat))
   for (x=1; x<=thickX; x++)
    {
     xmodPtr[-x] =  xmodPtr[0];
     xmodPtr[xdim+x-1] =  xmodPtr[xdim-1];
    }
  if ((ydim!=img.height()) || (!doRepeat))
   for (y=1; y<=thickY; y++)
    {
     ymodPtr[-y] =  ymodPtr[0];
     ymodPtr[ydim+y-1] =  ymodPtr[ydim-1];
    }

  /* make a list of points around the center averaging point, use circle, not sqaure checking */
  for (x=-thickX; x<=thickX; x++)
   for (y=-thickY; y<=thickY; y++)
    if ((x*x + y*y)<=(thickX*thickY))
     {
      checkX[numCheckPoints]=x;
      checkY[numCheckPoints]=y;
      checkXYbpl[numCheckPoints++]=y*bpl+x;
     }

  buffHalf = xdim * thickY*8;	/* allocate thickY*8 rows of data */
  if (!img.allocDrawBuff(buffHalf))
   return;
  data = img.data->imageData;
  for (y = 0; y < ydim; y++)
   {
    ybpl = y*bpl;
    if (img.data->changecount > (buffHalf/2))
     img.flushChanges(protColors, FALSE);
    for (x = 0; x < xdim; x++)
     {
      dataxy = &data[ybpl + x];
      /* presume the current point is most popular one */
      maxpos = *dataxy;
      memset(numPoints, 0, numColors*sizeof(unsigned short));

      /* summing up all the points without overrun check is faster */
      if ((x>thickX) && (x<(xdim-thickX)) && (y>thickY) && (y<(ydim-thickY)))
       for (i=0; i<numCheckPoints; i++)
	numPoints[dataxy[checkXYbpl[i]]]++;
      else
       for (i=0; i<numCheckPoints; i++)
	numPoints[data[ymodPtr[y+checkY[i]] + xmodPtr[x+checkX[i]] ]]++;

      /* search for most popular color only if current point has less than half neighbors of the same color */
      if (numPoints[maxpos]<=(numCheckPoints>>1))
       {
	for (i=0; i<numColors; i++)
	 if (!protColors[i])
	  if (numPoints[i]>numPoints[maxpos])
	   maxpos = i;
	if (maxpos != *dataxy)
	 img.setChangePosVal(img.data->changecount++, ybpl + x, maxpos);
       }
     }
   }
 }
 img.flushChanges(protColors, TRUE);
 img.freeDrawBuff();
}

void ApaintImageUtil::despecleContour(ApaintImage& img, int xin, int yin, int w, int h, bool *protColors, bool doRepeat, int what, int neighbours)
{
 int i, j, x, y, xdim, ydim;
 int xlimit, ylimit;
 int bpl, pos, bpp;
 uint  max, posval, maxval;
 unsigned char value[10];
 unsigned char count[10];
 int left, right, up, down, midx, midy, buffHalf;
 int *xtab, *ytab;
 int numEqualCenter; /* number of points equal to center */
 unsigned char *data;
 
 if (img.type() !=LOAD_CMAP) return;

 if ( w < 0 ) w = img.width();
 if ( h < 0 ) h = img.height();
 if ( xin < 0 ) { w += xin; xin = 0; }
 if ( yin < 0 ) { h += xin; xin = 0; }
 if ( xin + w > img.width() ) w = img.width() - xin;
 if ( yin + h > img.height() ) h = img.height() - yin;
 if ( w <= 0 || h <= 0 ) return;

 posval = 0;
 bpl = img.data->imageBpl;
 bpp = img.depth()/8;
 xdim = w;
 ydim = h;
 xlimit = xdim;
 ylimit = ydim;
 
 xtab = new int[xdim+2];
 ytab = new int[ydim+2];
 if (!xtab || !ytab)
  {
   qDebug("despecleContour: NULL pointer!");
   return;
  }
 for (i=0; i<xdim+2; i++)
  {
    if (i==0 && doRepeat)
      xtab[i] = xin+xdim-1;
    else if (i == 0)
      xtab[i] = xin;
    else if (i == xdim+1 && doRepeat)
      xtab[i] = xin;
    else if (i == xdim+1)
      xtab[i] = xin+xdim-1;
    else
      xtab[i] = xin+i-1;
  }
 for (i=0; i<ydim+2; i++)
  {
    if (i==0 && doRepeat)
      ytab[i] = yin+ydim-1;
    else if (i==0)
      ytab[i] = yin;
    else if (i == ydim+1 && doRepeat)
      ytab[i] = yin;
    else if (i == ydim+1)
      ytab[i] = yin+ydim-1;
    else
      ytab[i] = yin+i-1;
  }

 buffHalf = xdim * 6;	/* allocate 6 rows of data */
 if (!img.allocDrawBuff(buffHalf))
  {
   delete[] xtab;
   delete[] ytab;
   return;
  }

 data = img.data->imageData;
 for (y = 1 ; y <= ylimit; y++)
  {
   if (img.data->changecount > buffHalf)
    img.flushChanges(protColors, FALSE);
   midy = (ytab[y]) * bpl;
   up = (ytab[y - 1]) * bpl;
   down = (ytab[y + 1]) * bpl;

   for (x = 1; x <= xlimit; x++)
    {
     numEqualCenter = 1;
     midx = xtab[x];
     pos = midx*bpp + midy;
     left = xtab[x - 1];
     right = xtab[x + 1];

     posval = data[pos];
     /* on negative values of what, do it on all colors, otherwise just on the selected color */
     if ((what<0) || ((int) posval==what))
      { 
       if (posval==data[up + left])
	numEqualCenter++;
       if (posval==data[up + midx])
	{ numEqualCenter++; numEqualCenter++; }
       if (posval==data[up + right])
	numEqualCenter++;
       if (posval==data[midy + left])
	{ numEqualCenter++; numEqualCenter++; }
       if (posval==data[midy + right])
	{ numEqualCenter++; numEqualCenter++; }
       if (posval==data[down + left])
	numEqualCenter++;
       if (posval==data[down + midx])
	{ numEqualCenter++; numEqualCenter++; }
       if (posval==data[down + right])
	numEqualCenter++;
       if (numEqualCenter<=neighbours)
	{
	 memset(count, 0, sizeof(unsigned char)*10);
	 memset(value, 0, sizeof(unsigned char)*10);
	 for (j = 0; j < 9; j++)
	  {
	   switch (j)
   	    {
   	     case 0: posval = data[up + left]; break;
   	     case 1: posval = data[up + midx]; break;
   	     case 2: posval = data[up + right]; break;
   	     case 3: posval = data[midy + left]; break;
   	     case 4: posval = data[pos]; break;
   	     case 5: posval = data[midy + right]; break;
   	     case 6: posval = data[down + left]; break;
   	     case 7: posval = data[down + midx]; break;
   	     case 8: posval = data[down + right]; break;
   	    }

	   i = 0;
	   while ((value[i] != posval) && (count[i] != 0))
  	     i++;
	   value[i] = posval;
	   count[i]++;
	  }
	 maxval = value[0];
	 max = count[0];
	 i = 1;
	 while (count[i] > 0)
	  {
	   if (count[i] > max)
  	    { max = count[i]; maxval = value[i]; }
	   i++;
	  }
	 if ((maxval != data[pos]) && (max > 1))
          img.setChangePosVal(img.data->changecount++, pos, maxval);
	}
      }
    }
  }
 img.flushChanges(protColors, TRUE);
 img.freeDrawBuff();
 delete[] xtab;
 delete[] ytab;
}

void ApaintImageUtil::contourThick(ApaintImage& img, uint rmcol, int direction, bool *protColors, int numX, int numY, int angle, bool doRepeat, bool doShade, bool doLines, bool doGradient, bool doBidirectional, bool repeatOperation)
{
  int x, y, xdim, ydim, i;
  int xlimit, ylimit;
  int bpl, pos, bpp;
  int  up, down, midx, midy, buffHalf;
  int doit;
  int *xtab, *ytab, *xmod, *ymod;
  int gradientAngle, gradientLimit, randVal;

int num = (numX+numY)/2;
int num2 = max(numX, numY);

  bpl = img.data->imageBpl;
  bpp = img.depth()/8;
  xdim = img.width();
  ydim = img.height();
  xlimit = xdim;
  ylimit = ydim;

  if (img.type() != LOAD_CMAP)
   return;
  xtab = new int[xdim+2*num2];
  ytab = new int[ydim+2*num2];
  if (!xtab || !ytab)
   {
    qDebug("contourThick: NULL pointer!");
    return;
   }

  xmod = &xtab[num2];
  for (i=0; i<xdim; i++)
   xmod[i] = i;
  if (doRepeat)
   {
    memcpy(&xtab[num2+xdim], xmod, sizeof(int)*num2);
    memcpy(xtab, &xtab[xdim], sizeof(int)*num2);
   }
  else
   for (i=0; i<num2; i++)
    {
     xtab[i]=0;
     xtab[i+xdim+num2]=xdim-1;
    }

  ymod = &ytab[num2];
  for (i=0; i<ydim; i++)
   ymod[i] = i;
  if (doRepeat)
   {
    memcpy(&ytab[num2+ydim], ymod, sizeof(int)*num2);
    memcpy(ytab, &ytab[ydim], sizeof(int)*num2);
   }
  else
   for (i=0; i<num2; i++)
    {
     ytab[i]=0;
     ytab[i+ydim+num2]=ydim-1;
    }

  if (repeatOperation)
   buffHalf = xdim * 8;	/* allocate 8 rows of data; need less memory since operation will be called several times */
  else
   buffHalf = xdim * 20 * num2;	/* allocate 20 rows of data, plus size of repeated operation */
  if (!img.allocDrawBuff(buffHalf))
   return;

  for (y = 0 ; y < ylimit; y++)
   {
    if (img.data->changecount > buffHalf)
     img.flushChanges(protColors, FALSE);
    midy = ymod[y] * bpl;
    up = ymod[y-1] * bpl;
    down = ymod[y+1] * bpl;

    for (x = 0; x < xlimit; x++)
     {
      doit = CDIR_NONE;
      midx = xmod[x];
      pos = midy + midx*bpp;

      if (img.data->imageData[pos] != rmcol)
       {
        if (repeatOperation && doGradient)
         {
	  gradientAngle = (y + (angle*x)/45)%ylimit;
	  randVal = rand(); /* we only evaluate one bit, so one random value has enough bits for uncorrelated testing of all values */

          if (doBidirectional)
           {
            if ((direction & CDIR_LEFT_UP) && (img.data->imageData[down + xmod[x+1]] == (uchar)rmcol))
	     {
	      if (gradientAngle<(ylimit>>1))
	       {
        	if (gradientAngle<(rand()%(ylimit>>1)))
	         if (randVal&(1<<0))
	          doit |= CDIR_LEFT_UP;
               }
	      else
	       {
        	if ((gradientAngle-(ylimit>>1))>(rand()%(ylimit>>1)))
	         if (randVal&(1<<0))
	          doit |= CDIR_LEFT_UP;
               }
	     }

	    if ((direction & CDIR_UP) && (img.data->imageData[down+midx] == (uchar)rmcol))
	     {
	      if (gradientAngle<(ylimit>>1))
	       {
        	if (gradientAngle<(rand()%(ylimit>>1)))
	         if (randVal&(1<<1))
	          doit |= CDIR_UP;
               }
	      else
	       {
        	if ((gradientAngle-(ylimit>>1))>(rand()%(ylimit>>1)))
	         if (randVal&(1<<1))
	          doit |= CDIR_UP;
               }
             }

            if ((direction & CDIR_RIGHT_UP) && (img.data->imageData[down + xmod[x-1]] == (uchar)rmcol))
	     {
 	      if (gradientAngle<(ylimit>>1))
	       {
        	if (gradientAngle<(rand()%(ylimit>>1)))
	         if (randVal&(1<<2))
	          doit |= CDIR_RIGHT_UP;
               }
	      else
	       {
        	if ((gradientAngle-(ylimit>>1))>(rand()%(ylimit>>1)))
	         if (randVal&(1<<2))
	          doit |= CDIR_RIGHT_UP;
               }
	     }

	    if ((direction & CDIR_LEFT) && (img.data->imageData[midy + xmod[x+1]] == (uchar)rmcol))
	     {
 	      if (gradientAngle<(ylimit>>1))
	       {
        	if (gradientAngle<(rand()%(ylimit>>1)))
	         if (randVal&(1<<3))
	          doit |= CDIR_LEFT;
               }
	      else
	       {
        	if ((gradientAngle-(ylimit>>1))>(rand()%(ylimit>>1)))
	         if (randVal&(1<<3))
	          doit |= CDIR_LEFT;
               }
	     }

	    if ((direction & CDIR_RIGHT) && (img.data->imageData[midy + xmod[x-1]] == (uchar)rmcol))
	     {
 	      if (gradientAngle<(ylimit>>1))
	       {
        	if (gradientAngle<(rand()%(ylimit>>1)))
 	         if (randVal&(1<<4))
	          doit |= CDIR_RIGHT;
               }
	      else
	       {
        	if ((gradientAngle-(ylimit>>1))>(rand()%(ylimit>>1)))
	         if (randVal&(1<<4))
	          doit |= CDIR_RIGHT;
               }
	     }

            if ((direction & CDIR_LEFT_DOWN) && (img.data->imageData[up + xmod[x+1]] == (uchar)rmcol))
	     {
 	      if (gradientAngle<(ylimit>>1))
	       {
        	if (gradientAngle<(rand()%(ylimit>>1)))
	         if (randVal&(1<<5))
	         doit |= CDIR_LEFT_DOWN;
               }
	      else
	       {
        	if ((gradientAngle-(ylimit>>1))>(rand()%(ylimit>>1)))
	         if (randVal&(1<<5))
 	          doit |= CDIR_LEFT_DOWN;
               }
	     }

            if ((direction & CDIR_DOWN) && (img.data->imageData[up+midx] == (uchar)rmcol))
	     {
 	      if (gradientAngle<(ylimit>>1))
	       {
        	if (gradientAngle<(rand()%(ylimit>>1)))
	         if (randVal&(1<<6))
	          doit |= CDIR_DOWN;
               }
	      else
	       {
        	if ((gradientAngle-(ylimit>>1))>(rand()%(ylimit>>1)))
	         if (randVal&(1<<6))
	          doit |= CDIR_DOWN;
               }
	     }

            if ((direction & CDIR_RIGHT_DOWN) && (img.data->imageData[up + xmod[x-1]] == (uchar)rmcol))
	     {
 	      if (gradientAngle<(ylimit>>1))
	       {
        	if (gradientAngle<(rand()%(ylimit>>1)))
	         if (randVal&(1<<7))
 	          doit |= CDIR_RIGHT_DOWN;
               }
	      else
	       {
        	if ((gradientAngle-(ylimit>>1))>(rand()%(ylimit>>1)))
	         if (randVal&(1<<7))
	          doit |= CDIR_RIGHT_DOWN;
               }
	     }
	   }
	  else
           {
            if ((direction & CDIR_LEFT_UP) && (img.data->imageData[down + xmod[x+1]] == (uchar)rmcol))
             if (gradientAngle<(rand()%ylimit))
	      if (randVal&(1<<0))
	       doit |= CDIR_LEFT_UP;
	    if ((direction & CDIR_UP) && (img.data->imageData[down+midx] == (uchar)rmcol))
             if (gradientAngle<(rand()%ylimit))
	      if (randVal&(1<<1))
	       doit |= CDIR_UP;
            if ((direction & CDIR_RIGHT_UP) && (img.data->imageData[down + xmod[x-1]] == (uchar)rmcol))
             if (gradientAngle<(rand()%ylimit))
	      if (randVal&(1<<2))
               doit |= CDIR_RIGHT_UP;

	    if ((direction & CDIR_LEFT) && (img.data->imageData[midy + xmod[x+1]] == (uchar)rmcol))
             if (gradientAngle<(rand()%ylimit))
	      if (randVal&(1<<3))
    	       doit |=  CDIR_LEFT;
	    if ((direction & CDIR_RIGHT) && (img.data->imageData[midy + xmod[x-1]] == (uchar)rmcol))
             if (gradientAngle<(rand()%ylimit))
	      if (randVal&(1<<4))
    	       doit |= CDIR_RIGHT;

            if ((direction & CDIR_LEFT_DOWN) && (img.data->imageData[up + xmod[x+1]] == (uchar)rmcol))
             if (gradientAngle<(rand()%ylimit))
	      if (randVal&(1<<5))
	       doit |= CDIR_LEFT_DOWN;
            if ((direction & CDIR_DOWN) && (img.data->imageData[up+midx] == (uchar)rmcol))
             if (gradientAngle<(rand()%ylimit))
	      if (randVal&(1<<6))
               doit |= CDIR_DOWN;
            if ((direction & CDIR_RIGHT_DOWN) && (img.data->imageData[up + xmod[x-1]] == (uchar)rmcol))
             if (gradientAngle<(rand()%ylimit))
	      if (randVal&(1<<7))
	       doit |= CDIR_RIGHT_DOWN;
	   }
         }
        else // normal just one operation 
         {
	  if (repeatOperation && doShade)
	   {
	    randVal = rand(); /* we only evaluate one bit, so one random value has enough bits for uncorrelated testing of all values */
	    if ((direction & CDIR_LEFT_UP) && (img.data->imageData[down + xmod[x+1]] == (uchar)rmcol))
	     if (randVal&(1<<0))
	      doit |= CDIR_LEFT_UP;
	    if ((direction & CDIR_UP) && (img.data->imageData[down+midx] == (uchar)rmcol))
	     if (randVal&(1<<1))
	      doit |= CDIR_UP;
	    if ((direction & CDIR_RIGHT_UP) && (img.data->imageData[down + xmod[x-1]] == (uchar)rmcol))
	     if (randVal&(1<<2))
	      doit |= CDIR_RIGHT_UP;

	    if ((direction & CDIR_LEFT) && (img.data->imageData[midy + xmod[x+1]] == (uchar)rmcol))
	     if (randVal&(1<<3))
	      doit |=  CDIR_LEFT;
	    if ((direction & CDIR_RIGHT) && (img.data->imageData[midy + xmod[x-1]] == (uchar)rmcol))
	     if (randVal&(1<<4))
	      doit |= CDIR_RIGHT;

	    if ((direction & CDIR_LEFT_DOWN) && (img.data->imageData[up + xmod[x+1]] == (uchar)rmcol))
	     if (randVal&(1<<5))
	      doit |= CDIR_LEFT_DOWN;
	    if ((direction & CDIR_DOWN) && (img.data->imageData[up+midx] == (uchar)rmcol))
	     if (randVal&(1<<6))
	      doit |= CDIR_DOWN;
	    if ((direction & CDIR_RIGHT_DOWN) && (img.data->imageData[up + xmod[x-1]] == (uchar)rmcol))
	     if (randVal&(1<<7))
	      doit |= CDIR_RIGHT_DOWN;
	   }
	  else
	   {
            if ((direction & CDIR_LEFT_UP) && (img.data->imageData[down + xmod[x+1]] == (uchar)rmcol))
	     doit |= CDIR_LEFT_UP;
	    if ((direction & CDIR_UP) && (img.data->imageData[down+midx] == (uchar)rmcol))
	     doit |= CDIR_UP;
            if ((direction & CDIR_RIGHT_UP) && (img.data->imageData[down + xmod[x-1]] == (uchar)rmcol))
	     doit |= CDIR_RIGHT_UP;

	    if ((direction & CDIR_LEFT) && (img.data->imageData[midy + xmod[x+1]] == (uchar)rmcol))
    	     doit |=  CDIR_LEFT;
	    if ((direction & CDIR_RIGHT) && (img.data->imageData[midy + xmod[x-1]] == (uchar)rmcol))
    	     doit |= CDIR_RIGHT;

            if ((direction & CDIR_LEFT_DOWN) && (img.data->imageData[up + xmod[x+1]] == (uchar)rmcol))
	     doit |= CDIR_LEFT_DOWN;
            if ((direction & CDIR_DOWN) && (img.data->imageData[up+midx] == (uchar)rmcol))
	     doit |= CDIR_DOWN;
            if ((direction & CDIR_RIGHT_DOWN) && (img.data->imageData[up + xmod[x-1]] == (uchar)rmcol))
	     doit |= CDIR_RIGHT_DOWN;
	   }
         }

	if (doit)
	 {
	  img.setChangePosVal(img.data->changecount++, pos, rmcol);

	  if (((numX>1) || (numY>1)) && (!repeatOperation))
           {
	    if (doShade || doLines)
	     {
              if (doGradient)
	       {
                if (doBidirectional)
		 {
	          gradientAngle = (y + (angle*x)/45)%ylimit;
		  if (gradientAngle<(ylimit>>1))
		   gradientLimit = (gradientAngle*num2)/(ylimit>>1);
		  else
 		   gradientLimit = ((ylimit - gradientAngle)*num2)/(ylimit>>1);
		 }
		else
                 gradientLimit = (((y + (angle*x)/45)%ylimit)*num2)/ylimit;
	       }
	      else
	       gradientLimit = num2;

              if (gradientLimit>0)
	       {
        	if (doLines)
	         {
       	          if (doit & CDIR_UP)
        	   for (i=(rand()%gradientLimit); i; i--)
        	    img.setChangePosVal(img.data->changecount++, ymod[y-i]*bpl + midx, rmcol);
        	  if (doit & CDIR_DOWN)
        	   for (i=(rand()%gradientLimit); i; i--)
        	    img.setChangePosVal(img.data->changecount++, ymod[y+i]*bpl + midx, rmcol);
        	  if (doit & CDIR_LEFT)
        	   for (i=(rand()%gradientLimit); i; i--)
        	    img.setChangePosVal(img.data->changecount++, midy + xmod[x-i], rmcol);
        	  if (doit & CDIR_RIGHT)
        	   for (i=(rand()%gradientLimit); i; i--)
        	    img.setChangePosVal(img.data->changecount++, midy + xmod[x+i], rmcol);

        	  if (doit & CDIR_LEFT_UP)
        	   for (i=(rand()%gradientLimit); i; i--)
        	    img.setChangePosVal(img.data->changecount++, ymod[y-i]*bpl + xmod[x-i], rmcol);
        	  if (doit & CDIR_LEFT_DOWN)
        	   for (i=(rand()%gradientLimit); i; i--)
        	    img.setChangePosVal(img.data->changecount++, ymod[y+i]*bpl + xmod[x-i], rmcol);
        	  if (doit & CDIR_RIGHT_UP)
        	   for (i=(rand()%gradientLimit); i; i--)
        	    img.setChangePosVal(img.data->changecount++, ymod[y-i]*bpl + xmod[x+i], rmcol);
        	  if (doit & CDIR_RIGHT_DOWN)
        	   for (i=(rand()%gradientLimit); i; i--)
        	    img.setChangePosVal(img.data->changecount++, ymod[y+i]*bpl + xmod[x+i], rmcol);
                 }
        	if (doShade)
	         {
        	  if (doit & CDIR_UP)
        	   for (i=1; i<gradientLimit; i++)
		    if ((rand()%(gradientLimit<<8))>(i<<8))
        	     img.setChangePosVal(img.data->changecount++, ymod[y-i]*bpl + midx, rmcol);
        	  if (doit & CDIR_DOWN)
        	   for (i=1; i<gradientLimit; i++)
		    if ((rand()%(gradientLimit<<8))>(i<<8))
        	     img.setChangePosVal(img.data->changecount++, ymod[y+i]*bpl + midx, rmcol);
        	  if (doit & CDIR_LEFT)
        	   for (i=1; i<gradientLimit; i++)
		    if ((rand()%(gradientLimit<<8))>(i<<8))
        	    img.setChangePosVal(img.data->changecount++, midy + xmod[x-i], rmcol);
        	  if (doit & CDIR_RIGHT)
        	   for (i=1; i<gradientLimit; i++)
		    if ((rand()%(gradientLimit<<8))>(i<<8))
        	     img.setChangePosVal(img.data->changecount++, midy + xmod[x+i], rmcol);

        	  if (doit & CDIR_LEFT_UP)
        	   for (i=1; i<gradientLimit; i++)
		    if ((rand()%(gradientLimit<<8))>(i<<8))
        	     img.setChangePosVal(img.data->changecount++, ymod[y-i]*bpl + xmod[x-i], rmcol);
        	  if (doit & CDIR_LEFT_DOWN)
        	   for (i=1; i<gradientLimit; i++)
		    if ((rand()%(gradientLimit<<8))>(i<<8))
        	     img.setChangePosVal(img.data->changecount++, ymod[y+i]*bpl + xmod[x-i], rmcol);
        	  if (doit & CDIR_RIGHT_UP)
        	   for (i=1; i<gradientLimit; i++)
		    if ((rand()%(gradientLimit<<8))>(i<<8))
        	     img.setChangePosVal(img.data->changecount++, ymod[y-i]*bpl + xmod[x+i], rmcol);
        	  if (doit & CDIR_RIGHT_DOWN)
        	   for (i=1; i<gradientLimit; i++)
		    if ((rand()%(gradientLimit<<8))>(i<<8))
        	     img.setChangePosVal(img.data->changecount++, ymod[y+i]*bpl + xmod[x+i], rmcol);
                 }
	       }
	     }
	    else
	     {
              if (doGradient)
	       {
                if (doBidirectional)
		 {
	          gradientAngle = (y + (angle*x)/45)%ylimit;
		  if (gradientAngle<(ylimit>>1))
		   gradientLimit = (gradientAngle*num2)/(ylimit>>1);
		  else
 		   gradientLimit = ((ylimit - gradientAngle)*num2)/(ylimit>>1);
		 }
		else
                 gradientLimit = (((y + (angle*x)/45)%ylimit)*num2)/ylimit;

        	if (gradientLimit>0)
	         {
        	  if (doit & CDIR_UP)
        	   for (i=1; i<gradientLimit; i++)
        	    img.setChangePosVal(img.data->changecount++, ymod[y-i]*bpl + midx, rmcol);
        	  if (doit & CDIR_DOWN)
        	   for (i=1; i<gradientLimit; i++)
        	    img.setChangePosVal(img.data->changecount++, ymod[y+i]*bpl + midx, rmcol);
        	  if (doit & CDIR_LEFT)
        	   for (i=1; i<gradientLimit; i++)
        	    img.setChangePosVal(img.data->changecount++, midy + xmod[x-i], rmcol);
        	  if (doit & CDIR_RIGHT)
        	   for (i=1; i<gradientLimit; i++)
        	    img.setChangePosVal(img.data->changecount++, midy + xmod[x+i], rmcol);

        	  if (doit & CDIR_LEFT_UP)
        	   for (i=1; i<gradientLimit; i++)
        	    img.setChangePosVal(img.data->changecount++, ymod[y-i]*bpl + xmod[x-i], rmcol);
        	  if (doit & CDIR_LEFT_DOWN)
        	   for (i=1; i<gradientLimit; i++)
        	    img.setChangePosVal(img.data->changecount++, ymod[y+i]*bpl + xmod[x-i], rmcol);
        	  if (doit & CDIR_RIGHT_UP)
        	   for (i=1; i<gradientLimit; i++)
        	    img.setChangePosVal(img.data->changecount++, ymod[y-i]*bpl + xmod[x+i], rmcol);
        	  if (doit & CDIR_RIGHT_DOWN)
        	   for (i=1; i<gradientLimit; i++)
        	    img.setChangePosVal(img.data->changecount++, ymod[y+i]*bpl + xmod[x+i], rmcol);
		 }
               }
	      else
	       {
        	if (doit & CDIR_UP)
        	 for (i=1; i<numY; i++)
        	  img.setChangePosVal(img.data->changecount++, ymod[y-i]*bpl + midx, rmcol);
        	if (doit & CDIR_DOWN)
        	 for (i=1; i<numY; i++)
        	  img.setChangePosVal(img.data->changecount++, ymod[y+i]*bpl + midx, rmcol);
        	if (doit & CDIR_LEFT)
        	 for (i=1; i<numX; i++)
        	  img.setChangePosVal(img.data->changecount++, midy + xmod[x-i], rmcol);
        	if (doit & CDIR_RIGHT)
        	 for (i=1; i<numX; i++)
        	  img.setChangePosVal(img.data->changecount++, midy + xmod[x+i], rmcol);

        	if (doit & CDIR_LEFT_UP)
        	 for (i=1; i<num; i++)
        	  img.setChangePosVal(img.data->changecount++, ymod[y-i]*bpl + xmod[x-i], rmcol);
        	if (doit & CDIR_LEFT_DOWN)
        	 for (i=1; i<num; i++)
        	  img.setChangePosVal(img.data->changecount++, ymod[y+i]*bpl + xmod[x-i], rmcol);
        	if (doit & CDIR_RIGHT_UP)
        	 for (i=1; i<num; i++)
        	  img.setChangePosVal(img.data->changecount++, ymod[y-i]*bpl + xmod[x+i], rmcol);
        	if (doit & CDIR_RIGHT_DOWN)
        	 for (i=1; i<num; i++)
        	  img.setChangePosVal(img.data->changecount++, ymod[y+i]*bpl + xmod[x+i], rmcol);
	       }
	     }
           }
	 }
       }

    }
  }
 img.flushChanges(protColors, TRUE);
 img.freeDrawBuff();
 delete[] xtab;
 delete[] ytab;
}

void ApaintImageUtil::contourRemove(ApaintImage& img, uint rmcol, bool *protColors, bool doRepeat)
{
 int i, j, x, y, xdim, ydim;
 int xlimit, ylimit;
 int bpl, pos, bpp;
 uint  max, posval, maxval;
 uint value[10];
 uint count[10];
 int left, right, up, down, midx, midy, buffHalf;
 uchar *up32, *down32, *mid32;
 uint col=0;
 bool allDone = false;
 int *xtab, *ytab;

 if (protColors[rmcol]) return;


  posval = 0;
  bpl = img.data->imageBpl;
  bpp = img.depth()/8;
  xdim = img.width();
  ydim = img.height();
  xlimit = xdim;
  ylimit = ydim;

  xtab = new int[xdim+2];
  ytab = new int[ydim+2];
  if (!xtab || !ytab)
   {
    qDebug("contourRemove: NULL pointer!");
    return;
   }

  for (i=0; i<xdim+2; i++)
   {
    if (i==0 && doRepeat)
      xtab[i] = xdim-1;
    else if (i == 0)
      xtab[i] = 0;
    else if (i == xdim+1 && doRepeat)
      xtab[i] = 0;
    else if (i == xdim+1)
      xtab[i] = xdim-1;
    else
      xtab[i] = i-1;
   }
  for (i=0; i<ydim+2; i++)
   {
    if (i==0 && doRepeat)
      ytab[i] = ydim-1;
    else if (i==0)
      ytab[i] = 0;
    else if (i == ydim+1 && doRepeat)
      ytab[i] = 0;
    else if (i == ydim+1)
      ytab[i] = ydim-1;
    else
      ytab[i] = i-1;
   }

  buffHalf = xdim * 6;	/* allocate 6 rows of data */
  if (!img.allocDrawBuff(buffHalf))
   {
    delete[] xtab;
    delete[] ytab;
    return;
   }


  while (!allDone)
   {
     for (y = 1 ; y <= ylimit; y++)
      {
       if ((img.data->changecount) > buffHalf)
         img.flushChanges(protColors, FALSE);
       midy = ytab[y] * bpl;
       up = ytab[y - 1] * bpl;
       down = ytab[y + 1] * bpl;

       up32 = &img.data->imageData[up];
       down32 = &img.data->imageData[down];
       mid32 = &img.data->imageData[midy];

       for (x = 1; x <= xlimit; x++)
    	 {
         midx = xtab[x];
     	   pos = midx*bpp + midy;

         if (img.type() == LOAD_TRUE_COL)
           col = (mid32[midx*bpp] << 16) | (mid32[midx*bpp+1] << 8) | (mid32[midx*bpp+2]);
         else if (img.type() == LOAD_CMAP)
           col = mid32[midx];

     	  if (col == rmcol)
     	   {
       	    left = xtab[x - 1];
       	    right = xtab[x + 1];
       	    for (i = 0; i < 9; i++)
       	     {
               value[i] = rmcol;
               count[i] = 0;
             }
       	    for (j = 0; j < 8; j++)
       	     {
               if (img.type() == LOAD_TRUE_COL)
           	switch (j)
           	 {
           	  case 0: posval = (up32[left*bpp] << 16) | (up32[left*bpp+1] << 8) | (up32[left*bpp+2]); break;
           	  case 1: posval = (up32[midx*bpp] << 16) | (up32[midx*bpp+1] << 8) | (up32[midx*bpp+2]); break;
           	  case 2: posval = (up32[right*bpp] << 16) | (up32[right*bpp+1] << 8) | (up32[right*bpp+2]); break;
           	  case 3: posval = (mid32[left*bpp] << 16) | (mid32[left*bpp+1] << 8) | (mid32[left*bpp+2]); break;
           	  case 4: posval = (mid32[midx*bpp] << 16) | (mid32[midx*bpp+1] << 8) | (mid32[midx*bpp+2]); break;
           	  case 5: posval = (mid32[right*bpp] << 16) | (mid32[right*bpp+1] << 8) | (mid32[right*bpp+2]); break;
           	  case 6: posval = (down32[left*bpp] << 16) | (down32[left*bpp+1] << 8) | (down32[left*bpp+2]);break;
           	  case 7: posval = (down32[midx*bpp] << 16) | (down32[midx*bpp+1] << 8) | (down32[midx*bpp+2]); break;
           	  case 8: posval = (down32[right*bpp] << 16) | (down32[right*bpp+1] << 8) | (down32[right*bpp+2]);break;
           	 }
               else if (img.type() == LOAD_CMAP)
                switch (j)
           	 {
             	  case 0: posval = img.data->imageData[up + left]; break;
             	  case 1: posval = img.data->imageData[up + midx]; break;
             	  case 2: posval = img.data->imageData[up + right]; break;
             	  case 3: posval = img.data->imageData[midy + left]; break;
             	  case 4: posval = img.data->imageData[pos]; break;
             	  case 5: posval = img.data->imageData[midy + right]; break;
             	  case 6: posval = img.data->imageData[down + left]; break;
             	  case 7: posval = img.data->imageData[down + midx]; break;
             	  case 8: posval = img.data->imageData[down + right]; break;
           	 }
       	       if (posval != rmcol)
        	{
                 i = 0;
                 while ((value[i] != posval) && (value[i] != rmcol))
                  i++;
                 value[i] = posval;
                 count[i]++;
        	}
      	     }
       	     maxval = value[0];
       	     max = count[0];
       	     i = 1;
       	     while (value[i] != rmcol)
       	      {
       	       if (count[i] > max)
                { max = count[i]; maxval = value[i]; }
       	       i++;
       	      }
       	     if (maxval != rmcol)
              img.setChangePosVal(img.data->changecount++, pos, maxval);
          }
       }
     }
   if (!img.data->changecount)
    allDone = true;

   img.flushChanges(protColors, TRUE);
  }
  img.freeDrawBuff();
  delete[] xtab;
  delete[] ytab;
}

void ApaintImageUtil::contourPrun(ApaintImage& img, uint rmcol, bool *protColors, bool doRepeat)
{
  int  x, y, xdim, ydim, i, numIter, left, right;
  int xlimit, ylimit;
  int bpl, pos, bpp;
  int  up, down, midx, midy, buffHalf;
  uchar *data;
  bool doit;
  int *xtab, *ytab;
  uint chgCol;

  if (img.type() !=LOAD_CMAP)
   return;
   
  bpl = img.data->imageBpl;
  bpp = img.depth()/8;
  xdim = img.width();
  ydim = img.height();
  xlimit = xdim;
  ylimit = ydim;

  xtab = new int[xdim+2];
  ytab = new int[ydim+2];
  if (!xtab || !ytab)
  {
    qDebug("contourPrun: NULL pointer!");
    return;
  }

  for (i=0; i<xdim+2; i++)
  {
    if (i==0 && doRepeat)
      xtab[i] = xdim-1;
    else if (i == 0)
      xtab[i] = 0;
    else if (i == xdim+1 && doRepeat)
      xtab[i] = 0;
    else if (i == xdim+1)
      xtab[i] = xdim-1;
    else
      xtab[i] = i-1;
  }
  for (i=0; i<ydim+2; i++)
  {
    if (i==0 && doRepeat)
      ytab[i] = ydim-1;
    else if (i==0)
      ytab[i] = 0;
    else if (i == ydim+1 && doRepeat)
      ytab[i] = 0;
    else if (i == ydim+1)
      ytab[i] = ydim-1;
    else
      ytab[i] = i-1;
  }

  buffHalf = xdim * 6;	/* allocate 6 rows of data */
  if (!img.allocDrawBuff(buffHalf))
   {
    delete[] xtab;
    delete[] ytab;
    return;
   }

  data = img.data->imageData;
  for (numIter = 0; numIter<40; numIter++)
  {
    for (y = 1 ; y <= ylimit; y++)
    {
      if (img.data->changecount > buffHalf)
       img.flushChanges(protColors, FALSE);
      midy = ytab[y] * bpl;
      up = ytab[y-1] * bpl;
      down = ytab[y+1] * bpl;

      for (x = 1; x <= xlimit; x++)
      {
       doit = false;
       midx = xtab[x];
       pos = midy + midx*bpp;
       left = xtab[x - 1];
       right = xtab[x + 1];
       
       if (data[pos] == rmcol)
       {
         switch (numIter % 8)
         {
          case 0:
            if ((doit = (data[midy + left] == rmcol) && (data[up + midx] != rmcol) &&
               (data[up + right] != rmcol) && (data[midy + right] != rmcol) &&
               (data[down + right] != rmcol) && (data[down + midx] != rmcol)))
                 chgCol = data[midy + right];
            break;
          case 1:
            if ((doit = (data[up + midx] == rmcol) && (data[midx + right] != rmcol) &&
               (data[down + right] != rmcol) && (data[down + midx] != rmcol) &&
               (data[down + left] != rmcol) && (data[midy + left] != rmcol)))
                 chgCol= data[down + midx];
            break;
          case 2:
            if ((doit = (data[midy + right] == rmcol) && (data[down + midx] != rmcol) &&
                (data[down + left] != rmcol) && (data[midy + left] != rmcol) &&
                (data[up + left] != rmcol) && (data[up + midx] != rmcol)))
                   chgCol = data[midy + left];
            break;
          case 3:
            if ((doit = (data[down + midx] == rmcol) && (data[midy + left] != rmcol) &&
               (data[up + left] != rmcol) && (data[up + midx] != rmcol) &&
               (data[up + right] != rmcol) && (data[midy + right] != rmcol)))
                  chgCol = data[up + midx];
            break;
          case 4:
            if ((doit = (data[up + left] == rmcol) && (data[up + midx] != rmcol) &&
               (data[up + right] != rmcol) && (data[midy + left] != rmcol) &&
               (data[midy + right] != rmcol) && (data[down + left] != rmcol) &&
               (data[down + midx] != rmcol) && (data[down + right] != rmcol)))
                  chgCol = data[down + right];
            break;
          case 5:
            if ((doit = (data[up + right] == rmcol) && (data[up + left] != rmcol) &&
               (data[up + midx] != rmcol) && (data[midy + left] != rmcol) &&
               (data[midy + right] != rmcol) && (data[down + left] != rmcol) &&
               (data[down + midx] != rmcol) && (data[down + right] != rmcol)))
                  chgCol = data[down + left];
            break;
          case 6:
            if ((doit = (data[down + right] == rmcol) && (data[up + left] != rmcol) &&
               (data[up + midx] != rmcol) && (data[up + right] != rmcol) &&
               (data[midy + left] != rmcol) && (data[midy + right] != rmcol) &&
               (data[down + left] != rmcol) && (data[down + midx] != rmcol)))
                  chgCol = data[up + left];
            break;
          case 7:
            if ((doit = (data[down + left] == rmcol) && (data[up + left] != rmcol) &&
               (data[up + midx] != rmcol) && (data[up + right] != rmcol) &&
               (data[midy + left] != rmcol) && (data[midy + right] != rmcol) &&
               (data[down + midx] != rmcol) && (data[down + right] != rmcol)))
                  chgCol = data[up + right];
            break;
         }
         if (doit)
           img.setChangePosVal(img.data->changecount++, pos, chgCol);
       }
     }
   } 
  }
 img.flushChanges(protColors, TRUE);
 img.freeDrawBuff();
 delete[] xtab;
 delete[] ytab; 
}

void ApaintImageUtil::contourAdd(ApaintImage& img, uint rmcol, uint cmpcol, uint withCol, bool useWithCol, int direction, bool into, bool *protColors, bool doRepeat, bool doShade, int numX, int numY)
{
 int  x, y, xdim, ydim, left, right, i;
 int xlimit, ylimit;
 int bpl, pos, bpp;
 int  up, down, midx, midy, buffHalf;
 int doit = CDIR_NONE;
 int *ytab, *xtab, *xmod, *ymod;

int num = (numX+numY)/2;
int num2 = max(numX, numY);

 bpl = img.data->imageBpl;
 bpp = img.depth()/8;
 xdim = img.width();
 ydim = img.height();
 xlimit = xdim;
 ylimit = ydim;

 if (img.type() != LOAD_CMAP)
  return;

 xtab = new int[xdim+2*num2];
 ytab = new int[ydim+2*num2];

 if (!xtab || !ytab)
  {
   qDebug("contourAdd: NULL pointer!");
   return;
  }

 xmod = &xtab[num2];
 for (i=0; i<xdim; i++)
  xmod[i] = i;
 if (doRepeat)
  {
   memcpy(&xtab[num2+xdim], xmod, sizeof(int)*num2);
   memcpy(xtab, &xtab[xdim], sizeof(int)*num2);
  }
 else
  for (i=0; i<num2; i++)
   {
    xtab[i]=0;
    xtab[i+xdim+num2]=xdim-1;
   }

 ymod = &ytab[num2];
 for (i=0; i<ydim; i++)
  ymod[i] = i;
 if (doRepeat)
  {
   memcpy(&ytab[num2+ydim], ymod, sizeof(int)*num2);
   memcpy(ytab, &ytab[ydim], sizeof(int)*num2);
  }
 else
  for (i=0; i<num2; i++)
   {
    ytab[i]=0;
    ytab[i+ydim+num2]=ydim-1;
   }

 buffHalf = xdim * 6 * num2;	/* allocate 6 rows of data */
 if (!img.allocDrawBuff(buffHalf))
  return;

 for (y = 0 ; y < ylimit; y++)
  {
   if ((img.data->changecount) > buffHalf)
    img.flushChanges(protColors, FALSE);

   midy = ymod[y] * bpl;
   up = ymod[y - 1] * bpl;
   down = ymod[y + 1]* bpl;

   for (x = 0; x < xlimit; x++)
    {
     doit = false;
     midx = xmod[x];
     pos = midy + midx*bpp;
     left = xmod[x - 1];
     right = xmod[x + 1];

     if ((img.data->imageData[pos] != (uchar)rmcol) || (numX>1) || (numY>1) )
      {
       doit = CDIR_NONE;

       if ((direction & CDIR_LEFT) && ( (rmcol==cmpcol) || (img.data->imageData[right + midy]==cmpcol) ) &&
        (img.data->imageData[right + midy] != (uchar)rmcol) &&
                 (img.data->imageData[pos] != img.data->imageData[right + midy] &&
                 (img.data->imageData[pos] == withCol || !useWithCol)))
         doit |= CDIR_LEFT;

       if ((direction & CDIR_RIGHT) && ( (rmcol==cmpcol) || (img.data->imageData[left + midy]==cmpcol) ) &&
        (img.data->imageData[left + midy] != rmcol) &&
                 (img.data->imageData[pos] != img.data->imageData[left + midy] &&
                 (img.data->imageData[pos] == withCol || !useWithCol)))           
         doit |= CDIR_RIGHT;

       if ((direction & CDIR_UP) && ( (rmcol==cmpcol) || (img.data->imageData[down + midx]==cmpcol) ) &&
        (img.data->imageData[down + midx] != rmcol) &&
                 (img.data->imageData[pos] != img.data->imageData[down + midx] &&
                 (img.data->imageData[pos] == withCol || !useWithCol)))
         doit |= CDIR_UP;

       if ((direction & CDIR_DOWN) && ( (rmcol==cmpcol) || (img.data->imageData[up + midx]==cmpcol) ) &&
        (img.data->imageData[up + midx] != rmcol) &&
                 (img.data->imageData[pos] != img.data->imageData[up + midx] &&
                 (img.data->imageData[pos] == withCol || !useWithCol)))
         doit |= CDIR_DOWN;

       if ((direction & CDIR_LEFT_UP) && ( (rmcol==cmpcol) || (img.data->imageData[right + down]==cmpcol) ) &&
        (img.data->imageData[right + down] != (uchar)rmcol) &&
                 (img.data->imageData[pos] != img.data->imageData[right + down] &&
                 (img.data->imageData[pos] == withCol || !useWithCol)))
         doit |= CDIR_LEFT_UP;

       if ((direction & CDIR_RIGHT_UP) && ( (rmcol==cmpcol) || (img.data->imageData[left + down]==cmpcol) ) &&
        (img.data->imageData[left + down] != rmcol) &&
                 (img.data->imageData[pos] != img.data->imageData[left + down] &&
                 (img.data->imageData[pos] == withCol || !useWithCol)))
         doit |= CDIR_RIGHT_UP;

       if ((direction & CDIR_LEFT_DOWN) && ( (rmcol==cmpcol) || (img.data->imageData[right + up]==cmpcol) ) &&
        (img.data->imageData[right + up] != (uchar)rmcol) &&
                 (img.data->imageData[pos] != img.data->imageData[right + up] &&
                 (img.data->imageData[pos] == withCol || !useWithCol)))
         doit |= CDIR_LEFT_DOWN;

       if ((direction & CDIR_RIGHT_DOWN) && ( (rmcol==cmpcol) || (img.data->imageData[left + up]==cmpcol) ) &&
        (img.data->imageData[left + up] != rmcol) &&
                 (img.data->imageData[pos] != img.data->imageData[left + up] &&
                 (img.data->imageData[pos] == withCol || !useWithCol)))
         doit |= CDIR_RIGHT_DOWN;

      if (doit)
       {
	if (into)
	 {
          if (doShade)
	   {
            if (doit & CDIR_LEFT)
	     {
              img.setChangePosVal(img.data->changecount++, midy+right, rmcol);
	      if (numX>1)
               for (i=1; i<numX; i++)
		if (img.data->imageData[midy + (x+i+1)%xlimit]==cmpcol)
	         if ((rand()%(numX<<8))>(i<<8))
        	  img.setChangePosVal(img.data->changecount++, midy + (x+i+1)%xlimit, rmcol);
	     }
            if (doit & CDIR_RIGHT)
	     {
              img.setChangePosVal(img.data->changecount++, midy+left, rmcol);
	      if (numX>1)
               for (i=1; i<numX; i++)
		if (img.data->imageData[midy + (x-i-1 + xlimit*numX)%xlimit]==cmpcol)
	         if ((rand()%(numX<<8))>(i<<8))
         	  img.setChangePosVal(img.data->changecount++, midy + (x-i-1 + xlimit*numX)%xlimit, rmcol);
	     }
            if (doit & CDIR_UP)
	     {
              img.setChangePosVal(img.data->changecount++, midx+down, rmcol);
	      if (numY>1)
               for (i=1; i<numY; i++)
		if (img.data->imageData[((y+i+1)%ylimit)*bpl + midx]==cmpcol)
	         if ((rand()%(numY<<8))>(i<<8))
        	  img.setChangePosVal(img.data->changecount++, ((y+i+1)%ylimit)*bpl + midx, rmcol);
	     }
            if (doit & CDIR_DOWN)
	     {
              img.setChangePosVal(img.data->changecount++, midx+up, rmcol);
	      if (numY>1)
               for (i=1; i<numY; i++)
		if (img.data->imageData[((y-i-1+ylimit*numY)%ylimit)*bpl + midx]==cmpcol)
	         if ((rand()%(numY<<8))>(i<<8))
        	  img.setChangePosVal(img.data->changecount++, ((y-i-1+ylimit*numY)%ylimit)*bpl + midx, rmcol);
	     }
            if (doit & CDIR_LEFT_UP)
	     {
              img.setChangePosVal(img.data->changecount++, down+right, rmcol);
	      if (num>1)
               for (i=1; i<num; i++)
		if (img.data->imageData[((y+i+1)%ylimit)*bpl + (x+i+1)%xlimit]==cmpcol)
	         if ((rand()%(num<<8))>(i<<8))
        	  img.setChangePosVal(img.data->changecount++, ((y+i+1)%ylimit)*bpl + (x+i+1)%xlimit, rmcol);
	     }
            if (doit & CDIR_RIGHT_UP)
	     {
              img.setChangePosVal(img.data->changecount++, down+left, rmcol);
	      if (num>1)
               for (i=1; i<num; i++)
		if (img.data->imageData[((y+i+1)%ylimit)*bpl + (x-i-1 + xlimit*num)%xlimit]==cmpcol)
	         if ((rand()%(num<<8))>(i<<8))
        	  img.setChangePosVal(img.data->changecount++, ((y+i+1)%ylimit)*bpl + (x-i-1 + xlimit*num)%xlimit, rmcol);
	     }
            if (doit & CDIR_LEFT_DOWN)
	     {
              img.setChangePosVal(img.data->changecount++, up+right, rmcol);
	      if (num>1)
               for (i=1; i<num; i++)
		if (img.data->imageData[((y-i-1+ylimit*num)%ylimit)*bpl + (x+i+1)%xlimit]==cmpcol)
	         if ((rand()%(num<<8))>(i<<8))
        	  img.setChangePosVal(img.data->changecount++, ((y-i-1+ylimit*num)%ylimit)*bpl + (x+i+1)%xlimit, rmcol);
	     }
            if (doit & CDIR_RIGHT_DOWN)
	     {
              img.setChangePosVal(img.data->changecount++, up+left, rmcol);
	      if (num>1)
               for (i=1; i<num; i++)
		if (img.data->imageData[((y-i-1+ylimit*num)%ylimit)*bpl + (x-i-1 + xlimit*num)%xlimit]==cmpcol)
	         if ((rand()%(num<<8))>(i<<8))
        	  img.setChangePosVal(img.data->changecount++, ((y-i-1+ylimit*num)%ylimit)*bpl + (x-i-1 + xlimit*num)%xlimit, rmcol);
	     }
           }
	  else
	   {
            if (doit & CDIR_LEFT)
	     {
              img.setChangePosVal(img.data->changecount++, midy+right, rmcol);
	      if (numX>1)
               for (i=1; i<numX; i++)
		if (img.data->imageData[midy + (x+i+1)%xlimit]==cmpcol)
        	 img.setChangePosVal(img.data->changecount++, midy + (x+i+1)%xlimit, rmcol);
	     }
            if (doit & CDIR_RIGHT)
	     {
              img.setChangePosVal(img.data->changecount++, midy+left, rmcol);
	      if (numX>1)
               for (i=1; i<numX; i++)
		if (img.data->imageData[midy + (x-i-1 + xlimit*numX)%xlimit]==cmpcol)
        	 img.setChangePosVal(img.data->changecount++, midy + (x-i-1 + xlimit*numX)%xlimit, rmcol);
	     }
            if (doit & CDIR_UP)
	     {
              img.setChangePosVal(img.data->changecount++, midx+down, rmcol);
	      if (numY>1)
               for (i=1; i<numY; i++)
		if (img.data->imageData[((y+i+1)%ylimit)*bpl + midx]==cmpcol)
        	 img.setChangePosVal(img.data->changecount++, ((y+i+1)%ylimit)*bpl + midx, rmcol);
	     }
            if (doit & CDIR_DOWN)
	     {
              img.setChangePosVal(img.data->changecount++, midx+up, rmcol);
	      if (numY>1)
               for (i=1; i<numY; i++)
		if (img.data->imageData[((y-i-1+ylimit*numY)%ylimit)*bpl + midx]==cmpcol)
        	 img.setChangePosVal(img.data->changecount++, ((y-i-1+ylimit*numY)%ylimit)*bpl + midx, rmcol);
	     }
            if (doit & CDIR_LEFT_UP)
	     {
              img.setChangePosVal(img.data->changecount++, down+right, rmcol);
	      if (num>1)
               for (i=1; i<num; i++)
		if (img.data->imageData[((y+i+1)%ylimit)*bpl + (x+i+1)%xlimit]==cmpcol)
        	 img.setChangePosVal(img.data->changecount++, ((y+i+1)%ylimit)*bpl + (x+i+1)%xlimit, rmcol);
	     }
            if (doit & CDIR_RIGHT_UP)
	     {
              img.setChangePosVal(img.data->changecount++, down+left, rmcol);
	      if (num>1)
               for (i=1; i<num; i++)
		if (img.data->imageData[((y+i+1)%ylimit)*bpl + (x-i-1 + xlimit*num)%xlimit]==cmpcol)
        	 img.setChangePosVal(img.data->changecount++, ((y+i+1)%ylimit)*bpl + (x-i-1 + xlimit*num)%xlimit, rmcol);
	     }
            if (doit & CDIR_LEFT_DOWN)
	     {
              img.setChangePosVal(img.data->changecount++, up+right, rmcol);
	      if (num>1)
               for (i=1; i<num; i++)
		if (img.data->imageData[((y-i-1+ylimit*num)%ylimit)*bpl + (x+i+1)%xlimit]==cmpcol)
        	 img.setChangePosVal(img.data->changecount++, ((y-i-1+ylimit*num)%ylimit)*bpl + (x+i+1)%xlimit, rmcol);
	     }
            if (doit & CDIR_RIGHT_DOWN)
	     {
              img.setChangePosVal(img.data->changecount++, up+left, rmcol);
	      if (num>1)
               for (i=1; i<num; i++)
		if (img.data->imageData[((y-i-1+ylimit*num)%ylimit)*bpl + (x-i-1 + xlimit*num)%xlimit]==cmpcol)
        	 img.setChangePosVal(img.data->changecount++, ((y-i-1+ylimit*num)%ylimit)*bpl + (x-i-1 + xlimit*num)%xlimit, rmcol);
	     }
           }
	 }
	else
         {
          img.setChangePosVal(img.data->changecount++, pos, rmcol);

	  if ((numX>1) || (numY>1))
           {
            if (doShade)
	     {
              if (doit & CDIR_UP)
               for (i=1; i<numY; i++)
        	if (img.data->imageData[ymod[y-i]*bpl + midx]!=cmpcol)
	         if ((rand()%(numY<<8))>(i<<8))
        	  img.setChangePosVal(img.data->changecount++, ymod[y-i]*bpl + midx, rmcol);
              if (doit & CDIR_DOWN)
               for (i=1; i<numY; i++)
		if (img.data->imageData[ymod[y+i]*bpl + midx]!=cmpcol)
	         if ((rand()%(numY<<8))>(i<<8))
        	  img.setChangePosVal(img.data->changecount++, ymod[y+i]*bpl + midx, rmcol);
              if (doit & CDIR_LEFT)
               for (i=1; i<numX; i++)
		if (img.data->imageData[midy + xmod[x-i]]!=cmpcol)
	         if ((rand()%(numX<<8))>(i<<8))
        	  img.setChangePosVal(img.data->changecount++, midy + xmod[x-i], rmcol);
              if (doit & CDIR_RIGHT)
               for (i=1; i<numX; i++)
		if (img.data->imageData[midy + xmod[x+i]]!=cmpcol)
	         if ((rand()%(numX<<8))>(i<<8))
        	  img.setChangePosVal(img.data->changecount++, midy + xmod[x+i], rmcol);
              if (doit & CDIR_LEFT_UP)
               for (i=1; i<num; i++)
		if (img.data->imageData[ymod[y-i]*bpl + xmod[x-i]]!=cmpcol)
	         if ((rand()%(num<<8))>(i<<8))
        	  img.setChangePosVal(img.data->changecount++, ymod[y-i]*bpl + xmod[x-i], rmcol);
              if (doit & CDIR_LEFT_DOWN)
               for (i=1; i<num; i++)
		if (img.data->imageData[ymod[y+i]*bpl + xmod[x-i]]!=cmpcol)
	         if ((rand()%(num<<8))>(i<<8))
        	  img.setChangePosVal(img.data->changecount++, ymod[y+i]*bpl + xmod[x-i], rmcol);
              if (doit & CDIR_RIGHT_UP)
               for (i=1; i<num; i++)
		if (img.data->imageData[ymod[y-i]*bpl + xmod[x+i]]!=cmpcol)
	         if ((rand()%(num<<8))>(i<<8))
        	  img.setChangePosVal(img.data->changecount++, ymod[y-i]*bpl + xmod[x+i], rmcol);
              if (doit & CDIR_RIGHT_DOWN)
               for (i=1; i<num; i++)
		if (img.data->imageData[ymod[y+i]*bpl + xmod[x+i]]!=cmpcol)
	         if ((rand()%(num<<8))>(i<<8))
        	  img.setChangePosVal(img.data->changecount++, ymod[y+i]*bpl + xmod[x+i], rmcol);
	     }
            else
	     {
              if (doit & CDIR_UP)
               for (i=1; i<numY; i++)
		if (img.data->imageData[ymod[y-i]*bpl + midx]!=cmpcol)
        	 img.setChangePosVal(img.data->changecount++, ymod[y-i]*bpl + midx, rmcol);
              if (doit & CDIR_DOWN)
               for (i=1; i<numY; i++)
		if (img.data->imageData[ymod[y+i]*bpl + midx]!=cmpcol)
        	 img.setChangePosVal(img.data->changecount++, ymod[y+i]*bpl + midx, rmcol);
              if (doit & CDIR_LEFT)
               for (i=1; i<numX; i++)
		if (img.data->imageData[midy + xmod[x-i]]!=cmpcol)
        	 img.setChangePosVal(img.data->changecount++, midy + xmod[x-i], rmcol);
              if (doit & CDIR_RIGHT)
               for (i=1; i<numX; i++)
		if (img.data->imageData[midy + xmod[x+i]]!=cmpcol)
        	 img.setChangePosVal(img.data->changecount++, midy + xmod[x+i], rmcol);
              if (doit & CDIR_LEFT_UP)
               for (i=1; i<num; i++)
		if (img.data->imageData[ymod[y-i]*bpl + xmod[x-i]]!=cmpcol)
        	 img.setChangePosVal(img.data->changecount++, ymod[y-i]*bpl + xmod[x-i], rmcol);
              if (doit & CDIR_LEFT_DOWN)
               for (i=1; i<num; i++)
		if (img.data->imageData[ymod[y+i]*bpl + xmod[x-i]]!=cmpcol)
        	 img.setChangePosVal(img.data->changecount++, ymod[y+i]*bpl + xmod[x-i], rmcol);
              if (doit & CDIR_RIGHT_UP)
               for (i=1; i<num; i++)
		if (img.data->imageData[ymod[y-i]*bpl + xmod[x+i]]!=cmpcol)
        	 img.setChangePosVal(img.data->changecount++, ymod[y-i]*bpl + xmod[x+i], rmcol);
              if (doit & CDIR_RIGHT_DOWN)
               for (i=1; i<num; i++)
		if (img.data->imageData[ymod[y+i]*bpl + xmod[x+i]]!=cmpcol)
        	 img.setChangePosVal(img.data->changecount++, ymod[y+i]*bpl + xmod[x+i], rmcol);
	     }
           }
	 }
       }
     }
   }
  }
 img.flushChanges(protColors, TRUE);
 img.freeDrawBuff();
 delete[] xtab;
 delete[] ytab;
}


/* "Efficient Binary Image Thinning using Neighborhood Maps by Joseph M. Cychosz,
   in "Graphics Gems IV", Academic Press, 1994
   Direction masks:     N    S     W     E  */
static int masks[] = {0200, 0002, 0040, 0010};

/* True if pixel neighbor map indicates the pixel is 8-simple and
   not an end point and thus can be deleted.  The neighborhood
   map is defined as an integer of bits abcdefghi with a non-zero
   bit representing a non-zero pixel.  The bit assignment for the
   neighborhood is:  a b c
                     d e f
                     g h i */

static unsigned char deletePix[512] =
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 0,0,0,1,0,0,1,1,0,1,1,1,0,0,1,1,
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 0,0,1,1,1,0,1,1,0,0,1,1,0,0,1,1,
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 0,0,0,0,0,0,0,0,1,1,1,1,0,0,1,1,
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 0,0,0,0,0,0,0,0,0,0,1,1,0,0,1,1,
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 0,0,0,0,0,0,0,0,1,1,1,1,0,0,1,1,
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 1,0,1,1,1,0,1,1,1,1,1,1,1,1,1,1,
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 1,0,0,0,0,0,0,0,1,1,1,1,0,0,1,1,
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 1,0,1,1,1,0,1,1,1,1,1,1,1,1,1,1,
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 1,0,1,1,1,0,1,1,0,0,1,1,0,0,1,1,
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 0,0,0,0,0,0,0,0,0,0,1,1,0,0,1,1,
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 1,0,0,0,0,0,0,0,1,1,1,1,0,0,1,1,
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 1,0,1,1,1,0,1,1,1,1,1,1,1,1,1,1,
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 1,0,0,0,0,0,0,0,1,1,1,1,0,0,1,1,
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 1,0,1,1,1,0,1,1,1,1,1,1,1,1,1,1};

void ApaintImageUtil::contourThin(ApaintImage& img, int colInd, bool doRepeat)
{
#define PIX(y,x) img.data->imageData[bpl*(y)+(x)]
#define DEL_PIX(yy,xx,ii) \
 switch (ii) \
 { \
  case 0: \
	if ((yy) == ystart) \
	 PIX((yy),(xx)) = PIX(ypred,(xx)); \
	else \
	 PIX((yy),(xx)) = PIX((yy)-1,(xx)); \
	break; \
  case 1: \
	if ((yy) == yend) \
	 PIX((yy),(xx)) = PIX(ysucc,(xx)); \
	else \
	 PIX((yy),(xx)) = PIX((yy)+1,(xx)); \
	break; \
  case 2: \
	if ((xx) == xstart) \
	 PIX((yy),(xx)) = PIX((yy),xpred); \
	else \
	 PIX((yy),(xx)) = PIX((yy),(xx)-1); \
	break; \
  case 3: \
	if ((xx) == xend) \
	 PIX((yy),(xx)) = PIX((yy),xsucc); \
	else \
	 PIX((yy),(xx)) = PIX((yy),(xx)+1); \
 }
 unsigned int bpl = img.data->imageBpl;
 unsigned int xstart = 0;
 unsigned int ystart = 0;
 unsigned int xend = img.width() -1;
 unsigned int yend = img.height() -1;
 unsigned int xpred = (xstart + img.width() -1) % img.width();
 unsigned int ypred = (ystart + img.height() -1) % img.height();
 unsigned int xsucc = (xstart + img.width()) % img.width();
 unsigned int ysucc = (ystart + img.height()) % img.height();
 unsigned int j;

 unsigned int xsize;	// Image resolution
 unsigned int x, y;	// Pixel location
 unsigned int i;	// Pass index
 unsigned int pc = 0;	// Pass count
 unsigned int count = 1;// Deleted pixel count
 unsigned int p, q;	// Neighborhood maps of adjacent cells
 unsigned int *qb;	// Neighborhood maps of previous scanline
 unsigned int m;	// Deletion direction mask

 xsize = img.width();
 qb = (unsigned int *) malloc ((xsize + 1) * sizeof (unsigned int));

 while (count && pc < 10)
 {	/* Scan image while deletions */
   pc++;
   count = 0;
   for (i = 0; i < 4; i++)
    {
     m = masks[i];

     /* Build initial previous scan buffer */
     p = PIX (ypred, xpred) == colInd;
     for (j = 0, x = xstart; j < xsize; j++, x++)
       qb[j] = p = ((p << 1) & 0006) | (PIX (ypred, x) == colInd);
     qb[xsize] = ((p << 1) & 0006) | (PIX (ypred, xsucc) == colInd);

     q = qb[0];
     p = ((q << 2) & 0010) | (PIX (ystart, xpred) == colInd);
     for (j = 0, x = xstart; j < xsize; j++, x++)
      {
       q = qb[j];
       p = ((p << 1) & 0066) | ((q << 3) & 0010) | (PIX (ystart, x) == colInd);
       qb[j] = p;
      }
     q = qb[xsize];
     qb[xsize] = ((p << 1) & 0066) | ((q << 3) & 0010) | (PIX (ystart, xsucc) == colInd);

     /* Scan image for pixel deletion candidates */
     for (y = ystart; y < yend; y++)
      {
       q = qb[0];
       p = ((q << 2) & 0110) | (PIX (y + 1, xpred) == colInd);
       p = ((p << 1) & 0666) | ((q << 3) & 0110) | (PIX (y + 1, xstart) == colInd);
       qb[0] = p;
       for (j = 1, x = xstart; j < xsize; j++, x++)
       {
      	 q = qb[j];
      	 p = ((p << 1) & 0666) | ((q << 3) & 0110) | (PIX (y + 1, x + 1) == colInd);
      	 qb[j] = p;
      	 /*if ((reg) && (!XPointInRegion (reg, x, y)))
      	    continue;*/
      	 if (((p & m) == 0) && deletePix[p])
      	   { count++; DEL_PIX(y, x, i) }
       }
             /* Process right edge pixel */
       /*if ((reg) && (!XPointInRegion (reg, x, y)))
	      continue;*/
       q = qb[xsize];
       p = ((p << 1) & 0666) | ((q << 3) & 0110) | (PIX (y + 1, xsucc) == colInd);
       qb[xsize] = p;
       if ((p & m) == 0 && deletePix[p])
      	 { count++; DEL_PIX(y, xend, i) }
      }
     /* Process bottom scan line */
     q = qb[0];
     p = ((q << 2) & 0110) | (PIX (ysucc, xpred) == colInd);
     p = ((p << 1) & 0666) | ((q << 3) & 0110) | (PIX (ysucc, xstart) == colInd);

     for (j = 1, x = xstart; j < xsize; j++, x++)
     {
       /*if ((reg) && (!XPointInRegion (reg, x, yend)))
      	continue;*/
       q = qb[j];
       p = ((p << 1) & 0666) | ((q << 3) & 0110) | (PIX (ysucc, x + 1) == colInd);
       if ((p & m) == 0 && deletePix[p])
        { count++; DEL_PIX(yend, x, i) }
     }
     q = qb[xsize];
     p = ((p << 1) & 0666) | ((q << 3) & 0110) | (PIX (ysucc, xsucc) == colInd);
     /*if ((reg) && (!XPointInRegion (reg, xend, yend)))
      continue;*/
     if ((p & m) == 0 && deletePix[p])
      { count++; DEL_PIX(yend, xend, i) }
    }
  }
 free(qb);
#undef DEL_PIX
#undef PIX
}

void ApaintImageUtil::insertRemovePixelsVert(ApaintImage& img, int num, int ypos, uchar colIndx, QRgb col, 
                      bool doInsert, bool useDataBuf, uchar *dataBuf)
{
 ApaintImage dimg;
 uchar *srcBuf=NULL;
 int sbpl;
 bool deleteResource = false;

 if (useDataBuf && !dataBuf) return;
 if (ypos>img.height() || ypos<0) return;
 if (!doInsert)
  if (ypos+num>=img.height())
   num = img.height()-QMAX(1,ypos);

 if (doInsert)
  {
   if (useDataBuf)
    srcBuf = dataBuf;
   else
    {
     deleteResource  = true;
     srcBuf = new uchar[img.bpl()*num];
     if (!srcBuf)
      {
       qDebug("ApaintImageUtil::insertRemovePixelsVert: NULL pointer!");
       return;
      }
     int size = img.width()*num;
     if (img.type() == LOAD_CMAP)
      memset(srcBuf, colIndx, size);
     else if(img.type() == LOAD_TRUE_COL)
      {
       uchar rp = qRed(col);
       uchar gp = qGreen(col);
       uchar bp = qBlue(col);
       size*=3;
       for (int i = 0; i<size; i+=3)
        {
         srcBuf[i] = rp;
         srcBuf[i+1] = gp;
         srcBuf[i+2] = bp;
        }
      }
    }
  }
 if (doInsert)
  dimg.newImageKeepName(img.width(), img.height()+num, img.depth(), &img);
 else
  dimg.newImageKeepName(img.width(), img.height()-num, img.depth(), &img);
  /*copy color table*/
 if (img.type() == LOAD_CMAP)
  img.copyImageDataTo(&dimg);

 sbpl = img.bpl(); 

 memcpy(dimg.scanLine(0), img.scanLine(0), sbpl*ypos);
 if (doInsert)
  {
   memcpy(dimg.scanLine(ypos), srcBuf, sbpl*num);
   memcpy(dimg.scanLine(num+ypos), img.scanLine(ypos), sbpl*(img.height()-ypos));
  }
 else
  memcpy(dimg.scanLine(ypos), img.scanLine(ypos+num), sbpl*(img.height()-ypos-num)); 

 img = dimg;
 if (deleteResource)
  delete[] srcBuf;
}


void ApaintImageUtil::insertRemovePixelsHor(ApaintImage& img, int num, int xpos, uchar colIndx, QRgb col, 
                    bool doInsert, bool useDataBuf, uchar *dataBuf)
{
  ApaintImage dimg;
  int x, y;
  int bpp = img.depth()/8;
  uchar *dst, *src;
  uchar r, g, b;
  
  if (useDataBuf && !dataBuf) return;
  if (xpos>img.width() || xpos<0) return;
  if (!doInsert)
   {
    if (xpos+num>=img.width())
     num = img.width()-QMAX(1,xpos);
   }

  r = qRed(col);
  g = qGreen(col);
  b = qBlue(col);
  
  if (doInsert)
    dimg.newImageKeepName(img.width()+num, img.height(), img.depth(), &img);
  else
    dimg.newImageKeepName(img.width()-num, img.height(), img.depth(), &img);
  /*copy color table*/
  if (img.type() == LOAD_CMAP)
   img.copyImageDataTo(&dimg);
   
  uchar **srcData = img.jumpTable();
  uchar **dstData = dimg.jumpTable();
  
  if (img.type() == LOAD_TRUE_COL)
  {
      for (y=0; y<img.height(); y++)
      {
        memcpy(*(dstData+y), *(srcData+y), bpp*xpos);
        if (doInsert)
        {
           dst = *(dstData+y)+xpos*bpp;
           if (useDataBuf)
           {
             src = (uchar *)(dataBuf + y*num*bpp);
             for (x=0; x<num; x++)
             {
               *dst++ = *src++;
               *dst++ = *src++;
               *dst++ = *src++;
             }
           }
           else
           {
             for (x=0; x<num; x++)
             {
               *dst++ = r;
               *dst++ = g;
               *dst++ = b;
             }
           }
           memcpy(*(dstData+y)+(xpos+num)*bpp, *(srcData+y)+xpos*bpp, (img.width()-xpos)*bpp);
        }
        else //remove
         memcpy(*(dstData+y)+xpos*bpp, *(srcData+y)+(xpos+num)*bpp, QMAX(0, img.width()-xpos-num)*bpp);
      }
  }
  else if (img.type() == LOAD_CMAP)
  {
      for (y=0; y<img.height(); y++)
      {
        memcpy(*(dstData+y), *(srcData+y), xpos);
        if (doInsert)
        {
           dst = *(dstData+y)+xpos;
           if (useDataBuf)
           {
             src = (uchar *)(dataBuf + y*num);
             for (x=0; x<num; x++)
               *dst++ = *src++;
           }
           else
           {
             for (x=0; x<num; x++)
               *dst++ = colIndx;
           }
           memcpy(*(dstData+y)+xpos+num, *(srcData+y)+xpos, (img.width()-xpos));
        }
        else
          memcpy(*(dstData+y)+xpos, *(srcData+y)+xpos+num, QMAX(0, img.width()-xpos-num));
      }

  }
  img = dimg;
}

uchar *ApaintImageUtil::getVertLines(ApaintImage *img, int pos, int num, int &bufSize)
{
  if (!img) return 0;
  bufSize = img->bpl()*num;
  uchar *buf = new uchar[bufSize];
  if (!buf)
  {
    qDebug("ApaintUnRedo::putInsRemData : NULL pointer!");
    return 0;
  }
  memcpy(buf, img->scanLine(pos), num*img->bpl());
  return buf;
}

uchar *ApaintImageUtil::getHorLines(ApaintImage *img, int pos, int num, int &bufSize)
{
  if (!img) return 0;
  int bpp = img->depth()/8;
  int bufBpl = bpp*num;
  bufSize = img->height()*bufBpl;
  uchar *buf = new uchar[bufSize];
  if (!buf)
  {
    qDebug("ApaintUnRedo::putInsRemData : NULL pointer!");
    return 0;
  }
  for (int y=0; y<img->height(); y++)
    memcpy(buf+y*bufBpl, img->scanLine(y)+pos*bpp, bufBpl);    

  return buf;
}

bool ApaintImageUtil::qPixmapToImage(QPixmap &pixRect, ApaintImage &outImage, int xs, int ys, int wids, int heis, uint fgCol)
{
  QImage img1Bit;
  int x, y;
  uchar *bitLine, *imgLine;
  uchar **dataBit, **imgData;
  uchar colIndx;
  uchar rcol, bcol, gcol;

  if (pixRect.isNull()) return false;
  if (pixRect.width() != wids || pixRect.height() !=heis) return false;
  colIndx = 0;
  rcol = bcol = gcol =0;
  img1Bit=pixRect.convertToImage();
  dataBit=img1Bit.jumpTable();
  imgData=outImage.jumpTable();

  if (outImage.type() == LOAD_TRUE_COL)
   {   
    rcol = qRed(fgCol);
    bcol = qBlue(fgCol);
    gcol = qGreen(fgCol);    
   }
  else if (outImage.type() == LOAD_CMAP)
   colIndx = fgCol;   

  switch (outImage.type())
  {
    case LOAD_BW:
    {
      for (y=0; y<heis; y++)
      {
          bitLine=*(dataBit+y);
          imgLine=*(imgData+y+ys);
          if (img1Bit.bitOrder()==QImage::LittleEndian)
          {
           for (x=0; x<wids; x++)
            if (*(bitLine+(x>>3)) & 1 << (x & 7))
              *(imgLine+(x>>3)) |= 1 << (x & 7);
            else
              *(imgLine+(x>>3)) &= 254 << (x & 7);
          }
          else /*big endian*/
          {
           for (x=0; x<wids; x++)
            if (*(bitLine+(x>>3)) & 1<<(7-(x & 7)))
              *(imgLine+(x>>3)) |= 1<<(7-(x & 7));
            else
              *(imgLine+(x>>3)) &= 254 << (7 -(x & 7));
          }
      }
      break;
    }
    case LOAD_CMAP:
    {
        for (y=0; y<heis; y++)
        {
          bitLine=*(dataBit+y);
          imgLine=*(imgData+y+ys);
          if (img1Bit.bitOrder()==QImage::LittleEndian)
          {
           x=0;
           while (x<wids)
           {
             if (*(bitLine+(x>>3))==0)
             {
               x+=8;
               continue;
             }
             if (*(bitLine+(x>>3)) & 1<<(x & 7))
             {
                *(imgLine+x+xs)=(uchar)colIndx;
             }
             x++;
           }
          }
          else /*big endian*/
          {
           x=0;
           while (x<wids)
           {
             if (*(bitLine+(x>>3))==0)
             {
               x+=8;
               continue;
             }
             if (*(bitLine+(x>>3)) & 1<<(7-(x & 7)))
             {
                *(imgLine+x+xs)=(uchar)colIndx;
             }
             x++;
           }
          }
        }
        break;
    }
    case LOAD_TRUE_COL:
    {
         for (y=0; y<heis; y++)
         {
           bitLine=*(dataBit+y);
           imgLine=*(imgData+y+ys)+(xs*3);
           if (img1Bit.bitOrder()==QImage::LittleEndian)
           {
             x=0;
             while (x<wids)
             {
               if (*(bitLine+(x>>3)) == 0)
               {
                 x+=8;
                 imgLine+=24;
                 continue;
               }
               if (*(bitLine+(x>>3)) & 1<<(x & 7))
               {
                 *imgLine++=rcol;
                 *imgLine++=gcol;
                 *imgLine++=bcol;
               }
               else
               {
               imgLine++; imgLine++; imgLine++;
               }
               x++;
             }
           }
           else /*big endian*/
           {
             x=0;
             while (x<wids)
             {
               if (*(bitLine+(x>>3)) == 0)
               {
                 x+=8;
                 imgLine+=24;
                 continue;
               }
                if (*(bitLine+(x>>3)) & 1<<(7-(x & 7)))
                {
                  *imgLine++=rcol;
                  *imgLine++=gcol;
                  *imgLine++=bcol;
                }
                else
                {
                imgLine++; imgLine++; imgLine++;
                }
                x++;
              }
           }
         }
      break;
    }
  }
  return true;
}

/*
int ApaintImageUtil::getDarkestColor(QRgb *ctab, int num)
{
  int darkest = 3*255;
  int indx, val;
  indx = 0;  
  for (int i=0; i<num; i++)
  {
    val = qRed(ctab[i]) + qGreen(ctab[i]) + qBlue(ctab[i]);
    if (val<darkest)
    {
      indx = i;
      darkest = val;
    }
  }
  return indx;  
}
*/

int ApaintImageUtil::getLightestColor(QRgb *ctab, int num)
{
  int light = 0;
  int indx, val;
  indx = 0;
  for (int i=0; i<num; i++)
  {
    val = qRed(ctab[i]) + qGreen(ctab[i]) + qBlue(ctab[i]);
    if (val>light)
    {
      indx = i;
      light = val;
    }
  }
  return indx;   
}

ApaintImage ApaintImageUtil::getTextImage(int wid, int hei, QFont font, QString text, bool doAntiAlias)
{
  return (getTextImage(wid, hei, font, text, qRgb(255, 255, 255), qRgb(0, 0, 0), doAntiAlias));
}

ApaintImage ApaintImageUtil::getTextImage(int wid, int hei, QFont font, QString text, QRgb bg, QRgb fg, bool doAntiAlias)
{  
  QRgb black = fg;
  QRgb white = bg;
  QRect pixRect;
  const int blackIndx = 0;
  const int whiteIndx = 1;
  
  QFontMetrics fm(font);
  
  if (doAntiAlias)
    pixRect = QRect(0, 0, wid*4, hei*4);
  else
    pixRect = QRect(0, 0, wid, hei);
    
  QPixmap pix(pixRect.width(), pixRect.height(), 1);
  pix.fill(Qt::black);

  QPainter p(&pix);
  p.setPen(QPen(Qt::white));
  p.setFont(font);

  p.drawText(pixRect, Qt::AlignLeft | Qt::AlignVCenter, text);
  
  ApaintImage aimg8, aimg24;
  
  aimg8.newImage(pixRect.width(), pixRect.height(), 8);
  aimg24.newImage(wid, hei, 24);

  aimg8.setNumColors(2);
  aimg8.setColor(blackIndx ,black);
  aimg8.setColor(whiteIndx, white);
     
  aimg8.fill(whiteIndx);
  qPixmapToImage(pix, aimg8, 0, 0, pixRect.width(), pixRect.height(), blackIndx);

  if (doAntiAlias)
    antialiasImage(aimg8.scanLine(0), aimg8.bpl(), aimg24.scanLine(0), aimg24.bpl(), aimg8.getCmap(), aimg24.width(), aimg24.height());
  else
  {
    aimg8.setImageType(LOAD_TRUE_COL, 0, 0, 0, false);
    aimg24 = aimg8;
  }  
  return aimg24;
  
}

ApaintImage ApaintImageUtil::getColorsImage(ApaintImage &img, QFont font, int wid, int hei)
{
 int cwid, cwidAt;
 ApaintImage aimg, cimg;
 QRgb blackText, whiteText;
 QRgb *ctab = img.colorTable();
 int midVal = 127*3;
 QString numText;
 int *histoNum, *histoCol;
 int allPix = img.width()*img.height();
 
 blackText = qRgb(0, 0, 0);
 whiteText = qRgb(255, 255, 255);

 img.getHistogramData(&histoNum, &histoCol, false, 0);
 aimg.newImage(wid, hei, 24);
 cwid = wid / img.numColors();
 cwidAt = 0;
 for (int i=0; i<img.numColors(); i++)
  {   
   if (i == img.numColors()-1)
    cwid = wid - cwidAt;
   numText = QString("%1%").arg((int)(((double)histoCol[i]/(double)allPix)*100));
   if ((qRed(ctab[histoNum[i]]) + qGreen(ctab[histoNum[i]]) + qBlue(ctab[histoNum[i]])) < midVal)
    cimg = getTextImage(cwid, hei, font, numText, ctab[histoNum[i]], whiteText, false);
   else
    cimg = getTextImage(cwid, hei, font, numText, ctab[histoNum[i]], blackText, false);

   bitBlt(&aimg, cwidAt, 0, &cimg, 0, 0, cimg.width(), cimg.height(), false, false, 0, 0, 0);
   cwidAt+=cwid;  
  }

 return aimg;
}

bool ApaintImageUtil::getBgFgColors(ApaintImage &img, uint &bg, uint &fg)
{    
  int *histoNum, *histoCol;
  
  if (img.type() != LOAD_CMAP ) return false;

  img.getHistogramData(&histoNum, &histoCol, false, 0);
  
  bg = img.color(histoNum[0]);
  fg = img.color(histoNum[1]);
  
  return true;
}

ApaintImage ApaintImageUtil::doContourCleanup(ApaintImage &img, uint bg, uint fg)
{
  ApaintImage dImg;
  uchar *srcLine, *dstLine;
  uchar rbg, gbg, bbg;
  int dr, dg, db;

  if (img.type() != LOAD_TRUE_COL)
   return dImg;

  rbg = qRed(bg) ;
  gbg = qGreen(bg);
  bbg = qBlue(bg);
  
  dImg.newImage(img.width(), img.height(), 8);
  img.copyImageDataTo(&dImg);
  dImg.setNumColors(2);
  dImg.setColor(0, bg);
  dImg.setColor(1, fg);

  for (int y=0; y<img.height(); y++)
  {
    srcLine = img.scanLine(y);
    dstLine = dImg.scanLine(y);
    for (int x=0; x<img.width(); x++)
    {
     dr = srcLine[x*3]-rbg;
     dg = srcLine[x*3+1]-gbg;
     db = srcLine[x*3+2]-bbg;
      if ((dr*dr+dg*dg+db*db) < 30)
        dstLine[x] = 0;
      else
        dstLine[x] = 1;
    }
  }

  contourPrun(dImg, 1, 0, false);
  contourThin(dImg, 1, false);
  return dImg;
}

/**
* From color table (param ctab) select color that is the best match for 
* input color (param col).
* @param col Color you wish to have.
* @param indxTab Table with indexes to colors in color table.
* @param numIndx Number of indexes.
* @param ctab Color table.
* @return Color table index for color that is the closest to selected one.
*/

int ApaintImageUtil::getClosestColor(QRgb col, int *indxTab, int numIndx, QRgb *ctab)
{
  if (!indxTab || !ctab) return 0;

  int val, retIndx = 0;
  int minVal = INT_MAX;
  int dr, red = qRed(col);
  int dg, green = qGreen(col);
  int db, blue = qBlue(col);
  
  for (int i=0; i < numIndx; i++)
  {
   dr = red - qRed(ctab[indxTab[i]]);
   dg = green - qGreen(ctab[indxTab[i]]);
   db = blue - qBlue(ctab[indxTab[i]]);
   val = dr*dr + dg*dg + db*db;
   if (minVal > val)
    {
      minVal = val;
      retIndx = indxTab[i];
    }
  }

  return retIndx;
}

ApaintImage ApaintImageUtil::changeToColorTabImg(ApaintImage &orig, ApaintImage &trueColImg,
                              double angle, int dstWid, int dstHei, bool rotateContour, 
                              bool usingMask, QImage& mask)
{

  uint fg, bg;
  QRgb *ctab;

  if (rotateContour)
  {
    if (ApaintImageUtil::getBgFgColors(orig, bg, fg))
      return ApaintImageUtil::doContourCleanup(trueColImg, bg, fg);
  }
/*
  // simple algorithm

  bool usedCol[256];
  uint dtab[256];
  int numUsed, n=0;
  
  ctab = orig.colorTable();
  orig.getUsedColors(usedCol, numUsed, false, 0);
  for (int i=0; i<256; i++)
    if(usedCol[i])
    {
     dtab[n] = ctab[i];
     n++;
    }
  trueColImg.setImageType(orig.type(), dtab, numUsed, 0, false);
  trueColImg.setColorTable(ctab, orig.numColors());

  return trueColImg;
*/

  uchar *upLine, *line, *downLine;
  int  j, rIndx, x, y, yup, ydown;
  int tabIndx[9];
  int xpos[3];
  ApaintImage dstImg = orig.scale(dstWid, dstHei);

  QImage rot_mask = QImage(dstImg.width(), dstImg.height(), 1, 2, QImage::LittleEndian);
  rot_mask.fill(255);
  if (angle != 0)
  {
    dstImg = dstImg.rotate(angle, FILTER_NONE, true, rot_mask);
    if (usingMask)
      mask = rot_mask;
  }

  int wid = dstImg.width();
  int hei = dstImg.height();

  int buffHalf = wid * 6;
  //alocate drawing buffer for upper, middle and bottom line
  if (!dstImg.allocDrawBuff(buffHalf)) return dstImg;
  ctab = dstImg.colorTable();
  for (y=0; y < hei; y++)
   {
     if (dstImg.data->changecount >= buffHalf)
      dstImg.flushChanges(0, FALSE);

     yup = QMAX(0, y-1);
     ydown = QMIN(hei-1, y+1);
     upLine = dstImg.scanLine(yup);
     line = dstImg.scanLine(y);
     downLine = dstImg.scanLine(ydown);

     for (x=0; x < wid; x++)
     {
        if (!PIX_ON(rot_mask, x, y))
          continue;

        xpos[0] = x;                // mid
        xpos[1] = QMAX(0, x-1);     // left
        xpos[2] = QMIN(wid-1, x+1); // right
        for (j = 0; j < 3; j++)
        {
          tabIndx[j] = PIX_ON(rot_mask, xpos[j], yup) ? upLine[xpos[j]] : line[x];
          tabIndx[j+3] = PIX_ON(rot_mask, xpos[j], y) ? line[xpos[j]] : line[x];
          tabIndx[j+6] = PIX_ON(rot_mask, xpos[j], ydown) ? downLine[xpos[j]] : line[x];
        }
        rIndx = getClosestColor(trueColImg.pixel(x, y), tabIndx, 9, ctab);
        dstImg.setChangePosVal(dstImg.data->changecount, line[xpos[0]], rIndx);
        dstImg.data->changecount++;

     }
  }
  dstImg.flushChanges(0, TRUE);
  return dstImg;
}

QImage ApaintImageUtil::getMaskByColor(ApaintImage &img, uchar colIndx, QRect &area)
{
  int x, y;
  int minx, maxx, miny, maxy;
  uchar *line;

  if (img.type() != LOAD_CMAP) return 0;
  
  QImage validMask = QImage(img.width(), img.height(), 1 ,2, QImage::LittleEndian);
  validMask.fill(0);

  minx = miny = INT_MAX;
  maxx = maxy = 0;
  for (y=0; y<img.height(); y++)
   {
    line = img.scanLine(y);
    for (x=0; x<img.width(); x++)
      if (line[x] == colIndx)
       {
        SET_PIX(validMask, x, y);
        if (minx > x) minx = x;
        if (miny > y) miny = y;
        if (maxx < x) maxx = x;
        if (maxy < y) maxy = y;
       }
   }

  area = QRect(minx, miny, maxx - minx + 1, maxy - miny + 1);
  QImage retMask = QImage(area.width(), area.height(), 1 ,2, QImage::LittleEndian);
  retMask.fill(0);
  copy1BitImage(retMask, 0, 0, validMask, area.x(), area.y(), area.width(), area.height(), false, 0, 0);

  return retMask;
}

