/* 
 * $Id: kpixmapio.cpp,v 1.23 2002/03/04 00:51:51 lunakl Exp $
 *
 * This file is part of the KDE project, module kdeui.
 * Copyright (C) 2000 Geert Jansen <jansen@kde.org>.
 *
 * You can Freely distribute this program under the GNU Library General
 * Public License. See the file "COPYING.LIB" for the exact licensing terms.
 *
 * kpixmapio.cpp.old(in KDE): Fast pixmap <-> image conversion.
 * kpixmapio.cpp(NEW apaint): works faster, supports ApaintDoc file for fast layer sup. 
 * apaintio.cpp :  kpixmapio.cpp is renamed to apaintio.cpp.
 *                 This file has been changed by Matjaz Gutenberger for the use in ArahPaint4,
 *                 and convertToXImage(), drawGrid() has been optimized by Dusan Peterc
 *                 and finally heavily modified by Nedeljko Stefanovic
 *                 Removed handling of LOAD_BW image type on image display, since we do not allow this type of image
 */

#ifndef HAVE_MITSHM
#define HAVE_MITSHM
#endif

#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <stdlib.h>
#include <string.h>
#include <sys/utsname.h>
#include <libgen.h>
#include <cmath>

#include <qimage.h>
#include <qpixmap.h>
#include <qcolor.h>
#include <qglobal.h>
#include <stdio.h>
#include <qpaintdevice.h>
#include <qwidget.h>
#include "iconViewData.h"
#include "apaintio.h"
#include "apaintconfig.h"
#include <qfontmetrics.h>
#include "apaintBrushList.h"
#include "apaint.h"

#ifndef Q_WS_QWS
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#ifdef HAVE_MITSHM
#include <X11/extensions/XShm.h>
#endif
#ifdef __osf__
extern "C" int XShmQueryExtension(Display *display);
#endif
#else
#undef HAVE_MITSHM
#endif

#define BUF_Y 250
#define PIXEL_ON(src, sx, sy) (*(src->scanLine(sy)+(sx>>3)) & 1 << (sx & 7))
#define SET_PIXEL_ON(src, sx, sy) (*(src->scanLine(sy)+(sx>>3)) |= 1 << (sx & 7))
#define SET_PIXEL_OFF(src, sx, sy) (*(src->scanLine(sy)+(sx>>3)) &= 254 << (sx & 7))

struct ApaintIOPrivate
{
    int shmsize;
    int bytesPerPixel;
    int xmax;
#ifndef Q_WS_QWS
    XImage *ximage;
#ifdef HAVE_MITSHM
    XShmSegmentInfo *shminfo;
#endif
#else
    void *ximage;
#endif
};


//  From Qt: Returns the position of the lowest set bit in val.

typedef unsigned char uchar;
typedef unsigned int uint;

/*** ApaintIO ***/

static ApaintIO *objPtr = 0;

ApaintIO* ApaintIO::getObject()
{
  if (!objPtr)
   objPtr = new ApaintIO();

  return objPtr;  
}

void ApaintIO::deleteObject()
{
  if (objPtr)
   delete objPtr;
}

ApaintIO::ApaintIO()
{
 m_bShm = false;
 d = new ApaintIOPrivate;
 gridWid = gridHei =8;
 gridCol = 0;
#ifdef HAVE_MITSHM

 int ignore;
 if (XQueryExtension(qt_xdisplay(), "MIT-SHM", &ignore, &ignore, &ignore))
  {
   if (XShmQueryExtension(qt_xdisplay()))
    {
     struct utsname unixid;
     char *sysComputerName;
     if (uname(&unixid)>=0)
       sysComputerName = strdup(unixid.nodename);
     else
       sysComputerName = strdup("");
     char *dname = XDisplayString(qt_xdisplay());
     strtok(dname, ":");
     if (!strcmp(":0", dname))
      m_bShm = true;
     else if (!strcmp(":0.0", dname))
      m_bShm = true;
     else if (!strcmp("unix", dname))
      m_bShm = true;
     else if (!strcmp(sysComputerName, dname))
      m_bShm = true;
     else if (strstr(sysComputerName, ".local")) /* this works on OSX */
      m_bShm = true;
     else
      m_bShm = false;
     free(sysComputerName);
    }
  }
 if (!ApaintConfig::getUseSharedMemory())
  m_bShm = false;

 if (!m_bShm)
  {
   qDebug("MIT-SHM not available!\n");
   d->ximage = 0;
   d->shminfo = 0;
   d->shmsize = 0;
  }
  else
  {
   d->ximage = 0;
   d->shminfo = new XShmSegmentInfo;
  }
#else
 d->ximage = 0;    
#endif 
 d->xmax=XDisplayWidth(QPaintDevice::x11AppDisplay(), QPaintDevice::x11AppScreen());
 initXImage(d->xmax*4, BUF_Y); /* allocate it 4 times wider, to allow overwrites on extreme zoom in values */
 pix = new QPixmap(d->xmax*4, BUF_Y);
 d->bytesPerPixel = d->ximage->bits_per_pixel/8;
 transProtTable = new QImage(d->xmax*4, BUF_Y, 1, 2, QImage::LittleEndian);
 transProtTable->fill(0);
}


ApaintIO::~ApaintIO()
{
    destroyXImage();
#ifdef HAVE_MITSHM
    if (m_bShm)
    {
     destroyShmSegment();
     delete d->shminfo;
    }
#endif
    delete pix;
    delete d;
    delete transProtTable;
}

void ApaintIO::initXImage(int w, int h)
{
   createXImage(w, h);
   int size = d->ximage->bytes_per_line * d->ximage->height;
#ifdef HAVE_MITSHM
   if (m_bShm)
    {
     createShmSegment(size);
     d->ximage->data = d->shminfo->shmaddr;
    }
#endif
    return;
}

void ApaintIO::destroyXImage()
{
    if (d->ximage)
    {
    	XDestroyImage(d->ximage);
    	d->ximage = 0L;
    }
}


void ApaintIO::createXImage(int w, int h)
{
   char *imagedata=0;
   destroyXImage();
   if (m_bShm)
    {
#ifdef HAVE_MITSHM
      d->ximage = XShmCreateImage(qt_xdisplay(), (Visual *) QPaintDevice::x11AppVisual(),
	     QPaintDevice::x11AppDepth(), ZPixmap, 0L, d->shminfo, w, h);
#endif
     }
     else
     {
       d->ximage = XCreateImage(qt_xdisplay(), (Visual *) QPaintDevice::x11AppVisual(),
    	                          QPaintDevice::x11AppDepth(), ZPixmap, 0L, imagedata,
                                w, h, BitmapPad(qt_xdisplay()), 0 );          
       Q_CHECK_PTR(d->ximage);
       imagedata = (char *) malloc( (unsigned long) h*(d->ximage->bytes_per_line));
       Q_CHECK_PTR(imagedata);
       d->ximage->data = imagedata;
     }
}


QRect ApaintIO::getScaleRect(QRect pos, double scaleX, double scaleY)
{
  int x, y, w, h;
  
  x = (int)(pos.x()*scaleX);
  w = (int)((pos.right()+1)*scaleX) - x;
  y = (int)(pos.y()*scaleY);
  h = (int)((pos.bottom()+1)*scaleY) -y;
  return QRect(x, y, w, h);
}

void ApaintIO::fastIconDraw(IconList *ilist, QPainter *p, QFontMetrics &fm, int dx, int dy, int dw, int dh)
{
 
  int ypos, hei, fldx, fldy;
  bool drawRect;
  QPixmap *folderIcon = ilist->getFolderIcon();

  QRect pos, rRect;
  IconData *icData;
  int ty;
  QPen redP = QPen(Qt::red,2);
  QPen blackP = QPen(Qt::black,1);
  QPen grayP  = QPen(Qt::gray, 1);

  if (!dw || !dh) return;
  
  if (d->bytesPerPixel >= 2)
  {           
     for (ypos=0; ypos<dh; ypos+=BUF_Y)
     {
        hei = QMIN(BUF_Y, dh-ypos);
        QRect refreshRect(dx, dy+ypos, dw, hei);
        for (int icon=0; icon < ilist->iconNum(); icon++)
        {
           icData = ilist->getIconData(icon);
           pos = icData->getIconPos();
           drawRect = false;
           if (refreshRect.intersects(pos))
           {
             drawRect = true;
             rRect= refreshRect.intersect(pos);
             if (icData->isImage())
             {
               icData->renderIcon((unsigned char *)d->ximage->data, d->bytesPerPixel, d->ximage->bytes_per_line);
 #ifdef HAVE_MITSHM
               if (m_bShm)
                XShmPutImage(qt_xdisplay(), pix->handle(), qt_xget_temp_gc(qt_xscreen(), false), d->ximage,
                          rRect.x()-pos.x(), rRect.y()-pos.y(), 0, 0, rRect.width(), rRect.height(), false);
               else
 #endif
                XPutImage(qt_xdisplay(), pix->handle(), qt_xget_temp_gc(qt_xscreen(), false), d->ximage,
                          rRect.x()-pos.x(), rRect.y()-pos.y(), 0, 0, rRect.width(), rRect.height());
               XSync(qt_xdisplay(), false);
               p->drawPixmap(rRect.x(), rRect.y(), *pix, 0, 0, rRect.width(), rRect.height());
              }
              else if (icData->isDir() && folderIcon)
               {
                p->setFont(QFont (ApaintConfig::getFontName(), ApaintConfig::getFontSize()));
                p->fillRect(rRect, Qt::white);
                fldx = pos.x()+(QMAX(0, pos.width()-folderIcon->width()))/2;
                fldy = pos.y()+(QMAX(0, pos.height()-folderIcon->height()))/2;
                p->drawPixmap(fldx, fldy, *folderIcon, 0, 0, folderIcon->width(), folderIcon->height());
		if ( (!ilist->isBrushList()) && (ApaintConfig::getBrowseImageDisplay() & IMAGE_ICON_DIR_INFO))
		 {
                  if (icData->getNumDirs())
        	   p->drawText(fldx + folderIcon->width() + 2, fldy + fm.height() + 2, QString("%1").arg(icData->getNumDirs()));
		  if (icData->getNumFiles())
        	   p->drawText(fldx + 4, fldy + folderIcon->height() + fm.height(), QString("%1").arg(icData->getNumFiles()));
		 }
               }
            }
           p->setFont(QFont (ApaintConfig::getFontName(), ApaintConfig::getSmallFontSize()));
           /* draw textual data */
	   /* draw subdirectory name on top */
	   pos = icData->getIconPos();
	   if (refreshRect.intersects(pos) && icData->isImage() && (ApaintConfig::getBrowseImageDisplay() & IMAGE_ICON_SUB_DIR))
	    {
	     imageInfo *iI = icData->getImageInfo();
	     if (iI->subDirID)
	      {
               int i, numSlashes=0;
	       char subStr[250]=" ";

	       ty = pos.y()+fm.ascent()+1;
	       if (ilist->selectedIcon() == icon)
	        p->setBackgroundColor(Qt::yellow);
	       else
		{
		 QColor backc = Qt::white;

        	 if (icData->getImageInfo()->imageType==LOAD_BW)
	          backc = QColor(224, 224, 224); /* light grey */ 
        	 if (icData->getImageInfo()->imageType==LOAD_TRUE_COL)
	          backc = QColor(255, 224, 224); /* light pink */ 
        	 if (icData->getImageInfo()->imageType==LOAD_RGBA)
	          backc = QColor(224, 224, 255); /* light blue */ 
        	 p->setBackgroundColor(backc);
		}

	       p->setBackgroundMode(Qt::OpaqueMode);
	       strcat(subStr, getImageSubDir(iI->subDirID));
	       strcat(subStr, " ");
	       for (i = strlen(subStr); i; i--)
		if (subStr[i]=='/')
        	 numSlashes++;
	       for (i=0; i<=numSlashes; i++)
		{
		 p->drawText(pos.x()+2, ty, QString::fromLocal8Bit(basename(subStr)));
		 if (i!=numSlashes)
		  {
		   strcpy(subStr, dirname(subStr));
		   ty+=fm.ascent()+fm.descent();
		  }
		}
	      }
	    }

           pos = icData->getTextPos();
           ty = pos.y()+fm.ascent()-1;
           if (refreshRect.intersects(pos) && icData->isImage())
            {
             drawRect = true;
	     if (ilist->selectedIcon() == icon)
              p->fillRect(pos, Qt::yellow);
	     else
	      {
	       QColor backc = Qt::white;

               if (icData->getImageInfo()->imageType==LOAD_BW)
	        backc = QColor(224, 224, 224); /* light grey */ 
               if (icData->getImageInfo()->imageType==LOAD_TRUE_COL)
	        backc = QColor(255, 224, 224); /* light pink */ 
               if (icData->getImageInfo()->imageType==LOAD_RGBA)
	        backc = QColor(224, 224, 255); /* light blue */ 
               p->fillRect(pos, backc);
	       p->setBackgroundColor(backc);
	      }

             if ( (!ilist->isBrushList()) && (ApaintConfig::getBrowseImageDisplay() & IMAGE_ICON_FILENAME) )
              {
	       if (ApaintConfig::getBrowseImageDisplay() & IMAGE_ICON_NO_SUFFIX)
	        {
		 char *fname = strdup(icData->getName().local8Bit().data());
		 int len = strlen(fname);
	         int haveSuffix = TRUE;
	         int i;

		 for (i=2; i<7; i++)
		  if ((len>(i+1)) && haveSuffix)
		   if (fname[len-i]=='.')
		    { fname[len-i]=0; haveSuffix=FALSE; }

                 p->drawText(pos.x()+4, ty, QString("%1").arg(QString::fromLocal8Bit(fname)));
		 free(fname);
		}
	       else
                p->drawText(pos.x()+4, ty, icData->getName());
               ty+=fm.height();
              }
             if (ilist->isBrushList() || (ApaintConfig::getBrowseImageDisplay() & IMAGE_ICON_IMAGESIZE))
              {
               p->drawText(pos.x()+4, ty, icData->getInfo(false, !ilist->isBrushList()));
               ty+=fm.height();
              }
             if ((!ilist->isBrushList()) && (ApaintConfig::getBrowseImageDisplay() & IMAGE_ICON_FILESIZE))
              {
              p->drawText(pos.x()+4, ty, icData->getSize());
               ty+=fm.height();
              }
             if ((!ilist->isBrushList()) && (ApaintConfig::getBrowseImageDisplay() & IMAGE_ICON_FILEDATE))
              p->drawText(pos.x()+4, ty, icData->getDTime());
            }
           else
	    if (refreshRect.intersects(pos) && icData->isDir())
             {
              drawRect = true;
	      p->setFont(QFont(ApaintConfig::getFontName(), (strlen(icData->getName())<13) ? ApaintConfig::getFontSize() : ApaintConfig::getSmallFontSize()));
              p->fillRect(pos, Qt::white);
              p->drawText(pos.x()+4, pos.bottom()-fm.ascent()+1, icData->getName());
             }
           /*draw border around icon*/
           if (drawRect)
           {
            pos = icData->getIconPos() | icData->getTextPos();
            if (ilist->selectedIcon() == icon)
            {
             p->setPen(redP);
             p->drawRect(pos.x()+1, pos.y()+1, pos.width()-1, pos.height()-1);
             p->setPen(blackP);
            }
            else
            {
             p->setPen(grayP) ;
             p->drawRect(pos.x(), pos.y(), pos.width(), pos.height());
             p->setPen(blackP);
            }
           }
        }
        pos = ilist->getClearArea();
        if (refreshRect.intersects(pos))
         p->fillRect(refreshRect.intersect(pos), Qt::white);
     }
  } 
}

void ApaintIO::fastImageDraw(ApaintImage& img, int sx, int sy, QPainter *p,
                             int dx, int dy, int dw, int dh, double scaleX, double scaleY)
{
 int ypos, hei;
 QRect pos;
 QRect rRect;
 int scaledBufSize;
 
  if (!dw || !dh) return;
  if (img.isNull()) return;
  if (d->bytesPerPixel >= 2)
  {
     if (scaleY>=BUF_Y)
        scaledBufSize = BUF_Y;
     else if (scaleY>1)
        scaledBufSize = (int)(BUF_Y/scaleY)*(int)scaleY;
     else
        scaledBufSize = (int)((int)(BUF_Y*scaleY)/scaleY);

     pos = QRect((int)(sx*scaleX), (int)(sy*scaleY), (int)(img.width()*scaleX), (int)(img.height()*scaleY));
     for (ypos=0; ypos<dh; ypos+=scaledBufSize)
     {
       hei = QMIN(scaledBufSize, dh-ypos);
       QRect refreshRect(dx, dy+ypos, dw, hei);
       if (refreshRect.intersects(pos))
       {
         rRect = refreshRect.intersect(pos);
         convertToXImage(rRect.x()-refreshRect.x(), rRect.y()-refreshRect.y(),
                         img.width(), img.height(),
                         img.type(), img.depth(),
                         img.jumpTable(), img.colorTable(),
                         img.numColors(), rRect.x()-pos.x(), rRect.y()-pos.y(), rRect.width(), rRect.height(), pos.y(),
                         false, false, scaleX, scaleY,
                         false, 0,
                         0, 0);
                 
#ifdef HAVE_MITSHM
         if (m_bShm)
          XShmPutImage(qt_xdisplay(), pix->handle(), qt_xget_temp_gc(qt_xscreen(), false), d->ximage, 0, 0, 0, 0, rRect.width(), rRect.height(), false);
         else
#endif
          XPutImage(qt_xdisplay(), pix->handle(), qt_xget_temp_gc(qt_xscreen(), false), d->ximage, 0, 0, 0, 0, rRect.width(), rRect.height());
         XSync(qt_xdisplay(), false);
         p->drawPixmap(dx, dy+ypos, *pix, 0, 0, rRect.width(), rRect.height());
       }
     }
  } 
}

void ApaintIO::fastDocDraw(ApaintDoc *doc, QPainter *p, int dx, int dy, int dw, int dh, double scaleX, double scaleY, bool useGrid)
{
 int ypos, hei, sx, sy;
 QRect pos, posTile;
 QWMatrix mx;
 Tile *tile;
 QRect rRect;
 int imgsx, imgsy;
 int scaledBufSize;
 ApaintBrushList *bList;
 ApaintImage brushImg;
 QImage brushMask;

  if (!dw || !dh) return;
  mx.scale(scaleX, scaleY);
  if (d->bytesPerPixel >= 2)
  {
     if (scaleY>=BUF_Y)
        scaledBufSize = BUF_Y;
     else if (scaleY>1)
        scaledBufSize = (int)(BUF_Y/scaleY)*(int)scaleY;
     else
        scaledBufSize = (int)((int)(BUF_Y*scaleY)/scaleY);

     bList = doc->getBrushList();     
     for (ypos=0; ypos<dh; ypos+=scaledBufSize)
      {
       hei = QMIN(scaledBufSize, dh-ypos);
       QRect refreshRect(dx, dy+ypos, dw, hei);
       transProtTable->fill(0);

       for (int layer=0; layer < doc->numLayers(); layer++)
       {
    	bool semitransparent = apaint->transparentSelection(doc->layerList[layer])
    			&& doc->layerList[layer]->getType() != MainLayer;

        if (!doc->layerList[layer]->isDataLoaded()) continue;
        if (doc->layerList[layer]->isHidden()) continue;
        if (doc->layerList[layer]->imageBuf.isNull()) continue;

        imgsx = (int)(doc->layerList[layer]->imageBuf.width()*scaleX);
        imgsy = (int)(doc->layerList[layer]->imageBuf.height()*scaleY);
        pos = getScaleRect(doc->getLayerPos(layer), scaleX, scaleY);

        if (doc->layerList[layer]->isUsingTiles())
        {
           QPtrList<Tile>& tList = doc->layerList[layer]->getTileList();
           for (int i=0; i<(int)tList.count(); i++)
           {
             tile = tList.at(i);
             posTile = getScaleRect(tile->getPosMain(), scaleX, scaleY);

             if (refreshRect.intersects(posTile))
             {
               rRect = refreshRect.intersect(posTile);
               sx = rRect.x() - posTile.x();
               sy = rRect.y() - posTile.y();
               if (tile->mirrorHor())
                 sx = QMAX(0,imgsx-sx-rRect.width());
               if (tile->mirrorVert())
                 sy = QMAX(0,imgsy-sy-rRect.height());
               convertToXImage(rRect.x()-refreshRect.x(), rRect.y()-refreshRect.y(),
                               doc->layerList[layer]->imageBuf.width(), doc->layerList[layer]->imageBuf.height(),
                               doc->layerList[layer]->imageBuf.type(), doc->layerList[layer]->imageBuf.depth(),
                               doc->layerList[layer]->imageBuf.jumpTable(), doc->layerList[layer]->imageBuf.colorTable(),
                               doc->layerList[layer]->imageBuf.numColors(), sx, sy, rRect.width(), rRect.height(), sy,
                               tile->mirrorHor(), tile->mirrorVert(), scaleX, scaleY,
                               false, 0, doc->layerList[layer]->getTransColors(), doc->layerList[layer]->getProtectColors(),
                               semitransparent);
             }
             posTile = getScaleRect(tile->getPosSecond(), scaleX, scaleY);
             if (refreshRect.intersects(posTile))
             {
               rRect = refreshRect.intersect(posTile);
               sx = rRect.x() - posTile.x();
               sy = rRect.y() - posTile.y();
               if (tile->getSecondHor())
                 sx += (imgsx-posTile.width());
               else
                  sy += (imgsy-posTile.height());
               if (tile->mirrorHor())
                sx = imgsx-sx-rRect.width();
               if (tile->mirrorVert())
                sy = imgsy-sy-rRect.height();
               convertToXImage(rRect.x()-refreshRect.x(), rRect.y()-refreshRect.y(),
                               doc->layerList[layer]->imageBuf.width(), doc->layerList[layer]->imageBuf.height(),
                               doc->layerList[layer]->imageBuf.type(), doc->layerList[layer]->imageBuf.depth(),
                               doc->layerList[layer]->imageBuf.jumpTable(), doc->layerList[layer]->imageBuf.colorTable(),
                               doc->layerList[layer]->imageBuf.numColors(), sx, sy, rRect.width(), rRect.height(), sy,
                               tile->mirrorHor(), tile->mirrorVert(), scaleX, scaleY,
                               false, 0, doc->layerList[layer]->getTransColors(), doc->layerList[layer]->getProtectColors(),
                               semitransparent);
             }
           }
        }
        else
        {
          if (refreshRect.intersects(pos))
          {
            rRect = refreshRect.intersect(pos);
            convertToXImage(rRect.x()-refreshRect.x(), rRect.y()-refreshRect.y(), 
                            doc->layerList[layer]->imageBuf.width(), doc->layerList[layer]->imageBuf.height(),
                            doc->layerList[layer]->imageBuf.type(), doc->layerList[layer]->imageBuf.depth(),
                            doc->layerList[layer]->imageBuf.jumpTable(), doc->layerList[layer]->imageBuf.colorTable(),
                            doc->layerList[layer]->imageBuf.numColors(), rRect.x()-pos.x(), rRect.y()-pos.y(), rRect.width(), rRect.height(), pos.y(),
                            false, false, scaleX, scaleY,
                            doc->layerList[layer]->usingMask(), &doc->layerList[layer]->getMask(),
                            doc->layerList[layer]->getTransColors(), doc->layerList[layer]->getProtectColors(),
                               semitransparent);
          }
        }
       }
       if  (bList->doShowBrush() && bList->isValid())
       {
         brushImg = bList->getBrush();
         brushMask = bList->getBrushMask();
         bool useMask = !brushMask.isNull();
         pos = getScaleRect(bList->getPos(), scaleX, scaleY);
         if (refreshRect.intersects(pos))
          {
            rRect = refreshRect.intersect(pos);
            convertToXImage(rRect.x()-refreshRect.x(), rRect.y()-refreshRect.y(),
                            brushImg.width(), brushImg.height(), brushImg.type(), brushImg.depth(),
                            brushImg.jumpTable(), brushImg.colorTable(), brushImg.numColors(), 
                            rRect.x()-pos.x(), rRect.y()-pos.y(), rRect.width(), rRect.height(), pos.y(),
                            false, false, scaleX, scaleY, useMask, &brushMask,
                            doc->getLayer()->getTransColors(), doc->getLayer()->getProtectColors());
          }
         
       }
       if (useGrid && scaleX>=3 && scaleY>=3)
          drawGrid(dx, dy+ypos, dw, hei, scaleX, scaleY, (int)(doc->mainLayer->layerPos().height()*scaleY));

#ifdef HAVE_MITSHM         
       if (m_bShm)
        XShmPutImage(qt_xdisplay(), pix->handle(), qt_xget_temp_gc(qt_xscreen(), false), d->ximage,  0, 0, 0, 0, dw, hei, false);
       else
#endif
        XPutImage(qt_xdisplay(), pix->handle(), qt_xget_temp_gc(qt_xscreen(), false), d->ximage, 0, 0, 0, 0, dw, hei); 
       XSync(qt_xdisplay(), false);
       p->drawPixmap(dx, dy+ypos, *pix, 0, 0, dw, hei);

     }
   }
}
  
void ApaintIO::drawGrid(int dx, int dy, int dw, int dh, double dscaleX, double dscaleY, int imgHei)
{
  int x, y, i, lnum;
  int sizex;
  int bpl = d->ximage->bytes_per_line;
  char *data = d->ximage->data;
  uchar *dst, *dst8;
  unsigned short *dst16;
  QRgb *dst32;
  unsigned long gridCol32 = (gridCol<<16) | (gridCol<<8) | gridCol;
  unsigned short gridCol16 = ((gridCol&248)<<8) | ((gridCol&252)<<3) | (gridCol>>3);
  int *xm=0;
  bool *secondxm = 0;
  bool justSecondX, justSecondY;
  int scaleX, scaleY;
  /* we start drawing from bottom left corner! */
  double addx;
  
  if (!dw || !dh) return;

  scaleX = (int)(dscaleX);
  scaleY = (int)(dscaleY);
  if (scaleX < 3 || scaleY < 3) return;
  
  justSecondX = (scaleX<4);  // draw just second level grid
  justSecondY = (scaleY<4);  // draw just second level grid
  sizex = dw / scaleX;
  if (!sizex) return;
  xm = new int[sizex];
  secondxm = new bool[sizex];
  memset(xm, 0, sizex*sizeof(int));
  memset(secondxm, 0, sizex*sizeof(bool));

  addx = 0;
  i=0;
  lnum = 0;

  while (addx<dx)
   {
    addx+=dscaleX;
    lnum++;
   }
  while (addx<=dx+dw && i<sizex)
   {
    xm[i] = QMAX(0,ceil(addx-dx)*d->bytesPerPixel);
    secondxm[i] = ((lnum%gridWid) == 0);
    i++;
    lnum++;
    addx+=dscaleX;
   }

  lnum = 1;//ceil((imgHei+1)/dscaleY) == ceil(imgHei/dscaleY);

  for (int y1=imgHei-1; y1>=dy; --y1)
   {
    bool draw = ceil(y1/dscaleY) != ceil((y1+1)/dscaleY);

    if (draw)
     {
      ++lnum;
      draw = !justSecondY;
     }
    else
     if (ceil(y1/dscaleY) != ceil((y1-1)/dscaleY))
      draw = (lnum % gridHei == 0);

    if (draw)
     {
      if (y1>=dy+dh)
       continue;

      dst = (uchar*)data+(y1-dy)*bpl;
      if (d->bytesPerPixel==2)
       {
	dst16 = (unsigned short *)dst;
	for (x=0; x<(bpl>>1); x++)
	 *dst16++=gridCol16;
       }
      else
       memset(dst, gridCol, bpl);
     }
   }

   for (y=0; y<dh;y++)
    {
     dst = (uchar *)data+y*bpl;
     switch (d->bytesPerPixel)
      {
       case 2:
       for (x=0; x<sizex; x++)
        {
         dst16 = (unsigned short *)(dst+xm[x]);
         if (!justSecondX)
          *dst16 = gridCol16;
         if (secondxm[x] && x)
          *(--dst16) = gridCol16;
        }
        break;
       case 3:
	for (x=0; x<sizex; x++)
         {
          dst8 = dst+xm[x];
          if (!justSecondX)
           { *dst8++=gridCol; *dst8++=gridCol; *dst8++=gridCol; dst8-=3;}
          if (secondxm[x] && x)
           { *(--dst8)=gridCol; *(--dst8)=gridCol; *(--dst8)=gridCol; }
         }
         break;
       case 4:
        for (x=0; x<sizex; x++)
         {
           dst32 =(QRgb *) (dst+xm[x]);
           if (!justSecondX)
             *dst32 = gridCol32;
           if (secondxm[x] && x)
             *(--dst32) = gridCol32;
         }
        break;
      }
    }
  delete[] xm;
  delete[] secondxm;  
}

#define WRITE_COLOR_8(red, green, blue, semiTransparent)\
{\
	if (semiTransparent)\
	{\
		*dst8++ = (int)(red+*dst8) >> 1;\
		*dst8++ = (int)(green+*dst8) >> 1;\
		*dst8++ = (int)(blue+*dst8) >> 1;\
	} else\
	{\
		*dst8++ = red;\
		*dst8++ = green;\
		*dst8++ = blue;\
	}\
}

#define WRITE_COLOR_16(red, green, blue, semiTransparent)\
{\
	   int r = round(red*0.125);\
	   int g = round(green*0.25);\
	   int b = round(blue*0.125);\
\
	   if (semiTransparent)\
	   {\
		   r = (int)(r + (*dst16>>11)) >> 1;\
		   g = (int)(g + ((*dst16>>5)&63)) >> 1;\
		   b = (int)(b + ((*dst16)&31)) >> 1;\
	   }\
\
	   r = QMIN(r,31);\
	   g = QMIN(g,63);\
	   b = QMIN(b,31);\
\
	   *dst16++ = (r << 11) | (g << 5) | b;\
}

#define WRITE_COLOR_32(red, green, blue, semiTransparent)\
{\
 if (semiTransparent)\
  *dst32++ = qRgb((int)(red + qRed(*dst32)) >> 1, (int)(green + qGreen(*dst32)) >> 1, (int)(blue + qBlue(*dst32)) >> 1); \
 else\
  *dst32++ = qRgb(red, green, blue);\
}

#define WRITE_COLOR_32_SEMI_TRANSPARENT(red, green, blue)\
{\
  *dst32++ = qRgb((int)(red + qRed(*dst32)) >> 1, (int)(green + qGreen(*dst32)) >> 1, (int)(blue + qBlue(*dst32)) >> 1); \
}

#define READ_TC\
	int red = src[0];\
	int green = src[1];\
	int blue = src[2];

#define READ_8_TC READ_TC
#define READ_16_TC READ_TC
#define READ_32_TC READ_TC

#define READ_8_CM\
	QRgb color = cmap24[(*src)*3];\
	int red = qRed(color);\
	int green = qGreen(color);\
	int blue = qBlue(color);

#define READ_16_CM\
	QRgb color = cmap16[*src];\
	int red = qRed(color);\
	int green = qGreen(color);\
	int blue = qBlue(color);

#define READ_32_CM\
	QRgb color = clut[*src];\
	int red = qRed(color);\
	int green = qGreen(color);\
	int blue = qBlue(color);

#define SET_SRC_TC src = tmp+3*xpr;
#define SET_SRC_CM src = tmp+xpr;

#define NEXT8 {dst8 += 3;}
#define NEXT16 {dst16++;}
#define NEXT32 {dst32++;}

#define SET_DST\
 dst8  = (uchar *)(data + (y+dsty)*bpl + d->bytesPerPixel*dstx);\
 dst16 = (unsigned short *) (data+(y+dsty)*bpl + d->bytesPerPixel*dstx);\
 dst32 = (QRgb*)(data+(y+dsty)*bpl+d->bytesPerPixel*(dstx));

#define MASK_COND ((srcMask[xp>>3]&(1<<(xp&7)))==0)
#define PROT_COND_TC false
#define PROT_COND_CM false && (transProtTableY[x>>3]&(1<<(x&7)))
#define TRANS_COND_TC false
#define TRANS_COND_CM false && (transColors[tmp[xp]])
#define X_BEGIN_TC 0
#define X_END_TC (width)
#define X_BEGIN_CM dstx
#define X_END_CM (width+dstx)

#define ANTIALIASING(bits, type, useMask, semiTransparent)\
{\
 for (y=0; y<height; ++y)\
 {\
  SET_DST\
  transProtTableY = (unsigned char *)(transProtTable->scanLine(y+dsty));\
  float y_begin = (mirrory ? (imgHei-sy-y-1)/scaleY : (sy+y)/scaleY);\
  float y_end = (mirrory ? (imgHei-sy-y)/scaleY : (sy+y+1)/scaleY);\
\
  for (x=X_BEGIN_ ## type; x<X_END_ ## type; x++)\
  {\
   if (PROT_COND_ ## type)\
    continue;\
\
   float x_begin = (mirrorx ? (imgWid-sx-x-1)/scaleX : (sx+x)/scaleX);\
   float x_end = (mirrorx ? (imgWid-sx-x)/scaleX : (sx+x+1)/scaleX);\
   float coeff_sum = 0;\
   float red = 0, green = 0, blue = 0;\
\
   for (int yp=floor(y_begin); yp<y_end; ++yp)\
   {\
    uchar *tmp = mirrory ? *(line + (imgHei-yp) % imageHeight) : *(line + yp % imageHeight);\
\
	if (useMask)\
	 srcMask = mask->scanLine((int)yp % mask->height());\
\
    float hor_sum = 0;\
    float red_hor = 0;\
    float green_hor = 0;\
    float blue_hor = 0;\
\
    for (int xp=floor(x_begin), xpr=xp%imageWidth; xp<x_end; ++xp, ++xpr)\
    {\
     if (xpr == imageWidth)\
      xpr = 0;\
\
     if ((useMask && MASK_COND) || (TRANS_COND_ ## type))\
     {\
      NEXT ## bits\
\
      continue;\
     }\
\
     SET_SRC_ ## type\
     float coeff = 1;\
\
     if (xp<x_begin)\
      coeff -= x_begin-floor(x_begin);\
\
     if (xp>x_end-1)\
      coeff -= ceil(x_end)-x_end;\
\
     READ_ ## bits ## _ ## type;\
     red_hor += red*coeff;\
     green_hor += green*coeff;\
     blue_hor += blue*coeff;\
     hor_sum += coeff;\
    }\
\
    float coeff = 1;\
\
    if (yp<y_begin)\
     coeff -= y_begin-floor(y_begin);\
\
    if (yp>y_end-1)\
     coeff -= ceil(y_end)-y_end;\
\
    red += red_hor*coeff;\
    green += green_hor*coeff;\
    blue += blue_hor*coeff;\
    coeff_sum += coeff*hor_sum;\
   }\
\
   if (coeff_sum>0)\
   {\
    red /= coeff_sum;\
    green /= coeff_sum;\
    blue /= coeff_sum;\
    red = round(red);\
	green = round(green);\
	blue = round(blue);\
\
	if (red>255) red = 255;\
	if (green>255) green = 255;\
	if (blue>255) blue = 255;\
    WRITE_COLOR_ ## bits(red, green, blue, semiTransparent)\
   } else\
    NEXT ## bits;\
\
  }\
 }\
}

//srcOffy is used for correctly copying the same lines from src buffer when downscaling!
void ApaintIO::convertToXImage(int dstx, int dsty,
                                int imageWidth, int imageHeight, int imageType,
                                int imageDepth, uchar** imageLine, QRgb* imageColorTab, int numColors,
                                int sx, int sy, int sw, int sh, int srcOffY,
                                bool mirrorx, bool mirrory, double scaleX, double scaleY,
                                bool useMask, QImage *mask, bool *transColors, bool *protectColors,
				bool semiTransparent)
{
  int x, y;
  int width = sw, height = sh;
  int bpl = d->ximage->bytes_per_line;
  char *data = d->ximage->data;
  uchar **line = imageLine;
  uchar  *src,
  *dst8, *srcMask=0, cmap24[256*3], *cmap24ptr;

  unsigned short *dst16, cmap16[256];
  QRgb *dst32;
  QRgb *clut = imageColorTab;

  long *srcx, *xm;
  long *srcXptr, *xmPtr;
  unsigned char *transProtTableY;
  bool haveProtectedColors = FALSE;
  bool haveTransparentColors = FALSE;
  int nx = sx/scaleX;
  int ny = sy/scaleY;
  int imgWid = ((sx+sw)/scaleX+0.5) - 1;
  int imgHei = ((sy+sh)/scaleY+0.5) - 1;

  //srcOffy should not be less than 0
  srcOffY = QMAX(0, srcOffY);
  
  if (imageType == LOAD_CMAP)
   {
    if (protectColors)
     for (x=0; x<numColors; x++)
      if (protectColors[x])
       { haveProtectedColors=TRUE; break; }
    if (transColors)
     for (x=0; x<numColors; x++)
      if (transColors[x])
       { haveTransparentColors=TRUE; break; }
   }

  srcx = (long *) malloc(sizeof(unsigned long)*width);
  if (!srcx)
   return;
  xm = (long *) malloc(sizeof(unsigned long)*width);
  if (!xm)
   { free(srcx); return; }
  xmPtr = xm;
  if (useMask)/* main layer does not use mask so no mirroring */
   for (x=0; x<width; x++)
    *xmPtr++ = ((int)((sx+x)/scaleX))%mask->width();
  srcXptr = srcx;

  if (mirrorx)
   for (x=0; x<width; x++) {
    *srcXptr = (imgWid-(int)((sx+x)/scaleX)+nx) % imageWidth;
    if (*srcXptr >= imageWidth)
    	*srcXptr -= imageWidth;
    srcXptr++;
   }
  else
   for (x=0; x<width; x++) {
    *srcXptr = ((int)floor((sx+x)/scaleX))%imageWidth;
    if (*srcXptr < 0)
    	*srcXptr += imageWidth;
    srcXptr++;
   }

  /* prepare colormap pixel values in advance */
  if (imageType==LOAD_CMAP)
   {
    if (d->bytesPerPixel==2)
     for (x=0; x<256; x++)
      cmap16[x]= ((qRed(clut[x])&248)<<8) | ((qGreen(clut[x])&252)<<3) | (qBlue(clut[x])>>3);
    cmap24ptr = cmap24;
    if (d->bytesPerPixel==3)
     for (x=0; x<256; x++)
      {
       *cmap24ptr++ = qRed(clut[x]);
       *cmap24ptr++ = qGreen(clut[x]);
       *cmap24ptr++ = qBlue(clut[x]);
      }
   }

  srcXptr = srcx;
  if (imageDepth==24)
   for (x=0; x<width; x++)
    *srcXptr++ *= 3;

  xmPtr = xm;

  if ((scaleX<1 || scaleY<1) && apaint->antialiasView()) {
	  switch(d->bytesPerPixel)
	  {
		  case 2: switch(imageType) {
			  case LOAD_TRUE_COL:
				  ANTIALIASING(16, TC, useMask, semiTransparent)
				  break;
			  case LOAD_CMAP:
				  ANTIALIASING(16, CM, useMask, semiTransparent)
				  break;
			  }
		  case 3: switch(imageType) {
			  case LOAD_TRUE_COL:
				  ANTIALIASING(8, TC, useMask, semiTransparent)
				  break;
			  case LOAD_CMAP:
				  ANTIALIASING(8, CM, useMask, semiTransparent)
				  break;
			  }
		  case 4: switch(imageType) {
			  case LOAD_TRUE_COL:
				  ANTIALIASING(32, TC, useMask, semiTransparent)
				  break;
			  case LOAD_CMAP:
				  ANTIALIASING(32, CM, useMask, semiTransparent)
				  break;
			  }
	  }
  }
 else
  for (y=0; y<height; y++)
   {
    if (mirrory)    
     src = *(line + (imgHei-(int)((sy+y)/scaleY)+ny) % imageHeight);
    else
     src = *(line + ((int)((sy+y)/scaleY) % imageHeight));

    if (useMask)
     srcMask = mask->scanLine((int)((sy+y)/scaleY)%mask->height());

    dst32 = (QRgb*)(data+(y+dsty)*bpl+d->bytesPerPixel*(dstx));
    dst8  = (uchar *)(data + (y+dsty)*bpl + d->bytesPerPixel*dstx);
    dst16 = (unsigned short *) (data+(y+dsty)*bpl + d->bytesPerPixel*dstx);
    transProtTableY = (unsigned char *)(transProtTable->scanLine(y+dsty));
    srcXptr = srcx;
    xmPtr = xm;

    switch (d->bytesPerPixel)
     {
      case 2:
       if (imageType==LOAD_TRUE_COL)
       	{
       	 if (useMask)
           for (x=0; x<width; x++, srcXptr++, xmPtr++)
            if (srcMask[*xmPtr>>3]&(1<<(*xmPtr&7)))
             WRITE_COLOR_16(src[*srcXptr], src[*srcXptr+1], src[*srcXptr+2], semiTransparent)
            else
             dst16++;
       	 else
          for (x=0; x<width; x++, srcXptr++)
	   WRITE_COLOR_16(src[*srcXptr], src[*srcXptr+1], src[*srcXptr+2], semiTransparent)
         }
	else /* only (imageType == LOAD_CMAP) is possible */
         {
       	  if (useMask)
           for (x=dstx; x<(width+dstx); x++, srcXptr++, xmPtr++)
            if ((srcMask[*xmPtr>>3]&(1<<(*xmPtr&7))) && (!transColors[src[*srcXptr]]) && !(transProtTableY[x>>3]&(1<<(x&7))))
             {
              unsigned short color = cmap16[src[*srcXptr]];
              int red = (color&0xf800)>>8;
              int green = (color&0x07e0)>>3;
              int blue = (color&0x001f)<<3;

              WRITE_COLOR_16(red, green, blue, semiTransparent)
             }
	    else
             dst16++;
       	  else
           for (x=dstx; x<(width+dstx); x++, srcXptr++)
            if (!(transColors[src[*srcXptr]] || (transProtTableY[x>>3]&(1<<(x&7))) ))
             {
              unsigned short color = cmap16[src[*srcXptr]];
              int red = (color&0xf800)>>8;
              int green = (color&0x07e0)>>3;
              int blue = (color&0x001f)<<3;

              WRITE_COLOR_16(red, green, blue, semiTransparent)
             }
            else
             dst16++;
       	 }
       	break;

       case 3:
        if (imageType == LOAD_TRUE_COL)
         {
          if (useMask)
           for (x=0; x<width; x++, srcXptr++, xmPtr++)
            if (srcMask[*xmPtr>>3]&(1<<(*xmPtr&7)))
             WRITE_COLOR_8(src[*srcXptr], src[*srcXptr+1], src[*srcXptr+2], semiTransparent)
            else
             { dst8++; dst8++; dst8++; }
          else
           for (x=0; x<width; x++)
            WRITE_COLOR_8(src[*srcXptr], src[*srcXptr+1], src[*srcXptr+2], semiTransparent)
         }
	else /* only (imageType == LOAD_CMAP) is possible */
         {
          if (useMask)
           for (x=dstx; x<(width+dstx); x++, srcXptr++, xmPtr++)
            if ((srcMask[*xmPtr>>3]&(1<<(*xmPtr&7))) && (!transColors[src[*srcXptr]]) && !(transProtTableY[x>>3]&(1<<(x&7))))
             {
              cmap24ptr = &cmap24[src[*srcXptr]*3];
              WRITE_COLOR_8(cmap24ptr[0], cmap24ptr[1], cmap24ptr[2], semiTransparent)
             }
            else
             { dst8++; dst8++; dst8++; }
           else
            for (x=dstx; x<(width+dstx); x++, srcXptr++)
             if (!(transColors[src[*srcXptr]] || (transProtTableY[x>>3]&(1<<(x&7))) ))
              {
               cmap24ptr = &cmap24[src[*srcXptr]*3];
               WRITE_COLOR_8(cmap24ptr[0], cmap24ptr[1], cmap24ptr[2], semiTransparent)
              }
             else
              { dst8++; dst8++; dst8++; }
          }
       break;

      case 4:
       if  (imageType == LOAD_TRUE_COL)
       	{
         if (useMask)
	  {
	   if (semiTransparent)
            for (x=0; x<width; x++, srcXptr++, xmPtr++)
             if (srcMask[*xmPtr>>3]&(1<<(*xmPtr&7)))
              WRITE_COLOR_32_SEMI_TRANSPARENT(src[*srcXptr], src[*srcXptr+1], src[*srcXptr+2])
             else
              dst32++;
	   else
            for (x=0; x<width; x++, srcXptr++, xmPtr++)
             if (srcMask[*xmPtr>>3]&(1<<(*xmPtr&7)))
              *dst32++ = qRgb(src[*srcXptr], src[*srcXptr+1], src[*srcXptr+2]);
             else
              dst32++;
	  }
         else
	  {
	   if (semiTransparent)
            for (x=0; x<width; x++, srcXptr++)
             WRITE_COLOR_32_SEMI_TRANSPARENT(src[*srcXptr], src[*srcXptr+1], src[*srcXptr+2])
	   else
            for (x=0; x<width; x++, srcXptr++)
             *dst32++ = qRgb(src[*srcXptr], src[*srcXptr+1], src[*srcXptr+2]);
	  }
	}
       else /* only (imageType == LOAD_CMAP) is possible */
       	{
         QRgb color; /* allocate it here, so it is not allocated in tightest inner loop, which is called milions of times! */

         if (useMask)
	   {
	    if (semiTransparent)
             for (x=dstx; x<(width+dstx); x++, srcXptr++, xmPtr++)
              if ((srcMask[*xmPtr>>3]&(1<<(*xmPtr&7))) && (!transColors[src[*srcXptr]]) && !(transProtTableY[x>>3]&(1<<(x&7))) )
               {
                color = clut[src[*srcXptr]];
          	WRITE_COLOR_32_SEMI_TRANSPARENT(qRed(color), qGreen(color), qBlue(color))
               }
              else
               dst32++;
	    else
             for (x=dstx; x<(width+dstx); x++, srcXptr++, xmPtr++)
              if ((srcMask[*xmPtr>>3]&(1<<(*xmPtr&7))) && (!transColors[src[*srcXptr]]) && !(transProtTableY[x>>3]&(1<<(x&7))) )
	       *dst32++ = clut[src[*srcXptr]];
              else
               dst32++;
	   }
          else
           {
            if (haveTransparentColors)
	     {
	      if (semiTransparent)
               for (x=dstx; x<(width+dstx); x++, srcXptr++)
                if (!(transColors[src[*srcXptr]] || (transProtTableY[x>>3]&(1<<(x&7))) ))
		 {
		  color = clut[src[*srcXptr]];
		  WRITE_COLOR_32_SEMI_TRANSPARENT(qRed(color), qGreen(color), qBlue(color))
		 }
                else
         	 dst32++;
	      else
               for (x=dstx; x<(width+dstx); x++, srcXptr++)
                if (!(transColors[src[*srcXptr]] || (transProtTableY[x>>3]&(1<<(x&7))) ))
		 *dst32++ = clut[src[*srcXptr]];
                else
         	 dst32++;
             }
            else
	     {
              if (semiTransparent)
               for (x=dstx; x<(width+dstx); x++, srcXptr++)
                if (!(transProtTableY[x>>3]&(1<<(x&7))))
		 {
		  color = clut[src[*srcXptr]];
		  WRITE_COLOR_32_SEMI_TRANSPARENT(qRed(color), qGreen(color), qBlue(color))
		 }
                else
         	 dst32++;
	      else
               for (x=dstx; x<(width+dstx); x++, srcXptr++)
                if (!(transProtTableY[x>>3]&(1<<(x&7))))
		 *dst32++ = clut[src[*srcXptr]];
                else
         	 dst32++;
	     }
           }
        }
       break;
      } /* end big switch */

     /* set protected bits for this line */
     if ((imageType == LOAD_CMAP) && haveProtectedColors )
      {
       srcXptr = srcx;
       for (x=dstx; x<(width+dstx); x++)
        if (protectColors[src[*srcXptr++]])
         transProtTableY[x>>3] |= 1<<(x&7);
      }
   }
 free(srcx);
 free(xm);
}

#ifdef HAVE_MITSHM

void ApaintIO::destroyShmSegment()
{
 if (!ApaintConfig::getUseSharedMemory())
  return;
 XShmDetach(qt_xdisplay(), d->shminfo);
 shmdt(d->shminfo->shmaddr);
     shmctl(d->shminfo->shmid, IPC_RMID, 0);
 d->shmsize = 0;
}


void ApaintIO::createShmSegment(int size)
{
 if (!ApaintConfig::getUseSharedMemory())
  {
   m_bShm = false;
   return; 
  }
 d->shminfo->shmid = shmget(IPC_PRIVATE, size, IPC_CREAT|0600);
 if (d->shminfo->shmid < 0)
  {
     qWarning("Could not get shared memory segment.\n");
     m_bShm = false;
     return;
  }

 d->shminfo->shmaddr = (char *) shmat(d->shminfo->shmid, 0, 0);
 if (d->shminfo->shmaddr < 0)
  {
     qWarning("Could not attach shared memory segment.\n");
     m_bShm = false;
     shmctl(d->shminfo->shmid, IPC_RMID, 0);
     return;
  }

 d->shminfo->readOnly = false;
 if (!XShmAttach(qt_xdisplay(), d->shminfo))
  {
     qWarning("X-Server could not attach shared memory segment.\n");
     m_bShm = false;
     shmdt(d->shminfo->shmaddr);
     shmctl(d->shminfo->shmid, IPC_RMID, 0);
     return;
  }

 d->shmsize = size;
 XSync(qt_xdisplay(), false);
}


#else

void ApaintIO::createShmSegment(int) {}
void ApaintIO::destroyShmSegment() {}

#endif // HAVE_MITSHM
