/***************************************************************************
                          multiplepastedlg.cpp  -  description
                             -------------------
    begin                : Fri Aug 29 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dialogLayerRepeat.h"
#include <qgroupbox.h>
#include <qbuttongroup.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qbitmap.h>
#include <qpainter.h>
#include <qcombobox.h>
#include <qclipboard.h>
#include "apaintconfig.h"

extern char *getCommonPNG(const char *pixName);

MultiplePasteDlg::MultiplePasteDlg(QWidget *parent, const char *name, bool modal, ApaintDoc *pdoc): MultiplePasteDialogUI(parent, name, modal)
{
  validLayer = false;
  dlgOn = false;
  previewOn = false;
  setCaption(ApaintConfig::getText(99));
  bgRepMode->setTitle(ApaintConfig::getText(142));
  bgDistanceMode->setTitle(ApaintConfig::getText(617));
  gbHorizontal->setTitle(ApaintConfig::getText(46));
  lbSelWidth->setText(ApaintConfig::getText(WIDTH_MSG)+":");
  lbPasteTimes->setText(ApaintConfig::getText(101)+":");
  bgOffHor->setTitle(ApaintConfig::getText(100));
  lbOffHor->setText(ApaintConfig::getText(147));
  cbOffWid->setCurrentItem(0);
  cbOffWid->setCurrentText(ApaintConfig::getText(ABSOLUTE_MSG));
  lbDistanceHor->setText(ApaintConfig::getText(173)+":");
  bgAlignHor->setTitle(ApaintConfig::getText(174));
  gbVertical->setTitle(ApaintConfig::getText(47));
  lbSelHeight->setText(ApaintConfig::getText(HEIGHT_MSG)+":");
  lbPasteTimes_2->setText(ApaintConfig::getText(101)+":");
  bgOffVert->setTitle(ApaintConfig::getText(100));
  cbOffHei->setCurrentItem(0);
  cbOffHei->setCurrentText(ApaintConfig::getText(ABSOLUTE_MSG));
  lbOffVert->setText(ApaintConfig::getText(147));
  lbDistanceVert->setText(ApaintConfig::getText(173)+":");
  bgAlignVert->setTitle(ApaintConfig::getText(174));
  btnOK->setText(ApaintConfig::getText(OK_MSG));
  btnOK->setDefault(false);
  btnPreview->setText(ApaintConfig::getText(PREVIEW_MSG));
  btnClose->setText(ApaintConfig::getText(86));

  /* there must be a better way to reference child widgets */
  bgRepMode->find(0)->setPixmap(QPixmap(getCommonPNG("repeat-block")));
  bgRepMode->find(1)->setPixmap(QPixmap(getCommonPNG("repeat-brick")));
  bgRepMode->find(2)->setPixmap(QPixmap(getCommonPNG("repeat-drop")));

  bgDistanceMode->find(0)->setPixmap(QPixmap(getCommonPNG("motif-distance1")));
  bgDistanceMode->find(1)->setPixmap(QPixmap(getCommonPNG("motif-distance2")));

  bgAlignHor->find(0)->setPixmap(QPixmap(getCommonPNG("motif-horiz-left")));
  bgAlignHor->find(1)->setPixmap(QPixmap(getCommonPNG("motif-horiz-center")));
  bgAlignHor->find(2)->setPixmap(QPixmap(getCommonPNG("motif-horiz-right")));

  bgAlignVert->find(0)->setPixmap(QPixmap(getCommonPNG("motif-vert-up")));
  bgAlignVert->find(1)->setPixmap(QPixmap(getCommonPNG("motif-vert-center")));
  bgAlignVert->find(2)->setPixmap(QPixmap(getCommonPNG("motif-vert-down")));

  distanceWid = distanceHei = 0;
  setDistanceHor(0);
  setDistanceVert(0);
  setRepMode(Brick);
  setDistanceMode(Alignment);
  Q_CHECK_PTR(pdoc);
  doc = pdoc;
  connect(bgRepMode, SIGNAL(clicked(int)), this, SLOT(setRepMode(int)));
  connect(bgDistanceMode, SIGNAL(clicked(int)), this, SLOT(setDistanceMode(int)));
  connect(sbDistanceHor, SIGNAL(valueChanged(int)), this, SLOT(distHorChanged(int)));
  connect(sbDistanceVert, SIGNAL(valueChanged(int)), this, SLOT(distVertChanged(int)));
  connect(sbPasteTimesWid, SIGNAL(valueChanged(int)), this, SLOT(pasteTimes(int)));
  connect(sbPasteTimesHei, SIGNAL(valueChanged(int)), this, SLOT(pasteTimes(int)));  
  connect(btnOK, SIGNAL(clicked()), this, SLOT(btnOkClicked()));
  connect(btnPreview, SIGNAL(clicked()), this, SLOT(btnPreviewClicked()));
  connect(btnClose, SIGNAL(clicked()), this, SLOT(btnCloseClicked()));
  connect(bgAlignHor, SIGNAL(clicked(int)), this, SLOT(setAlignHorMode(int)));
  connect(bgAlignVert, SIGNAL(clicked(int)), this, SLOT(setAlignVertMode(int)));
  connect(cbOffWid, SIGNAL(activated(int)), this, SLOT(setOffsetX(int)));
  connect(cbOffHei, SIGNAL(activated(int)), this, SLOT(setOffsetY(int)));
  connect(sbOffsetHor, SIGNAL(valueChanged(int)), this, SLOT(offsetChanged(int)));
  connect(sbOffsetVert, SIGNAL(valueChanged(int)), this, SLOT(offsetChanged(int)));
  connect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer *)));
  connect(doc, SIGNAL(transProtColorsChanged()), this, SLOT(doUpdate()));
}

MultiplePasteDlg::~MultiplePasteDlg()
{
  
}

void MultiplePasteDlg::layerChangedSlot(Layer *layer)
{
  validLayer = getData(layer);
  setupDlg();
}

void MultiplePasteDlg::offsetChanged(int val)
{
  doUpdate();
}

void MultiplePasteDlg::pasteTimes(int val)
{
  doUpdate();
}

void MultiplePasteDlg::setOffsetX(int indx)
{
 int val; 
 if (!indx)
 {
   sbOffsetHor->setEnabled(true);
   return;
 }
 val = imgData.width()/(indx+1);
 sbOffsetHor->setValue(val);
 sbOffsetHor->setEnabled(false);
 doUpdate();
}

void MultiplePasteDlg::setOffsetY(int indx)
{
 int val; 
 if (!indx)
 {
   sbOffsetVert->setEnabled(true);
   return;
 }
 val = imgData.height()/(indx+1);
 sbOffsetVert->setValue(val);
 sbOffsetVert->setEnabled(false);
 doUpdate();
}

void MultiplePasteDlg::setRepMode(int mode)
{
  rmode = (RepeatMode) mode;
  bgOffHor->setEnabled(false);
  bgOffVert->setEnabled(false);
  if (mode == HorMove)
   bgOffHor->setEnabled(true);
  else if (mode == VertMove)
   bgOffVert->setEnabled(true);

  doUpdate();  
}

void MultiplePasteDlg::setDistanceMode(int mode)
{
  bool savePreview = previewOn;
  previewOn = false;
  dmode = (DistanceMode)mode;
  setDistanceMinMax();
  if (mode == Alignment)
  {
   bgAlignHor->setEnabled(true);
   bgAlignVert->setEnabled(true);
   setDistanceHor(distanceWid);
   setDistanceVert(distanceHei);
  }
  else
  {
   bgAlignHor->setEnabled(false);
   bgAlignVert->setEnabled(false);
   setDistanceHor(0);
   setDistanceVert(0);
  }
  previewOn = savePreview;
  doUpdate();
}

void MultiplePasteDlg::distHorChanged(int val)
{
  distanceWid = val;
  if (dmode == Alignment )
  {
    if (distanceWid<imgData.width())
     bgAlignHor->setEnabled(false);
    else
     bgAlignHor->setEnabled(true);
  }
  doUpdate();
}

void MultiplePasteDlg::distVertChanged(int val)
{
  distanceHei = val;
  if (dmode == Alignment )
  {
    if (distanceHei<imgData.height())
     bgAlignVert->setEnabled(false);
    else
     bgAlignVert->setEnabled(true);
  }
  doUpdate();
}

void MultiplePasteDlg::setAlignHorMode(int mode)
{
  alignHorMode = (AlignmentModeHor) mode;
  doUpdate();
}

void MultiplePasteDlg::setAlignVertMode(int mode)
{
  alignVertMode = (AlignmentModeVert) mode;
  doUpdate();
}


void MultiplePasteDlg::setDistanceHor(int val)
{
  sbDistanceHor->setValue(val);
}

void MultiplePasteDlg::setDistanceVert(int val)
{
  sbDistanceVert->setValue(val);
}

void MultiplePasteDlg::btnCloseClicked()
{
  close();
}

void MultiplePasteDlg::btnOkClicked()
{
  if (!previewOn)
  {
    doc->putUndoStack(ChangeLayerForm);
    doAction();
  }
  else
   doUndo(false);
   
  close();
}

void MultiplePasteDlg::btnPreviewClicked()
{
  if (!previewOn)
  {
    previewOn = true;
    btnPreview->setText(ApaintConfig::getText(UNDO_MSG));
    doc->putUndoStack(ChangeLayerForm);
    doAction();
  }
  else
   doUndo(true);
}

void MultiplePasteDlg::doRestore()
{
  disconnect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer *)));
  QRect oldRec = doc->getLayerPos();
  doc->layerSetNewData(imgData, &maskData);
  doc->refreshLayer(oldRec, doc->getLayerPos(), 3);
  connect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer *)));

}

void MultiplePasteDlg::doUndo(bool makeUndo)
{
 if (previewOn)
 {
   btnPreview->setText(ApaintConfig::getText(PREVIEW_MSG));
   previewOn = false;
   if (makeUndo)
   {
    doRestore();
   }
  }
}

void MultiplePasteDlg::doUpdate()
{
  if (previewOn)
   doAction();
}

void MultiplePasteDlg::doAction()
{
  ApaintImage dimg;
  QImage mask;
  int i, j, offx, offy, wid, hei, imgWid, imgHei, dstx, dsty, srcWid, srcHei,doffx, doffy;
  int distx, disty, distxLeft, distxRight, distyTop, distyBottom, alignWid, alignHei;
  QRect oldRec;
  
  if (!validLayer) return;
  
  Layer *selLayer = doc->getLayer();
  if (!selLayer) return;
  
  distxLeft = distxRight = distyTop = distyBottom = 0;
  didPaste = true;
  imgWid = imgData.width();
  imgHei = imgData.height();
  alignWid = distx = getDistanceX();
  alignHei = disty = getDistanceY();
  if (dmode == Alignment )
  {
    distx-=imgData.width();
    disty-=imgData.height();
    if (alignWid>=imgWid)
    {
      switch (alignHorMode)
      {
        case LeftHor:
          distxLeft = 0;
          distxRight = distx;
        break;
        case RightHor:
          distxLeft = distx;
          distxRight = 0;
        break;
        case MidHor:
          distxLeft = distx/2;
          distxRight = distx/2;
        break;
      }
    }
    else
    {
       distxLeft = 0;
       distxRight = distx;
    }
    
    if (alignHei>=imgHei)
    {
      switch (alignVertMode)
      {
        case TopVert:
          distyTop = 0;
          distyBottom = disty;
        break;
        case BottomVert:
          distyTop = disty;
          distyBottom = 0;
        break;
        case MidVert:
          distyTop = disty/2;
          distyBottom = disty/2;
        break;
      }
    }
    else
    {
      distyTop = 0;
      distyBottom = disty; 
    }
  }
  else
  {
    distxLeft = 0;
    distxRight = distx;
    distyTop = 0;
    distyBottom = disty; 
  }
  if (distx<0)
    wid = (getPasteTimesX()-1)*(distx+imgData.width())+imgData.width();
  else
    wid = getPasteTimesX()*(distx+imgData.width());
  if (disty<0)
    hei = (getPasteTimesY()-1)*(disty+imgData.height())+imgData.height();
  else
    hei = getPasteTimesY()*(disty+imgData.height());
  offx = getOffsetX();
  offy = getOffsetY();
    

  dimg.newImage(wid, hei, imgData.depth());
  dimg.setColorTable(imgData.colorTable(), imgData.numColors());
  /*set final mask*/
  mask = QImage(wid, hei, 1, 2, QImage::LittleEndian);
  mask.fill(0);
  
  if (rmode == Brick || rmode == HorMove)
  {
    if (rmode == Brick) offx = 0;
    
    for (i=0; i<getPasteTimesY(); i++)
    {
      /*draw first interrupted part*/
     doffx = (i*offx)%imgWid;
     if (doffx)
     {
       bitBlt(&dimg, 0, distyTop+i*(imgHei+distyTop+distyBottom), &imgData, imgWid-doffx, 0, doffx, imgHei,
              true, true, &maskData, 0, selLayer->getTransColors());
       copy1BitImage(mask, 0, distyTop+i*(imgHei+distyTop+distyBottom),
                    maskData, imgWid-doffx, 0, doffx, imgHei, true, &imgData, selLayer->getTransColors());
       /*must add right border*/
       doffx+=distxRight;
     }
     /*draw the rest*/
     for (j=0; j<getPasteTimesX(); j++)
     {
       dstx = doffx+distxLeft+j*(imgWid+distxLeft+distxRight);
       srcWid = imgWid;
       if (dstx+srcWid>wid)
         srcWid = wid-distxLeft-distxRight;
           
       bitBlt(&dimg, dstx, distyTop+i*(imgHei+distyTop+distyBottom), &imgData, 0, 0, srcWid, imgHei,
              true, true, &maskData, 0, selLayer->getTransColors());
       copy1BitImage(mask, dstx, distyTop+i*(imgHei+distyTop+distyBottom),
                     maskData, 0, 0, srcWid, imgHei, true, &imgData, selLayer->getTransColors());

     }
    }
  }
  else if (rmode == VertMove)
  {
    for (j=0; j<getPasteTimesX(); j++)
    {
     doffy = (j*offy)%imgHei;
     if (doffy)
     {
       bitBlt(&dimg, distxLeft+j*(imgWid+distxLeft+distxRight), 0, &imgData, 0, imgHei-doffy, imgWid, doffy,
              true, true, &maskData, 0, selLayer->getTransColors());
       copy1BitImage(mask, distxLeft+j*(imgWid+distxLeft+distxRight), 0,
                    maskData, 0, imgHei-doffy, imgWid, doffy, true, &imgData, selLayer->getTransColors());
       doffy+=distyBottom;
     }
     for (i=0; i<getPasteTimesY(); i++)     
     {
       dsty = doffy+distyTop+i*(imgHei+distyTop+distyBottom);
       srcHei = imgHei;
       if (dsty+srcHei>hei)
        srcHei = hei - distyTop - distyBottom;

       bitBlt(&dimg, distxLeft+j*(imgWid+distxLeft+distxRight), dsty, &imgData, 0, 0, imgWid, srcHei,
              true, true, &maskData, 0, selLayer->getTransColors());
       copy1BitImage(mask, distxLeft+j*(imgWid+distxLeft+distxRight), dsty,
                    maskData, 0, 0, imgWid, srcHei, true, &imgData, selLayer->getTransColors());
     }
    }
  }

  disconnect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer *)));  
  selLayer->setGeometry(RectangleG);
  oldRec = selLayer->layerPos();
  doc->layerSetNewData(dimg, &mask);
  doc->refreshLayer(oldRec, doc->getLayerPos(), 3);
  connect(doc, SIGNAL(layerChanged(Layer *)), this, SLOT(layerChangedSlot(Layer *)));
}

void MultiplePasteDlg::setDistanceMinMax()
{
 if (imgData.isNull()) return;
 QRect pos = doc->getMainLayer()->layerPos();
 if (dmode == Alignment)
 {
  sbDistanceHor->setMinValue(0);
  sbDistanceVert->setMinValue(0); 
  sbDistanceHor->setMaxValue(pos.width());
  sbDistanceVert->setMaxValue(pos.height());
  sbDistanceHor->setValue(imgData.width());
  sbDistanceVert->setValue(imgData.height());
 }
 else
 {
  sbDistanceHor->setMinValue(0);
  sbDistanceVert->setMinValue(0);
  sbDistanceHor->setMaxValue(pos.width());
  sbDistanceVert->setMaxValue(pos.height()); 
 }
}

void  MultiplePasteDlg::setupDlg()
{
 if (!validLayer)
 {
  lbSelWidthVal->setText(QString("0"));
  lbSelHeightVal->setText(QString("0"));
  btnOK->setEnabled(false);
  btnPreview->setEnabled(false);
 }
 else
 {   
  lbSelWidthVal->setText(QString("%1").arg(imgData.width()));
  lbSelHeightVal->setText(QString("%1").arg(imgData.height()));
  setDistanceMinMax();
  setOffsetX(cbOffWid->currentItem());
  setOffsetY(cbOffHei->currentItem());
  btnOK->setEnabled(true);
  btnPreview->setEnabled(true);
 }
 didPaste = false;
}


bool MultiplePasteDlg::getData(Layer *layer)
{
   if (!dlgOn) return false;
   doUndo(false);
   if (!layer) return false;
   if (layer->getType() == MainLayer)
   {
     return false;    
   }
   
   imgData = layer->imageBuf;
   if (layer->usingMask())
   {
     maskData = layer->getMask();
   }
   else
   { 
     maskData = QImage(imgData.width(), imgData.height(), 1, 2, QImage::LittleEndian);
     maskData.fill(1);
   }   
   return true;
}

void MultiplePasteDlg::showDlg()
{
  dlgOn = true;
  doc->protectLayer(true);
  layerChangedSlot(doc->getLayer());
  show();
}

void MultiplePasteDlg::doOnClose()
{
  dlgOn = false;
  doc->protectLayer(false);
  doUndo(true); 
}

void MultiplePasteDlg::closeEvent(QCloseEvent *e)
{
  doOnClose();
  MultiplePasteDialogUI::closeEvent(e);
}

void MultiplePasteDlg::hideEvent(QHideEvent *e)
{
  doOnClose();
  MultiplePasteDialogUI::hideEvent(e);
}
