/***************************************************************************
                          layermovetool.cpp  -  description
                             -------------------
    begin                : Thu Jan 22 2004
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "toolLayerMove.h"
#include "apaintdoc.h"
#include "apaintUtil.h"
#include "apaint.h"

extern char *getCommonPNG(const char *pixName);

LayerMoveTool::LayerMoveTool(ToolType typ, ApaintDoc *docp, PropObject *propObj):ToolBase(typ, propObj)
{
  setToggleBtn(QPixmap(getCommonPNG("transform-move")), tr("Move Tool"));
  Q_CHECK_PTR(docp);
  doc = docp;
  if (propObj)
  {
    setUsepWidget(true);
  }
  mainCur=QCursor(ArrowCursor);
  allCur = QCursor(Qt::SizeAllCursor);
  normCur = QCursor();
  layerPressed = inSelection = false;
}

LayerMoveTool::~LayerMoveTool()
{
  
}

void LayerMoveTool::begin()
{
  ToolBase::begin();
  layerPressed =inSelection= false;
}



void LayerMoveTool::selectLayer(int x, int y)
{
 for (int lnum=doc->numLayers()-1; lnum >=1 ; lnum--)
 {
   if (doc->getLayer(lnum)->pointVisible(x, y))
   {
    emit putLayerNum(doc->getLayerNum()); /*put on undo stack*/
    doc->setActiveLayer(lnum);
    return;
   }
 }
}

void LayerMoveTool::setupCursor(int x, int y)
{
  bool inReg = doc->isLayerPointVisible(x, y);
  if (inReg && mainCur.handle() != allCur.handle())
  {
    emit setCursor(allCur);
    mainCur = allCur;
  }
  else if (!inReg && mainCur.handle() != normCur.handle())
  {
    emit unsetCursor();
    mainCur = normCur;
  } 
}

void LayerMoveTool::mousePressed(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
  lastx = (int)(e->pos().x()/zoomX);
  lasty = (int)(e->pos().y()/zoomY);
  selectLayer(lastx, lasty);
  Layer *layer = doc->getLayer();
  if (layer != doc->getMainLayer() && doc->isLayerPointVisible(lastx, lasty))
  {
   doc->putUndoStack(ChangeLayerPos);
   layerPressed = true;
   apaint->selectionMoving(true, layer);
  }
  else if (e->button() == RightButton)
  {       
    ApaintDocUtil::destroyCurLayer(doc);
  }
}

void LayerMoveTool::mouseReleased(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
  layerPressed = false;
  apaint->selectionMoving(false, doc->getLayer());
}

void LayerMoveTool::mouseMoved(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
  drawPointer(pc, e->x(), e->y());
   
  int x = (int)(e->pos().x()/zoomX);
  int y = (int)(e->pos().y()/zoomY);
  
  setupCursor(x, y);

  drawPointer(pc, e->x(), e->y());
  
  if (layerPressed)
    ApaintDocUtil::moveCurLayer(doc, lastx - x, lasty - y, QMAX(2, (int)(zoomX/2)));

  drawPointer(pc, e->x(), e->y());

  lastx = x;
  lasty = y; 
}

void LayerMoveTool::keyPress(QKeyEvent *e, QPainter *pc, QPainter *pi)
{
  int dx, dy, inc;
  bool keyOK = false;

  dx = 0; dy =0;
  if (e->state() == ShiftButton)
   inc = 10;
  else
   inc = 1;
  if (e->key() == Qt::Key_Left) {dx+=inc; keyOK = true;}
  if (e->key() == Qt::Key_Right) {dx-=inc; keyOK = true;}
  if (e->key() == Qt::Key_Up) {dy+=inc; keyOK = true;}
  if (e->key() == Qt::Key_Down) {dy-=inc; keyOK = true;}

  if (!keyOK) return;

  /* do not move the ground layer */
  if (doc->getLayer() == doc->getMainLayer())
   return;

  ApaintDocUtil::moveCurLayer(doc, dx, dy, QMAX(2, (int)(zoomX/2)));
  lastx+=dx;
  lasty+=dy;
  
}
