/***************************************************************************
                          linetool.h  -  description
                             -------------------
    begin                : Wed Feb 26 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LINETOOL_H
#define LINETOOL_H

#include <cmath>
#include <toolbase.h>
#include <qpainter.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include "apaintdoc.h"

enum LineType {normalLine, rbLine, measure};  

class DrawInRepeat;

class MeasureToolProp: public QWidget {
  Q_OBJECT  // Enable signals and slots
public:
   MeasureToolProp(QWidget *parent, const char *name);
   void measureChanged(int, int, double, double);
   void drawMeasure(ApaintDoc*, QPainter*, int, int, int, int, double, double);
   int getSizeX() { return sbSizeX->value(); }
   int getSizeY() { return sbSizeY->value(); }
   int getValue() { return sbValue->value(); }
   bool doVarColors() {return cbVarColors->isChecked();}
   bool doVarSize() {return cbVarSize->isChecked();}
   bool doGradient() {return cbGradient->isChecked();}
public slots:  // This slots section is public
  void moveLeftClicked();
  void moveRightClicked();
  void moveUpClicked();
  void moveDownClicked();
  void tiltLeftClicked();
  void tiltRightClicked();
  void tiltUpClicked();
  void tiltDownClicked();
  void splitHorizontalClicked();
  void splitVerticalClicked();
  void joinHorizontalClicked();
  void joinVerticalClicked();

private:
  QLabel *point, *cm, *in;
  QPushButton *moveLeft, *moveRight, *moveUp, *moveDown;
  QPushButton *tiltLeft, *tiltRight, *tiltUp, *tiltDown;
  QPushButton *splitHorizontal, *splitVertical, *joinHorizontal, *joinVertical;
  QSpinBox *sbSizeX, *sbSizeY, *sbValue;
  QLabel *lbSizeX, *lbSizeY, *lbValue;
  QCheckBox *cbVarColors, *cbVarSize, *cbGradient;
protected:
  void hideEvent(QHideEvent *);
};

class LineTool : public ToolBase  {

enum DrawType { NormalDraw, FillDraw};

public: 
	LineTool(DrawInRepeat *drawp, ApaintDoc*, ToolType type, LineType typ, PropObject *propObj);
	~LineTool();

 virtual void mousePressed(QMouseEvent *e, QPainter *pc, QPainter *pi);
 virtual void mouseReleased(QMouseEvent *e, QPainter *pc, QPainter *pi);
 virtual void mouseMoved(QMouseEvent *e, QPainter *pc, QPainter *pi);
 virtual void drawPointer(QPainter *p, int x, int y);
 virtual bool doChangeTool(ToolChangeType chgType);
 virtual QPixmap getToolPix();
 virtual void end();
 virtual void begin();
 virtual void showCommonProp(bool enable);
 void afterRefresh(QPainter*);

private:

 void endLineDraw(QPainter *pi, bool useFg);
 
 int sx, sy;
 int lx, ly;
 int mx, my;
 int lineWid;
 bool drawStarted;
 QPointArray polygon;
 int pointsNum;
 QBrush *polygonBrush;
 DrawType drawType;
 LineType ltyp;
 QBrush noBrush, solidBrush, selBrush;
 ButtonState btnPressed;
 DrawInRepeat *draw;
 MeasureToolProp *mProp;
 ApaintDoc *doc;
};

class StraightLineTool : public ToolBase  {

enum DrawType { NormalDraw, FillDraw};

public:
	StraightLineTool(DrawInRepeat *drawp, ApaintDoc*, PropObject *propObj);
	~StraightLineTool();

 virtual void mousePressed(QMouseEvent *e, QPainter *pc, QPainter *pi);
 virtual void mouseReleased(QMouseEvent *e, QPainter *pc, QPainter *pi);
 virtual void mouseMoved(QMouseEvent *e, QPainter *pc, QPainter *pi);
 virtual bool doChangeTool(ToolChangeType chgType);
 virtual QPixmap getToolPix();
 virtual void end();
 virtual void begin();
 virtual void showCommonProp(bool enable);

private:

 void endLineDraw(QPainter *pi, bool useFg);

 bool straightLine;
 bool activeTool;
 int sx, sy;
 int lx, ly;
 int mx, my;
 int lineWid;
 bool drawStarted;
 QPointArray polygon;
 int pointsNum;
 QBrush *polygonBrush;
 DrawType drawType;
 LineType ltyp;
 QBrush noBrush, solidBrush, selBrush;
 ButtonState btnPressed;
 DrawInRepeat *draw;
 MeasureToolProp *mProp;
 ApaintDoc *doc;
};

#endif
