/***************************************************************************
                          toolMagicWand.h  -  description
                             -------------------
    begin                : Mon Aug 2 2004
    copyright            : (C) 2004-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MAGIC_WAND_H
#define MAGIC_WAND_H

#include <toolbase.h>
#include <qimage.h>
#include <qcolor.h>
#include <qpainter.h>
#include "apaintdoc.h"
#include "toolSelect.h"

class DrawInRepeat;
class Layer;
class QVBoxLayout;
class QLabel;
class QHBoxLayout;
class QSpinBox;
class QCheckBox;

class MagicToolProp: public QWidget {
   Q_OBJECT
public:
   MagicToolProp(QWidget *parent, const char *name, SelectProp *selectionProp);
   ~MagicToolProp();

   int getTreshold();
   bool isEightWayOn() const;
   void show();
   void hide();

signals:
  void valueChanged(int val);
  void selectWholeImage(bool toggle);

private:
  SelectProp *selectProp;
  QSpinBox *sbTreshold;
  QCheckBox *cbWholeImage;
  QCheckBox *eightWay;
  QLabel *lbTreshold;
  QVBoxLayout  *vbox;
  QHBoxLayout  *hbox;
};


class MagicWandTool : public ToolBase  {
  Q_OBJECT
public:
	MagicWandTool(DrawInRepeat *drawp, ApaintDoc *docp, ToolType type, PropObject *propObj);
	~MagicWandTool();

  virtual void mousePressed(QMouseEvent *e, QPainter *pc, QPainter *pi);
  virtual void mouseMoved(QMouseEvent *e, QPainter *pc, QPainter *pi);
  virtual void mouseReleased(QMouseEvent *e, QPainter *pc, QPainter *pi);
  virtual void keyPress(QKeyEvent *e, QPainter *pc, QPainter *pi);   
  virtual void begin();
  virtual void end();

signals:
  void setCursor(QCursor &cur);
  void unsetCursor();

private slots:
  void layerChangedSlot(Layer *layer);
  void enableSelectByColor(bool enable);

private:
  // fills img and returns area of the fill
  QRect fillBitArea(int x, int y, QRect pos, ApaintImage& img);
  QRect selectByColor(int x, int y, QRect pos, ApaintImage& img);
  bool validPoint(int xin, int yin, uchar red, uchar green, uchar blue, int treshold, ApaintImage &img);
  void setPixCol(int xin, int yin);
  void createLayer(int x, int y);
  void destroyLayer();
  void beginTool();
  void endTool();
  void setupCursor(int x, int y);
  
  ButtonState btnPressed;
  ApaintDoc *doc;
  DrawInRepeat *draw;
  QImage validMask;
  bool layerCreated;
  bool leftBtnDown, rightBtnDown;
  bool doSelectByColor;
  SelectState state;
  int lastx, lasty, clickx, clicky;
  QCursor allCur, mainCur, normCur;
  MagicToolProp *mwProp;
};

#endif
