/***************************************************************************
                          selecttool.h  -  description
                             -------------------
    begin                : Fri Apr 18 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SELECTTOOL_H
#define SELECTTOOL_H

#include <toolbase.h>
#include <qbitmap.h>
#include <qcheckbox.h>
#include <qlayout.h>
#include "apaintdoc.h"

enum SelectType{Rectangle, Circle};

class QVBoxLayout;
class QLabel;
class QHBoxLayout;
class QSpinBox;
class QCheckBox;

class SelectProp: public QWidget {
   Q_OBJECT
public:
   SelectProp(QWidget *parent, const char *name);
   ~SelectProp();
   bool isTransparentMove() const;
   bool isTransparentPaste() const;
   void setAlignEnabled(bool set);
   void setPosX(int val);
   void setPosY(int val);

private slots:
	void transparentButtonToggled(bool);
	void alignLeftClicked(void);
	void alignRightClicked(void);
	void alignTopClicked(void);
	void alignBottomClicked(void);
	void alignCenterXClicked(void);
	void alignCenterYClicked(void);

protected slots:
   void posXChanged(int val);
   void posYChanged(int val);

private:
  QCheckBox *transparentMove;
  QCheckBox *transparentPaste;
  QPushButton *alignLeft;
  QPushButton *alignRight;
  QPushButton *alignBottom;
  QPushButton *alignTop;
  QPushButton *alignCenterX;
  QPushButton *alignCenterY;
  QLabel *lbPosX, *lbPosY;
  QSpinBox *sbPosX, *sbPosY;
};

class SelectTool : public ToolBase  {
  Q_OBJECT
public: 
	SelectTool(ToolType tooltype, SelectType sel, ApaintDoc *docp, PropObject *propObj);
	~SelectTool();

 virtual bool doChangeTool(ToolChangeType);
 virtual QPixmap getToolPix();
 virtual void mousePressed(QMouseEvent *e, QPainter *pc, QPainter *pi);
 virtual void mouseReleased(QMouseEvent *e, QPainter *pc, QPainter *pi);
 virtual void mouseMoved(QMouseEvent *e, QPainter *pc, QPainter *pi);
 virtual void keyPress(QKeyEvent *e, QPainter *pc, QPainter *pi);
 virtual void beforeRefresh(QPainter *p);
 virtual void afterRefresh(QPainter *p);
 virtual void begin(); 
 virtual void end();

 void endSelection();
 
 
signals:
 void setCursor(QCursor &cur);
 void unsetCursor();
 void showSize(QRect size);
 void clearSize();

public slots:
 void layerChangedSlot(Layer *layer);
 
private:
 void drawScreenRect(QPainter *p, int x1, int y1, int x2, int y2);
 void setArea(bool newArea);
 void doOp(QPainter *pc, int x, int y);
 void selectOp(int x, int y);
 void drawRec(QPainter *pc, int x, int y);
 void beginTool();
 void endTool();
 bool pointInImage(int x, int y);

 SelectState state;
 SelectType selType;
 MousePos mpos;
 bool leftBtnDown, rightBtnDown, areaClick, layerCreated;
 int startx, starty, lastx, lasty;
 ApaintDoc *doc;
 QCursor horCur, verCur, allCur, mainCur;
};

class UnselectTool : public ToolBase {

public:
  UnselectTool(ToolType typ, ApaintDoc *docp);
  ~UnselectTool(){};

   virtual void doAction();

private:

 ApaintDoc *doc;
};  

#endif
