/***************************************************************************
                          spraytool.cpp  -  description
                             -------------------
    begin                : Wed Mar 26 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "toolSpray.h"
#include <stdlib.h>
#include <math.h>
#include <qtimer.h>
#include "apaintdoc.h"
#include "apaintdrawrepeat.h"
#include "apaintconfig.h"
#include "qlabel.h"
#include "apaintUtil.h"

extern char *getCommonPNG(const char *pixName);

#define NUM_PT  30
#define SIZE    30

SprayPropWid::SprayPropWid(QWidget *parent, const char *name):SprayPropWidgetUI(parent, name)
{
  lbNumPoints->setText(ApaintConfig::getText(183));
  lbRad->setText(ApaintConfig::getText(184));
  cbFromCenter->setText(ApaintConfig::getText(781));
  cbFromCenter->setChecked(TRUE);
}

SprayTool::SprayTool(DrawInRepeat *drawp, ToolType type, SprayType styp, ApaintDoc *docp, PropObject *propObj):ToolBase(type, propObj)
{
  drawStart=false;
  cur=QCursor(ArrowCursor);
  stype = styp;
  if (stype == NormalSpray)
    setToggleBtn(QPixmap(getCommonPNG("draw-airbrush")), tr("Spray Tool"));
  else if (stype == MixSpray)
    setToggleBtn(QPixmap(getCommonPNG("draw-smudge")), tr("Spray Tool"));
    
  time= new QTimer(this);
  stype = styp;
  if (stype == NormalSpray)
    connect(time, SIGNAL(timeout()), this ,SLOT(drawPoints()));
  else if (stype == MixSpray)
    connect(time, SIGNAL(timeout()), this ,SLOT(mixPoints()));

  if (propObj)
   {
    setUsepWidget(true);
    prop = (SprayPropWid*) propObj->pWidget();
    prop->setNumPoints(NUM_PT);
    prop->setRadius(SIZE);
   }
  Q_CHECK_PTR(drawp);
  draw = drawp;
  Q_CHECK_PTR(docp);
  doc = docp;
  protCol = 0;
  setAutoScroll(false);
}

SprayTool::~SprayTool()
{
 delete time;
}

void SprayTool::begin()
{
  CommonProp *cp = CommonProp::getObject();
  aspectDrawSetting = cp->doAspectDraw();
  if (aspectDrawSetting)
   cp->enableAspectDrawProp(false);
}

void SprayTool::end()
{
 CommonProp *cp = CommonProp::getObject();
 if (!cp->doAspectDraw() && aspectDrawSetting) 
   cp->enableAspectDrawProp(true); 
}

void SprayTool::drawPoints()
{
  int i, x, y, x1, y1, rx, ry, alfaInt, size, numPts, radius2;
  double alfa, ax, dax, ay, day;
  bool fromCenter = prop->getFromCenter();
  int pointOK = TRUE;
  
  ax = dax = doc->getAspectX();
  ay = day = doc->getAspectY();
  if ((ax == 1) && (ay == 1))
   {
    ax = aspectX;
    ay = aspectY;
   }

  Layer *selLayer = doc->getLayer();
  if (!selLayer) return;
  
  if (!selLayer->layerPos().contains(xm, ym)) return;
  
  QPainter pw(selLayer->getPixmap());
  pw.translate(-selLayer->drawAreaPos().x(), -selLayer->drawAreaPos().y());
  size=prop->getRadius();
  numPts = prop->getNumPoints();
  radius2 = max(1, (size/2)*(size/2));

  for (i=0; i<numPts;)
   {
    if (fromCenter)
     {
      alfaInt=rand()%3600;
      alfa = alfaInt*M_PI/1800;
      rx = ry =(rand()% size)/2;
      rx=(int)(rx / ax);
      ry=(int)(ry / ay);
      x=xm+(int)(rx*sin(alfa));
      y=ym+(int)(ry*cos(alfa));
     }
    else
     {
      rx = (rand()%size) - (size/2);
      ry = (rand()%size) - (size/2);
      pointOK = ((rx*rx+ry*ry)<radius2);
      if (pointOK)
       {
	rx=(int)(rx / ax);
	ry=(int)(ry / ay);      
        x = xm + rx;
        y = ym + ry;
       }
     }
    if (pointOK)
     {
      i++; /* increment loop counter */
      draw->transPoint(x, y, &x1, &y1);
      pw.drawPoint((int)(x1*dax), (int)(y1*day));
      if (pointSize)
       switch (pointSize)
	{
	 case 2:
	 draw->transPoint(x+1, y, &x1, &y1);
	 pw.drawPoint((int)(x1*dax), (int)(y1*day));      
	 break;
	 case 3:
	 draw->transPoint(x, y+1, &x1, &y1);
	 pw.drawPoint((int)(x1*dax), (int)(y1*day));      
	 break;
	 case 4:
	 draw->transPoint(x+1, y, &x1, &y1);
	 pw.drawPoint((int)(x1*dax), (int)(y1*day));      
	 draw->transPoint(x, y+1, &x1, &y1);
	 pw.drawPoint((int)(x1*dax), (int)(y1*day));      
	 draw->transPoint(x+1, y+1, &x1, &y1);
	 pw.drawPoint((int)(x1*dax), (int)(y1*day));      
	 break;
	 case 5:
	 draw->transPoint(x+1, y, &x1, &y1);
	 pw.drawPoint((int)(x1*dax), (int)(y1*day));      
	 draw->transPoint(x, y+1, &x1, &y1);
	 pw.drawPoint((int)(x1*dax), (int)(y1*day));      
	 draw->transPoint(x-1, y, &x1, &y1);
	 pw.drawPoint((int)(x1*dax), (int)(y1*day));      
	 draw->transPoint(x, y-1, &x1, &y1);
	 pw.drawPoint((int)(x1*dax), (int)(y1*day));      
	 break;
	 case 6:
	 draw->transPoint(x+1, y, &x1, &y1);
	 pw.drawPoint((int)(x1*dax), (int)(y1*day));      
	 draw->transPoint(x, y+1, &x1, &y1);
	 pw.drawPoint((int)(x1*dax), (int)(y1*day));      
	 draw->transPoint(x-1, y, &x1, &y1);
	 pw.drawPoint((int)(x1*dax), (int)(y1*day));      
	 draw->transPoint(x, y-1, &x1, &y1);
	 pw.drawPoint((int)(x1*dax), (int)(y1*day));      
	 draw->transPoint(x-1, y-1, &x1, &y1);
	 pw.drawPoint((int)(x1*dax), (int)(y1*day));      
	 draw->transPoint(x+1, y+1, &x1, &y1);
	 pw.drawPoint((int)(x1*dax), (int)(y1*day));      
	 draw->transPoint(x-1, y+1, &x1, &y1);
	 pw.drawPoint((int)(x1*dax), (int)(y1*day));      
	 draw->transPoint(x+1, y-1, &x1, &y1);
	 pw.drawPoint((int)(x1*dax), (int)(y1*day));      
	 break;
	}
      }
   }
  x = xm-size/2;
  y = ym-size/2;
  x1 = xm+size/2;
  y1 = ym+size/2;
  draw->refreshDirect(x, y, x1-x, y1-y, (btnPressed == LeftButton), true ,false);
}

uint SprayTool::getPixel(int x, int y, ApaintImage *img)
{
  if (img->type() == LOAD_CMAP)
      return img->pixelIndex(x,y);
  else if (img->type() == LOAD_TRUE_COL)
      return img->pixel(x, y);

  return 0;
}

void SprayTool::mixPoints()
{
  int i, x, y, x1, y1, x2, y2, rx, ry, alfaInt, size, numPts, radius2;
  double alfa, ax, dax, ay, day;
  uint pixel, pixel1;
  Layer *selLayer = doc->getLayer();
  bool fromCenter = prop->getFromCenter();
  int pointOK = TRUE;

  if (!selLayer) return;
  if (!selLayer->layerPos().contains(xm, ym)) return;

  ax = dax = doc->getAspectX();
  ay = day = doc->getAspectY();
  if ((ax == 1) && (ay == 1))
   {
    ax = aspectX;
    ay = aspectY;
   }
  
  QImage& maskImg = selLayer->getMask();
  bool usingMask = selLayer->usingMask();
  QRect br = selLayer->layerPos();
  
  QPainter pw(selLayer->getPixmap());
  size=prop->getRadius();
  numPts = prop->getNumPoints();
  radius2 = max(1, (size/2)*(size/2));
  
  for (i=0; i<numPts;)
   {
    if (fromCenter)
     {
      alfaInt=rand()%3600;
      alfa = alfaInt*M_PI/1800;

      rx=ry=rand()%size/2;
      rx=(int)(rx/ax);
      ry=(int)(ry/ay);
      x=xm+(int)(rx*sin(alfa))-br.x();
      y=ym+(int)(ry*cos(alfa))-br.y();
     }
    else
     {
      rx = (rand()%size) - (size/2);
      ry = (rand()%size) - (size/2);
      pointOK = ((rx*rx+ry*ry)<radius2);
      if (pointOK)
       {
	rx=(int)(rx / ax);
	ry=(int)(ry / ay);      
        x = xm + rx;
        y = ym + ry;
       }
     }
    if (x<0) x=0;
    if (x>=br.width()) x = br.width()-1;
    if (y<0) y=0;
    if (y>=br.height()) y = br.height()-1;
    draw->transPoint(x, y, &x1, &y1);
    
    if (!Layer::pointVisible(maskImg, x1, y1) && usingMask) continue;
    pixel = getPixel(x1, y1, &selLayer->imageBuf);

    if (fromCenter)
     {
      alfaInt=rand()%3600;
      alfa = alfaInt*M_PI/1800;

      rx=ry=rand()%size/2;
      rx=(int)(rx/ax);
      ry=(int)(ry/ay);
      x=xm+(int)(rx*sin(alfa))-br.x();
      y=ym+(int)(ry*cos(alfa))-br.y();
     }
    else
     {
      rx = (rand()%size) - (size/2);
      ry = (rand()%size) - (size/2);
      pointOK = ((rx*rx+ry*ry)<radius2);
      if (pointOK)
       {
	rx=(int)(rx / ax);
	ry=(int)(ry / ay);      
        x = xm + rx;
        y = ym + ry;
       }
     }

    if (x<0) x=0;
    if (x>=br.width()) x = br.width()-1;
    if (y<0) y=0;
    if (y>=br.height()) y = br.height()-1;
    draw->transPoint(x, y, &x2, &y2);

    if (pointOK)
     {
      i++; /* increment loop counter */
    
      if (!Layer::pointVisible(maskImg, x2, y2) && usingMask) continue;
      pixel1 = getPixel(x2, y2, &selLayer->imageBuf);

      if (selLayer->imageBuf.depth() == 8 && protCol)
       {
	if (!protCol[pixel1] && !protCol[pixel])
	 {
	  selLayer->imageBuf.setPixel(x2, y2, pixel);
	  selLayer->imageBuf.setPixel(x1, y1, pixel1);
	  pw.drawPoint((int)(x1*dax), (int)(y1*day));
	  pw.drawPoint((int)(x2*dax), (int)(y2*day));
	 }
       }
      else
       {
	selLayer->imageBuf.setPixel(x2, y2, pixel);
	selLayer->imageBuf.setPixel(x1, y1, pixel1);
	pw.drawPoint((int)(x1*dax), (int)(y1*day));
	pw.drawPoint((int)(x2*dax), (int)(y2*day));
       }
    }
  }
  x = xm-size/2;
  y = ym-size/2;
  x1 = xm+size/2;
  y1 = ym+size/2;
  draw->refreshImageDraw(x, y, x1-x, y1-y, false);

}

void SprayTool::drawPointer(QPainter *p, int x, int y)
{
  int sx, sy;
  double ax = doc->getAspectX();
  double ay = doc->getAspectY();
  if ((ax == 1) && (ay == 1))
   {
    ax = aspectX;
    ay = aspectY;
   }
  sx = (int)(prop->getRadius()/(2*ax));
  sy = (int)(prop->getRadius()/(2*ay));
  
  p->setRasterOp(NotROP);
  p->drawEllipse((int)(x-sx*zoomX), (int)(y-sy*zoomY), (int)(2*sx*zoomX), (int)(2*sy*zoomY));
}

void SprayTool::mousePressed(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
 if (drawStart) return;
 btnPressed = e->button();
 if (btnPressed == MidButton) return;
 drawStart=true;
 protCol = doc->getLayer()->getProtectColors();
 
 xm=(int)(e->x()/zoomX);
 ym=(int)(e->y()/zoomY);

 time->start(25, false);

 pointSize = min(6, pi->pen().width());
}

void SprayTool::mouseMoved(QMouseEvent *e, QPainter *pc, QPainter *pi)
{  
 if (!drawStart) return;
 time->stop();

 xm=(int)(e->pos().x()/zoomX);
 ym=(int)(e->pos().y()/zoomY);

 if (stype == NormalSpray)
  drawPoints();
 else if (stype == MixSpray)
  mixPoints();
 time->start(50, false);
 
}

void SprayTool::mouseReleased(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
 if (!drawStart) return;
 int x, y, x1, y1;
 
 drawStart=false;

 xm=(int)(e->pos().x()/zoomX);
 ym=(int)(e->pos().y()/zoomX);
 
 time->stop(); 
 x = xm-SIZE/2;
 y = ym-SIZE/2;
 x1 = xm+SIZE/2;
 y1 = ym+SIZE/2;
 
 if (stype == NormalSpray)
  draw->refreshDirect(x, y, x1-x, y1-y, (btnPressed == LeftButton), true, true);
 else
  draw->refreshImageDraw(x, y, x1-x, y1-y, true);
}

