/***************************************************************************
                          toolbase.cpp  -  description
                             -------------------
    begin                : Sun Feb 2 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <math.h>
#include <qpen.h>
#include <qpainter.h>
#include "apaint.h"
#include "toolbase.h"

PropObject::PropObject(QWidget *w)
{
 validShow = true;
 if (!w)
  {
   widgetOK = false;
   propWidget = 0;
  }
 else
  {
   widgetOK = true;
   propWidget = w;
  }
}

void PropObject::show()
{
  if (!widgetOK || !validShow) return;
  propWidget->show();
  emit shown();  
}

void PropObject::hide()
{
  if (!widgetOK) return;
  propWidget->hide();
  emit hidden();
}

void PropObject::setValidShow(bool enable)
{
 validShow = enable;
 if (validShow)
  show();
 else
  hide();
}


ToolBase::ToolBase(ToolType typ, PropObject *propObj)
{
  active=false;
  zoomX=zoomY=invZoomX=invZoomY=oldZoomX=oldZoomY=1;
  pObj =  propObj;
  setAutoScroll(true);
  usepWidget = false;
  type = typ;
  aspectX = aspectY = 1;
  changeType = NoChange;
}

ToolBase::~ToolBase(){
}

PropObject *ToolBase::pObject()
{
  return (pObj);
}

void ToolBase::setPropWindow(PropObject *propObj)
{
 pObj =  propObj;
 usepWidget = true; 
}

void ToolBase::showPropWindow(bool enable)
{
  if (!pObj) return;
  if (enable)
    pObj->show();
  else
    pObj->hide();  
}

void ToolBase::showCommonProp(bool enable)
{
  
}

void ToolBase::enablePropWindow(bool enable)
{
  if (!pObj) return;
  pObj->setValidShow(enable);
}

bool ToolBase::usePropertyWin()
{
 return usepWidget;
}

void ToolBase::setUsepWidget(bool use)
{
 usepWidget=use; 
}

void ToolBase::mousePressed(QMouseEvent *e, QPainter *pc, QPainter *pi)
{
  
}

void ToolBase::mouseReleased(QMouseEvent *e, QPainter *pc, QPainter *pi)
{

}

void ToolBase::mouseMoved(QMouseEvent *e, QPainter *pc, QPainter *pi)
{

}

void ToolBase::mouseDoubleClick(QMouseEvent *e, QPainter *pc, QPainter *pi)
{

}

void ToolBase::keyPress(QKeyEvent *e, QPainter *pc, QPainter *pi)
{
  
}
void ToolBase::keyRelease(QKeyEvent *e, QPainter *pc, QPainter *pi)
{

}

void ToolBase::drawPointer(QPainter *p, int x, int y)
{
 QWidget *widget = (QWidget*)(p->device());

 if (apaint->crossHairCursor())
 {
  QSize size = apaint->getTotalSize();

  if (&(widget->cursor()) != &blankCursor) widget->setCursor(blankCursor);
  p->setRasterOp(XorROP);
  p->setBrush(QColor(qRgb(128,128,255)));
  p->drawRect(0, floor(y/zoomY)*zoomY, ceil((floor(x/zoomX)-1)*zoomX)+1, ceil(zoomY)+1);
  p->drawRect(ceil((floor(x/zoomX)+2)*zoomX), floor(y/zoomY)*zoomY, size.width()-ceil((floor(x/zoomX)+2)*zoomX)+1, ceil(zoomY)+1);
  p->drawRect(ceil(floor(x/zoomX)*zoomX), 0, ceil(zoomX)+1, ceil((floor(y/zoomY)-1)*zoomY)+1);
  p->drawRect(ceil(floor(x/zoomX)*zoomX), ceil((floor(y/zoomY)+2)*zoomY), ceil(zoomX)+1, size.height()-ceil((floor(y/zoomY)+2)*zoomY)+1);
 }
 else if (&(widget->cursor()) != &cur) widget->setCursor(cur);
}

void ToolBase::beforeRefresh(QPainter *p)
{
  
}

void ToolBase::afterRefresh(QPainter *p)
{
  
}

bool ToolBase::doChangeTool(ToolChangeType chgType)
{
  return false;
}

QPixmap ToolBase::getToolPix()
{
  return toolPix;
}

void ToolBase::doAction()
{
  
}

void ToolBase::begin()
{
 active=true;
}

void ToolBase::end()
{
 active=false;
}

void ToolBase::resetTool()
{
  
}

bool ToolBase::isActive()
{
  return active;
}

void ToolBase::activateSlot()
{
  emit activated(this);
}

void ToolBase::activateSlotDouble()
{
  emit activatedDouble(this); 
}

QCursor& ToolBase::getCursor()
{
 return (cur);
}

void ToolBase::setToggleBtn(QPixmap pix, QString text)
{
  toolPix = pix;
  toolName = text;
}

void ToolBase::setToggleBtn(QPixmap pix, QPixmap pix2)
{
  toolPix = pix;
  toolPix2 = pix2; 
}

void ToolBase::setZoom(double zx, double zy)
{
  oldZoomX=zoomX;
  oldZoomY=zoomY;
  zoomX=zx;
  zoomY=zy;
  invZoomX=1/zx;
  invZoomY=1/zy;
}

void ToolBase::setAspectRatio(double ax, double ay)
{
  aspectX = ax;
  aspectY = ay;
}

QRect ToolBase::normalRect(int xs1, int ys1, int xs2, int ys2)
{
  int x1, y1, x2, y2;

  x1=QMIN(xs1, xs2);
  y1=QMIN(ys1, ys2);
  x2=QMAX(xs1, xs2);
  y2=QMAX(ys1, ys2);
  return QRect(x1, y1, x2-x1+1, y2-y1+1);
}

QRect ToolBase::normalQuadRect(int xs1, int ys1, int xs2, int ys2, double aspx, double aspy)
{
 int x1, y1, ax, ay;

  ax = ay = QMAX(QABS(xs2-xs1), QABS(ys2-ys1));
  ax = (int)((double)ax/aspx);
  ay = (int)((double)ay/aspy);
  
  if (xs1>xs2)
    x1 = xs1 - ax;
  else
    x1 = xs1;
  
  if (ys1>ys2)
   y1 = ys1 - ay;
  else
   y1 = ys1;
  return QRect(x1, y1, ax, ay);

}

QRect ToolBase::normalCircle(int xs1, int ys1, int xs2, int ys2, double aspx, double aspy)
{
 int newRx, newRy;
 int relx, rely;
 
 newRx = newRy = (int) sqrt((float)((xs1-xs2)*(xs1-xs2)+(ys1-ys2)*(ys1-ys2)));
 newRx=(int)(newRx/aspx);
 newRy=(int)(newRy/aspy);

 relx=xs1-newRx;
 rely=ys1-newRy;
 return QRect(relx, rely, newRx*2, newRy*2);
}

QRect ToolBase::normalEllipse(int xs1, int ys1, int xs2, int ys2)
{
  int lupx, lupy;

  lupx = xs1-(xs2-xs1);
  lupy = ys1-(ys2-ys1);

  return QRect(lupx, lupy, xs2-lupx, ys2-lupy).normalize();
}
