/***************************************************************************
                          drawtoolbar.cpp  -  description
                             -------------------
    begin                : Tue Feb 25 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "widgetDrawToolbar.h"
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qwindowsstyle.h>
#include <qtooltip.h>

#define NUM_BTNS_COL    5

ToolPushButton::ToolPushButton(QWidget *parent, const char *name):QPushButton(parent, name)
{ 
}

ToolPushButton::~ToolPushButton()
{
}

void ToolPushButton::mouseDoubleClickEvent(QMouseEvent *e)
{
 emit doubleClicked();
}


DrawToolBar::DrawToolBar(QWidget *parent, const char* name, ToolsCtrl *tc):QWidget(parent,name)
{
  curBtn = -1;
  Q_CHECK_PTR(tc);
  tCtrl=tc;
  canChangeTool = true;
  btnBack = QColor(228, 228, 228);
  btnGrp = new QButtonGroup(0, "drawToolBarBtnGroup");
  connect(btnGrp, SIGNAL(clicked(int)), this, SLOT(btnGrpClicked(int)));
}

DrawToolBar::~DrawToolBar()
{
 delete btnGrp;
}

void DrawToolBar::enableToolChange(bool enable)
{
 canChangeTool = enable; 
}

void DrawToolBar::btnGrpClicked(int i)
{
 QButton *btn;
 if (!canChangeTool)
 {
   btn = btnGrp->find(i);
   btn->toggle();
   return;
 }
 ToolBase *tb = toolList.at(i);
 if (tb->getType() == ActionTool)
  return;
 if (curBtn!=-1)
 {
    btn = btnGrp->find(curBtn);
    if (curBtn!=i)
      btnChgColor((ToolPushButton *)btn, false);
    btn->toggle();
 }
 if (curBtn != i)
 {
  curBtn = i;
  btn = btnGrp->find(curBtn);
  btnChgColor((ToolPushButton *)btn, true);
 }
}

void DrawToolBar::btnChgColor(ToolPushButton *btn, bool active)
{
 if (!btn) return;
 if (active)
 {
  btnBack = btn->paletteBackgroundColor();  
  btn->setPaletteBackgroundColor(QColor(210, 200, 10));
 }
 else
  btn->setPaletteBackgroundColor(btnBack);
}

void DrawToolBar::addTool(ToolBase *tool, QString toolTip)
{
  ToolPushButton *tButton= new ToolPushButton(this, "tButton");
  if (tool->getType()!=ActionTool)
    tButton->setToggleButton(true);
  tButton->setPixmap(tool->getToolPix());
  if (toolTip != "")
    QToolTip::add(tButton, toolTip);
  btnList.append(tButton);
  btnGrp->insert(tButton);  
  connect(tButton, SIGNAL(clicked()), tool, SLOT(activateSlot()));
  connect(tool, SIGNAL(activated(QObject *)), this, SLOT(activateTool(QObject *)));
  connect(tButton, SIGNAL(doubleClicked()), tool, SLOT(activateSlotDouble()));
  connect(tool, SIGNAL(activatedDouble(QObject *)), this, SLOT(changeTool(QObject *)));
  toolList.append(tool);
}

void DrawToolBar::endToolAdd()
{
 ToolPushButton *btn;
 int num = 0;
 int col, row, i, j;
  
 num = btnList.count();
 col = NUM_BTNS_COL;
 row = num / col;
 if (num%col>0) row++;
 gridLay = new QGridLayout(this, row, col);
 btn = btnList.first();
 for (i=0; i<row; i++)
  for (j=0; j<col; j++)
   {
    if (btn)
     {
      gridLay->addWidget(btn, i, j);
      btn->setEnabled(false);
     }
    btn = btnList.next();
    if (!btn) continue;
   }
}

void DrawToolBar::resetBtns()
{
 ToolPushButton *pb;
 for (pb = btnList.first(); pb; pb = btnList.next())
  if (pb->state())
  {
   pb->toggle();
   btnChgColor(pb, false);
  }
 curBtn = -1;
}

void DrawToolBar::enableTool(ToolBase *tool, bool enable)
{
 int ret = toolList.find(const_cast<const ToolBase*>(tool));

 if (ret != -1) btnList.at(ret)->setEnabled(enable);
}

void DrawToolBar::changeTool(QObject *sender)
{
  ToolBase *tool;
  ToolBase *tb=(ToolBase *)sender;
  ToolChangeType chgTyp = tb->getChangeType();

  /* do not change tool function on double click of action tools */
  if (tb->getType()==ActionTool)
   return;

  for (int i=0; i<(int)toolList.count(); i++)
   {
    tool = toolList.at(i);
    if (tool->doChangeTool(chgTyp))
     {
      QButton *btn = btnList.at(i);
      btn->setPixmap(tool->getToolPix());
     }
   }
  activateTool(sender);
}

void DrawToolBar::activateTool(QObject *sender)
{
  if (!canChangeTool) return;
  ToolBase *tb=(ToolBase *)sender;
 
  if (tb->getType() != ActionTool)
    tCtrl->unregisterTool();
  if (!tb->isActive())
  {
   tCtrl->registerTool(tb);
  }
}

void DrawToolBar::setTool(ToolBase *tool, bool activate)
{
  ToolBase *tb;
  ToolPushButton *pb;
  int i;  
  
  if (!canChangeTool) return;
  if (activate && !tool->isActive())
   tCtrl->registerTool(tool);
  else if (!activate && tool->isActive())
   tCtrl->unregisterTool();
  else
   return;
  
  for (tb = toolList.first(), pb = btnList.first(), i=0; tb ; tb = toolList.next(), pb = btnList.next(), i++)
   {
    if (tb == tool)
     {
      pb->toggle();
      btnGrpClicked(i);
     }
   }
}

