/***************************************************************************
                          penbrushset.cpp  -  description
                             -------------------
    begin                : Wed Apr 16 2003
    copyright            : (C) 2003-2011 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "widgetPenBrushSet.h"
#include <qpainter.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <qbitmap.h>
#include "apaintconfig.h"
#include "apaintdoc.h"
#include <qdrawutil.h>
#include "dialogColorSelect.h"
#include <qdialog.h>
#include "apaintBrushList.h"
#include "toolMagicWand.h"

extern char *getCommonPNG(const char *pixName);

#define NO_BRUSH      0
#define SOLID_BRUSH   1
#define CROSS_PATTERN 2

#define SIZEX           35
#define SIZEY           25
#define COLOR_MED       381
#define MAX_LINE_WIDTH  500

/*******************ColorChangeBtn*********************************/
ColorChangeBtn::ColorChangeBtn(QWidget *parent, const char *name):QWidget(parent, name)
{
 colDlg = new ArahColorDialog( parent, name);
 connect(colDlg, SIGNAL(colorChg(QRgb)) , this, SLOT(colorChgSlot(QRgb)));
 connect(this, SIGNAL(setCurrentColor(QRgb)), colDlg, SLOT(colorChgSlot(QRgb)));
 arrow = new QPixmap(getCommonPNG("swap-colors"));

 setMinimumSize(sizeHint());
 fg=QBrush(Qt::black, SolidPattern);
 bg=QBrush(Qt::white, SolidPattern);
 selected=Foreground;
 bgNum = fgNum =-1;
 refresh(0);
}

void ColorChangeBtn::refresh(int maxCol)
{
 if (bgNum>=maxCol && fgNum>=maxCol)
  bgNum = fgNum = -1;
 emit fgChanged(fg.color().rgb());
 emit bgChanged(bg.color().rgb());
 emit fgChangedIndx(fgNum);
 emit bgChangedIndx(bgNum);
 repaint(0);
}

void ColorChangeBtn::setBgFgIndxCol(int fgn, QRgb fgc, int bgn, QRgb bgc)
{
 selected = Foreground;
 fg=QBrush(QColor(fgc), SolidPattern);
 fgNum = fgn;
 emit fgChangedIndx(fgNum);

 bg=QBrush(QColor(bgc), SolidPattern);
 bgNum = bgn;
 emit bgChangedIndx(bgNum);
 repaint(0);
}

QSize ColorChangeBtn::sizeHint() const
{
 return(QSize(80, 60));
}

ColorChangeBtn::~ColorChangeBtn()
{
 delete arrow;
}

void ColorChangeBtn::colorChgSlot(QRgb c)
{
  QColor newcol = QColor(c);

  if (newcol.isValid())
   {
    disconnect(this, SIGNAL(setCurrentColor(QRgb)), colDlg, SLOT(colorChgSlot(QRgb)));
    if (bgNum == fgNum && bgNum!=-1 && fgNum!=-1)
     {
      setForeground(newcol);
      setBackground(newcol);
     }
    else if (selected == Foreground)
     setForeground(newcol);
    else
     setBackground(newcol);
    connect(this, SIGNAL(setCurrentColor(QRgb)), colDlg, SLOT(colorChgSlot(QRgb)));
  }
}

void ColorChangeBtn::setSelected(WhichColor c)
{
  selected=c;
}

QColor ColorChangeBtn::foreground() const
{
 return (fg.color());
}

QColor ColorChangeBtn::background() const
{
 return (bg.color());
}

void ColorChangeBtn::setForeground(const QColor &c)
{
 fg=QBrush(c, SolidPattern);
 repaint(0);

 emit fgChanged(c.rgb());
 emit fgChangedIndx(fgNum);
 emit setColorSig(c.rgb());
}

void ColorChangeBtn::setBackground(const QColor &c)
{
 bg=QBrush(c, SolidPattern);
 repaint(0);

 emit bgChanged(c.rgb());
 emit bgChangedIndx(bgNum);
 emit setColorSig(c.rgb());
}

void ColorChangeBtn::setColorSlot(QRgb rgb, int colNum)
{
 if (selected == Foreground)
  {
   fg=QBrush(QColor(rgb), SolidPattern);
   fgNum = colNum;
   emit fgChangedIndx(fgNum);
  }

 if (selected == Background)
  {
   bg=QBrush(QColor(rgb), SolidPattern);
   bgNum = colNum;
   emit bgChangedIndx(bgNum);
  }
  refreshed = false;
  emit setCurrentColor(rgb);
  repaint(0);

 /* force refresh of both foreground and background, as it could be copied from one to another */
 if (fgNum==bgNum)
  {
   WhichColor oldSelected = selected;
   
   selected =  Foreground;
   fg=QBrush(QColor(rgb), SolidPattern);
   emit fgChangedIndx(fgNum);
   refreshed = false;
   emit setCurrentColor(rgb);

   selected = Background;
   bg=QBrush(QColor(rgb), SolidPattern);
   emit bgChangedIndx(bgNum);
   refreshed = false;
   emit setCurrentColor(rgb);

   selected = oldSelected;
   repaint(0);
  }
}

void ColorChangeBtn::setColorSlot(QRgb rgb)
{
  if (selected==Foreground)
  {
   fg=QBrush(QColor(rgb), SolidPattern);
   emit fgChanged(rgb);
   emit fgChangedIndx(fgNum);
  }
  else
  {
   bg=QBrush(QColor(rgb), SolidPattern);
   emit bgChanged(rgb);
   emit bgChangedIndx(bgNum);
  }
  emit setCurrentColor(rgb);
  repaint(0);
}

void ColorChangeBtn::geometry(QRect &fgRect, QRect &bgRect, QRect &colRect)
{
 fgRect = QRect(width()/2-SIZEX/2-7, height()/2-SIZEY/2-7, SIZEX, SIZEY);
 bgRect = QRect(width()/2-SIZEX/2+10, height()/2-SIZEY/2+10, SIZEX, SIZEY);
 colRect = QRect(fgRect.right()+5, fgRect.top(), arrow->width(), arrow->height());
}

void ColorChangeBtn::drawNumber(QPainter *p,  QRect r, int n, WhichColor ct)
{
  QColor col;
  if (ct == Foreground)
   col = fg.color();
  else
   col = bg.color();

  int light = col.red()+col.green()+col.blue();
  if (light<COLOR_MED)
   p->setPen(QPen(white));
  else
   p->setPen(QPen(black));

  p->drawText(r.x(), r.y(), r.width(), r.height(), AlignCenter, QString("%1").arg(n + ApaintConfig::getFirstPaletteColor()));
}

void ColorChangeBtn::paintEvent(QPaintEvent *e)
{
  QRect fgRect, bgRect, colRect;
  QPainter p(this);

  geometry(fgRect, bgRect, colRect);
  qDrawShadeRect(&p, bgRect, colorGroup(), selected == Background, 2, 0, &bg);
  qDrawShadeRect(&p, fgRect, colorGroup(), selected == Foreground, 2, 0, &fg);
  p.setPen(colorGroup().shadow());
  p.drawPixmap(fgRect.right()+5, fgRect.top(), *arrow);
  if (fgNum!=-1 && bgNum!=-1)
   {
    drawNumber(&p, fgRect, fgNum, Foreground);
    drawNumber(&p, bgRect, bgNum, Background);
   }
}

void ColorChangeBtn::doOnClickEvent(QMouseEvent *e, bool dclick)
{
  QRect fgRect, bgRect, colRect;
  QPoint cpos=e->pos();

  geometry(fgRect, bgRect, colRect);
  if (fgRect.contains(cpos))
   {
    selected=Foreground;
    emit setColorSigIndx(fgNum);
    if (dclick)
     colDlg->show();
   }
  else if (bgRect.contains(cpos))
   {
    selected=Background;
    emit setColorSigIndx(bgNum);
    if (dclick)
     colDlg->show();
   }
  else if (colRect.contains(cpos))
   changeFgGb(); 

  repaint(0);
}

void ColorChangeBtn::changeFgGb()
{
  QBrush b;
  b=fg;
  fg=bg;
  bg=b;
  int tmp = fgNum;
  fgNum = bgNum;
  bgNum = tmp;
  emit fgChanged(fg.color().rgb());
  emit bgChanged(bg.color().rgb());
  emit fgChangedIndx(fgNum);
  emit bgChangedIndx(bgNum);
  if(selected == Foreground)
   emit setColorSigIndx(fgNum);
  else
   emit setColorSigIndx(bgNum);
}

void ColorChangeBtn::showColorDialog()
{
  colDlg->show();
}


void ColorChangeBtn::mouseDoubleClickEvent(QMouseEvent *e)
{
  doOnClickEvent(e, true);
}

void ColorChangeBtn::mousePressEvent(QMouseEvent *e)
{
  doOnClickEvent(e, false);
}



/****************************COORDS WIDGET************************************************/

CoordsWidget::CoordsWidget(QWidget *parent, const char *name):QFrame(parent, name)
{
 //setBackgroundMode(Qt::NoBackground);
 setFrameShape(StyledPanel);
 setFrameShadow(Sunken);
 hbx = new QHBoxLayout();
 hby = new QHBoxLayout();
 hbox = new QHBoxLayout();
 vbox = new QVBoxLayout(this, 1, 1, "cordsWidLayout");
  
 lbXtext = new  QLabel(QString(tr("X: ")), this);
 lbXtext->setAlignment(Qt::AlignRight);
 lbYtext = new  QLabel(QString(tr("Y: ")), this);
 lbYtext->setAlignment(Qt::AlignRight);
 lbx = new QLabel(this);
 lby = new QLabel(this);
 lbSelection = new QLabel(this);
 lbSelection->setAlignment(Qt::AlignHCenter);

 hbx->addWidget(lbXtext);
 hbx->addWidget(lbx);
 hby->addWidget(lbYtext);
 hby->addWidget(lby);

 hbox->addLayout(hbx);
 hbox->addLayout(hby);
 
 vbox->addLayout(hbox);
 vbox->addWidget(lbSelection);

 showCoords(0, 0);
}

CoordsWidget::~CoordsWidget()
{

}

/* zzzz */
void CoordsWidget::clearSelection()
{
 lbSelection->setText("");
}

void CoordsWidget::showSelection(QRect size)
{
 int xpos, ypos;

 xpos = size.x()+1;
 ypos = mainSize.height() - size.y() - size.height() + 1;
 
 selSize = QString(tr("%1x%2")).arg(size.width()).arg(size.height());
 lbSelection->setText(selSize);
 selectProp->setPosX(xpos);
 selectProp->setPosY(ypos);
}

void CoordsWidget::showCoords(int x, int y)
{
  if (!mainSize.contains(x, y)) return;
  
  if (x>=0)
    xv.setNum(x+1);
  if (y>=0)
    yv.setNum(mainSize.height()-y);

  lbx->setText(xv);
  lby->setText(yv);
}

void CoordsWidget::setRGB(uchar r, uchar g, uchar b, uchar indx)
{
  if (!indx)
    selSize = QString("(%2, %3, %4)").arg(r, 3).arg(g, 3).arg(b, 3);
  else
    selSize = QString("%1(%2, %3, %4)").arg(indx - 1 + ApaintConfig::getFirstPaletteColor(), 3).arg(r, 3).arg(g, 3).arg(b, 3);
  lbSelection->setText(selSize);
}

void CoordsWidget::setSelectProp(SelectProp *sProp)
{
 selectProp = sProp;
}

/**********************PenBrushSet class**********************************/

PenBrushSet::PenBrushSet(QWidget *parent, const char *name):QFrame(parent, name)
{ 
 vbox = new QVBoxLayout(this);
 hbox = new QHBoxLayout();
 lineSize = new QSpinBox(this, "line size widget");
// lineSize->setFocusPolicy(NoFocus);
 lineSize->setMinValue(1);
 lineSize->setMaxValue(MAX_LINE_WIDTH);
 connect(lineSize, SIGNAL(valueChanged(int)), this, SLOT(lineSizeSlot(int)));
 brushStyle = new  QCheckBox(this);
 brushStyle->setText(ApaintConfig::getText(722));
 brushStyle->setFocusPolicy(NoFocus);
 connect(brushStyle, SIGNAL(clicked()), this, SLOT(brushStyleSlot()));
 sTop = new QSpacerItem( 0, 5, QSizePolicy::Minimum, QSizePolicy::Expanding );
// sMid = new QSpacerItem( 0, 2, QSizePolicy::Minimum, QSizePolicy::Expanding );
 sBot = new QSpacerItem( 0, 5, QSizePolicy::Minimum, QSizePolicy::Expanding );
 sRight = new  QSpacerItem(5, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);

 vbox->addItem(sTop);
 hbox->addWidget(lineSize);
 hbox->addItem(sRight);
 vbox->addLayout(hbox);
 //vbox->addItem(sMid);
 vbox->addWidget(brushStyle);
 vbox->addItem(sBot);

 noB = QBrush(Qt::NoBrush);
 outSolid = QBrush(Qt::SolidPattern);
 outSolid.setColor(white);
}

PenBrushSet::~PenBrushSet()
{

}

/*QSize PenBrushSet::sizeHint() const
{
  return (QSize(70, 60));
}
*/
void PenBrushSet::initPenBrush()
{
  brushStyleSlot();
  lineSizeSlot(0);
}

void PenBrushSet::switchBrushStyle()
{
  brushStyle->setChecked(!brushStyle->isChecked());
  brushStyleSlot();
}

void PenBrushSet::brushStyleSlot()
{
 ApaintBrushList *bList = ApaintBrushList::getObject();
 bList->enableBrush(brushStyle->isChecked());  
}

void PenBrushSet::lineSizeSlot(int indx)
{
 if (indx==1)
  indx=0; 
 QPen p(white, indx);
 p.setCapStyle(Qt::RoundCap);
 emit lineSizeChg(p);  
}

void PenBrushSet::incLineSize()
{
  lineSize->setValue(lineSize->value()+1);
}

void PenBrushSet::decLineSize()
{
  lineSize->setValue(lineSize->value()-1); 
}

int PenBrushSet::getLineSize()
{
  return(lineSize->value()); 
}


/************************COLORUTIL CLASS******************************************/

PenBrushUtil::PenBrushUtil(QWidget *parent, const char *name, ApaintDoc *docp):QWidget(parent, name)
{
 doc = docp;
 ccBtn = new ColorChangeBtn(this, "ColorChangeBtn");
 pbSet = new PenBrushSet(this, "penBrushSet");
 sMid = new QSpacerItem(10, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);
 sLeft = new QSpacerItem(10, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);
 sRight = new QSpacerItem(10, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);
 cwid = new CoordsWidget(this, "coords widget");

 hbl = new QHBoxLayout();
 hbl->addItem(sLeft);
 hbl->addWidget(pbSet);
 hbl->addItem(sMid);
 hbl->addWidget(ccBtn);
 hbl->addItem(sRight);
 hbl2 = new QHBoxLayout();
 hbl2->addWidget(cwid);
 bl = new QVBoxLayout(this);
 bl->addLayout(hbl);
 bl->addLayout(hbl2);

 connect(ccBtn, SIGNAL(setColorSigIndx(int)), this, SIGNAL(setColorSigIndx(int)));
 connect(ccBtn, SIGNAL(setColorSig(QRgb)), this, SLOT(sendColorSig(QRgb)));
 connect(ccBtn, SIGNAL(fgChanged(QRgb)), this, SIGNAL(fgChanged(QRgb)));
 connect(ccBtn, SIGNAL(bgChanged(QRgb)), this, SIGNAL(bgChanged(QRgb)));
 connect(ccBtn, SIGNAL(fgChangedIndx(int)), this, SIGNAL(fgChangedIndx(int)));
 connect(ccBtn, SIGNAL(bgChangedIndx(int)), this, SIGNAL(bgChangedIndx(int)));

 connect(pbSet, SIGNAL(lineSizeChg(const QPen &)), this, SIGNAL(lineSizeChg(const QPen&)));
 connect(pbSet, SIGNAL(brushChg(const QBrush&)), this, SIGNAL(brushChg(const QBrush&)));
}

PenBrushUtil::~PenBrushUtil()
{
}

void PenBrushUtil::putInWidget(QWidget *w)
{
  bl->addWidget(w);
}

void PenBrushUtil::changeFgBg()
{
  ccBtn->changeFgGb();
}

void PenBrushUtil::showColorDlg()
{
  ccBtn->showColorDialog();
}

void PenBrushUtil::switchBrush()
{
  pbSet->switchBrushStyle(); 
}

void PenBrushUtil::colorChgSlot(QRgb col)
{
 ccBtn->colorChgSlot(col);
}

void PenBrushUtil::showCoords(int x, int y)
{
  cwid->showCoords(x, y);
}

void PenBrushUtil::setSelectProp(SelectProp *sProp)
{
 cwid->setSelectProp(sProp);
}

void PenBrushUtil::initPenBrush()
{
  pbSet->initPenBrush();
}

void PenBrushUtil::sendColorSig(QRgb rgb)
{
  emit  setColorSig(rgb, true);
}

void PenBrushUtil::setColorSlot(QRgb rgb)
{
  ccBtn->setColorSlot(rgb);
}

void PenBrushUtil::setColorSlot(QRgb rgb, int i)
{
 ccBtn->setColorSlot(rgb, i);
}

uint PenBrushUtil::foreground() const
{
  QColor c = ccBtn->foreground();
  return (c.rgb());
}

uint PenBrushUtil::background() const
{
  QColor c = ccBtn->background();
  return (c.rgb());
}

void PenBrushUtil::layerChangedSlot(Layer *sel)
{
  /*save mainLayer size*/
//  if (sel->getType() == MainLayer)
//   cwid->setMainSize(sel->layerPos());
  /* must do it every time, or it crashes on undo of crop to selection */
  cwid->setMainSize(doc->getMainLayer()->layerPos());

  if (sel->getImageType() == LOAD_TRUE_COL)
    ccBtn->refresh(0);
  if (sel->getType() == SelectionLayer || sel->getType() == ImageLayer)
   cwid->showSelection(sel->layerPos());
  else
   cwid->clearSelection();
}

void PenBrushUtil::showSizeSlot(QRect pos)
{
  cwid->showSelection(pos);
}

void PenBrushUtil::clearSizeSlot()
{
  cwid->clearSelection();
}

void PenBrushUtil::setBgFgIndxCol(int fgn, QRgb fgc, int bgn, QRgb bgc)
{
  ccBtn->setBgFgIndxCol(fgn, fgc, bgn, bgc);
}

void PenBrushUtil::incLineSize()
{
  pbSet->incLineSize();
}

void PenBrushUtil::decLineSize()
{
  pbSet->decLineSize();
}

int PenBrushUtil::getLineSize()
{
  return(pbSet->getLineSize()); 
}

void PenBrushUtil::setRGB(uchar r, uchar g, uchar b, uchar indx)
{
  cwid->setRGB(r, g, b, indx);
}

