#include "gimp-print.h"
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include "print.h"

static char *dstFile = NULL;
static char *commandStr = NULL;
static int doRemove = FALSE;

static FILE *fout, *fgimp;
static char *fname, *fnamegimp;
static int horDpi;

static printBlock *pBlock=NULL;
static unsigned short *lineToBlock;
static int numPrintBlocks;

static const char *Image_get_appname(stp_image_t *image);
static void Image_progress_conclude(stp_image_t *image);
static void Image_note_progress(stp_image_t *image, double current, double total);
static void Image_progress_init(stp_image_t *image);
static stp_image_status_t Image_get_row(stp_image_t *image, unsigned char *data, int row);
static int Image_height(stp_image_t *image);
static int Image_width(stp_image_t *image);
static int Image_bpp(stp_image_t *image);
static void Image_init(stp_image_t *image);
static stp_image_t theImage =
{
  Image_init,
  /* we do not provide functions for: reset, transpose, hflip, vflip, crop, ccw, cw, 180 */
  NULL,	NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  Image_bpp,
  Image_width,
  Image_height,
  Image_get_row,
  Image_get_appname,
  Image_progress_init,
  Image_note_progress,
  Image_progress_conclude,
  NULL
};

double global_c_level;
double global_c_gamma = 1.0;
double global_m_level;
double global_m_gamma = 1.0;
double global_y_level;
double global_y_gamma = 1.0;
double global_k_gamma = 1.0;
double global_gamma = 1.0;
double ink_limit;
double density;
char *printer;
char *ink_type;
char *stp_resolution;
char *media_type;
char *media_size;
char *dither_algorithm = "Ordered";

int printx, printy, offsetx, offsety;

static void writefunc(void *file, const char *buf, size_t bytes)
{
 FILE *prn = (FILE *)file;
 fwrite(buf, 1, bytes, prn);
}

static int Image_bpp(stp_image_t *image)
{ return(3); }

static int Image_width(stp_image_t *image)
{ return printx; }

static int Image_height(stp_image_t *image)
{ return printy; }

static void Image_init(stp_image_t *image)
{ }

static void Image_progress_init(stp_image_t *image)
{ }

/* progress display */
static void Image_note_progress(stp_image_t *image, double current, double total)
{
// fprintf(stderr, ".");
}

static void
Image_progress_conclude(stp_image_t *image)
{
// fprintf(stderr, "\n");
}

static const char *Image_get_appname(stp_image_t *image)
{ return("aspawngp: Arahne's interface to gimp-print"); }


static stp_image_status_t Image_get_row(stp_image_t *image, unsigned char *data, int row)
{
 int x;
 unsigned char *dat, *cmapPtr;
 printBlock *pb = &pBlock[lineToBlock[row]];

 if (!pb->data)
  {
   /* read the print block structure */
   if (!fread(pb, sizeof(printBlock), 1, fout))
    return(STP_IMAGE_ABORT);
   /* read a block of data from file if we do not have it yet */
   pb->data = malloc(pb->bpl*pb->unreadLines);
   if (!pb->data)
    return(STP_IMAGE_ABORT);
   /* we presume that reading of rows is close to consecutive - we must not jump by more than one block height */
   if (pb->unreadLines!=fread(pb->data, pb->bpl, pb->unreadLines, fout))
    return(STP_IMAGE_ABORT);
//fprintf(stderr, "reading block %d lines %d type %d\n", lineToBlock[row], pb->unreadLines, pb->bandType);
  }
 dat = &pb->data[pb->bpl*(row-pb->ypos)];
 /* now interpret the image type and fill in the data with the correct format of data */

 switch (pb->bandType)
  {
   case IMG_BW:   
    for (x=0; x<printx; x++)
     if (!(dat[x>>3]&(128>>(x&7))))
      { *data++ = 255; *data++ = 255; *data++ = 255; }
     else
      { *data++ = 0; *data++ = 0; *data++ = 0; }
    break;
   case IMG_CMAP:   
    for (x=0; x<printx; x++)
     {
      cmapPtr = &pb->cmap[(*dat++)*3];
      *data++ = *cmapPtr++; *data++ = *cmapPtr++; *data++ = *cmapPtr;
     }
    break;
   case IMG_TRUE_COL:   
    memcpy(data, dat, printx*3);
    break;
  }

 /* if we have read all the lines from this block, then free it from the memory */
 pb->unreadLines--;
 if (pb->unreadLines==0)
  {
//fprintf(stderr, "freed block %d\n", lineToBlock[row]);
   free(pb->data);
   pb->data=NULL;
  }
 return(STP_IMAGE_OK);
}

/* use it to remove new line character */
static void removeLast(char *str)
{
 str[strlen(str)-1]=0;
}

static void readParameterFile(FILE *fp)
{
 char buf[500];
 int i, j, num, pos;

 fgets(buf, 500, fp); removeLast(buf);
 fname = strdup(buf);

 fgets(buf, 500, fp); removeLast(buf);
 if (strlen(buf))
  dstFile = strdup(buf);
 fgets(buf, 500, fp); removeLast(buf);
 if (strlen(buf))
  commandStr = strdup(buf);
 fgets(buf, 500, fp); removeLast(buf);
 doRemove = atoi(buf);

 fgets(buf, 500, fp); removeLast(buf);
 sscanf(buf, "%d %d %d %d", &printx, &printy, &offsetx, &offsety);

 fgets(buf, 500, fp); removeLast(buf);
 i = atoi(buf); /* this is bidirectional printing, but we do not use it so far */

 fgets(buf, 500, fp); removeLast(buf);
 sscanf(buf, "%lf %lf %lf %lf %lf %lf %lf %lf %lf %lf", &global_c_level, &global_c_gamma, &global_m_level, &global_m_gamma,  &global_y_level, &global_y_gamma, &global_k_gamma, &global_gamma, &ink_limit, &density);

 fgets(buf, 500, fp); removeLast(buf);
 printer = strdup(buf);

 fgets(buf, 500, fp); removeLast(buf);
 ink_type = strdup(buf);

 fgets(buf, 500, fp); removeLast(buf);
 media_type = strdup(buf);

 fgets(buf, 500, fp); removeLast(buf);
 media_size = strdup(buf);

 fgets(buf, 500, fp); removeLast(buf);
 stp_resolution = strdup(buf);

 fgets(buf, 500, fp); removeLast(buf);
 horDpi = atoi(buf);

 fgets(buf, 500, fp); removeLast(buf);
 numPrintBlocks = atoi(buf);

 pBlock = calloc(sizeof(printBlock), numPrintBlocks);
 pos=0;
 lineToBlock = calloc(sizeof(unsigned short), printy);
 for (i=0; i<numPrintBlocks; i++)
  {
   fgets(buf, 500, fp); removeLast(buf);
   num = atoi(buf);
   for (j=0; j<num; j++)
    if (pos<printy)
     lineToBlock[pos++] = (unsigned short) i;
  } 
}

/* return -1 if process can't be created, 0 on failure, 1 on success */
static int executeCommand(char *command)
{
 int status;

 status = system(command);
 if(status<0)
  return(-1);
 if (WIFEXITED(status))
  if (WEXITSTATUS(status)==0)
   return(TRUE);
  else
   return(FALSE);
 else
  return(FALSE);
}

/* does the proper thing (copy, move or print) with the finished print file */
static int enqPrintFile(char *srcFile)
{
 char command[500];
 int status;

 if (!commandStr) /* need to move the file from srcFile to dstFile */
  {
   if (rename(srcFile, dstFile)==0)
    return(PRT_OK);
   else /* rename fails across filesystems, so use mv command */
    strcpy(command, "/bin/mv -f");
  }
 else
  strcpy(command, commandStr); /* use the provided command */

 /* take the command, add srcfile (and dstfile, if present) */
 strcat(command, " \"");
 strcat(command, srcFile);
 strcat(command, "\" ");
 if (dstFile)
  {
   strcat(command, " \"");
   strcat(command, dstFile);
   strcat(command, "\" ");
  }
 status = executeCommand(command);

 if (doRemove)
  if (access(dstFile, F_OK)==0)
   remove(dstFile);
 if (dstFile)
  free(dstFile);
 if (access(srcFile, F_OK)==0)
  remove(srcFile);

 if (status<0)
  return(PRT_INTERNAL_ERROR);
 else
  return((status==0) ? PRT_IO_ERROR : PRT_OK);
}

int main(int argc, char *argv[])
{
 FILE *fparam;
 stp_vars_t v;
 stp_printer_t the_printer;
 stp_papersize_t pt;
 int left, right, top, bottom;
 const stp_printfuncs_t *printfuncs;
 int x, y;
 int filedes = -1;

 if (argc<1)
  {
   printf("missing parameter file\n");
   return(-1);
  }
 if ((fparam = fopen(argv[1], "rb" )) == NULL)
// if ((fparam = fopen("/tmp/param", "rb" )) == NULL)
  {
   printf("cannot open parameter file %s\n", argv[1]);
   return(-1);
  }


 readParameterFile(fparam);
 fclose(fparam);
 remove(argv[1]);

 if ((fout = fopen(fname, "rb" )) == NULL)
  {
   printf("cannot open input file %s\n", fname);
   return(-1);
  }

 fnamegimp = strdup("/tmp/plibgpXXXXXX");
 if ((filedes=mkstemp(fnamegimp)) < 0 )
  {
   printf("cannot open temporary print file\n");
   return(-1);
  }
 else
  fgimp = fdopen(filedes, "wb");

 stp_init();
 v = stp_allocate_vars();
 the_printer = stp_get_printer_by_driver(printer);
 if (!the_printer)
  {
   printf("gimp-print unsupported printer: %s\n", printer);
   return(-1);
  }

 stp_set_printer_defaults(v, the_printer, NULL);
 stp_set_outfunc(v, writefunc);
 stp_set_errfunc(v, writefunc);
 stp_set_outdata(v, fgimp);
 stp_set_errdata(v, stderr);
 printfuncs = stp_printer_get_printfuncs(the_printer);
 stp_set_density(v, (float) density);
 stp_set_resolution(v, stp_resolution);
 stp_set_ink_type(v, ink_type);
 stp_set_media_type(v, media_type);

 if (media_size)
  stp_set_media_size(v, media_size);
 if (dither_algorithm)
  stp_set_dither_algorithm(v, dither_algorithm);

 stp_set_output_type(v, OUTPUT_COLOR);

 pt = stp_get_papersize_by_name(stp_get_media_size(v));
 if (!pt)
   {
    fprintf(stderr, "Papersize %s unknown\n", media_size);
    return(PRT_INTERNAL_ERROR);
   }

 (printfuncs->imageable_area)(the_printer, v, &left, &right, &bottom, &top);
 (printfuncs->describe_resolution)(the_printer, stp_get_resolution(v), &x, &y);
 if (x < 0)
   x = 300;
 if (y < 0)
   y = 300;

 /* stp_papersize_get_height(pt); you may need this function in the future */

 /* we always render things at 1:1 size */
 if (horDpi==150)
  stp_set_scaling(v, -150); /* this means we are saying we have 150 dpi */
 if (horDpi==300)
  stp_set_scaling(v, -300); /* this means we are saying we have 300 dpi */
 if (horDpi==600)
  stp_set_scaling(v, -600); /* this means we are saying we have 600 dpi */
 if (horDpi==1200)
  stp_set_scaling(v, -1200); /* this means we are saying we have 1200 dpi */
 if (horDpi==360)
  stp_set_scaling(v, -360); /* this means we are saying we have 360 dpi */
 if ((horDpi==720)||(horDpi==1440))
  stp_set_scaling(v, -720); /* this means we are saying we have 720 dpi */

 stp_set_left(v, (int) (offsetx/10.0));
 stp_set_top(v, (int) (offsety/10.0));
 stp_set_orientation(v, ORIENT_PORTRAIT);

 stp_merge_printvars(v, stp_printer_get_printvars(the_printer));
 if (stp_printer_get_printfuncs(the_printer)->verify(the_printer, v))
   (stp_printer_get_printfuncs(the_printer)->print)(the_printer, &theImage, v);
 else
  printf("wrong gimp-print parameters\n");

 fclose(fout);
 fclose(fgimp);
 remove(fname); /* remove it immediately so we lower the necessary disk space */
 free(fname);
 fname=NULL;
 return(enqPrintFile(fnamegimp));
}
