#if !defined(__MINGW32__) && !defined(__APPLE__)
#define GUTENP
#include <sys/types.h>
#include <unistd.h>
#include <sys/wait.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "print.h"
/* check src/gutenprint-xxx/src/main/papers.xml for valid paper sizes */

static FILE *fout;
static char *fname;

/* the supported classes of printers; in variable printerClass */
#define CLASS_FAKE	0
#define CLASS_R200	1
#define CLASS_R360	2
#define CLASS_DX3850	3
#define CLASS_D120	4
#define CLASS_R2400	5
#define CLASS_R290	6
#define CLASS_R800	7
#define CLASS_DX8400	8
#define CLASS_P50	9
#define CLASS_T60	10
#define CLASS_R1900	11
#define CLASS_R2000	12
#define CLASS_R3000	13

static int printerClass = 0;
static int horDpi;

static unsigned short *blockLines;
static int numPrintBlocks;

static double global_c_level = 1.0;
static double global_c_gamma = 1.0;
static double global_m_level = 1.0;
static double global_m_gamma = 1.0;
static double global_y_level = 1.0;
static double global_y_gamma = 1.0;
static double global_k_gamma = 1.0;
static double global_gamma = 1.0;
static double ink_limit = 1.0;
static double density = 1.0;
static char printer[100];
static char ink_type[30];
static char stp_resolution[50];
/* PhotoCMYK CMYK */
static char media_type[50];
/* Plain Photo */
static char media_size[30];
/* Letter Legal A0 A1 A2 A3 A4 A5 */
static int printx, printy, offsetx, offsety;

static void fillCommon(printerMode *printerMod, int *numModes, printerDensity *prtDensityX,
 int numDensX, printerDensity *prtDensityY, int numDensY, int *dpiVariable, char **profileName)
{
 int i;

 printerMod[0].printType = IMG_TRUE_COL;
 printerMod[1].printType = IMG_BW;
 *numModes = 2;
 *dpiVariable = FALSE;
 for (i=1; i<numDensX; i++)
  {
   prtDensityX[i].dpi = prtDensityX[i-1].dpi/2;
   prtDensityX[i].pixels = prtDensityX[i-1].pixels/2;
  }
 for (i=1; i<numDensY; i++)
  {
   prtDensityY[i].dpi = prtDensityY[i-1].dpi/2;
   prtDensityY[i].pixels = prtDensityY[i-1].pixels/2;
  }
 switch (printerClass)
  {
   case CLASS_R200:   *profileName="StylusR200GP"; break;
   case CLASS_R360:   *profileName="StylusR360"; break;
   case CLASS_DX3850: *profileName="StylusDX3850"; break;
   case CLASS_D120:   *profileName="StylusD120"; break;
   case CLASS_R3000:  *profileName="StylusR3000"; break;
   case CLASS_R1900:  /* use R2400 profile until we get the measurements */
   case CLASS_R2000:  /* use R2400 profile until we get the measurements */
   case CLASS_R2400:  *profileName="StylusR2400"; break;
   case CLASS_R290:   *profileName="screen"; break;
   case CLASS_R800:   *profileName="StylusR800"; break;
   case CLASS_DX8400: *profileName="StylusDX8400"; break;
   case CLASS_P50:    *profileName="StylusP50"; break;
   case CLASS_T60:    *profileName="screen"; break;
   case CLASS_FAKE:   *profileName="screen"; break;
  }
}

/* return -1 if process can't be created, 0 on failure, 1 on success */
static int executeCommand(char *command)
{
 int status;

 status = system(command);
 if(status<0)
  return(-1);
 if (WIFEXITED(status))
  if (WEXITSTATUS(status)==0)
   return(TRUE);
  else
   return(FALSE);
 else
  return(FALSE);
}

static int initPrint(int printType, int complexImage, int xpixels,
 int ypixels, int xdpi, int ydpi, int xmargin, int ymargin)
{
 fname = strdup("/tmp/prnXXXXXX");
 mkstemp(fname);
 if (fname==NULL)
  return(PRT_IO_ERROR);
 if ((fout = fopen(fname, "wb" )) == NULL)
  return(PRT_IO_ERROR);

 horDpi = xdpi/1000;
 printx = xpixels;
 printy = ypixels;
 offsetx = xmargin;
 offsety = ymargin;
 blockLines = calloc(sizeof(unsigned short), ypixels/2); /* assume at least 2 pixels per block */
 if (!blockLines)
  return(PRT_INTERNAL_ERROR);
 numPrintBlocks=0;
 return(PRT_OK);
}

static int printBand(int xpixels, char *data, int bpl, int ypos,
	int lines, char bandType, unsigned char *cmap)
{
 printBlock prtBlock;

 prtBlock.bandType=bandType;
 prtBlock.unreadLines=lines;
 prtBlock.bpl=bpl;
 prtBlock.ypos=ypos;
 if (bandType==IMG_CMAP)
  memcpy(&prtBlock.cmap, cmap, 256*3);
 blockLines[numPrintBlocks] = (unsigned short) lines;

 /* first write printBlock structure */
 /* no danger for source/binary compatibility, since the file will always be written */
 /* and read by the same program. */
 if (!fwrite(&prtBlock, 1, sizeof(printBlock), fout))
  return(PRT_IO_ERROR);

 /* then the actual image block data */
 if (lines!=fwrite(data, bpl, lines, fout))
  return(PRT_IO_ERROR);

 numPrintBlocks++;
 return(PRT_OK);
}

static void writeParamFile(FILE *fp)
{
 int i;

 if (printerClass==CLASS_FAKE)
  {
   global_c_level = global_m_level = global_y_level = 1.0;
   ink_limit = 1.0;
   density = 1.0;
  }

 if (printerClass==CLASS_R200)
  {
   global_c_level = global_m_level = global_y_level = 1.0;
   ink_limit = 1.0;
   density = 1.0;


/*
   if (horDpi==1440)
    {
     global_c_level = global_m_level = global_y_level = 1.0;
     ink_limit = 1.1;
     density = 1.2;
    }
   else // 720 and 360 dpi settings
    {
     global_c_level = global_m_level = global_y_level = 1.2;
     ink_limit = 1.1;
     density = 1.2;
    }
*/
  }

 if (printerClass==CLASS_R360)
  {
   global_c_level = global_m_level = global_y_level = 1.0;
   ink_limit = 1.0;
   density = 1.0;
  }

 if (printerClass==CLASS_DX3850)
  {
   global_c_level = global_m_level = global_y_level = 1.0;
   ink_limit = 1.0;
   density = 1.0;
  }

 if (printerClass==CLASS_D120)
  {
   global_c_level = global_m_level = global_y_level = 1.0;
   ink_limit = 1.0;
   density = 1.0;
  }

 if ( (printerClass==CLASS_R2400) || (printerClass==CLASS_R1900) || (printerClass==CLASS_R2000) )
  {
   global_c_level = global_m_level = global_y_level = 1.1;
   ink_limit = 1.1;
   density = 1.0;
  }

 if ((printerClass==CLASS_R3000) )
  {
/*
   if (horDpi==2880)
    {
     global_c_level = global_m_level = global_y_level = 1.0;
     ink_limit = 0.8;
     density = 0.8;
    }
   else
*/
    {
     global_c_level = global_m_level = global_y_level = 1.0;
     ink_limit = 0.7;
     density = 0.7;
    }
  }

 if (printerClass==CLASS_R290)
  {
   global_c_level = global_m_level = global_y_level = 1.0;
   ink_limit = 1.0;
   density = 1.0;
  }

 if (printerClass==CLASS_R800)
  {
   global_c_level = global_m_level = global_y_level = 1.0;
   ink_limit = 1.0;
   density = 1.0;
  }

 if (printerClass==CLASS_R800)
  {
   global_c_level = global_m_level = global_y_level = 1.0;
   ink_limit = 1.0;
   density = 1.0;
  }

 if ((printerClass==CLASS_P50) || (printerClass==CLASS_T60))
  {
   if (horDpi==2880)
    {
     global_c_level = global_m_level = global_y_level = 1.2;
     ink_limit = 1;
     density = 1;   
    }
   else if (horDpi==1440)
    {
     global_c_level = global_m_level = global_y_level = 1.2;
     ink_limit = 1;
     density = 0.9;   
    }
   else if (horDpi==720)
    {
     global_c_level = global_m_level = global_y_level = 1.0;
     ink_limit = 0.7;
     density = 0.7;
    }
  }
 if (ink_limit <= 0 || ink_limit > 2.0 ||
     global_c_level <= 0 || global_c_level > 10 ||
     global_m_level <= 0 || global_m_level > 10 ||
     global_y_level <= 0 || global_y_level > 10  )
  printf("Print options out of range!\n");

/* 360sw 360swuni */
/* 720sw 720swuni 720hquni */
/* 1440x720sw 1440x720swuni */
/* 2880x720sw  2880x720swuni 1440x720hq2 */
/* 1440x1440sw */
  if (horDpi==300)
   strcpy(stp_resolution, "300dpi");
  if (horDpi==600)
   {
    if (strstr(printer, "bjc-"))
     strcpy(stp_resolution, "600x600dpi");
    else
     strcpy(stp_resolution, "600dpi");
   }
  if (horDpi==1200)
   strcpy(stp_resolution, "1200x1200dpi");
  if (horDpi==360)
   {
    if (strcmp(printer, "escp2-pro-xl")==0)
     strcpy(stp_resolution, "360mw");
    else
     strcpy(stp_resolution, "360sw");
   }
  if (horDpi==720)
   {
    if ((strcmp(printer, "escp2-9000")==0) || (strcmp(printer, "escp2-5500")==0) || (strcmp(printer, "escp2-pro-xl")==0))
     strcpy(stp_resolution, "720mw");
    else
     strcpy(stp_resolution, "720sw");
   }
  if (horDpi==1440)
   strcpy(stp_resolution, "1440x720sw");
  if (horDpi==2880)
   {
    if ( (printerClass==CLASS_P50) || (printerClass==CLASS_T60) || (printerClass==CLASS_R3000) )
     strcpy(stp_resolution, "2880x1440sw");
    else
     strcpy(stp_resolution, "2880x720sw");
   }
 if (printerClass==CLASS_FAKE)
  strcpy(media_type, "Plain");
 else
  if ((printerClass==CLASS_R2400) || (printerClass==CLASS_R1900) || (printerClass==CLASS_R2000))
   strcpy(media_type, "MatteHeavy");
  else
   strcpy(media_type, "Coated");

printf("media %s resolution %s limits cmy %f limit %f density %f\n", media_type, stp_resolution, global_c_level, ink_limit, density);

 /* start of writing parameters file */
 fprintf(fp, "%s\n", fname);
 fprintf(fp, "%s\n", getRegistredDstFile() ? getRegistredDstFile() : "");
 fprintf(fp, "%s\n", getRegistredCommandStr() ? getRegistredCommandStr() : "");
 fprintf(fp, "%d\n", getRegistredDoRemove());
 fprintf(fp, "%d %d %d %d\n", printx, printy, offsetx, offsety);
 fprintf(fp, "%d\n", getBidirectionalPrinting());
 fprintf(fp, "%f %f %f %f %f %f %f %f %f %f\n", global_c_level, global_c_gamma, global_m_level, global_m_gamma,  global_y_level, global_y_gamma, global_k_gamma, global_gamma, ink_limit, density);
 fprintf(fp, "%s\n", printer);
 fprintf(fp, "%s\n", ink_type);
 fprintf(fp, "%s\n", media_type);
 fprintf(fp, "%s\n", media_size);
 fprintf(fp, "%s\n", stp_resolution);
 fprintf(fp, "%d\n", horDpi);
 fprintf(fp, "%d\n", numPrintBlocks);
 for (i=0; i<numPrintBlocks; i++)
  fprintf(fp, "%d\n", blockLines[i]);
}

static int endPrint(int printStatus)
{
 if (printStatus==PRT_OK)
  {
   char *pfname;
   FILE *pfout;
   char command[300];
   int status;
   int filedes = -1;

   fflush(fout);
   fclose(fout);
   pfname = strdup("/tmp/prnXXXXXX");
   if ((filedes = mkstemp(pfname)) < 0)
    {
     free(pfname);
     return(PRT_IO_ERROR);
    }
   else
    close(filedes); /* we just need temporary filename, we open it below */
   if ((pfout = fopen(pfname, "wb" )) == NULL)
    {
     free(pfname);
     return(PRT_IO_ERROR);
    }

   writeParamFile(pfout);
   fclose(pfout);

   sprintf(command, "../common/agutenp %s &", pfname);
   free(pfname);
   status = executeCommand(command);
   if (status<0)
    return(PRT_INTERNAL_ERROR);
   else
    return((status==0) ? PRT_IO_ERROR : PRT_OK);
  }
 else
  {
   fclose(fout);
   remove(fname);
   free(fname);
   fname=NULL;
  }
 return(printStatus);
}


static void getPrtInfoPIXMAiP4000(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA iP4000 (gutenprint)";
 strcpy(printer, "bjc-iP4000");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAiP4100(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA iP4100 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-iP4100");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAiP4200(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA iP4200 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-iP4200");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAiP4300(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA iP4300 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-iP4300");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAiP4500(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA iP4500 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-iP4500");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAiP5000(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA iP5000 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-iP5000");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAiP5200(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA iP5200 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-iP5200");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAiP5300(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA iP5300 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-iP5300");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}


static void getPrtInfoPIXMAiP6000D(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA iP6000D (gutenprint)";
 strcpy(printer, "bjc-PIXMA-iP6000D");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAiP6700(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA iP6700 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-iP6700");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAiP8500(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA iP8500 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-iP8500");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAiX5000(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA iX5000 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-iX5000");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAPro9500(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA Pro9500 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-Pro9500");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAMP150(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA MP150 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-MP150");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAMP160(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA MP160 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-MP160");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAMP170(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA MP170 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-MP170");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAMP180(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA MP180 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-MP180");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAMP500(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA MP500 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-MP500");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAMP520(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA MP520 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-MP520");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAMP610(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA MP610 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-MP610");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAMP700(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA MP700 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-MP700");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAMP710(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA MP710 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-MP710");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAMP730(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA MP730 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-MP730");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAMP740(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA MP740 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-MP740");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAMP750(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA MP750 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-MP750");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAMP760(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA MP760 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-MP760");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAMP770(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA MP770 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-MP770");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAMP780(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA MP780 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-MP780");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAMP790(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA MP790 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-MP790");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPIXMAMP830(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 1;
 *numDensitY = 1;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Canon PIXMA MP830 (gutenprint)";
 strcpy(printer, "bjc-PIXMA-MP830");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoC88(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 11550;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi =  1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "StylusC88 (gutenprint)";
 strcpy(printer, "escp2-c88");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoR200(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_R200;
 prtDensityX[0].pixels = 11550;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi =  1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "StylusR200 (gutenprint)";
 strcpy(printer, "escp2-r200");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoR230(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 11550;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi =  1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "StylusR230 (gutenprint)";
 strcpy(printer, "escp2-r230");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoR270(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 11550;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi =  1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "StylusR270 (gutenprint)";
 strcpy(printer, "escp2-r270");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoR285(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 11550;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi =  1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "StylusR285 (gutenprint)";
 strcpy(printer, "escp2-r285");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoR290(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_R290;
 prtDensityX[0].pixels = 11550;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi =  1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 0;
 *prtName = "StylusR290 (gutenprint)";
 strcpy(printer, "escp2-r290");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoR800(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_R800;
 prtDensityX[0].pixels = 11550;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi =  1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "StylusR800 (gutenprint)";
 strcpy(printer, "escp2-r800");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoB300(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 11550;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi =  1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Stylus B-300 (gutenprint)";
 strcpy(printer, "escp2-b300");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoB500DN(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 11550;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi =  1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Stylus B-500DN (gutenprint)";
 strcpy(printer, "escp2-b500dn");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoP50(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_P50;
 prtDensityX[0].pixels = 11550*2;
 prtDensityY[0].pixels = 8300*2;
 prtDensityX[0].dpi =  2880000;
 prtDensityY[0].dpi =  1440000;
 *numDensitX = 4;
 *numDensitY = 3;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Stylus Photo P50/T50 (gutenprint)";
 strcpy(printer, "escp2-t50");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoT60(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_T60;
 prtDensityX[0].pixels = 11550*2;
 prtDensityY[0].pixels = 8100*2;
 prtDensityX[0].dpi =  2880000;
 prtDensityY[0].dpi =  1440000;
 *numDensitX = 4;
 *numDensitY = 3;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Stylus Photo T60 (gutenprint)";
 strcpy(printer, "escp2-t60");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfo1400(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 16600;
 prtDensityY[0].pixels = 11900;
 prtDensityX[0].dpi =  1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "Stylus1400 (gutenprint)";
 strcpy(printer, "escp2-1400");
 strcpy(ink_type, "");
 strcpy(media_size, "w907h1275"); /* just a lolvely name for Super A3 */
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoEX3(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 16600;
 prtDensityY[0].pixels = 11900;
 prtDensityX[0].dpi =  1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "StylusPhotoEX3 (gutenprint)";
 strcpy(printer, "escp2-ex3");
 strcpy(ink_type, "");
 strcpy(media_size, "w907h1275"); /* just a lolvely name for Super A3 */
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}


static void getPrtInfoR1800(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 16600;
 prtDensityY[0].pixels = 11900;
 prtDensityX[0].dpi =  1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "StylusR1800 (gutenprint)";
 strcpy(printer, "escp2-r1800");
 strcpy(ink_type, "");
 strcpy(media_size, "w907h1275"); /* just a lovely name for Super A3 */
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoR1900(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_R1900;
 prtDensityX[0].pixels = 16600;
 prtDensityY[0].pixels = 11900;
 prtDensityX[0].dpi =  1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "StylusR1900 (gutenprint)";
 strcpy(printer, "escp2-r1900");
 strcpy(ink_type, "");
 strcpy(media_size, "w907h1275"); /* just a lovely name for Super A3 */
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoR2000(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_R2000;
 prtDensityX[0].pixels = 16600;
 prtDensityY[0].pixels = 11900;
 prtDensityX[0].dpi =  1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "StylusR2000 (gutenprint)";
 strcpy(printer, "escp2-r2000");
 strcpy(ink_type, "");
 strcpy(media_size, "w907h1275"); /* just a lovely name for Super A3 */
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoR2400(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_R2400;
 prtDensityX[0].pixels = 16600;
 prtDensityY[0].pixels = 11900;
 prtDensityX[0].dpi =  1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "StylusR2400 (gutenprint)";
 strcpy(printer, "escp2-r2400");
 strcpy(ink_type, "");
 strcpy(media_size, "w907h1275"); /* just a lovely name for Super A3 */
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoR3000(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_R3000;
 prtDensityX[0].pixels = 16600*2;
 prtDensityY[0].pixels = 11900*2;
 prtDensityX[0].dpi =  1440000*2;
 prtDensityY[0].dpi =  720000*2;
 *numDensitX = 4;
 *numDensitY = 3;
 *nonprintableTop = 3;
 *nonprintableBottom = 4;
 *nonprintableLeft = 3;
 *prtName = "StylusR3000 (gutenprint)";
 strcpy(printer, "escp2-r3000");
 strcpy(ink_type, "");
 strcpy(media_size, "w907h1275"); /* just a lovely name for Super A3 */
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoR350(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_R360;
 prtDensityX[0].pixels = 11550;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi =  1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 0;
 *prtName = "StylusR350 (gutenprint)";
 strcpy(printer, "escp2-r350");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoR360(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_R360;
 prtDensityX[0].pixels = 11550;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi =  1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 0;
 *prtName = "StylusR360 (gutenprint)";
 strcpy(printer, "escp2-r360");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoR390(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_R360;
 prtDensityX[0].pixels = 11550;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi =  1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 0;
 *prtName = "StylusR390 (gutenprint)";
 strcpy(printer, "escp2-r390");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoD68(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 11550;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi =  1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 0;
 *prtName = "StylusD68 (gutenprint)";
 strcpy(printer, "escp2-d68");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoDX3850(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_DX3850;
 prtDensityX[0].pixels = 11550;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi =  1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 0;
 *prtName = "StylusDX3850 (gutenprint)";
 strcpy(printer, "escp2-dx3850");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoDX8400(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_DX8400;
 prtDensityX[0].pixels = 11550;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi =  1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 0;
 *prtName = "StylusDX8400 (gutenprint)";
 strcpy(printer, "escp2-dx8400");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoD120(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_D120;
 prtDensityX[0].pixels = 11550;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi =  1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 0;
 *prtName = "StylusD120 (gutenprint)";
 strcpy(printer, "escp2-d120");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoME300(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 11550;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi =  1440000;
 prtDensityY[0].dpi =  720000;
 *numDensitX = 3;
 *numDensitY = 2;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 0;
 *prtName = "Stylus ME 300 (gutenprint)";
 strcpy(printer, "escp2-me300");
 strcpy(ink_type, "");
 strcpy(media_size, "A4");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPS2A4(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 5950;
 prtDensityY[0].pixels = 8300;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 3;
 *numDensitY = 3;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 0;
 *prtName = "PostScript A4 (gutenprint)";
 strcpy(printer, "ps2");
 strcpy(ink_type, "");
// strcpy(media_size, "A4");
 strcpy(media_size, "");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

static void getPrtInfoPS2A3(printerMode *printerMod, int *numModes,
 printerDensity *prtDensityX, int *numDensitX,
 printerDensity *prtDensityY, int *numDensitY,
 int *nonprintableTop, int *nonprintableBottom, int *nonprintableLeft,
 int *dpiVariable, char **prtName, char **profileName)
{
 printerClass = CLASS_FAKE;
 prtDensityX[0].pixels = 8300;
 prtDensityY[0].pixels = 11900;
 prtDensityX[0].dpi = 600000;
 prtDensityY[0].dpi = 600000;
 *numDensitX = 3;
 *numDensitY = 3;
 *nonprintableTop = 0;
 *nonprintableBottom = 4;
 *nonprintableLeft = 0;
 *prtName = "PostScript A3 (gutenprint)";
 strcpy(printer, "ps2");
 strcpy(ink_type, "");
// strcpy(media_size, "A3");
 strcpy(media_size, "");
 fillCommon(printerMod, numModes, prtDensityX, *numDensitX,
  prtDensityY, *numDensitY, dpiVariable, profileName);
}

DGUTENP void registerPIXMAiP4000GutP(void)
{ registerPrinter(*getPrtInfoPIXMAiP4000, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAiP4100GutP(void)
{ registerPrinter(*getPrtInfoPIXMAiP4100, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAiP4200GutP(void)
{ registerPrinter(*getPrtInfoPIXMAiP4200, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAiP4300GutP(void)
{ registerPrinter(*getPrtInfoPIXMAiP4300, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAiP4500GutP(void)
{ registerPrinter(*getPrtInfoPIXMAiP4500, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAiP5000GutP(void)
{ registerPrinter(*getPrtInfoPIXMAiP5000, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAiP5200GutP(void)
{ registerPrinter(*getPrtInfoPIXMAiP5200, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAiP5300GutP(void)
{ registerPrinter(*getPrtInfoPIXMAiP5300, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAiP6000DGutP(void)
{ registerPrinter(*getPrtInfoPIXMAiP6000D, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAiP6700GutP(void)
{ registerPrinter(*getPrtInfoPIXMAiP6700, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAiP8500GutP(void)
{ registerPrinter(*getPrtInfoPIXMAiP8500, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAiX5000GutP(void)
{ registerPrinter(*getPrtInfoPIXMAiX5000, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAPro9500GutP(void)
{ registerPrinter(*getPrtInfoPIXMAPro9500, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAMP150GutP(void)
{ registerPrinter(*getPrtInfoPIXMAMP150, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAMP160GutP(void)
{ registerPrinter(*getPrtInfoPIXMAMP160, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAMP170GutP(void)
{ registerPrinter(*getPrtInfoPIXMAMP170, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAMP180GutP(void)
{ registerPrinter(*getPrtInfoPIXMAMP180, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAMP500GutP(void)
{ registerPrinter(*getPrtInfoPIXMAMP500, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAMP520GutP(void)
{ registerPrinter(*getPrtInfoPIXMAMP520, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAMP610GutP(void)
{ registerPrinter(*getPrtInfoPIXMAMP610, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAMP700GutP(void)
{ registerPrinter(*getPrtInfoPIXMAMP700, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAMP710GutP(void)
{ registerPrinter(*getPrtInfoPIXMAMP710, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAMP730GutP(void)
{ registerPrinter(*getPrtInfoPIXMAMP730, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAMP740GutP(void)
{ registerPrinter(*getPrtInfoPIXMAMP740, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAMP750GutP(void)
{ registerPrinter(*getPrtInfoPIXMAMP750, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAMP760GutP(void)
{ registerPrinter(*getPrtInfoPIXMAMP760, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAMP770GutP(void)
{ registerPrinter(*getPrtInfoPIXMAMP770, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAMP780GutP(void)
{ registerPrinter(*getPrtInfoPIXMAMP780, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAMP790GutP(void)
{ registerPrinter(*getPrtInfoPIXMAMP790, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPIXMAMP830GutP(void)
{ registerPrinter(*getPrtInfoPIXMAMP830, *initPrint, *printBand, *endPrint); }

DGUTENP void registerStylusC88GutP(void)
{ registerPrinter(*getPrtInfoC88, *initPrint, *printBand, *endPrint); }

DGUTENP void registerStylusR200GutP(void)
{ registerPrinter(*getPrtInfoR200, *initPrint, *printBand, *endPrint); }

DGUTENP void registerStylusR230GutP(void)
{ registerPrinter(*getPrtInfoR230, *initPrint, *printBand, *endPrint); }

DGUTENP void registerStylusR270GutP(void)
{ registerPrinter(*getPrtInfoR270, *initPrint, *printBand, *endPrint); }

DGUTENP void registerStylusR285GutP(void)
{ registerPrinter(*getPrtInfoR285, *initPrint, *printBand, *endPrint); }

DGUTENP void registerStylusR290GutP(void)
{ registerPrinter(*getPrtInfoR290, *initPrint, *printBand, *endPrint); }

DGUTENP void registerStylusR800GutP(void)
{ registerPrinter(*getPrtInfoR800, *initPrint, *printBand, *endPrint); }

DGUTENP void registerStylusB300GutP(void)
{ registerPrinter(*getPrtInfoB300, *initPrint, *printBand, *endPrint); }

DGUTENP void registerStylusB500DNGutP(void)
{ registerPrinter(*getPrtInfoB500DN, *initPrint, *printBand, *endPrint); }

DGUTENP void registerStylusPhotoP50GutP(void)
{ registerPrinter(*getPrtInfoP50, *initPrint, *printBand, *endPrint); }

DGUTENP void registerStylusPhotoT60GutP(void)
{ registerPrinter(*getPrtInfoT60, *initPrint, *printBand, *endPrint); }

DGUTENP void registerStylusR1800GutP(void)
{ registerPrinter(*getPrtInfoR1800, *initPrint, *printBand, *endPrint); }

DGUTENP void registerStylusR1900GutP(void)
{ registerPrinter(*getPrtInfoR1900, *initPrint, *printBand, *endPrint); }

DGUTENP void registerStylusR2000GutP(void)
{ registerPrinter(*getPrtInfoR2000, *initPrint, *printBand, *endPrint); }

DGUTENP void registerStylusR2400GutP(void)
{ registerPrinter(*getPrtInfoR2400, *initPrint, *printBand, *endPrint); }

DGUTENP void registerStylusR3000GutP(void)
{ registerPrinter(*getPrtInfoR3000, *initPrint, *printBand, *endPrint); }

DGUTENP void registerStylus1400GutP(void)
{ registerPrinter(*getPrtInfo1400, *initPrint, *printBand, *endPrint); }

DGUTENP void registerStylusPhotoEX3GutP(void)
{ registerPrinter(*getPrtInfoEX3, *initPrint, *printBand, *endPrint); }

DGUTENP void registerStylusR350GutP(void)
{ registerPrinter(*getPrtInfoR350, *initPrint, *printBand, *endPrint); }

DGUTENP void registerStylusR360GutP(void)
{ registerPrinter(*getPrtInfoR360, *initPrint, *printBand, *endPrint); }

DGUTENP void registerStylusR390GutP(void)
{ registerPrinter(*getPrtInfoR390, *initPrint, *printBand, *endPrint); }

DGUTENP void registerStylusD68GutP(void)
{ registerPrinter(*getPrtInfoD68, *initPrint, *printBand, *endPrint); }

DGUTENP void registerStylusDX3850GutP(void)
{ registerPrinter(*getPrtInfoDX3850, *initPrint, *printBand, *endPrint); }

DGUTENP void registerStylusDX8400GutP(void)
{ registerPrinter(*getPrtInfoDX8400, *initPrint, *printBand, *endPrint); }

DGUTENP void registerStylusD120GutP(void)
{ registerPrinter(*getPrtInfoD120, *initPrint, *printBand, *endPrint); }

DGUTENP void registerStylusME300GutP(void)
{ registerPrinter(*getPrtInfoME300, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPS2A4GutP(void)
{ registerPrinter(*getPrtInfoPS2A4, *initPrint, *printBand, *endPrint); }

DGUTENP void registerPS2A3GutP(void)
{ registerPrinter(*getPrtInfoPS2A3, *initPrint, *printBand, *endPrint); }
#endif
