/* print.h */

#ifndef _PRINT_H_
#define _PRINT_H_

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

#ifndef min
#define max(a,b) 		((a)<(b) ? (b) : (a))
#define min(a,b) 		((a)>(b) ? (b) : (a))
#endif

#define MAX_PRINT 65520 /* maximal x or y printable pixels for all printers */

#define IMG_BW            0 /* 1=black */
			    /* 128 = 10000000 binary = leftmost pixel black */
#define IMG_CMAP          1 /* 8 bits, colormapped image; up to 256 colors */
#define IMG_TRUE_COL_DITH 2 /* 24 bits rgbrgbrgb... will be dithered when printed */
#define IMG_TRUE_COL      3 /* 24 bits rgbrgbrgb... will print on trueColor system */
#define IMG_RGBA	  7 /* 32 bits rgbargba... RGB with 8 bit alpha channel */
#define IMG_LAB		  8 /* 24 bits LabLab... L scaled from 0-100 to 0-255, a and b scaled from +-127 to 0-255 */

#define PRT_OK             0
#define PRT_USER_ABORT     1
#define PRT_NO_MEMORY      2
#define PRT_IO_ERROR       3
#define PRT_INTERNAL_ERROR 4
#define PRT_EXEC_ERROR     5
#define PRT_WRONG_IMG      6

/* defines 0-6 are equal to PRT_OK .. PRT_WRONG_IMG */
#define PTXT_WAIT		10
#define PTXT_PRINTING_WAIT	11
#define PTXT_CANCEL_PRINT	12
#define PTXT_SAVING_WAIT	13
#define PTXT_CANCEL_SAVE	14
#define PTXT_PRINTER		15
#define PTXT_PRINT_MODE		16
#define PTXT_DENSITY_X		17
#define PTXT_DENSITY_Y		18
#define PTXT_PAPER_WARNING	19
#define PTXT_PRINT_TO_FILE	20
#define PTXT_PRINT_TO_FILE_OK	21
#define PTXT_MODE_BW		22
#define PTXT_MODE_CMAP		23
#define PTXT_MODE_COLOR_DITHER	24
#define PTXT_MODE_COLOR_TRUE	25
#define PTXT_DENSITY		26
#define PTXT_MODE_RGBA		27
#define PTXT_MODE_LAB		28

#define MAX_PTXT 		30

typedef struct {
	int printType; /* IMG_BW ... IMG_TRUE_COL */
} printerMode;

typedef struct {
	int pixels; /* number of printable pixels */
	int dpi; /* dots per inch *1000 */
} printerDensity;

/* returns printer info */
typedef void (*getPrinterInfoProc) (
	printerMode *printerMod, /* fill in the preallocated printer modes */
	int *numModes, /* number of modes returned */
	printerDensity *prtDensityX, /* fill in the preallocated X density table */
        int *numDensitX, /* number of X densities returned */
	printerDensity *prtDensityY, /* fill in the preallocated Y density table */
        int *numDensitY, /* number of Y densities returned */
	int *nonprintableTop, /* nonprintable mm on top for paper handling */
	int *nonprintableBottom, /* nonprintable mm on bottom */
	int *nonprintableLeft, /* nonprintable mm on left */
	int *dpiVariable, /* print density is user variable */
        char **prtName, /* string containing printer's brand name */
        char **profileName /* string containing printer's profile name */
	);

/* application returns next band to be printed */
typedef int (*getNextBandProc) (
	int xpixels, /* number of horizontal pixels */
	int xdpi, /* horizontal dots per inch *1000 */
	int ydpi, /* vertical dots per inch *1000 */
	int ypos, /* requested y position of band within ytotal */
	int ytotal, /* total length of image to be printed in pixels */
	char desiredImageType, /* IMG_BW ... IMG_TRUE_COL */
	char **data, /* return data; to be freed by caller!!! */
        int  *bpl, /* bytes per line */
	int  *lines, /* number of return lines */
	char *returnImageType, /* IMG_BW ... IMG_TRUE_COL */
	unsigned char *cmap /* [256*3]; rgbrgbrgb; caller should preallocate it */
	);

/* printer driver processes the band */
typedef int (*printBandProc) (
	int xpixels, /* number of horizontal pixels */
	char *data, /* data; processBand must free it */
        int  bpl, /* bytes per line */
        int  ypos, /* current y position on page */
	int  lines, /* number of lines in this band */
	char bandType, /* IMG_BW ... IMG_TRUE_COL */
	unsigned char *cmap /* [256*3]; rgbrgbrgb */
	);

/* initializes the printer to the desired print type and page size */
/* opens oputput temporary files */
/* TRUE on success, FALSE otherwise */
typedef int (*initPrintProc) (
	int printType, /* IMG_BW ... IMG_TRUE_COL */
	int complexImage, /* TRUE or FALSE */
	int xpixels,
	int ypixels,
	int xdpi, /* horizontal dots per inch *1000 */
	int ydpi, /* vertical dots per inch *1000 */
	int xmargin, /* in pixels from top of printable page */
	int ymargin /* in pixels from left printable border */
	);

/* emits printer dependent end of page/eject page code, */
/* closes temporary files, enques the files in the print queue, removes them; */
/* returns TRUE if enqueued OK, FALSE othervise */
typedef int (*endPrintProc) (
	int normal /* PRT_OK ... PRT_WRONG_IMG */
	);

typedef struct
{
 unsigned char *data;
 unsigned char cmap[256*3];
 unsigned char bandType;
 unsigned int unreadLines;
 unsigned int bpl;
 unsigned int ypos;
} printBlock;


/* still need to make proper defines for functions below  ... */
extern void registerPNG(void);
extern void registerGIF(void);
extern void registerJPEG(void);
extern void registerPbmPlus(void);
extern void registerPostScript(void);
extern void registerTIFF(void);
extern void registerBMP(void);

#if !defined(__MINGW32__) && !defined(__APPLE__)
#define NUM_PRINTERS	111 /* must match the number of functions above */
#else
#define NUM_PRINTERS	24  /* must match the number of functions above */
#endif

#ifndef REGISTER
#define DREGISTER extern
#else
#define DREGISTER
#endif

/* P U B L I C   F U N C T I O N S */

/* registers printer with printer driver */
DREGISTER void registerPrinter(getPrinterInfoProc getPrtInfo,
 initPrintProc initPrt, printBandProc prtBand, endPrintProc endPrt);

/* allocates and returns the dither matrix of size 256*256 and distribution */
/* of values ranging from 0 to 255 according to the value of distr[256] */
/* array. The sum of all values in the distribution array must be 65536 */
DREGISTER unsigned char *makeDither(unsigned short *distr);

/* returns the number of printer drivers, gets the location of download files */
DREGISTER int initPrintLib(const char *path);

/* sets bidirectional printing */
DREGISTER void setBidirectionalPrinting(int set);

/* gets bidirectinal printing setting */
DREGISTER int getBidirectionalPrinting(void);

/* return the default suffix for print file in the given printer format, used only for graphical */
/* formats, printers return NULL */
DREGISTER char *getPrintFileSuffix(int numPrinter);

/* adds/replaces the suffix to the given filename. Warning! filename must have extra space for the length with */
/* the suffix! separator is used only if different than 0 */ 
DREGISTER void fixFilenameSuffix(char *fname, char separator, char *suffix);

/* sets the functions of currently selected printer */
DREGISTER void setPrinterFunctions(int numPrinter);

/* cleanup of temporary files */ 
DREGISTER void removeFileFreeFilename(char *filename, int prtStatus);

/* set the parameters for the action at the end of printing: either */
/* - copy the temporary print file to dFile (using command comStr) */
/* - print the temporary print file (using command comStr) */
/* and optionally remove the source file and free the filename */
DREGISTER void setEnqueuePrintFile(char *dFile, char *comStr, int removeSource);

/* does the proper thing (copy, move or print) with the finished print file */
DREGISTER int enqueuePrintFile(char *srcFile);

/* function takes bigger 8 bit image and reduces it to a smaller 24 bit image, resulting in antialiased view */
/* src must be 8bit image, dst must be 24 bit image */
/* source image must always be 4 times bigger in both dimensions */
DREGISTER void antialiasImage(unsigned char *srcImage, int srcBPL, unsigned char *dstImage, int dstBPL, unsigned char *cmap, int dstWid, int dstHei);

/* copy or move the file and set write permissions to the destination */
/* we used to fork a mv command, but this no longer works properly over CIFS networks */
/* anyway, we should be able to copy it on our own and we also avoid a fork */
DREGISTER int copyMoveFile(char *file1, char *file2, int move, int allowWriteToAll);

/* use these function to get the client settings of what to do with resulting print file */
DREGISTER char *getRegistredDstFile(void);
DREGISTER char *getRegistredCommandStr(void);
DREGISTER int getRegistredDoRemove(void);

#ifndef PTIFF
#define DPTIFF extern
#else
#define DPTIFF
#endif

/* choose between default LZW or DEFLATE; if you don't set it, we use DEFLATE */
DPTIFF void setTiffLZW(int set);

#ifndef EPSON
#define DEPSON extern
#else
#define DEPSON
#endif

/*-----------------------COMMANDS------------------------------------*/
DEPSON unsigned char *ResetPrn(unsigned char *bufPtr);
DEPSON unsigned char *FormFeed(unsigned char *bufPtr);
DEPSON unsigned char *SetColorCMYK(unsigned char CMYKcomp, unsigned char *bufPtr);
DEPSON unsigned char *SetColorCCMMYK(unsigned char CCMMYKcomp, unsigned char *bufPtr);
DEPSON unsigned char *SetGraphicsModeEpson(unsigned char *bufPtr);
DEPSON unsigned char *SetMicroWeave(int microWeaveOn, unsigned char *bufPtr);
DEPSON unsigned char *SetUnit(unsigned char unitS, unsigned char *bufPtr);
DEPSON unsigned char *SetPageLengthU(int PageLength, unsigned char *bufPtr);
DEPSON unsigned char *SetDirection(unsigned char d, unsigned char *bufPtr);
DEPSON unsigned char *SetPageFormat(int topM, int bottomM, unsigned char *bufPtr);
DEPSON unsigned char *SetAbsXPos(int position, unsigned char *bufPtr);
DEPSON unsigned char *SetRelXPos(int position, unsigned char *bufPtr);
DEPSON unsigned char *SetRelXPosNew(int position, unsigned int unit, unsigned char *bufPtr);
DEPSON unsigned char *SetAbsYPos(int position, unsigned char *bufPtr);
DEPSON unsigned char *SetRelYPos(int position, unsigned char *bufPtr);
DEPSON unsigned char *SetDotSize(unsigned char dotSize, unsigned char *bufPtr);
DEPSON unsigned char *PrintRasterGraphics(char compression, char vDotDens, char hDotDens,
		     char m, int width, unsigned char *bufPtr);
DEPSON unsigned char *SetResolutionRasterImage(int xdpi, int ydpi, unsigned char *bufPtr);
DEPSON unsigned char *TransferRasterImage(char color, char compression, char bitsPerPixel, int width, int height, unsigned char *bufPtr);
DEPSON unsigned char *SelectPrintingMode(char mode, unsigned char *bufPtr);
DEPSON unsigned char *SelectPrintingSpeed(char speed, unsigned char *bufPtr);

/*-----------------------COMMON FUNCTIONS------------------------------------*/
DEPSON unsigned char *compressLine(unsigned char *lnPtr, unsigned char *printBuf, 
 unsigned char *RLLBuf, int diff);

DEPSON unsigned char *linearizeDither(float *point, float bleed);
DEPSON void setDots(unsigned int *field, unsigned char *m);

#ifndef STYLUS
#define DSTYLUS extern
#else
#define DSTYLUS
#endif

DSTYLUS void registerStylus(void);
DSTYLUS void registerStylusPro(void);
DSTYLUS void registerStylusProXL(void);
DSTYLUS void registerStylus800(void);
DSTYLUS void registerStylus1520(void);
DSTYLUS void registerStylusPhoto(void);
DSTYLUS void registerStylusPhoto700(void);
DSTYLUS void registerStylusPhotoEX(void);
DSTYLUS void registerStylus5000(void);
DSTYLUS void registerStylus850(void);
DSTYLUS void registerStylus3000(void);
DSTYLUS void registerStylusPhoto750(void);
DSTYLUS void registerStylusPhoto1200(void);
DSTYLUS void registerStylusPhoto870(void);
DSTYLUS void registerStylusPhoto1270(void);
DSTYLUS void registerStylusPhoto790(void);
DSTYLUS void registerStylusPhoto890(void);
DSTYLUS void registerStylusPhoto1290(void);

#if !defined(__MINGW32__) && !defined(__APPLE__)

#ifndef GIMPP
#define DGIMPP extern
#else
#define DGIMPP
#endif

DGIMPP void registerStylusProXLGP(void);
DGIMPP void registerStylus3000GP(void);
DGIMPP void registerStylusPro5500GP(void);
DGIMPP void registerStylus9000GP(void);
DGIMPP void registerStylusPhoto870GP(void);
DGIMPP void registerStylusPhoto1270GP(void);
DGIMPP void registerStylusPhoto890GP(void);
DGIMPP void registerStylusPhoto1290GP(void);
DGIMPP void registerStylusC70GP(void);
DGIMPP void registerStylusC80GP(void);
DGIMPP void registerStylus670GP(void);
DGIMPP void registerStylus1160GP(void);
DGIMPP void registerStylusCX3200GP(void);
DGIMPP void registerStylusPhoto830GP(void);
DGIMPP void registerStylusPhoto935GP(void);
DGIMPP void registerStylusPhoto950GP(void);
DGIMPP void registerStylusPhoto2100GP(void);
DGIMPP void registerStylusPhoto2200GP(void);
DGIMPP void registerHPDeskJet1100CGP(void);
DGIMPP void registerHPDeskJet1120CGP(void);
DGIMPP void registerHPDeskJet1220CGP(void);
DGIMPP void registerHPDeskJet850CGP(void);
DGIMPP void registerHPCP1700CGP(void);
DGIMPP void registerStylus880GP(void);
DGIMPP void registerStylusC64GP(void);
DGIMPP void registerStylusC82GP(void);
DGIMPP void registerStylusR200GP(void);
DGIMPP void registerStylusR300GP(void);
DGIMPP void registerStylusRX500GP(void);
DGIMPP void registerStylusRX600GP(void);

#ifndef GUTENP
#define DGUTENP extern
#else
#define DGUTENP
#endif

DGUTENP void registerPIXMAiP4000GutP(void);
DGUTENP void registerPIXMAiP4100GutP(void);
DGUTENP void registerPIXMAiP4200GutP(void);
DGUTENP void registerPIXMAiP4300GutP(void);
DGUTENP void registerPIXMAiP4500GutP(void);
DGUTENP void registerPIXMAiP5000GutP(void);
DGUTENP void registerPIXMAiP5200GutP(void);
DGUTENP void registerPIXMAiP5300GutP(void);
DGUTENP void registerPIXMAiP6000DGutP(void);
DGUTENP void registerPIXMAiP6700GutP(void);
DGUTENP void registerPIXMAiP8500GutP(void);
DGUTENP void registerPIXMAiX5000GutP(void);
DGUTENP void registerPIXMAPro9500GutP(void);
DGUTENP void registerPIXMAMP150GutP(void);
DGUTENP void registerPIXMAMP160GutP(void);
DGUTENP void registerPIXMAMP170GutP(void);
DGUTENP void registerPIXMAMP180GutP(void);
DGUTENP void registerPIXMAMP500GutP(void);
DGUTENP void registerPIXMAMP520GutP(void);
DGUTENP void registerPIXMAMP610GutP(void);
DGUTENP void registerPIXMAMP700GutP(void);
DGUTENP void registerPIXMAMP710GutP(void);
DGUTENP void registerPIXMAMP730GutP(void);
DGUTENP void registerPIXMAMP740GutP(void);
DGUTENP void registerPIXMAMP750GutP(void);
DGUTENP void registerPIXMAMP760GutP(void);
DGUTENP void registerPIXMAMP770GutP(void);
DGUTENP void registerPIXMAMP780GutP(void);
DGUTENP void registerPIXMAMP790GutP(void);
DGUTENP void registerPIXMAMP830GutP(void);
DGUTENP void registerStylusC88GutP(void);
DGUTENP void registerStylusR200GutP(void);
DGUTENP void registerStylusR230GutP(void);
DGUTENP void registerStylusR270GutP(void);
DGUTENP void registerStylusR285GutP(void);
DGUTENP void registerStylusR290GutP(void);
DGUTENP void registerStylusR800GutP(void);
DGUTENP void registerStylusB300GutP(void);
DGUTENP void registerStylusB500DNGutP(void);
DGUTENP void registerStylusPhotoP50GutP(void);
DGUTENP void registerStylusPhotoT60GutP(void);
DGUTENP void registerStylusR1800GutP(void);
DGUTENP void registerStylusR1900GutP(void);
DGUTENP void registerStylusR2000GutP(void);
DGUTENP void registerStylusR2400GutP(void);
DGUTENP void registerStylusR3000GutP(void);
DGUTENP void registerStylus1400GutP(void);
DGUTENP void registerStylusPhotoEX3GutP(void);
DGUTENP void registerStylusR350GutP(void);
DGUTENP void registerStylusR360GutP(void);
DGUTENP void registerStylusR390GutP(void);
DGUTENP void registerStylusD68GutP(void);
DGUTENP void registerStylusDX3850GutP(void);
DGUTENP void registerStylusDX8400GutP(void);
DGUTENP void registerStylusD120GutP(void);
DGUTENP void registerStylusME300GutP(void);
DGUTENP void registerPS2A4GutP(void);
DGUTENP void registerPS2A3GutP(void);

#endif /* ! __MINGW32__ */



#endif /* _PRINT_H_ */
