/***************************************************************************
                          apaintimage.cpp  -  description
                             -------------------
    begin                : Mon May 5 2003
    copyright            : (C) 2003-2014 Arahne, d.o.o.
    email                : arahne@arahne.si
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public RHSLLicense as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <qcolor.h>
#include <qpixmap.h>
#include <qwmatrix.h>
#include <qimage.h>
#include "apaintconfig.h"
#include "apaintUtil.h"
#include "apaintimage.h"
#include "apaintimageUtil.h"
#include "apaint.h"

#define DONT_CHANGE   -1
#define SWAP(a,b) {a^=b; b^=a; a^=b;}
#define equal(str1,str2)	!strcasecmp((str1),(str2))

static int trueColorToCmap = FALSE;

RemapArray::RemapArray()
{
 setLinear();
}

RemapArray::~RemapArray()
{
}

void RemapArray::setLinear()
{
 for (int i=0; i<256; i++)
  tab[i] = i;
}

void RemapArray::setVal(int pt, uchar val)
{
 if (pt>=0 && pt<256)
  tab[pt] = val;
}

uchar RemapArray::getVal(int pt)
{
 if (pt>=0 && pt<256)
  return(tab[pt]);
 else
  return 0;
}

ApaintImage::ApaintImage()
{
 data = new ImageSharedData;
 data->nullImage=true;
 data->refCount=1;
}

ApaintImage::ApaintImage(const ApaintImage &other)
{
 data = other.data;
 data->refCount++;
}

ApaintImage::~ApaintImage()
{
 if (--data->refCount == 0)
 {  
  freeImageData();
  delete data;
 }
}

ApaintImage& ApaintImage::operator=(const ApaintImage& other)
{
 if (this!=&other)
  {
   other.data->refCount++;
   if (--data->refCount == 0)
   {
    freeImageData();
    delete data;
   }
   data=other.data;
  }
 return *this; 
}

void ApaintImage::freeImageData()
{
 if (!data->nullImage)
  {
   delete[] data->imageData;
   free(data->jumpTableData);
   data->nullImage=true;
  }
}


void ApaintImage::copyImageDataTo(ApaintImage *img)
{
 if (!img)
  return;
 memcpy(img->data->colorTableData, data->colorTableData, sizeof(QRgb)*256);
 memcpy(img->data->histogramNum, data->histogramNum, sizeof(int)*256);
 memcpy(img->data->histogramCol, data->histogramCol, sizeof(int)*256);
 memcpy(img->data->cmap, data->cmap, sizeof(unsigned char)*256*3);
 img->data->numColorsData=data->numColorsData;
 img->data->imageXdpi = data->imageXdpi;
 img->data->imageYdpi = data->imageYdpi;
 img->data->imageName = data->imageName;
}

ApaintImage ApaintImage::copy()
{
 return (copy(0, 0, width(), height()));
}

/*
ApaintImage ApaintImage::copyOffsetX(int x, int y, int w, int h, int offx)
{
 ApaintImage image;

 if ( w <= 0 || h <= 0 )
  return image;

 image.newImage(w, h, depth());
 if (x<offx)
  {
   int ow = offx-x;
   bitBlt(&image, 0, 0, this, width()-ow, y, ow, h, true, false, 0, 0, 0);
   bitBlt(&image, ow, 0, this, 0, y, w-ow, h, true, false, 0, 0, 0);
  }
 else
  bitBlt(&image, 0, 0, this, x-offx, y, w, h, true, false, 0, 0, 0);
 copyImageDataTo(&image);
 return image; 
}

ApaintImage ApaintImage::copyOffsetY(int x, int y, int w, int h, int offy)
{
  ApaintImage image;

  if ( w <= 0 || h <= 0 )
   return image;

  image.newImage(w, h, depth());
  if (y<offy)
  {
   int oh = offy-y;
   bitBlt(&image, 0, 0, this, x, height()-oh, w, oh, true, false, 0, 0, 0);
   bitBlt(&image, 0, oh, this, x, 0, w, h-oh, true, false, 0, 0, 0);
  }
  else
    bitBlt(&image, 0, 0, this, x, y-offy, w, h, true, false, 0, 0, 0);
  copyImageDataTo(&image);
  return image; 
}
*/

ApaintImage ApaintImage::copy(int x, int y, int w, int h)
{
  ApaintImage image;

  if ( w <= 0 || h <= 0 )
   return image;

  image.newImage(w, h, depth());

  /* clear the buffer so result will be predictable on short writes in bitBlt */  
  memset(image.scanLine(0), 0 , image.numBytes());

  bitBlt(&image, 0, 0, this, x, y, w, h, true, false, 0, 0, 0);
  copyImageDataTo(&image);
  return image;
}

void ApaintImage::horShift(int x)
{
  ApaintImage image;
  if (x<0 || x>width())
   return ;

  image.newImage(width(), height(), depth());
  bitBlt(&image, 0, 0, this, x, 0, width()-x, height(), true, false, 0, 0, 0);
  bitBlt(&image, width()-x, 0, this, 0, 0, x, height(), true, false, 0, 0, 0);
  copyImageDataTo(&image);
  operator=(image);
}

void ApaintImage::vertShift(int y)
{
 ApaintImage image;
 if (y<0 || y>height())
   return ;

  image.newImage(width(), height(), depth());
  bitBlt(&image, 0, 0, this, 0, y, width(), height()-y, true, false, 0, 0, 0);
  bitBlt(&image, 0, height()-y, this, 0, 0, width(), y, true, false, 0, 0, 0);
  copyImageDataTo(&image);
  operator=(image); 
}

ApaintImage ApaintImage::changeToBW(int treshHold)
{
 ApaintImage dImage;
 int x, y, size;
 uint whiteCol, blackCol, col;
 
 treshHold*=3;

 whiteCol = qRgb(255, 255, 255);
 blackCol = qRgb(0 ,0, 0);
 dImage.newImage( width(), height(), depth());
 dImage.setNumColors(2); 
 dImage.setColor(0, whiteCol);
 dImage.setColor(1, blackCol);
 
 for(x= 0; x<width(); x++)
  for (y=0; y<height(); y++)
  {
     col = color(*(scanLine(y)+x));
     size = qRed(col) + qBlue(col) + qGreen(col);
     if (size>=treshHold)
      *(dImage.scanLine(y)+x) = (uchar)0;
     else
      *(dImage.scanLine(y)+x) = (uchar)1;
  }
  dImage.data->imageXdpi = data->imageXdpi;
  dImage.data->imageYdpi = data->imageYdpi;
  dImage.data->imageName = data->imageName;
  return dImage;
}

    
ApaintImage ApaintImage::resize(int wid, int hei, uint bgCol)
{
  ApaintImage dImage;

  if (isNull())
   return(dImage);

  dImage.newImage( wid, hei, depth());
  dImage.fill(bgCol);
  copyImageDataTo(&dImage);
  bitBlt(&dImage, 0, 0, this, 0, 0, wid, hei, true, false, 0, 0, 0);
  return dImage;
}

void ApaintImage::mirrorHorizontal()
{
 flipImageX(data->imageData, type(),  width(), height(), bpl());
}

void ApaintImage::mirrorVertical()
{
 flipImageY(data->imageData, height(), bpl());
}

void ApaintImage::mirrorExplode()
{
 explodeImage(data->imageData, type(),  width(), height(), bpl());
}

void ApaintImage::mirrorFlip(bool noRotate)
{
 ApaintImage dImg;

 dImg.newImage(height(), width(), depth());
 flipImageXY(noRotate, data->imageData, dImg.data->imageData, type(), data->imgWid, data->imgHei, data->imageBpl);
 copyImageDataTo(&dImg);
 operator=(dImg); 
}

void ApaintImage::duplicate(int duplicateType)
{
 int doubleX = (duplicateType<DUPLICATE_Y1) ? 2 : 1;
 int doubleY = (duplicateType<DUPLICATE_Y1) ? 1 : 2;
 int minusX = (duplicateType==DUPLICATE_X3) ? 1 : 0;
 int minusY = (duplicateType==DUPLICATE_Y4) ? 1 : 0;
 ApaintImage dImg;

 dImg.newImage(width()*doubleX-minusX, height()*doubleY-minusY, depth());
 duplicateImage(duplicateType, data->imageData, dImg.data->imageData, type(), data->imgWid, data->imgHei, data->imageBpl);
 copyImageDataTo(&dImg);
 operator=(dImg); 
}

void ApaintImage::scale2X()
{
 ApaintImage dImg;

 dImg.newImage(width()*2, height()*2, depth());
 scale2XImage(data->imageData, dImg.data->imageData, type(), data->imgWid, data->imgHei, data->imageBpl, dImg.data->imageBpl);
 copyImageDataTo(&dImg);
 operator=(dImg); 
}

void ApaintImage::scale3X()
{
 ApaintImage dImg;

 dImg.newImage(width()*3, height()*3, depth());
 scale3XImage(data->imageData, dImg.data->imageData, type(), data->imgWid, data->imgHei, data->imageBpl, dImg.data->imageBpl);
 copyImageDataTo(&dImg);
 operator=(dImg); 
}

void ApaintImage::shrink2X()
{
 ApaintImage dImg;

 dImg.newImage(width()/2, height()/2, depth());
 shrink2XImage(data->imageData, dImg.data->imageData, type(), data->imgWid, data->imgHei, data->imageBpl, dImg.data->imageBpl);
 copyImageDataTo(&dImg);
 operator=(dImg); 
}

void ApaintImage::shrink3X()
{
 ApaintImage dImg;

 dImg.newImage(width()/3, height()/3, depth());
 shrink3XImage(data->imageData, dImg.data->imageData, type(), data->imgWid, data->imgHei, data->imageBpl, dImg.data->imageBpl);
 copyImageDataTo(&dImg);
 operator=(dImg); 
}

void ApaintImage::shrink4X()
{
 ApaintImage dImg;

 dImg.newImage(width()/4, height()/4, depth());
 shrink4XImage(data->imageData, dImg.data->imageData, type(), data->imgWid, data->imgHei, data->imageBpl, dImg.data->imageBpl);
 copyImageDataTo(&dImg);
 operator=(dImg); 
}

/* should also be used for undo of duplicate */
void ApaintImage::halve(int halveType)
{
 int halveX = (halveType==HALVE_X) ? 2 : 1;
 int halveY = (halveType==HALVE_Y) ? 2 : 1;
 ApaintImage dImg;

 dImg.newImage(width()/halveX, height()/halveY, depth());
 halveImage(halveType, data->imageData, dImg.data->imageData, data->imgHei, data->imageBpl, dImg.data->imageBpl);
 copyImageDataTo(&dImg);
 operator=(dImg); 
}

void ApaintImage::shrink(int shrinkType)
{
 int shrinkX = (shrinkType==SHRINK_X) ? 2 : 1;
 int shrinkY = (shrinkType==SHRINK_Y) ? 2 : 1;
 ApaintImage dImg;

 dImg.newImage(width()/shrinkX, height()/shrinkY, depth());
 shrinkImage(shrinkType, data->imageData, dImg.data->imageData, type(), data->imgWid, data->imgHei, data->imageBpl, dImg.data->imageBpl);
 copyImageDataTo(&dImg);
 operator=(dImg); 
}

void ApaintImage::multiplyX(int times)
{
 ApaintImage dImg;

 dImg.newImage(width()*times, height(), depth());
 multiplyXImage(times, data->imageData, dImg.data->imageData, type(), data->imgWid, data->imgHei, data->imageBpl, dImg.data->imageBpl);
 copyImageDataTo(&dImg);
 operator=(dImg); 
}

void ApaintImage::multiplyY(int times)
{
 ApaintImage dImg;

 dImg.newImage(width(), height()*times, depth());
 multiplyYImage(times, data->imageData, dImg.data->imageData, data->imgHei, data->imageBpl, dImg.data->imageBpl);
 copyImageDataTo(&dImg);
 operator=(dImg); 
}

// classical scale
ApaintImage ApaintImage::scale(int dw, int dh)
{
  int *mapBuf;
  int x, y, sh, sw;
  uchar *src, *dstLine, **srcLine;
  ApaintImage dImg;
  
  if ( dw <= 0 || dh <= 0 )
   return copy();

  dImg.newImage(dw, dh, depth());
  copyImageDataTo(&dImg);
  sw = width();
  sh = height();
  mapBuf = new int[dw];
  if (!mapBuf) return copy();

  for (x=0; x<dw; x++)
    mapBuf[x] =  (int)((sw*x)/dw);

  if (type() == LOAD_TRUE_COL)
   for (x=0; x<dw; x++)
      mapBuf[x]*=3;

  srcLine = jumpTable();
  for (y=0; y<dh; y++)
  {
    src = *(srcLine+(int)((sh*y)/dh));
    dstLine = dImg.scanLine(y);
    if (type() == LOAD_CMAP)
    {
      for (x=0; x<dw; x++)
      {
        *dstLine++ = src[mapBuf[x]];
      }
    }
    else if (type() == LOAD_TRUE_COL)
     {
      for (x=0; x<dw; x++)
       {
        *dstLine++ = src[mapBuf[x]];
        *dstLine++ = src[mapBuf[x]+1];
        *dstLine++ = src[mapBuf[x]+2];
       }
     }
  }
  delete[] mapBuf;
  
  return dImg;
}

/*
ApaintImage ApaintImage::downScaleWithRatio(int dw, int dh)
{
  double sx, sy;
  int ndw, ndh;

  if (dw>width() || dh>height())
   return scale(dw, dh);
   
  sx = width() / dw;
  sy = height() / dh;

  if (sx > sy)
  {
    ndh = (int)(height() / sx);
    ndw = dw;
  }
  else
  {
    ndw = (int)(width() / sy);
    ndh = dh;
  }

  ApaintImage scaledImg = scale(ndw, ndh);
  ApaintImage dImg;
  dImg.newImage(dw, dh, depth());
  memset(dImg.scanLine(0), 255, dImg.numBytes());
  bitBlt(&dImg, 0, 0, &scaledImg, 0, 0, scaledImg.width(), scaledImg.height(), false, false, 0, 0, 0);
  return dImg;
}
*/

// returns true-col image if we use filtering with rotation!!
ApaintImage ApaintImage::rotate(double angle, ResizeFilterType ftyp, bool usingMask, QImage &mask)
{
  QImage tmpMask;
  ApaintImage dImage;
  if ( isNull() )
    return dImage;
  int swid = width();
  int shei = height();
  double xyRatio;

  if ((xdpi()!=0) && (ydpi()!=0))
   xyRatio = (double)xdpi()/(double)ydpi();
  else
   xyRatio = 1.0;

  QRgb *ctab=0;
  if (type() == LOAD_CMAP)
    ctab = colorTable();
  QWMatrix mx, rmx;
  mx.reset();
  MatrixUtil::translatemx(&mx, -(swid/2), -(shei/2));
  MatrixUtil::rotatemx(&mx, angle, xyRatio);
  MatrixUtil::translatemx(&mx, (swid/2), (shei/2));
  rmx = QPixmap::trueMatrix( mx, swid, shei );
  QPointArray a( QRect(0, 0, swid, shei) );
  a = rmx.map( a );
  QRect dstRect = a.boundingRect().normalize();
  int dwid = dstRect.width();
  int dhei = dstRect.height();

  // create image
  if (ftyp == FILTER_NONE)
    dImage.newImage( dwid, dhei, depth());
  else
    dImage.newImage( dwid, dhei, 24);
  copyImageDataTo(&dImage);
  memset(dImage.scanLine(0), 0, dImage.numBytes());
  
  bool invertible;
  rmx = rmx.invert(&invertible);
  if (!invertible)
   return copy();

  // create mask
  if (usingMask)
  {
   tmpMask = QImage(dwid, dhei, 1, 2, QImage::LittleEndian);
   tmpMask.fill(0);
  }

  int *mapBuf = new int[swid];
  if (!mapBuf) return copy();

  for (int x=0; x<swid; x++)
      mapBuf[x]=3*x;
 
  if (ftyp == FILTER_NONE)
  {
     xFormHelper(rmx, depth(), dImage.scanLine(0), dImage.bytesPerLine(), dhei,
                       scanLine(0), bytesPerLine(), swid, shei, usingMask, mask, tmpMask);
     if (usingMask)
      mask = tmpMask;
    delete[] mapBuf;
    return dImage;
  }

  // else use bilinear filtering. It should do the job:)
  uchar *dstLine, *srcLine, *srcLine2; 
  double srcx, srcy;
  int isrcx, isrcy, isrcx2, isrcy2, x, y;
  for (y=0; y<dhei; y++)
  {
    dstLine = dImage.scanLine(y);
    if (type() == LOAD_TRUE_COL)
    {
      for (x = 0; x<dwid; x++)
      {
        srcx = rmx.m11()*x + rmx.m21()*y + rmx.dx();
        isrcx =(int) (srcx);
        srcx = srcx-isrcx;
        isrcx2 = isrcx+1;
        srcy = rmx.m12()*x + rmx.m22()*y + rmx.dy();
        isrcy =(int) (srcy);
        srcy = srcy-isrcy;
        isrcy2 = isrcy+1;
        if (isrcx >=0 && isrcx < swid && isrcy >=0 && isrcy < shei)
        {
          if (usingMask)
            if (PIX_ON(mask, isrcx, isrcy))
              SET_PIX(tmpMask, x, y);

          if (isrcx2 == swid) isrcx2 = swid-1;
          if (isrcy2 == shei) isrcy2 = shei-1;
          isrcx =mapBuf[isrcx];
          isrcx2 =mapBuf[isrcx2];          
          srcLine = scanLine(isrcy);
          srcLine2 = scanLine(isrcy2);
          *dstLine++ = QMAX(0, QMIN(255, (int)((1-srcx)*(1-srcy)*srcLine[isrcx] + (srcx)*(1-srcy)*srcLine[isrcx2] +
                       (1-srcx)*(srcy)*srcLine2[isrcx] + (srcx)*(srcy)*srcLine2[isrcx2])));
          *dstLine++ = QMAX(0, QMIN(255, (int)((1-srcx)*(1-srcy)*srcLine[isrcx+1] + (srcx)*(1-srcy)*srcLine[isrcx2+1] +
                       (1-srcx)*(srcy)*srcLine2[isrcx+1] + (srcx)*(srcy)*srcLine2[isrcx2+1])));
          *dstLine++ = QMAX(0, QMIN(255, (int)((1-srcx)*(1-srcy)*srcLine[isrcx+2] + (srcx)*(1-srcy)*srcLine[isrcx2+2] +
                       (1-srcx)*(srcy)*srcLine2[isrcx+2] + (srcx)*(srcy)*srcLine2[isrcx2+2]))); 
        }
        else
        {dstLine++; dstLine++; dstLine++;}
      }
    }
    else if (type() == LOAD_CMAP)
    {
      for (x = 0; x<dwid; x++)
      {
        srcx = rmx.m11()*x + rmx.m21()*y + rmx.dx();
        isrcx =(int) (srcx);
        srcx = srcx-isrcx;
        isrcx2 = isrcx+1;
        srcy = rmx.m12()*x + rmx.m22()*y + rmx.dy();
        isrcy =(int) (srcy);
        srcy = srcy-isrcy;
        isrcy2 = isrcy+1;
        if (isrcx >=0 && isrcx < swid && isrcy >=0 && isrcy < shei)
         {
          if (usingMask)
            if (PIX_ON(mask, isrcx, isrcy))
              SET_PIX(tmpMask, x, y);

          if (isrcx2 == swid) isrcx2 = swid-1;
          if (isrcy2 == shei) isrcy2 = shei-1;
          srcLine = scanLine(isrcy);
          srcLine2 = scanLine(isrcy2);
          *dstLine++ = QMAX(0, QMIN(255,(int)((1-srcx)*(1-srcy)*qRed(ctab[srcLine[isrcx]]) + (srcx)*(1-srcy)*qRed(ctab[srcLine[isrcx2]]) +
                       (1-srcx)*(srcy)*qRed(ctab[srcLine2[isrcx]]) + (srcx)*(srcy)*qRed(ctab[srcLine2[isrcx2]]))));
          *dstLine++ = QMAX(0, QMIN(255,(int)((1-srcx)*(1-srcy)*qGreen(ctab[srcLine[isrcx]]) + (srcx)*(1-srcy)*qGreen(ctab[srcLine[isrcx2]]) +
                       (1-srcx)*(srcy)*qGreen(ctab[srcLine2[isrcx]]) + (srcx)*(srcy)*qGreen(ctab[srcLine2[isrcx2]]))));
          *dstLine++ = QMAX(0, QMIN(255,(int)((1-srcx)*(1-srcy)*qBlue(ctab[srcLine[isrcx]]) + (srcx)*(1-srcy)*qBlue(ctab[srcLine[isrcx2]]) +
                       (1-srcx)*(srcy)*qBlue(ctab[srcLine2[isrcx]]) + (srcx)*(srcy)*qBlue(ctab[srcLine2[isrcx2]]))));
         }
        else
        { dstLine++; dstLine++; dstLine++; }
      }
    }
  }
  delete[] mapBuf;
  if (usingMask)
   mask = tmpMask;
  return dImage;
}

ApaintImage ApaintImage::perspectiveXForm(int dstw, int dsth, double mx[3][3], bool usingMask, QImage &mask)
{
  ApaintImage dImage;
  int x, y, sx, sy, sw, sh;
  double q;
  uchar *dstLine, *srcLine;
  
  sw = width();
  sh = height();
  
  dImage.newImage(dstw, dsth, depth());  
  copyImageDataTo(&dImage);
  if (type() == LOAD_CMAP)
    memset(dImage.scanLine(0), 0 , dImage.numBytes());
  else if (type() == LOAD_TRUE_COL)
    memset(dImage.scanLine(0), 255 , dImage.numBytes());
  
  if (usingMask)
  {
    mask = QImage(dstw, dsth, 1, 2, QImage::LittleEndian);
    mask.fill(1);    
  }
  
  for (y=0; y<dsth; y++)
  {
     dstLine = dImage.scanLine(y);
     if (type() == LOAD_CMAP)
     {
       for (x=0; x<dstw; x++)
       {
         q =  x*mx[0][2] + y*mx[1][2] + mx[2][2];
         sx = (int)((x*mx[0][0] + y*mx[1][0] + mx[2][0])/q);
         sy = (int)((x*mx[0][1] + y*mx[1][1] + mx[2][1])/q);
         if (sx>=0 && sx<sw && sy>=0 && sy<sh)
         {
           srcLine = scanLine(sy);
           dstLine[x] =  srcLine[sx];           
         }
       }
     }
     else if (type() == LOAD_TRUE_COL)
     {
       for (x=0; x<dstw; x++)
       {
         q =  x*mx[0][2] + y*mx[1][2] + mx[2][2];
         sx = (int)((x*mx[0][0] + y*mx[1][0] + mx[2][0])/q);
         sy = (int)((x*mx[0][1] + y*mx[1][1] + mx[2][1])/q);
         if (sx>=0 && sx<sw && sy>=0 && sy<sh)
         {
           srcLine = scanLine(sy);
           *dstLine++ =  srcLine[sx*3];
           *dstLine++ =  srcLine[sx*3+1];
           *dstLine++ =  srcLine[sx*3+2];           
         } 
       }
     }
  }  
  return dImage;
}

ApaintImage ApaintImage::xForm(const QWMatrix &mx, int dstWid, int dstHei, bool usingMask, QImage &mask)
{
  ApaintImage dImage;
  QImage tmpMask;
  
  if ( isNull() )
    return dImage;

  // source image data
  int ws = width();
  int hs = height();
  int sbpl = bytesPerLine();
  uchar *sptr = scanLine(0);

  // target image data
  int wd;
  int hd;
  int bpp = depth();

  // compute size of target image
  QWMatrix mat = QPixmap::trueMatrix( mx, ws, hs );
  if ( mat.m12() == 0.0F && mat.m21() == 0.0F )
  {
      if (!dstHei)
       hd = qRound( mat.m22() * hs );
      else
       hd = dstHei;

      if (!dstWid)
       wd = qRound( mat.m11() * ws );      
      else
       wd = dstWid;
       
      hd = QABS( hd );
      wd = QABS( wd );
  }
  else
  {     // rotation or shearing
      QPointArray a( QRect(0, 0, ws, hs) );
      a = mat.map( a );
      QRect r = a.boundingRect().normalize();
      wd = r.width();
      hd = r.height();
  }

  bool invertible;
  mat = mat.invert( &invertible );		// invert matrix
  if ( hd == 0 || wd == 0 || !invertible )	// error, return null image
    return dImage;

  dImage.newImage( wd, hd, depth());
  copyImageDataTo(&dImage);
  memset(dImage.scanLine(0), 255 , dImage.numBytes());

  // create mask
  if (usingMask)
  {
   tmpMask = QImage(wd, hd, 1, 2, QImage::LittleEndian);
   tmpMask.fill(0);
  }
  
  int dbpl = dImage.bytesPerLine();
  xFormHelper( mat, bpp, dImage.scanLine(0), dbpl, hd, sptr, sbpl, ws, hs, usingMask, tmpMask, mask);
  if (usingMask)
   mask = tmpMask;
  return dImage;  
}

// fast transform function
void ApaintImage::xFormHelper(QWMatrix &trueMat, int depth, uchar *dptr, int dbpl, int dHeight,
                              uchar *sptr, int sbpl, int sWidth, int sHeight, bool useMask, QImage &oldMask, QImage &mask)
{
   int m11 = (int)(trueMat.m11()*65536.0);
   int m12 = (int)(trueMat.m12()*65536.0);
   int m21 = (int)(trueMat.m21()*65536.0);
   int m22 = (int)(trueMat.m22()*65536.0);
   int dx  = (int)(trueMat.dx() *65536.0);
   int dy  = (int)(trueMat.dy() *65536.0);

   int m21ydx = dx + QABS(m11)/2;
   int m22ydy = dy + QABS(m22)/2;
   uint trigx;
   uint trigy;
   uint maxws = sWidth<<16;
   uint maxhs = sHeight<<16;
   int dstx;

   for ( int y=0; y<dHeight; y++ )
   {
      trigx = m21ydx;
      trigy = m22ydy;
      uchar *maxp = dptr + dbpl;
      dstx = 0;
       switch ( depth )
       {
         case 8:
         while ( dptr < maxp )
         {
           if ( trigx < maxws && trigy < maxhs )
           {
             if (useMask)
              if (PIX_ON(oldMask, trigx>>16, trigy>>16))
                SET_PIX(mask, dstx, y);
             *dptr = *(sptr+sbpl*(trigy>>16)+(trigx>>16));
           }
           trigx += m11;
           trigy += m12;
           dptr++;
           dstx++;
         }
         break;

         case 24:
         {
           uchar *p2;
           while ( dptr < maxp )
           {
             if ( trigx < maxws && trigy < maxhs )
             {
               if (useMask)
                 if (PIX_ON(oldMask, trigx>>16, trigy>>16))
                   SET_PIX(mask, dstx, y);
               p2 = sptr+sbpl*(trigy>>16) + ((trigx>>16)*3);
               dptr[0] = p2[0];
               dptr[1] = p2[1];
               dptr[2] = p2[2];
             }
             trigx += m11;
             trigy += m12;
             dptr += 3;
             dstx++;
           }
         }
         break;
        }
        m21ydx += m21;
        m22ydy += m22;
    }
}
                   

// if ctab is set noise must be 0, otherwise it won't work
void ApaintImage::setImageType(int imgType, QRgb *ctab, int numColors, int noise, bool dither)
{
  ApaintImage img;
  bool doSetupColTab;
  int curType=type();

  /* convert colormap image into grey one by just modifying the color palette */
  if ((imgType==LOAD_GREYSCALE) && (curType==LOAD_CMAP))
   {
    int i, val, numColors =  ApaintImage::numColors();
    QRgb color; 

    for (i=0; i<numColors; i++)
     {
      color = ApaintImage::color(i);
      val = (305*qRed(color) + 510*qGreen(color) + 205*qBlue(color) + 512)>>10;
      if (val>255)
       val=255;
      ApaintImage::setColor(i, qRgb(val, val, val));
     }
    return;
   }
  if (curType==imgType) return;
  
  if (curType==LOAD_TRUE_COL)
   {
    switch (imgType)
    { /* convert 24 bit per pixel image into greyscale colormap image */
      case LOAD_GREYSCALE:
       {
        int x,y, val;
        unsigned char *srcData, *dstData;

        img.newImage(width(), height(), 8);
        img.data->numColorsData=255;
	for (x=0; x<256; x++)
	 img.data->cmap[x*3] = img.data->cmap[x*3+1] = img.data->cmap[x*3+2] = x;
        for (y=0; y<height(); y++)
	 {
	  srcData = &data->imageData[y*data->imageBpl];
	  dstData = &img.data->imageData[y*img.data->imageBpl];
          for (x=width(); x; x--)
	   {
	    val =  305 * *srcData++; /* red */
	    val += 510 * *srcData++; /* green */
	    val += 205 * *srcData++; /* blue */
	    val >>= 10; /* shift instead of divide */
	    if (val>255)
	     val=255;
	    *dstData++ = val;
	   }
         }
        img.setUpColorTable(TRUE);
       }
      break;
      case LOAD_CMAP:
        img.newImage(width(), height(), 8);
        img.data->numColorsData=numColors;
        doSetupColTab = true;
        if (noise == 0)
        {
         if (dither)
          {
           if (ctab)
            {
              img.setExactColorTable(ctab, numColors);
              img.saveColorTable();
              doSetupColTab = false;
            }
           else
            colorQuantize(data->imageData, data->imageBpl, 0, 0, width(), height(), img.data->cmap, 0, &img.data->numColorsData);
            colorQuantizeDitherFS(data->imageData, data->imageBpl, img.data->imageData, img.data->imageBpl,
                                width(), height(),width(), height(), img.data->cmap, img.data->numColorsData, 16, 0);
          }
         else
          {
            if (ctab)
            {
              img.setExactColorTable(ctab, numColors);
              img.saveColorTable();
              trueColorRemapToCmap(data->imageData, data->imageBpl, img.data->imageData,img.data->imageBpl,
                                  width(), height(), img.data->cmap, img.data->numColorsData, 0);
              doSetupColTab = false;
            }
            else
              colorQuantize(data->imageData, data->imageBpl, img.data->imageData, img.data->imageBpl, width(), height(),
                          img.data->cmap, 0, &img.data->numColorsData);
          }
        }
       else
        {
          colorQuantize(data->imageData, data->imageBpl, 0, 0, width(), height(), img.data->cmap, 0, &img.data->numColorsData);
          trueColorRemapToCmap(data->imageData, data->imageBpl, img.data->imageData,img.data->imageBpl,
                        width(), height(), img.data->cmap, img.data->numColorsData, noise);

        }
        if (doSetupColTab)
         img.setUpColorTable(TRUE);

      break;
    }
   }
  else if (curType==LOAD_CMAP)
   {
    switch (imgType)
     {
      case LOAD_TRUE_COL:
       {
        img.newImage(width(), height(), 24);              
        cmap2TrueCol(img.data->imageData, scanLine(0));
	img.data->imageBpl = width()*3;
	data->imageBpl = width()*3;
        img.data->numColorsData=0;
       }
      break;
     }
   }

  img.data->imageXdpi = data->imageXdpi;
  img.data->imageYdpi = data->imageYdpi;
  img.data->imageName = data->imageName;
  operator=(img);
}

int ApaintImage::depth()
{
 switch (data->imageType)
  {
   case LOAD_BW:
    return 1;
   case LOAD_CMAP:
    return 8;
    break;
   case LOAD_TRUE_COL:
    return 24;
    break;
  }
 return 0; 
}

void ApaintImage::newImage(int wid, int hei, int depth)
{
  data->imgHei=hei;
  data->imgWid=wid;
  switch (depth)
   {
    case 24:
     data->imageType=LOAD_TRUE_COL;
     break;
    case 8:
     data->imageType=LOAD_CMAP;
     break;
    case 1:
     data->imageType=LOAD_BW;
     break;
    default:
     data->imageType=LOAD_TRUE_COL;
   }
  if (initImageLoad()==LOAD_OK)
   {    
    data->numColorsData=0;
    data->nullImage=false;
    data->imageName = QString(ApaintConfig::getText(10));
    data->imageXdpi = 100000;
    data->imageYdpi = 100000;
   }
  else
   data->nullImage=true;
}

void ApaintImage::newImageKeepName(int wid, int hei, int depth, ApaintImage *oldImg)
{
  data->imgHei=hei;
  data->imgWid=wid;
  switch (depth)
   {
    case 24:
     data->imageType=LOAD_TRUE_COL;
     break;
    case 8:
     data->imageType=LOAD_CMAP;
     break;
    case 1:
     data->imageType=LOAD_BW;
     break;
    default:
     data->imageType=LOAD_TRUE_COL;
   }
  if (initImageLoad()==LOAD_OK)
   {    
    data->numColorsData=0;
    data->nullImage=false;
    data->imageName = oldImg->data->imageName;
    data->imageXdpi = oldImg->data->imageXdpi;
    data->imageYdpi = oldImg->data->imageYdpi;
   }
  else
   data->nullImage=true;
}

int ApaintImage::initImageLoad()
{
 data->bandh = data->imgHei;

 if (!data->nullImage)
  freeImageData();

 switch (data->imageType)
  {
   case LOAD_BW:
    data->imageBpl = (data->imgWid+7)/8;
    break;
   case LOAD_CMAP:
    data->imageBpl = data->imgWid;    
    break;
   case LOAD_TRUE_COL:
    data->imageBpl = data->imgWid*3;
    break;
   case LOAD_RGBA:
    data->imageBpl = data->imgWid*4;
    break;
  }
 data->imageData = new uchar[data->imgHei*data->imageBpl];
 if (!data->imageData)
  return(LOAD_NO_MEMORY);

 data->jumpTableData = (uchar**)malloc(data->imgHei*sizeof(uchar*));
 for (int j=0; j<data->imgHei; j++)
  data->jumpTableData[j] = data->imageData+j*data->imageBpl;
 return(LOAD_OK);
}

/* data->jumpTableData should not be allocated or you have memory leak. */
/* used only at pasting image from clipboard becouse jumptable is pointer and data is not transfered! */
  
void ApaintImage::recalcJumpTable()
{
 data->jumpTableData = (uchar**)malloc(data->imgHei*sizeof(uchar*));
 for (int j=0; j<data->imgHei; j++)
  data->jumpTableData[j] = data->imageData+j*data->imageBpl;
}

int ApaintImage::bandImageLoad(int lines, char reverseLoad)
{
 return(LOAD_OK);
}

static void reverseImageY(unsigned char *data, int bpl, int sizeY)
{
 int y;
 unsigned char line[bpl], *data1, *data2;

 if (!data)
  return;
 for (y=0; y<(sizeY>>1); y++)
  {
   data1 = &data[y*bpl];
   data2 = &data[(sizeY-y-1)*bpl];
   memcpy(line, data1, bpl);
   memcpy(data1, data2, bpl);
   memcpy(data2, line, bpl);
  }
}

int ApaintImage::endImageLoad(int status, char reverseLoad)
{
 if (status==LOAD_OK)
  {
   data->nullImage=false;   
   if (reverseLoad)
    reverseImageY(data->imageData, data->imageBpl, data->imgHei);
   switch (data->imageType)
    {
     case LOAD_CMAP:
      setUpColorTable(!apaint->keepOriginalColormap());
      break;
     case LOAD_BW:
      bwImagToCmap();
      break;
     case LOAD_RGBA:
      data->imageData = collapseRGBA(data->imageData, data->imgWid, data->imgHei, &data->imageBpl, FALSE);
      data->imageType = LOAD_TRUE_COL;
      /* bpl has changed and data pointer was reallocated, so we need to recalculate the jump table */
      for (int j=0; j<data->imgHei; j++)
       data->jumpTableData[j] = data->imageData+j*data->imageBpl;
      /* no break on purpose */
     case LOAD_TRUE_COL:
      data->numColorsData=0;
      break;
    }
  }
 else
  freeImageData();
 return(status);
}

void ApaintImage::bwImagToCmap()
{
 ApaintImage dimg;
 int x, y;
 uchar *srcy, *dsty;

 if (type() != LOAD_BW) return;
 
 dimg.newImage(width(), height(), 8);
 dimg.data->numColorsData = 2;
 dimg.data->colorTableData[0] = qRgb(0, 0, 0);
 dimg.data->colorTableData[1] = qRgb(255, 255, 255);
 
 for ( y=0; y<height(); y++)
 {
   srcy =  &data->imageData[y*data->imageBpl];
   dsty = &dimg.data->imageData[y*data->imgWid];
   for ( x=0; x<width(); x++)
   if (srcy[x>>3] & (128 >> (x & 7)))
     dsty[x]= 0;
    else
     dsty[x] = 1;
 }
 operator=(dimg);
}


void ApaintImage::saveColorTable()
{
  int i;

  for (i=0; i<numColors(); i++)
  {
     data->cmap[i*3]=(unsigned char)qRed(data->colorTableData[i]);
     data->cmap[i*3+1]=(unsigned char)qGreen(data->colorTableData[i]);
     data->cmap[i*3+2]=(unsigned char)qBlue(data->colorTableData[i]);
  }
}

/*
int ApaintImage::saveFixColorTable(int num)
{
  int val, newNum;
  newNum = 0;
  for (int i=0; i<num; i++)
  {
     val = data->histogramNum[i];
     data->cmap[i*3]=(unsigned char)qRed(data->colorTableData[val]);
     data->cmap[i*3+1]=(unsigned char)qGreen(data->colorTableData[val]);
     data->cmap[i*3+2]=(unsigned char)qBlue(data->colorTableData[val]);
     newNum++;
  }
 return newNum;
}
*/

void ApaintImage::changeColtabToBW(int treshHold)
{
  uint whiteCol, blackCol;
  int size;

  if (data->imageType!=LOAD_CMAP) return;

  whiteCol = qRgb(255, 255, 255);
  blackCol = qRgb(0 ,0, 0);

  treshHold*=3;
  for (int i=0; i<256; i++)
  {
     size = qRed(data->colorTableData[i]) + qBlue(data->colorTableData[i]) + qGreen(data->colorTableData[i]);
     if (size>=treshHold)
       data->colorTableData[i] = whiteCol;
     else
       data->colorTableData[i] = blackCol;
  }
}


/* used col table must be size 0..255 */
void ApaintImage::getUsedColors(bool *usedCol, int &num, bool useMask, QImage *mask)
{
 int x, y;
 uchar *line;

 /* init data */
 for (int i=0; i<256; i++)
  usedCol[i] = false;  
 num=0;
 
 /* check the colors in image */
 for (y=0; y<height(); y++)
  {
   line=scanLine(y);
   if (useMask)
    for (x=0; x<width(); x++)
     {
      if (!PIX_ON(*mask, x, y))
       continue;
      usedCol[line[x]]=true;
     }
   else
    for (x=0; x<width(); x++)
     usedCol[*line++]=true;
  }
 for (int i=0; i<256; i++)
  if (usedCol[i])
   num++;
}

void ApaintImage::setUpColorTable(bool reduce)
{
  int num; 
  bool validColor[256];
  
  if (data->imageType!=LOAD_CMAP) return;

  if (reduce)
   getUsedColors(validColor, num, false, 0);
  resampleColorTable(validColor, reduce ? num : 256);
}

void ApaintImage::resampleColorTable(bool *validColor, int num)
{
  int i, val;
  RemapArray transTable;
  data->numColorsData=num;

  /*resample colorTable to start from 0 without gaps*/
  for (i=0, val = 0; i<256; i++)
   if ((num==256) || validColor[i])
    {
     transTable.setVal(i, val);
     data->colorTableData[val]=qRgb(data->cmap[i*3], data->cmap[i*3+1], data->cmap[i*3+2]);
     val++;
    }
  if (num<256)
   remapData(transTable);
  //refreshHistogram(); 
}

void ApaintImage::mergeColors()
{
 int j, i, newPtr;
 RemapArray transTable;
 bool validColor[256];
 QRgb newTab[256];

 if (data->imageType!=LOAD_CMAP) return; 

 transTable.setLinear();
 newPtr = 0;
 for (i=0; i<256; i++)
   validColor[i] = false;

 for (i=0; i<numColors(); i++)
 {
  if (!validColor[i])
  {
   validColor[i] = true;
   newTab[newPtr] = data->colorTableData[i];
   for (j=0; j<numColors(); j++)
     if (data->colorTableData[i] == data->colorTableData[j])
     {
      transTable.setVal(j, newPtr);
      validColor[j] = true;
     }
   newPtr++;
  }
 }
 for (i=0; i<newPtr; i++)
  data->colorTableData[i] = newTab[i];
  
 data->numColorsData = newPtr; 
 remapData(transTable);
 saveColorTable();
}

void ApaintImage::refreshHistogram(bool useMask, QImage *mask)
{
  int x,y, val;
  int maxPos, tmp;
  uchar *line;

  if (data->imageType!=LOAD_CMAP) return;

  /*init histogram*/
  for (x=0; x<256; x++)
  {
    data->histogramCol[x] = 0;
    data->histogramNum[x] = x;
  }
  /*get it...*/
  if (useMask && mask)
  {
    for (y=0; y<height(); y++)
    {
      line=scanLine(y);
      for (x=0; x<width(); x++)
        if (PIX_ON(*mask, x, y))
        {
          val=(int)*(line+x);
          data->histogramCol[val]++;
        }
    }
  }
  else
  {
    for (y=0; y<height(); y++)
    {
      line=scanLine(y);
      for (x=0; x<width(); x++)
      {
        val=(int)*(line+x);
        data->histogramCol[val]++;
      }
    }
  }
 /*sort histogram*/

 for (x=0; x<256; x++)
 {
  maxPos = x; 
  for (y = x; y<256; y++)
   if (data->histogramCol[maxPos]<data->histogramCol[y])
     maxPos = y;
  if (maxPos!=x)
  {
   tmp = data->histogramCol[x];
   data->histogramCol[x] = data->histogramCol[maxPos];
   data->histogramCol[maxPos] = tmp;
   tmp = data->histogramNum[x];
   data->histogramNum[x] = data->histogramNum[maxPos];
   data->histogramNum[maxPos] = tmp;
  }
 }
}

void ApaintImage::getHistogramData(int **histoNum, int **histoCol, bool useMask, QImage *mask)
{
  refreshHistogram(useMask, mask);
  *histoNum = (int *)data->histogramNum;
  *histoCol = (int *)data->histogramCol;
}

void ApaintImage::fill(uint col)
{
  fill(0, 0, width(), height(), col, false, 0);
}

void ApaintImage::fill(int xr, int yr, int w, int h, uint col, bool useMask, QImage *mask)
{
 int x,y;
 int r,g,b;
 uchar *dst;

 r=qRed(col);
 g=qGreen(col);
 b=qBlue(col);

 if (xr<0) xr = 0;
 if (yr<0) yr = 0;
 if (xr+w>width()) w = width() - xr;
 if (yr+h>height()) h = height() - yr;

 for (y=yr; y<(yr+h); y++)
  if (data->imageType==LOAD_TRUE_COL)
   {
    dst=scanLine(y)+xr*3;
    if (useMask)
     {
      for (x=0; x<w; x++)
       if (*(mask->scanLine(y-yr)+(x>>3)) & 1 << (x & 7))
        {
         *dst++=r;
         *dst++=g;
         *dst++=b;
        }
        else
       {dst++;dst++;dst++;}
     }
    else
     {
      if ((r==g)&&(g==b))
       {
	memset(dst, (uchar)r, w*3);
	*dst+=w*3;
       }
      else
       for (x=0; x<w; x++)
	{
         *dst++=r;
         *dst++=g;
         *dst++=b;
	}
     }
   }
  else if (data->imageType==LOAD_CMAP)
   {
    dst=scanLine(y)+xr;
    if (useMask)
     {
      for (x=0; x<w; x++)
       if (*(mask->scanLine(y-yr)+(x>>3)) & 1 << (x & 7))
        *dst++=(uchar)col;
       else
        dst++;
     }
    else
     memset(dst, (uchar)col, w);
   }
}

void ApaintImage::cmap2TrueCol(uchar *dst, uchar *src)
{
 for (int y=0; y<height(); y++)
  for (int x=0; x<width(); x++)
   {
    *dst++=qRed(data->colorTableData[*src]);
    *dst++=qGreen(data->colorTableData[*src]);
    *dst++=qBlue(data->colorTableData[*src]);    
    src++;
   }
}

int ApaintImage::indexLastUsedColor()
{
 int num;
 bool useColors[256];
 
 getUsedColors(useColors, num, false, 0);
 for (int i=255; i>=0; i--)
  if (useColors[i]) return i;

 return -1; //no used colors....like that could ever happen
}

bool ApaintImage::setExactNumColors(int num)
{
  if (num>indexLastUsedColor())
  {
   data->numColorsData = num;
   return true;
  }
  return false;
}

void ApaintImage::setNumColors(int i)
{
  int j, num;
  bool usedColors[256];
  
  if (i<0 || i>256) return;
  
  if (i>=numColors())
   {
    for (j=numColors(); j<i; j++)
      data->colorTableData[j]=qRgb(rand()&255, rand()&255, rand()&255);
    data->numColorsData=i; 
   }
  else
  {
    saveColorTable();
    /*first check if we have to do color quantization*/
    getUsedColors(usedColors, num, false, 0);
    if (num<=i)
    {
      resampleColorTable(usedColors, num);
      data->numColorsData = i;
      return;
    }
    /*we have to do color reduction*/
    colorQuantize(0, 0, data->imageData, data->imageBpl, width(), height(), data->cmap, 0, &i);
     /*refresh colorTable*/
    resampleColorTable(usedColors, num);
    data->numColorsData = i;
  }
}

void ApaintImage::removeUnusedColors()
{
  saveColorTable();
  setUpColorTable(TRUE);
}

void ApaintImage::setExactColorTable(QRgb *ctab, int num)
{
  if (type() != LOAD_CMAP) return;
  data->numColorsData = num;
  for (int i=0; i<num; i++)
   data->colorTableData[i] = ctab[i];
}

void ApaintImage::setColorTable(QRgb *ctab, int num)
{
 int i,j, minIndx ,d, mind;
 RemapArray transTab;

 for (i=0; i<numColors(); i++)
 {
  minIndx = 0;
  mind = getDistance(data->colorTableData[i], ctab[minIndx]);
  for (j=1; j<num; j++)
  {
   d = getDistance(data->colorTableData[i], ctab[j]);
   if (mind>d)
   {
     minIndx = j;
     mind = d;
   }
  }
  transTab.setVal(i, minIndx);
 }

 for (i=0; i<num; i++)
  data->colorTableData[i] = ctab[i];

 remapData(transTab);
 data->numColorsData = num;
}


QRgb ApaintImage::color(int i, int *realPos)
{
 

 if (i<numColors())
 {
  *realPos = i; 
  return (data->colorTableData[i]);
 }

 *realPos = 0;
 return (0);
}

QRgb ApaintImage::color(int i)
{
 if (i<numColors())
  return (data->colorTableData[i]);
 return (0);
}

void ApaintImage::setColor(int i, QRgb col)
{
 if (i<numColors())
   data->colorTableData[i]=col;
}

void ApaintImage::setRandomColors(int num)
{
 int ncol;
 
 if (numColors()<num)
  ncol = numColors();
 else
  ncol = num;

 for (int i=0; i<ncol; i++)
  data->colorTableData[i] = qRgb(rand()&255, rand()&255, rand()&255);
}

QRect ApaintImage::rect()
{
  return QRect(0, 0, width(), height());
}

QRgb ApaintImage::pixel(int x, int y)
{
 uchar *src;
 int srcIndx;

 switch (data->imageType)
 {
   case LOAD_TRUE_COL:
    src=(scanLine(y)+x*3);
    return (qRgb(*src, *(src+1), *(src+2)));
   case LOAD_CMAP:
    srcIndx= (int)*(scanLine(y)+x);
    return (data->colorTableData[srcIndx]);
 }
 return 0;
}

int ApaintImage::pixelIndex(int x, int y)
{
 if (data->imageType!=LOAD_CMAP) return(0);

 return((int)*(scanLine(y)+x));
}

void ApaintImage::setPixel(int x, int y, uint val)
{
 uchar *src;
 switch (data->imageType)
 {
   case LOAD_TRUE_COL:
    src=(scanLine(y)+x*3);
    *src=      qRed(val);
    *(src+1) = qGreen(val);
    *(src+2) = qBlue(val);
   return;
   case LOAD_CMAP:
    *(scanLine(y)+x)=(uchar)val;
   return;
 }
}

bool ApaintImage::mergeColorTables(ApaintImage *src, int *transTable)
{
 int srcNum = src->numColors();
 int dstNum = numColors();
 int i, j, numAvail;
 bool *colorAvailable;

 numAvail = srcNum;
 /*remove duplicated colors*/
 colorAvailable = new bool[srcNum];
 if (!colorAvailable)
 {
   qDebug("ApaintImage::mergeColorTables: pointer is NULL!");
   return false;
 }
 for (i=0; i<srcNum; i++)
  colorAvailable[i] = true;
 for (i = 0; i<dstNum; i++)
  for (j = 0; j<srcNum; j++)
   if (colorAvailable[j])
   {
     if (getDistance(color(i), src->color(j))==0)
     {
       colorAvailable[j] = false;
       numAvail--;
       transTable[j] = i;
     }
    }
 if (!numAvail)
 {
   delete[] colorAvailable;
   return true;
 }
 /*if too much color go back*/
 if (dstNum+numAvail>256)
 {
   delete[] colorAvailable;
   return false;
 }
 /*merge col tables*/
 i=0; j=dstNum; 
 while (i<srcNum)
 {
    if (colorAvailable[i])
    {
     data->colorTableData[j] = src->color(i);
     transTable[i] = j;
     j++;     
    }
    i++;
  }
 /*srcNum is decreased*/ 
 data->numColorsData = dstNum+numAvail;
 delete[] colorAvailable;
 return true;
}

void ApaintImage::remapData(RemapArray &transTable)
{
 uchar *limit, *dst;
 int len = data->imageBpl*data->imgHei;

 dst = data->imageData;
 limit = &dst[len];
 dst--;
 while (++dst < limit)
  *dst = transTable.getVal(*dst);
}

unsigned long ApaintImage::getDistance(QRgb col1, QRgb col2)
{
 int dr, dg, db;
 int r1, g1, b1;
 int r2, g2, b2;

 r1 = qRed(col1); g1 = qGreen(col1); b1 = qBlue(col1);
 r2 = qRed(col2); g2 = qGreen(col2); b2 = qBlue(col2);

 dr = abs (r1 - r2);
 dg = abs (g1 - g2);
 db = abs (b1 - b2);

 return(dr * dr + dg * dg + db * db);
}

bool ApaintImage::isNull()
{
 return(data->nullImage);
}

bool ApaintImage::load(const QString &fname)
{
 ApaintImageIO *io = ApaintImageIO::getObject();
 bool res;
 
 res=io->loadFile(fname.local8Bit().data());
 if (res)
  operator=(io->image());
 data->imageName=fname;
 return res;
}

bool ApaintImage::save(QString &fname)
{
 ApaintImageIO *io = ApaintImageIO::getObject();

 saveColorTable();
 if (!io->saveFile(fname.local8Bit().data(), this))
  return false;    
 data->imageName=fname;
 return true;
}

void ApaintImage::setChangePosVal(int changecount, int pos, uint val)
{
 data->changepos[changecount] = pos;
 if (type() == LOAD_TRUE_COL)
  {
   data->changeval[changecount*3] = val>>16;
   data->changeval[changecount*3+1] = (val>>8)&255;
   data->changeval[changecount*3+2] = val&255;
  }
 else if (type() == LOAD_CMAP)
  data->changeval[changecount] = (uchar)val;
}

/* flushAll at the end, otherwise only flush half as we go through the image, so that current operation does not */
/* influence the operations, which we are still going to do */
/* without flush all, do not flush the initial quarter of the changes, since they will influence the image on wrap around */
void ApaintImage::flushChanges(bool *protColors, bool flushAll)
{
 int i;
 int numFlush = flushAll ? data->changecount : data->changecount/2;
 int startFlush = flushAll ? 0 : data->changecount/4;
 int endFlush = startFlush + numFlush;

 if (type() == LOAD_CMAP)
  {
   if (protColors)
    {
     for (i = startFlush; i < endFlush; i++)
      if (!protColors[data->imageData[data->changepos[i]]])
       data->imageData[data->changepos[i]] = data->changeval[i];
    }
   else
    for (i = startFlush; i < endFlush; i++)
     data->imageData[data->changepos[i]] = data->changeval[i];
  }
 else
  for (i = startFlush; i < endFlush; i++)
   memcpy(&data->imageData[data->changepos[i]], &data->changeval[i*3], 3);

 if ((!flushAll) && (data->changecount))
  {
   if (type() == LOAD_CMAP)
    {
     memcpy(&data->changepos[startFlush], &data->changepos[numFlush+startFlush], sizeof(unsigned long)*(data->changecount-numFlush));
     memcpy(&data->changeval[startFlush], &data->changeval[numFlush+startFlush], sizeof(uchar)*data->changecount-numFlush);
    }
   else
    {
     memcpy(&data->changepos[startFlush], &data->changepos[numFlush+startFlush], sizeof(unsigned long)*(data->changecount-numFlush));
     memcpy(&data->changeval[startFlush*3], &data->changeval[(numFlush+startFlush)*3], 3*(data->changecount-numFlush));
    }
  }
 data->changecount -= numFlush;
}

bool ApaintImage::allocDrawBuff(int size)
{
 int bpp = depth()/8;
 data->changecount = 0;
 data->changepos = (long unsigned *) malloc (size * 2 * sizeof (unsigned long));
 data->changeval = (uchar *)malloc (size * 2 * bpp);
 if ((!data->changepos) || (!data->changeval))
  {
   freeDrawBuff();
   return false;
  }
 return true;
}

void ApaintImage::freeDrawBuff()
{
 if (data->changepos)
   free(data->changepos);
 if (data->changeval)
   free(data->changeval); 
}

uchar *ApaintImage::getCmap()
{
  saveColorTable();
  return(data->cmap);
}


QDataStream &operator<<( QDataStream &s, ApaintImage &aimg )
{
 s << (Q_INT32) aimg.width();
 s << (Q_INT32) aimg.height();
 s << (Q_INT32) aimg.depth();
 s << (Q_INT32) aimg.numColors();
 s.writeRawBytes((char *)aimg.data->colorTableData, 3*256*4);
 s.writeRawBytes((char *)aimg.scanLine(0),  aimg.bytesPerLine()*aimg.height());

 return s;
}

QDataStream &operator>>( QDataStream &s, ApaintImage &aimg )
{
 int bpl;
 Q_INT32 w, h, d, nc;
 s >> w;
 s >> h;
 s >> d;
 s >> nc;
 bpl = w*(d/8);
 aimg.newImage(w, h, d);
 s.readRawBytes((char *)aimg.data->colorTableData, 3*256*4);
 s.readRawBytes((char *)aimg.scanLine(0), bpl*h);
 aimg.data->numColorsData = nc;
 aimg.saveColorTable();

 return s;
}



/**********************************************************/
/****************PRINT DATA PRODUCER***********************/
/**********************************************************/

PrintDataProducer::PrintDataProducer(int pageWidth, int pageHeight, int bandHei, char imageType, bool landscape)
{
  mlandscape = landscape;

  mPageWidth = pageWidth;
  mPageHeight = pageHeight;  
  mbandHei = bandHei;
  mimgType = imageType;
  mprtStatus = PRT_OK;
  mFilename = "";
  mSaveToFile = false;
  initData();
}

PrintDataProducer::~PrintDataProducer()
{
 if (data)
  delete[] data;
}

void PrintDataProducer::initData()
{
 eod = false;
 mcurLine = toLine  = 0;
 switch (mimgType)
 {
   case IMG_BW:
    mdepth = 1;
    break;
   case IMG_CMAP:
    mdepth = 8;    
    break;
   case IMG_TRUE_COL:
    mdepth = 24;
    break;
 }
 if (mdepth == 1)
 {
  mbpp = 1;
  mbpl = (mPageWidth+7)/8;
 }
 else
 {
  mbpp = mdepth/8;
  mbpl = mbpp*mPageWidth;
 }

 data = new uchar[mbpl*mbandHei];
 if (!data)
 {
   qDebug("PrintDataProducer::initData() > pointer NULL!");
   eod = true;
   mbandHei = 0;
 }
 else
   mprtStatus = PRT_OK;
}

uchar *PrintDataProducer::curDataPtr()
{
 return data;
}

void PrintDataProducer::nextBand()
{

}

uchar *PrintDataProducer::getColorMap()
{
  return 0;
}

QRgb *PrintDataProducer::getColorTable()
{
  return 0;
}

int PrintDataProducer::getNumColors()
{
  return 0;
}

/**********************************************************/
/****************PRINT DATA PRODUCER IMAGE*****************/
/**********************************************************/

PrintDataProducerImage::PrintDataProducerImage(ApaintImage &img, int pageWidth, int pageHeight, int imgWidth, int imgHeight,
    int bandHei, char imageType, bool landscape):PrintDataProducer(pageWidth, pageHeight, bandHei, imageType, landscape)
{
  mImgWidth = imgWidth;
  mImgHeight =  imgHeight;
  mimg = img;
}

PrintDataProducerImage::~PrintDataProducerImage()
{

}

void PrintDataProducerImage::nextBand()
{
  int x, y, srcx, mapBufSize, srcImgWid;
  int *mapBufx;
  uchar **mapBufy=NULL;
  uchar *dstLine, *src;
  uchar **srcLine;
  QRgb *colTab;
  int pixSize;
  int treshHold = 127*3;

  if (mimg.isNull())
  {
   mprtStatus = PRT_INTERNAL_ERROR;
   eod = true;
  }
  if (mimg.type() == IMG_TRUE_COL && mimgType!=LOAD_TRUE_COL)
  {
   mprtStatus = PRT_WRONG_IMG;
   eod = true;
  }
  if (eod) return;

  colTab = mimg.colorTable();
  mcurLine = toLine;
  toLine = QMIN(mcurLine+mbandHei, mPageHeight);
  srcLine = mimg.jumpTable();

  if (mlandscape)
  {
    mapBufSize = mPageHeight;
    srcImgWid = mImgHeight;
  }
  else
  {
    mapBufSize = mPageWidth;
    srcImgWid = mImgWidth;
  }
 
  mapBufx = new int[mapBufSize];
  if (!mapBufx)
  {
    eod = true;
    return;
  }

  for (x=0; x<mapBufSize; x++)
    mapBufx[x] =  (int)((mimg.width()*(x%srcImgWid))/srcImgWid);

  if (mimg.type() == LOAD_TRUE_COL)
   for (x=0; x<mapBufSize; x++)
      mapBufx[x]*=3;

  if (mlandscape)
   {
    mapBufy = (uchar **)malloc(mPageWidth*sizeof(uchar*));
    if (!mapBufy)
     {
      eod = true;
      delete[] mapBufx;
      return;
     }
    for (x=0; x<mPageWidth; x++)
      mapBufy[x] = *(srcLine+(int)((mimg.height()*(mImgWidth-1-(x%mImgWidth)))/mImgWidth));
   }
    
  if (mimgType  == IMG_BW)
   memset(data, 0, mbandHei*mbpl);

  for (y=mcurLine; y<toLine; y++)
  {
    src = *(srcLine+(int)((mimg.height()*(y%mImgHeight))/mImgHeight));
    dstLine = data+(y-mcurLine)*mbpl;
    if (mimgType  == IMG_CMAP && mimg.type()== LOAD_CMAP)
    {
      for (x=0; x<mPageWidth; x++)
      {
       if (mlandscape)
        {
         src = mapBufy[x];
         srcx = y;
        }
       else
         srcx = x;
       *dstLine++ = src[mapBufx[srcx]];
      }
    }
    else if (mimgType  == IMG_BW && mimg.type()== LOAD_CMAP)
    {
      for (x=0; x<mPageWidth; x++)
      {
        if (mlandscape)
         {
          src = mapBufy[x];
          srcx = y;
         }
        else
          srcx = x;

        pixSize = qRed(colTab[src[mapBufx[srcx]]]) + qBlue(colTab[src[mapBufx[srcx]]]) + qGreen(colTab[src[mapBufx[srcx]]]);
        if (pixSize<treshHold)
          *(dstLine + (x >> 3)) |= (128 >> (x & 7));
      }
    }
    else if (mimgType  == IMG_TRUE_COL)
    {
      if (mimg.type() == LOAD_TRUE_COL)
      {
        for (x=0; x<mPageWidth; x++)
        {
          if (mlandscape)
           {
            src = mapBufy[x];
            srcx = y;
           }
          else
            srcx = x;

          *dstLine++ = src[mapBufx[srcx]];
          *dstLine++ = src[mapBufx[srcx]+1];
          *dstLine++ = src[mapBufx[srcx]+2];
        }
      }
      else if (mimg.type() == LOAD_CMAP)
      {
        for (x=0; x<mPageWidth; x++)
        {
          if (mlandscape)
           {
            src = mapBufy[x];
            srcx = y;
           }
          else
            srcx = x;

          *dstLine++ = qRed(colTab[src[mapBufx[srcx]]]);
          *dstLine++ = qGreen(colTab[src[mapBufx[srcx]]]);
          *dstLine++ = qBlue(colTab[src[mapBufx[srcx]]]);
        }
      }
    }
  }
  if (toLine == mPageHeight) eod = true;
  delete[] mapBufx;
  if (mlandscape)
    free(mapBufy);
  
  emit setProgress(this);
}
uchar *PrintDataProducerImage::getColorMap()
{
 if (mimg.type() != LOAD_CMAP)
  return 0;

 return mimg.getCmap();
}

QRgb *PrintDataProducerImage::getColorTable()
{
 if (mimg.type() != LOAD_CMAP)
  return 0;

 return mimg.colorTable(); 
}

int PrintDataProducerImage::getNumColors()
{
  if (mimg.type() != LOAD_CMAP)
  return 0;

 return mimg.numColors();
}

/**********************************************************/
/****************IMAGE LOADER******************************/
/**********************************************************/

extern "C" getPrinterInfoProc  getPrinterInfo;
extern "C" initPrintProc   initPrint;
extern "C" printBandProc   printBand;
extern "C" endPrintProc    endPrint;

extern "C" tryLoadProc tryLoad;
extern "C" loadBandProc loadBand;
extern "C" endLoadProc endLoad;

/* copy the file and set write permissions to the destination */
/* unused
static bool copyFile(char *file1, char *file2, int allowWriteToAll)
{
 FILE *inF, *outF;
 int c;

 if (equal(file1, file2))
  return(true);
 if (access(file1, R_OK)!=0)
  return(false);
 if ((inF = fopen(file1, "rb" )) == NULL)
  return(false);
 if ((outF = fopen(file2, "wb" )) == NULL)
  {
   fclose(inF);
   return(false);
  }
 while (!feof(inF))
  {
   c = fgetc(inF);
   if (!feof(inF))
    fputc(c, outF);
  }
 fflush(outF);
 fclose(inF);
 fclose(outF);
 if (allowWriteToAll)
  chmod(file2, 00666);
 return(true);
}
*/

static ApaintImageIO *objPtr = 0;

ApaintImageIO::ApaintImageIO()
{
  xmargin=ymargin=0;
  numPrinters = 0;
  pcapType = "";
  setupPrinterData();
  numLoaders = returnLoaderFunctions(-1, NULL);
}

ApaintImageIO::~ApaintImageIO()
{

}

ApaintImageIO *ApaintImageIO::getObject()
{
  if (!objPtr)
   objPtr = new ApaintImageIO();

  return objPtr;
}

void ApaintImageIO::setupPrinterData()
{
 numPrinters =  initPrintLib("../common/");
 pInfo = new PrinterInfo[numPrinters];
 if (!pInfo)
 {
   numPrinters = 0;
   return;
 }
 for (int i=0; i<numPrinters; i++)
  {
   setPrinterFunctions(i);
   getPrinterInfo(pInfo[i].prtModes, &pInfo[i].numModes, pInfo[i].prtDensityX, &pInfo[i].numDensitX, pInfo[i].prtDensityY,
   &pInfo[i].numDensitY, &pInfo[i].nonprintableTop, &pInfo[i].nonprintableBottom, &pInfo[i].nonprintableLeft,
   &pInfo[i].dpivariable, &pInfo[i].printerName, &pInfo[i].profileName);
  }
}

printerMode *ApaintImageIO::getPrinterModes(int i, int *num)
{
  if (i<0 || i>=numPrinters) return 0;
  *num = pInfo[i].numModes;
  return pInfo[i].prtModes;  
}

printerDensity *ApaintImageIO::getPrtDensityX(int i, int *num)
{
  if (i<0 || i>=numPrinters) return 0;
  *num = pInfo[i].numDensitX;
  return pInfo[i].prtDensityX;
}

printerDensity *ApaintImageIO::getPrtDensityY(int i, int *num)
{
  if (i<0 || i>=numPrinters) return 0;
  *num = pInfo[i].numDensitY;
  return pInfo[i].prtDensityY;
}

char *ApaintImageIO::getPrinterName(int i)
{
  if (i<0 || i>=numPrinters) return 0;
  return pInfo[i].printerName;
}

/*
char *ApaintImageIO::getProfileName(int i)
{
  if (i<0 || i>=numPrinters) return 0;
  return pInfo[i].profileName;
}
*/

int ApaintImageIO::getPrinterMode(int prt, int i)
{
  if (i<0 || i>=numPrinters) return 0;
  
  return pInfo[prt].prtModes[i].printType;
}

int ApaintImageIO::getDensityX(int prt, int i)
{
  if (i<0 || i>=numPrinters) return 0;

  return pInfo[prt].prtDensityX[i].dpi;
}

int ApaintImageIO::getDensityY(int prt, int i)
{
  if (i<0 || i>=numPrinters) return 0;

  return pInfo[prt].prtDensityY[i].dpi;
}

int ApaintImageIO::getNonPrintableLeft(int prt)
{
  return pInfo[prt].nonprintableLeft;
}

int ApaintImageIO::getNonPrintableTop(int prt)
{
  return pInfo[prt].nonprintableTop;
}
                
char const *ApaintImageIO::getImageInfo(char *filename, int *x, int *y, unsigned char *loaderID, int *coreFile)
{
 int i, numInterlace, status;
 struct stat statBuf;

 if (stat(filename, &statBuf))
  return("");
 else if (statBuf.st_size<MIN_IMAGE_FILSIZE)
  return("");

 if (access(filename, R_OK)==0)
  if (filename[strlen(filename)-1]!='/')
   {
    *loaderID=returnLoaderFunctions(0, filename);
    if (*loaderID!=UNKNOWN_LOADER) /* first try with the loader suggested on filename heuristics */
     {
      status = tryLoad(filename, &aimg.data->imageType, x, y, &aimg.data->imageXdpi, &aimg.data->imageYdpi, aimg.data->cmap, loaderInfo, &reverseLoad, &numInterlace, coreFile, 0);
      if (status == LOAD_OK)
       { endLoad(); return(loaderInfo); }
      else
       if (status == LOAD_NOT_IMAGE)
        return("");
     }
    /* then try all the other loaders */
    for (i=1; i<numLoaders; i++)
     if (i!=*loaderID) /* don't try with the same loader again */
      {
       returnLoaderFunctions(i, filename);
       status = tryLoad(filename, &aimg.data->imageType, x, y, &aimg.data->imageXdpi, &aimg.data->imageYdpi, aimg.data->cmap, loaderInfo, &reverseLoad, &numInterlace, coreFile, 0);
       if (status == LOAD_OK)
	{ endLoad(); *loaderID=i; return(loaderInfo); }
       else
	if (status == LOAD_NOT_IMAGE)
         return("");
      }
   }
 return("");
}

void ApaintImageIO::processBand()
{
 uchar *data;

 data=aimg.data->imageData;
 
 if (loadStatus!=LOAD_OK)
  {
   endLoad();
   aimg.endImageLoad(loadStatus, reverseLoad);
   return;
  }

 loadStatus = loadBand(data, aimg.data->bandh, aimg.data->imageBpl);
 if (loadStatus==LOAD_OK)
  loadStatus = aimg.bandImageLoad(aimg.data->bandh, reverseLoad);
 data = &data[aimg.data->imageBpl*aimg.data->bandh]; 
 endLoad();
 aimg.endImageLoad(loadStatus, reverseLoad);
}

int ApaintImageIO::loadImage(char *filename, int desiredimageType, unsigned char *loaderID)
{
 int i, dummy, numInterlace;
 struct stat statBuf;

 /* don't try to load files, which are too small to be an image */
 if (stat(filename, &statBuf))
  return(LOAD_NOT_IMAGE);
 else if (statBuf.st_size<MIN_IMAGE_FILSIZE)
  return(LOAD_NOT_IMAGE);
 else
  loadStatus = LOAD_OK;

 if (access(filename, R_OK)!=0)
  loadStatus = LOAD_IO_ERROR;
 else
  for (i=1; i<numLoaders; i++)
   {
    if ((i==1)&&(*loaderID!=UNKNOWN_LOADER))
     i= *loaderID;
    returnLoaderFunctions(i, filename);
    aimg.data->imageType = desiredimageType;
    loadStatus = tryLoad(filename, &aimg.data->imageType, &aimg.data->imgWid, &aimg.data->imgHei, &aimg.data->imageXdpi, &aimg.data->imageYdpi, aimg.data->cmap, loaderInfo, &reverseLoad, &numInterlace, &dummy, TRUE);
    if (loadStatus==LOAD_OK)
     {
      *loaderID = i;

      i=numLoaders; /* to exit loop */
      loadStatus = aimg.initImageLoad();
      if (loadStatus==LOAD_OK)
       processBand();
      else
       endLoad();
     }
    else
     if (loadStatus==LOAD_NOT_IMAGE)
      return(loadStatus);
   }
 if (aimg.data->imageXdpi<0)
  aimg.data->imageXdpi=0;
 if (aimg.data->imageYdpi<0)
  aimg.data->imageYdpi=0;

 if (loadStatus == LOAD_OK && (aimg.data->imageXdpi == 0 || aimg.data->imageYdpi == 0))
  aimg.data->imageXdpi = aimg.data->imageYdpi = 100000;

 return(loadStatus);
}

bool ApaintImageIO::loadFile(char *filename)
{
 int x, y, dummy;
 unsigned char loaderID;

 aimg = ApaintImage();
 if (strlen(getImageInfo(filename, &x, &y, &loaderID, &dummy)))
  if ((x<MAX_IMAGE)&&(y<MAX_IMAGE) && (x>0)&&(y>0))
   if (LOAD_OK==loadImage(filename, LOAD_TRUE_COL, &loaderID))
    return true;

  return false;
}

bool ApaintImageIO::getFileInfo(char *filename, char *loaderID, int *changeImg, int imgType)
{
  char *pfx;
  if (strlen(filename)==0) return false;
  QString qpfx = getPfx(filename);
  pfx = (char *)(qpfx.ascii());

  if (equal(pfx, "jpg") || equal(pfx, "jpeg"))
   {
    strcpy(loaderID, "JPEG");
    printType=IMG_TRUE_COL;
    *changeImg= LOAD_TRUE_COL;
   }
  else if (equal(pfx, "gif"))
   {
    strcpy(loaderID, "GIF");
    printType=IMG_CMAP;
    *changeImg= LOAD_CMAP;
   }
  else if (equal(pfx, "bmp"))
   {
    strcpy(loaderID, "BMP");
    printType=IMG_CMAP;
    *changeImg= LOAD_CMAP;
   }
  else if (equal(pfx, "tif") || equal(pfx, "tiff"))
   {
    strcpy(loaderID, "TIFF");
    if (imgType==LOAD_TRUE_COL)
     printType=IMG_TRUE_COL;
    else
     printType=imgType;
    *changeImg= imgType;
   }
  else 
   {
    strcpy(loaderID, "PNG");
    if (imgType==LOAD_TRUE_COL)
     printType=IMG_TRUE_COL;
    else
     printType=imgType;
    *changeImg= imgType;
   }
 return true;
}

void ApaintImageIO::setPrintCapType(QString pcap)
{
  pcapType = pcap;
}

void ApaintImageIO::selectPrinter(int printerID)
{
 setPrinterFunctions(printerID);
 getPrinterInfo(pInfo[printerID].prtModes, &pInfo[printerID].numModes, pInfo[printerID].prtDensityX,
                &pInfo[printerID].numDensitX, pInfo[printerID].prtDensityY, &pInfo[printerID].numDensitY,
                &pInfo[printerID].nonprintableTop, &pInfo[printerID].nonprintableBottom, &pInfo[printerID].nonprintableLeft,
                &pInfo[printerID].dpivariable, &pInfo[printerID].printerName, &pInfo[printerID].profileName);
 
}

bool ApaintImageIO::selectSaverPrinter(char *findPrinter)
{
 int i;
 for (i=0; i<numPrinters; i++)
 {
   if (equal(findPrinter, pInfo[i].printerName))
   {
    selectPrinter(i);
    return(true);
   }
 }
 return(false);
}

void ApaintImageIO::processSaveBand()
{
 int imgType;

 if (prtStatus!=PRT_OK)
  {
   prtStatus = endPrint(prtStatus);
   return;
  }
 if (aimg.data->imageType==LOAD_TRUE_COL)
  imgType=IMG_TRUE_COL;
 else
  imgType=aimg.data->imageType;

 if (prtStatus==PRT_OK)
  prtStatus = printBand(aimg.data->imgWid, (char *)aimg.data->imageData, aimg.data->imageBpl, 0, aimg.data->imgHei, imgType, aimg.data->cmap);
}

bool ApaintImageIO::saveFile(char *filename, ApaintImage *img)
{
 char id[6];
 int chgImgType;

 /* XXXXX should call function setEnqueuePrintFile() with proper parameters */
 setEnqueuePrintFile(filename, 0, 1);
 aimg=*img;
 if (!getFileInfo(filename, id, &chgImgType, aimg.data->imageType))
  return false;
 aimg.setImageType(chgImgType, 0, 256, 0, false);
 *img = aimg;

 if (!selectSaverPrinter(id))
   return false;
 prtStatus = initPrint(printType, 0, aimg.data->imgWid, aimg.data->imgHei, aimg.data->imageXdpi, aimg.data->imageYdpi, xmargin, ymargin);
 if (prtStatus==PRT_OK)
   processSaveBand();

 prtStatus = endPrint(prtStatus);

 /* should read .arahne configuration file and change permissions only if so configured */
 if (/* fixWritePermissions && */ (prtStatus==PRT_OK))
  if (chmod(filename, 00666)!=0)
   printf("ArahPaint: No write permissions %s\n", filename);

 return(prtStatus==PRT_OK?true:false);
}

void ApaintImageIO::processPrintBand(QPtrList<PrintDataProducer> &pdpList)
{
 PrintDataProducer *pdp;
 int lastCurLine = 0;
 
 if (prtStatus!=PRT_OK)
 {
  prtStatus = endPrint(prtStatus);
  return;
 }
 for (int i=0; i<(int)pdpList.count(); i++)
 {
   pdp = pdpList.at(i);
   while (!pdp->endOfData() && (prtStatus==PRT_OK))
   {     
    pdp->nextBand();    
    prtStatus = pdp->getPrtStatus();

    if (prtStatus == PRT_OK)
      prtStatus = printBand(pdp->pageWidth(), (char *)pdp->curDataPtr(), pdp->bpl(), lastCurLine+pdp->curLine(),
                            pdp->lastLine() - pdp->curLine(), pdp->printType(), pdp->getColorMap());
   }
   lastCurLine+=pdp->lastLine();
 }
}

bool ApaintImageIO::printFile(int printerID, QPtrList<PrintDataProducer> &pdpList, int numCopies,
                              int xdpi, int ydpi, int xmargin, int ymargin)
{
 QString prtCmd = "";
 char pcmd[500];
 char *prtFilename;
 int allHeight = 0;
 PrintDataProducer *pdp=NULL;

 /* XXXXX should call function setEnqueuePrintFile() with proper parameters */
 /* some of the code below should be changed */

 for (int i=0; i<(int)pdpList.count(); i++)
  {
   pdp = pdpList.at(i);
   allHeight+=pdp->pageHeight();
  }

 /* do a fancy true color to cmap conversion */
/* unfinished, need to fix processPrintBand stuff so that it will allocate full color image */
// trueColorToCmap = (printType==IMG_CMAP);
 
 if (pdp->saveToFile())
  prtFilename = (char *)pdp->getFilename().ascii();
 else
  {
   prtFilename = (char *) malloc(300);
   if (prtFilename)
    {
     strcpy(prtFilename, "/tmp/apaintXXXXXX");
     if (mkstemp(prtFilename)<1)
      {
       qDebug("ApaintImageIO::printFile: Can't create temporary file");
       return(false);
      } 
    }
  }

 if(!prtFilename)
  {
   qDebug("ApaintImageIO::printFile: NULL pointer");
   return(false);
  }
 if(!pdp->saveToFile())
  {
   prtCmd = ApaintConfig::getPrintCommand(pcapType);
   sprintf(pcmd, (char *)prtCmd.ascii(), numCopies);

   /* clear the raw option from print command, if we are printing to postscript printer */
   if (strstr(getPrinterName(printerID), "PostScript") || strstr(getPrinterName(printerID), "TIFF"))
    {
     int i;
     char *optionPos = strstr(pcmd, "-o raw");

     if (optionPos)
      for (i=0; i<6; i++)
       optionPos[i]=' ';

     /* add the cups options for positioning the image on the page */
     if (strstr(getPrinterName(printerID), "TIFF"))
      {
       char cupsPosition[200];
       
       sprintf(cupsPosition, " -o position=top-left -o page-left=%d -o page-top=%d -o ppi=%d ", xmargin*1000*72/xdpi, ymargin*1000*72/ydpi, xdpi/1000);
       strcat(pcmd, cupsPosition);
      }
    }
   setEnqueuePrintFile(0, pcmd, 1);
  }
 else
  setEnqueuePrintFile(prtFilename, 0, 0);
   
 selectPrinter(printerID);

 prtStatus = initPrint(trueColorToCmap ? IMG_TRUE_COL : pdp->printType(), 0, pdp->pageWidth(), allHeight, xdpi, ydpi, xmargin, ymargin);
 
 if (prtStatus==PRT_OK)
  processPrintBand(pdpList); 
 
 prtStatus = endPrint(prtStatus);
 return(prtStatus==PRT_OK?true:false);
}

QString getDirName(QString str)
{
 QString ret = "";
 int i = str.findRev("/");
 if (i>0)
  ret = str.left(i);
  
 return ret;
}

QString getPfx(char *name)
{
 QString pfx = QString(name);
 QStringList lst = QStringList::split(".",pfx);
 if (lst.count()<2)
  return QString("");

 return (lst[lst.count()-1].lower());
}

bool brushFilename(QString &name)
{
 return(strstr(name.ascii(), "brush") && strstr(name.ascii(), "apaint"));
}

bool fixPfx(QString &name)
{
 QStringList lst = QStringList::split(".",name);

 if (lst.count()<2)
  {
   /* if we are editing brush in temporary directory, do not add .png suffix */
   if (brushFilename(name))
    return true;
//   name.append(".png");
   name.append("."+ApaintConfig::getImageFormat());
   return true;
  }
 QString pfx = lst[lst.count()-1].lower();

 bool validPfx = (pfx == "gif") || (pfx == "png") || (pfx == "bmp") || (pfx == "tif") || (pfx == "tiff") || (pfx == "jpg") || (pfx == "jpeg");
 if (!validPfx)
  {
   /* if we are editing brush in temporary directory, do not add .png suffix */
   if (brushFilename(name))
    return true;
//   name.append(".png");
   name.append("."+ApaintConfig::getImageFormat());
   return true;
  }
 return false;
}

void chgPfx(QString &name, QString pfx)
{
  QStringList lst = QStringList::split(".",name);
  if (lst.count()<2)
   return;
  lst[lst.count()-1] = pfx;
  name = lst.join(".");
}

void bitBltAddCmap(ApaintImage *dst, int dx, int dy, ApaintImage *src, int sx, int sy, int sw, int sh,
           bool sameImage, bool useMask, QImage *mask, bool *protectCol, bool *transCol, int groundColors, int layerColors)
{
  int line;
  uchar **dstData, **srcData, **maskData;
 
  /*init*/
  maskData = 0;
  /*check params...*/
  if ( sw < 0 ) sw = src->width();
  if ( sh < 0 ) sh = src->height();
  if ( sx < 0 ) { dx -= sx; sw += sx; sx = 0; }
  if ( sy < 0 ) { dy -= sy; sh += sy; sy = 0; }
  if ( dx < 0 ) { sx -= dx; sw += dx; dx = 0; }
  if ( dy < 0 ) { sy -= dy; sh += dy; dy = 0; }
  if ( sx + sw > src->width() ) sw = src->width() - sx;
  if ( sy + sh > src->height() ) sh = src->height() - sy;
  if ( dx + sw > dst->width() ) sw = dst->width() - dx;
  if ( dy + sh > dst->height() ) sh = dst->height() - dy;
  if ( sw <= 0 || sh <= 0 ) return; // Nothing left to copy

 dstData=dst->jumpTable();
 srcData=src->jumpTable(); 
 if (useMask)
  maskData = mask->jumpTable();

 if (dst->type()!=LOAD_CMAP || src->type() != LOAD_CMAP)
  {
   int dstType = dst->type();

   if (dstType != LOAD_TRUE_COL)
    dst->setImageType(LOAD_TRUE_COL, 0, 0, 0, false);

   for (line=0; line<sh; line++)
    for (int x = 0; x<sw; x++)
     {
      if ((src->type() == LOAD_CMAP) && (transCol[*(*(srcData+line+sy)+(x+sx))]))
       continue;
      if (!useMask || (*(*(maskData+line+sy)+((x+sx)>>3)) & 1 << ((x+sx) & 7)))
       {
	QRgb c1 = dst->pixel(x+dx,line+dy);
	QRgb c2 = src->pixel(x+sx,line+sy);
	int red = (qRed(c1)+qRed(c2))>>1;
	int green = (qGreen(c1)+qGreen(c2))>>1;
	int blue = (qBlue(c1)+qBlue(c2))>>1;
	dst->setPixel(x+dx,line+dy,qRgb(red,green,blue));
       }
     }
   if (dstType != LOAD_TRUE_COL)
    dst->setImageType(LOAD_CMAP, 0, 256, 0, false);
   return;
  }

 for (line=0; line<sh; line++)
  {
   if (useMask)
    for (int x = 0; x<sw; x++)
     {
      if (protectCol)
        if (protectCol[*(*(dstData+line+dy)+(x+dx))])
         continue;
      if (transCol)
        if (transCol[*(*(srcData+line+sy)+(x+sx))])
         continue;

      if (*(*(maskData+line+sy)+((x+sx)>>3)) & 1 << ((x+sx) & 7))
       *(*(dstData+line+dy)+(x+dx)) = groundColors + *(*(dstData+line+dy)+(x+dx)) + (*(*(srcData+line+sy)+(x+sx)))*groundColors;
     }
   else
    for (int x = 0; x<sw; x++)
     {
      if (protectCol)
       if (protectCol[*(*(dstData+line+dy)+(x+dx))])
        continue;
      if (transCol)
       if (transCol[*(*(srcData+line+sy)+(x+sx))])
        continue;
       *(*(dstData+line+dy)+(x+dx)) = groundColors + *(*(dstData+line+dy)+(x+dx)) + (*(*(srcData+line+sy)+(x+sx)))*groundColors;
     }
  }
}

void bitBlt(ApaintImage *dst, int dx, int dy, ApaintImage *src, int sx, int sy, int sw, int sh,
            bool sameImage, bool useMask, QImage *mask, bool *protectCol, bool *transCol)
{
  int line, bpp, dstBpp;;
  uchar **dstData, **srcData, **maskData;
  int transTable[256];
  QRgb *colorTab=NULL;

  /*init*/
  maskData = 0;
  /*check params...*/
  if ( sw < 0 ) sw = src->width();
  if ( sh < 0 ) sh = src->height();
  if ( sx < 0 ) { dx -= sx; sw += sx; sx = 0; }
  if ( sy < 0 ) { dy -= sy; sh += sy; sy = 0; }
  if ( dx < 0 ) { sx -= dx; sw += dx; dx = 0; }
  if ( dy < 0 ) { sy -= dy; sh += dy; dy = 0; }
  if ( sx + sw > src->width() ) sw = src->width() - sx;
  if ( sy + sh > src->height() ) sh = src->height() - sy;
  if ( dx + sw > dst->width() ) sw = dst->width() - dx;
  if ( dy + sh > dst->height() ) sh = dst->height() - dy;
  if ( sw <= 0 || sh <= 0 ) return; // Nothing left to copy
  
  for (int i=0; i<256; i++)
   transTable[i]=i;

  if (src->depth()!=dst->depth())
  {
   switch (dst->type())
    {
     case  LOAD_TRUE_COL:
      if (src->type() == LOAD_CMAP)
       colorTab = src->colorTable();
     break;
     case LOAD_CMAP:
      /*VERY UGLY HACK ...MUST BE CHANGED IN THE FUTURE!*/
      dst->setImageType(LOAD_TRUE_COL, 0, 0, 0, false);
      bitBlt(dst, dx, dy, src, sx, sy, sw, sh, sameImage, useMask, mask, protectCol, transCol);
      dst->setImageType(LOAD_CMAP, 0, 256, 0, false);
      return;
    }
  }
  else if (!sameImage) /*if both imgs are cmap than we have to join their colormaps*/
   {
     if (dst->type()==LOAD_CMAP)
     {
      if (!dst->mergeColorTables(src, transTable))
      { /* same as above...very slow but it works good enough */
        dst->setImageType(LOAD_TRUE_COL, 0, 0, 0, false);
        bitBlt(dst, dx, dy, src, sx, sy, sw, sh, sameImage, useMask, mask, protectCol, transCol);
        dst->setImageType(LOAD_CMAP, 0, 256, 0, false);
        return;
      }
     }
  }

 dstData=dst->jumpTable();
 srcData=src->jumpTable(); 
 if (useMask)
   maskData = mask->jumpTable();

  bpp=src->depth()/8;
  dstBpp = dst->depth()/8;

  for (line=0; line<sh; line++)
  {
   if (useMask)
   {
     if (dst->type()==LOAD_CMAP)
     {
       for (int x = 0; x<sw; x++)
       {
         if (protectCol)
           if (protectCol[*(*(dstData+line+dy)+(x+dx))])
            continue;
         if (transCol)
           if (transCol[*(*(srcData+line+sy)+(x+sx))])
            continue;

         if (*(*(maskData+line+sy)+((x+sx)>>3)) & 1 << ((x+sx) & 7))
           *(*(dstData+line+dy)+(x+dx))= transTable[*(*(srcData+line+sy)+(x+sx))];
        }
     }
     else if (dst->type()==LOAD_TRUE_COL)
     {
       if (src->type()==LOAD_CMAP)
       {
         for (int x = 0; x<sw; x++)
         {
           if (transCol)
             if (transCol[*(*(srcData+line+sy)+(x+sx))])
              continue;
           if (*(*(maskData+line+sy)+((x+sx)>>3)) & 1 << ((x+sx) & 7))
           {
             *(*(dstData+line+dy)+(x+dx)*dstBpp) = qRed(colorTab[*(*(srcData+line+sy)+(x+sx))]);
             *(*(dstData+line+dy)+(x+dx)*dstBpp+1) = qGreen(colorTab[*(*(srcData+line+sy)+(x+sx))]);
             *(*(dstData+line+dy)+(x+dx)*dstBpp+2) = qBlue(colorTab[*(*(srcData+line+sy)+(x+sx))]);
           }
         }
       }
       else if (src->type() == LOAD_TRUE_COL)
       {
         for (int x = 0; x<sw; x++)
          if (*(*(maskData+line+sy)+((x+sx)>>3)) & 1 << ((x+sx) & 7))
           memcpy(*(dstData+line+dy)+(x+dx)*dstBpp, *(srcData+line+sy)+(x+sx)*bpp, bpp);
       }
     }
   }
   else
    {
     if (dst->type()==LOAD_CMAP)
      {
       unsigned char *sPtr = *(srcData+line+sy)+sx;
       unsigned char *dPtr = *(dstData+line+dy)+dx;

       if (protectCol || transCol)
	for (int x = 0; x<sw; x++)
         {
          if (protectCol)
           if (protectCol[dPtr[x]])
            continue;
          if (transCol)
           if (transCol[sPtr[x]])
            continue;

          dPtr[x]= transTable[sPtr[x]];
         }
        else
	 for (int x = sw; x; x--)
          *dPtr++= transTable[*sPtr++];
      }
     else if (dst->type()==LOAD_TRUE_COL)
     {
      if (src->type() == LOAD_CMAP)
       {
        unsigned char *sPtr = *(srcData+line+sy)+sx;
        unsigned char *dPtr = *(dstData+line+dy);

        if (transCol)
         for (int x = 0; x<sw; x++)
          {
           if (transCol[sPtr[x]])
            continue;
           dPtr[(x+dx)*dstBpp] = qRed(colorTab[sPtr[x]]);
           dPtr[(x+dx)*dstBpp+1] = qGreen(colorTab[sPtr[x]]);
           dPtr[(x+dx)*dstBpp+2] = qBlue(colorTab[sPtr[x]]);  
          }
	else
	 {
	  if (dstBpp==3)
	   {
	    dPtr += dx*dstBpp;
            for (int x = 0; x<sw; x++)
             {
              *dPtr++ = qRed(colorTab[sPtr[x]]);
              *dPtr++ = qGreen(colorTab[sPtr[x]]);
              *dPtr++ = qBlue(colorTab[sPtr[x]]);  
             }
	   }
	  else
           for (int x = 0; x<sw; x++)
            {
             dPtr[(x+dx)*dstBpp] = qRed(colorTab[sPtr[x]]);
             dPtr[(x+dx)*dstBpp+1] = qGreen(colorTab[sPtr[x]]);
             dPtr[(x+dx)*dstBpp+2] = qBlue(colorTab[sPtr[x]]);  
            }
	 }
       }
       else /* LOAD_TRUE_COL */
        {
         unsigned char *sPtr = *(srcData+line+sy);
         unsigned char *dPtr = *(dstData+line+dy);

         if (dstBpp==bpp)
          memcpy(&dPtr[dx*dstBpp], &sPtr[sx*bpp], bpp*sw);
	 else
          for (int x = 0; x<sw; x++)
           memcpy(&dPtr[(x+dx)*dstBpp], &sPtr[(x+sx)*bpp], bpp);
	}
      }
    }
  }
}

QImage flipQImage(QImage *src)
{
 QImage dImg; 
 dImg = QImage(src->height(), src->width(), src->depth(), src->numColors(), src->bitOrder());
 dImg.fill(0);
 
 int w=src->width();
 int h=src->height();

 for (int sy = 0; sy < h; sy++)
  for (int sx = 0; sx < w; sx++)
    if (*(src->scanLine(sy)+(sx>>3)) & 1 << (sx & 7))
      *(dImg.scanLine(sx)+(sy>>3)) |= 1 << (sy & 7);

 return dImg; 
}

QImage explodeQImage(QImage *src)
{
 QImage dImg; 
 int halfw = src->width()/2;
 int halfh = src->height()/2;
 int dx, dy;
 dImg = QImage(src->width(), src->height(), src->depth(), src->numColors(), src->bitOrder());
 dImg.fill(0);
 
 int w=src->width();
 int h=src->height();

 for (int sy = 0; sy < h; sy++)
  {
   dy = (sy + halfh)%h;
   dx = halfw;
   for (int sx = 0; sx < w; sx++, dx++)
    {
     if (dx>w)
      dx=0;
     if (*(src->scanLine(sy)+(sx>>3)) & 1 << (sx & 7))
       *(dImg.scanLine(dy)+(sx>>3)) |= 1 << (sx & 7);
    }
  }
 return dImg; 
}

/* duplicates the mask horizontally and vertically with halfdrop and mirror */
QImage duplicateQImage(int duplicateType, QImage *src)
{
 QImage dImg;
 int doubleX = (duplicateType<DUPLICATE_Y1) ? 2 : 1;
 int doubleY = (duplicateType<DUPLICATE_Y1) ? 1 : 2;
 int minusX = (duplicateType==DUPLICATE_X3) ? 1 : 0;
 int minusY = (duplicateType==DUPLICATE_Y4) ? 1 : 0;
 dImg = QImage(src->width()*doubleX-minusX, src->height()*doubleY-minusY, src->depth(), src->numColors(), src->bitOrder());
 dImg.fill(0);
 int xx, copyOffset=0;
 int swid=src->width();
 int shei=src->height();

 /* in all cases we first copy the original mask */
 /* vertical duplicate must keep the original at the bottom */
 if ((duplicateType>DUPLICATE_Y1) && (duplicateType<DUPLICATE_Y10))
  copyOffset = shei;
 if (duplicateType==DUPLICATE_Y4)
  copyOffset--;

 for (int sy = 0; sy < shei; sy++)
  for (int sx = 0; sx < swid; sx++)
   if (*(src->scanLine(sy)+(sx>>3)) & 1 << (sx & 7))
     *(dImg.scanLine(sy+copyOffset)+(sx>>3)) |= 1 << (sx & 7);

 switch (duplicateType)
  {
   case DUPLICATE_X1:
   case DUPLICATE_X10:
    for (int sy = 0; sy < shei; sy++)
     for (int sx = 0; sx < swid; sx++)
      if (*(src->scanLine(sy)+(sx>>3)) & 1 << (sx & 7))
       *(dImg.scanLine(sy)+((sx+swid)>>3)) |= 1 << ((sx+swid) & 7);
    break;
   case DUPLICATE_X2:
    for (int sy = 0; sy < shei; sy++)
     for (int sx = 0; sx < swid; sx++)
      if (*(src->scanLine(sy)+(sx>>3)) & 1 << (sx & 7))
       *(dImg.scanLine(sy)+((swid - sx - 1 +swid)>>3)) |= 1 << ((swid - sx - 1 + swid) & 7);
    break;
   case DUPLICATE_X3:
    for (int sy = 0; sy < shei; sy++)
     for (int sx = 0; sx < swid-1; sx++)
      if (*(src->scanLine(sy)+(sx>>3)) & 1 << (sx & 7))
       *(dImg.scanLine(sy)+((swid - sx - 2 +swid)>>3)) |= 1 << ((swid - sx - 2 + swid) & 7);
    break;
   case DUPLICATE_X4:
    for (int sy = 0; sy < shei; sy++)
     for (int sx = 0; sx < swid; sx++)
      if (*(src->scanLine(shei - sy - 1)+(sx>>3)) & 1 << (sx & 7))
       *(dImg.scanLine(sy)+((sx+swid)>>3)) |= 1 << ((sx+swid) & 7);
    break;
   case DUPLICATE_X5:
    for (int sy = 0; sy < shei; sy++)
     for (int sx = 0; sx < swid; sx++)
      if (*(src->scanLine(shei - sy - 1)+(sx>>3)) & 1 << (sx & 7))
       *(dImg.scanLine(sy)+((swid - sx - 1 + swid)>>3)) |= 1 << ((swid - sx - 1 + swid) & 7);
    break;
   case DUPLICATE_X6:
    for (int sy = 0; sy < shei; sy++)
     for (int sx = 0; sx < swid; sx++)
      if (*(src->scanLine((sy + shei/2)%shei)+(sx>>3)) & 1 << (sx & 7))
       *(dImg.scanLine(sy)+((sx+swid)>>3)) |= 1 << ((sx+swid) & 7);
    break;
   case DUPLICATE_X7:
    for (int sy = 0; sy < shei; sy++)
     for (int sx = 0; sx < swid; sx++)
      if (*(src->scanLine((sy + shei/2)%shei)+(sx>>3)) & 1 << (sx & 7))
       *(dImg.scanLine(sy)+((swid - sx - 1 + swid)>>3)) |= 1 << ((swid - sx - 1 + swid) & 7);
    break;
   case DUPLICATE_X8:
    for (int sy = 0; sy < shei; sy++)
     for (int sx = 0; sx < swid; sx++)
      if (*(src->scanLine((shei*3 + shei/2 - sy - 1)%shei)+(sx>>3)) & 1 << (sx & 7))
       *(dImg.scanLine(sy)+((sx+swid)>>3)) |= 1 << ((sx+swid) & 7);
    break;
   case DUPLICATE_X9:
    for (int sy = 0; sy < shei; sy++)
     for (int sx = 0; sx < swid; sx++)
      if (*(src->scanLine((shei*3 + shei/2 - sy - 1)%shei)+(sx>>3)) & 1 << (sx & 7))
       *(dImg.scanLine(sy)+((swid - sx - 1 + swid)>>3)) |= 1 << ((swid - sx - 1 + swid) & 7);
    break;

   case DUPLICATE_Y1:
    for (int sy = 0; sy < shei; sy++)
     for (int sx = 0; sx < swid; sx++)
      if (*(src->scanLine(sy)+(sx>>3)) & 1 << (sx & 7))
       *(dImg.scanLine(sy+shei)+(sx>>3)) |= 1 << (sx & 7);
    break;
   case DUPLICATE_Y2:
    for (int sy = 0; sy < shei; sy++)
     for (int sx = 0; sx < swid; sx++)
      if (*(src->scanLine(sy)+(sx>>3)) & 1 << (sx & 7))
       *(dImg.scanLine(sy)+((swid - sx - 1)>>3)) |= 1 << ((swid - sx - 1) & 7);
    break;
   case DUPLICATE_Y3:
    for (int sy = 0; sy < shei; sy++)
     for (int sx = 0; sx < swid; sx++)
      if (*(src->scanLine(shei - sy - 1)+(sx>>3)) & 1 << (sx & 7))
       *(dImg.scanLine(sy)+(sx>>3)) |= 1 << (sx & 7);
   case DUPLICATE_Y4:
    for (int sy = 0; sy < shei-1; sy++)
     for (int sx = 0; sx < swid; sx++)
      if (*(src->scanLine(sy+1)+(sx>>3)) & 1 << (sx & 7))
       *(dImg.scanLine(shei-sy-2)+(sx>>3)) |= 1 << (sx & 7);


    break;
   case DUPLICATE_Y5:
    for (int sy = 0; sy < shei; sy++)
     for (int sx = 0; sx < swid; sx++)
      {
       xx = sx+(swid>>1);
       if (xx>=swid)
	xx-=swid;
       if (*(src->scanLine(sy)+(sx>>3)) & 1 << (sx & 7))
	 *(dImg.scanLine(sy)+(xx>>3)) |= 1 << (xx & 7);
      }
    break;
   case DUPLICATE_Y6:
    for (int sy = 0; sy < shei; sy++)
     for (int sx = 0; sx < swid; sx++)
      {
       xx = sx+(swid>>1);
       if (xx>=swid)
	xx-=swid;
       xx = swid-xx-1;
       if (*(src->scanLine(sy)+(sx>>3)) & 1 << (sx & 7))
	*(dImg.scanLine(sy)+(xx>>3)) |= 1 << (xx & 7);
      }
    break;
   case DUPLICATE_Y7:
    for (int sy = 0; sy < shei; sy++)
     for (int sx = 0; sx < swid; sx++)
      {
       xx = sx+(swid>>1);
       if (xx>=swid)
	xx-=swid;
       if (*(src->scanLine(sy)+(sx>>3)) & 1 << (sx & 7))
	 *(dImg.scanLine(shei - sy - 1)+(xx>>3)) |= 1 << (xx & 7);
      }
    break;
   case DUPLICATE_Y8:
    for (int sy = 0; sy < shei; sy++)
     for (int sx = 0; sx < swid; sx++)
      {
       xx = sx+(swid>>1);
       if (xx>=swid)
	xx-=swid;
       xx = swid-xx-1;
       if (*(src->scanLine(sy)+(sx>>3)) & 1 << (sx & 7))
	 *(dImg.scanLine(shei - sy - 1)+(xx>>3)) |= 1 << (xx & 7);
      }
    break;
 }
 return dImg; 
}

#define A_BSRC ((sline0[x0>>3]&(1 << (x0 & 7)))!=0)
#define B_BSRC ((sline0[x >>3]&(1 << (x  & 7)))!=0)
#define C_BSRC ((sline0[x1>>3]&(1 << (x1 & 7)))!=0)
#define D_BSRC ((sline1[x0>>3]&(1 << (x0 & 7)))!=0)
#define E_BSRC ((sline1[x >>3]&(1 << (x  & 7)))!=0)
#define F_BSRC ((sline1[x1>>3]&(1 << (x1 & 7)))!=0)
#define G_BSRC ((sline2[x0>>3]&(1 << (x0 & 7)))!=0)
#define H_BSRC ((sline2[x >>3]&(1 << (x  & 7)))!=0)
#define I_BSRC ((sline2[x1>>3]&(1 << (x0 & 7)))!=0)

#define E1_BDST_SET1 dline0[(x<<1)>>3]     |=   (1 << ((x<<1)     & 7));
#define E2_BDST_SET1 dline0[((x<<1)+1)>>3] |=   (1 << (((x<<1)+1) & 7));
#define E3_BDST_SET1 dline1[(x<<1)>>3]     |=   (1 << ((x<<1)     & 7));
#define E4_BDST_SET1 dline1[((x<<1)+1)>>3] |=   (1 << (((x<<1)+1) & 7));
#define E1_BDST_SET0 dline0[(x<<1)>>3]     &= (~(1 << ((x<<1)     & 7)));
#define E2_BDST_SET0 dline0[((x<<1)+1)>>3] &= (~(1 << (((x<<1)+1) & 7)));
#define E3_BDST_SET0 dline1[(x<<1)>>3]     &= (~(1 << ((x<<1)     & 7)));
#define E4_BDST_SET0 dline1[((x<<1)+1)>>3] &= (~(1 << (((x<<1)+1) & 7)));
#define E1_BDST_SET(x) { if (x) E1_BDST_SET1 else E1_BDST_SET0 } 
#define E2_BDST_SET(x) { if (x) E2_BDST_SET1 else E2_BDST_SET0 } 
#define E3_BDST_SET(x) { if (x) E3_BDST_SET1 else E3_BDST_SET0 } 
#define E4_BDST_SET(x) { if (x) E4_BDST_SET1 else E4_BDST_SET0 } 

#define EE1_BDST_SET1 dline0[(x*3)>>3]     |= (1 << ((x*3)   & 7));
#define EE2_BDST_SET1 dline0[(x*3+1)>>3]   |= (1 << ((x*3+1) & 7));
#define EE3_BDST_SET1 dline0[(x*3+2)>>3]   |= (1 << ((x*3+2) & 7));
#define EE4_BDST_SET1 dline1[(x*3)>>3]     |= (1 << ((x*3)   & 7));
#define EE5_BDST_SET1 dline1[(x*3+1)>>3]   |= (1 << ((x*3+1) & 7));
#define EE6_BDST_SET1 dline1[(x*3+2)>>3]   |= (1 << ((x*3+2) & 7));
#define EE7_BDST_SET1 dline2[(x*3)>>3]     |= (1 << ((x*3)   & 7));
#define EE8_BDST_SET1 dline2[(x*3+1)>>3]   |= (1 << ((x*3+1) & 7));
#define EE9_BDST_SET1 dline2[(x*3+2)>>3]   |= (1 << ((x*3+2) & 7));
#define EE1_BDST_SET0 dline0[(x*3)>>3]     &= (~(1 << ((x*3)   & 7)));
#define EE2_BDST_SET0 dline0[(x*3+1)>>3]   &= (~(1 << ((x*3+1) & 7)));
#define EE3_BDST_SET0 dline0[(x*3+2)>>3]   &= (~(1 << ((x*3+2) & 7)));
#define EE4_BDST_SET0 dline1[(x*3)>>3]     &= (~(1 << ((x*3)   & 7)));
#define EE5_BDST_SET0 dline1[(x*3+1)>>3]   &= (~(1 << ((x*3+1) & 7)));
#define EE6_BDST_SET0 dline1[(x*3+2)>>3]   &= (~(1 << ((x*3+2) & 7)));
#define EE7_BDST_SET0 dline2[(x*3)>>3]     &= (~(1 << ((x*3)   & 7)));
#define EE8_BDST_SET0 dline2[(x*3+1)>>3]   &= (~(1 << ((x*3+1) & 7)));
#define EE9_BDST_SET0 dline2[(x*3+2)>>3]   &= (~(1 << ((x*3+2) & 7)));
#define EE1_BDST_SET(x) { if (x) EE1_BDST_SET1 else EE1_BDST_SET0 } 
#define EE2_BDST_SET(x) { if (x) EE2_BDST_SET1 else EE2_BDST_SET0 } 
#define EE3_BDST_SET(x) { if (x) EE3_BDST_SET1 else EE3_BDST_SET0 } 
#define EE4_BDST_SET(x) { if (x) EE4_BDST_SET1 else EE4_BDST_SET0 } 
#define EE5_BDST_SET(x) { if (x) EE5_BDST_SET1 else EE5_BDST_SET0 } 
#define EE6_BDST_SET(x) { if (x) EE6_BDST_SET1 else EE6_BDST_SET0 } 
#define EE7_BDST_SET(x) { if (x) EE7_BDST_SET1 else EE7_BDST_SET0 } 
#define EE8_BDST_SET(x) { if (x) EE8_BDST_SET1 else EE8_BDST_SET0 } 
#define EE9_BDST_SET(x) { if (x) EE9_BDST_SET1 else EE9_BDST_SET0 } 

/* scales the mask two times */
QImage scale2XQImage(QImage *src)
{
 QImage dImg;
 dImg = QImage(src->width()*2, src->height()*2, src->depth(), src->numColors(), src->bitOrder());
 dImg.fill(0);
 unsigned char *sline0, *sline1, *sline2;
 unsigned char *dline0, *dline1;
 int x0, x1;
 int swid=src->width();
 int shei=src->height();

 for (int y = 0; y < shei; y++)
  {
   sline0 = src->scanLine(((y-1)<0) ? shei-1 : y-1);
   sline1 = src->scanLine(y);
   sline2 = src->scanLine(((y+1) < shei) ? y+1 : 0);

   dline0 = dImg.scanLine(y<<1);
   dline1 = dImg.scanLine((y<<1) + 1);

   for (int x = 0; x < swid; x++)
    {
     if (x)
      x0=x-1;
     else
      x0 = swid-1;
     x1 = x+1;
     if (x1>=swid)
      x1=0;

     /* basic scale2x, thanks to http://scale2x.sourceforge.net */
     if (E_BSRC)
      {
       E1_BDST_SET1
       E2_BDST_SET1
       E3_BDST_SET1
       E4_BDST_SET1
      }
     if ( (B_BSRC!=H_BSRC) && (D_BSRC!=F_BSRC) )
      {
       if (D_BSRC==B_BSRC)
        E1_BDST_SET(D_BSRC)
       if (B_BSRC==F_BSRC)
        E2_BDST_SET(F_BSRC)
       if (D_BSRC==H_BSRC)
        E3_BDST_SET(D_BSRC)
       if (H_BSRC==F_BSRC)
        E4_BDST_SET(F_BSRC)
      }
  
      // Modification for smooth diagonals, not 2x2 jagged
      // XX?
      // .XX
      // ..X
      if ((A_BSRC==B_BSRC) && (A_BSRC==E_BSRC) && (A_BSRC==F_BSRC) && (A_BSRC==I_BSRC) &&
	  (A_BSRC!=D_BSRC) && (A_BSRC!=G_BSRC) && (A_BSRC!=H_BSRC))
       E3_BDST_SET(E_BSRC)
      // ?XX
      // XX.
      // X..
      if ((C_BSRC==B_BSRC) && (C_BSRC==E_BSRC) && (C_BSRC==D_BSRC) && (C_BSRC==G_BSRC) &&
	  (C_BSRC!=F_BSRC) && (C_BSRC!=H_BSRC) && (C_BSRC!=I_BSRC) )
       E4_BDST_SET(E_BSRC)

      // Motification to keep square angles, not eaten/rounded
      // ?..
      // .XX
      // .X?
      if ((E_BSRC==F_BSRC) && (E_BSRC==H_BSRC) &&
	  (E_BSRC!=B_BSRC) && (E_BSRC!=C_BSRC) && (E_BSRC!=D_BSRC) && (E_BSRC!=G_BSRC))
       E1_BDST_SET(E_BSRC)
      // ..?
      // XX.
      // ?X.
      if ((E_BSRC==D_BSRC) && (E_BSRC==H_BSRC) &&
	  (E_BSRC!=A_BSRC) && (E_BSRC!=B_BSRC) && (E_BSRC!=F_BSRC) && (E_BSRC!=I_BSRC))
       E2_BDST_SET(E_BSRC)
      // .X?
      // .XX
      // ?..
      if ((E_BSRC==B_BSRC) && (E_BSRC==F_BSRC) &&
	  (E_BSRC!=A_BSRC) && (E_BSRC!=D_BSRC) && (E_BSRC!=H_BSRC) && (E_BSRC!=I_BSRC))
       E3_BDST_SET(E_BSRC)
      // ?X.
      // XX.
      // ..?
      if ((E_BSRC==B_BSRC) && (E_BSRC==D_BSRC) &&
	  (E_BSRC!=C_BSRC) && (E_BSRC!=F_BSRC) && (E_BSRC!=G_BSRC) && (E_BSRC!=H_BSRC))
       E4_BDST_SET(E_BSRC)
    }
  }
 return dImg; 
}

/* scales the mask three times */
QImage scale3XQImage(QImage *src)
{
 QImage dImg;
 dImg = QImage(src->width()*3, src->height()*3, src->depth(), src->numColors(), src->bitOrder());
 dImg.fill(0);
 unsigned char *sline0, *sline1, *sline2;
 unsigned char *dline0, *dline1, *dline2;
 int x0, x1;
 int swid=src->width();
 int shei=src->height();

 for (int y = 0; y < shei; y++)
  {
   sline0 = src->scanLine(((y-1)<0) ? shei-1 : y-1);
   sline1 = src->scanLine(y);
   sline2 = src->scanLine(((y+1) < shei) ? y+1 : 0);

   dline0 = dImg.scanLine(y*3);
   dline1 = dImg.scanLine(y*3 + 1);
   dline2 = dImg.scanLine(y*3 + 2);

   for (int x = 0; x < swid; x++)
    {
     if (x)
      x0=x-1;
     else
      x0 = swid-1;
     x1 = x+1;
     if (x1>=swid)
      x1=0;

     /* basic scale3x, thanks to http://scale2x.sourceforge.net */
     if (E_BSRC)
      {
       EE1_BDST_SET1
       EE2_BDST_SET1
       EE3_BDST_SET1
       EE4_BDST_SET1
       EE5_BDST_SET1
       EE6_BDST_SET1
       EE7_BDST_SET1
       EE8_BDST_SET1
       EE9_BDST_SET1
      }
     if ( (B_BSRC!=H_BSRC) && (D_BSRC!=F_BSRC) )
      {
       if (D_BSRC==B_BSRC)
        EE1_BDST_SET(D_BSRC)
       if ( ( (D_BSRC==B_BSRC) && (E_BSRC!=C_BSRC) ) || ( (B_BSRC==F_BSRC) && (E_BSRC!=A_BSRC) ) )
        EE2_BDST_SET(B_BSRC)
       if (B_BSRC==F_BSRC)
        EE3_BDST_SET(F_BSRC)
       if ( ( (D_BSRC==B_BSRC) && (E_BSRC!=G_BSRC) ) || ( (D_BSRC==H_BSRC) && (E_BSRC!=A_BSRC) ) )
        EE4_BDST_SET(D_BSRC)
       if ( ( (B_BSRC==F_BSRC) && (E_BSRC!=I_BSRC) ) || ( (H_BSRC==F_BSRC) && (E_BSRC!=C_BSRC) ) )
        EE6_BDST_SET(F_BSRC)
       if (D_BSRC==H_BSRC)
        EE7_BDST_SET(D_BSRC)
       if ( ( (D_BSRC==H_BSRC) && (E_BSRC!=I_BSRC) ) || ( (H_BSRC==F_BSRC) && (E_BSRC!=G_BSRC) ) )
        EE8_BDST_SET(H_BSRC)
       if (H_BSRC==F_BSRC)
        EE9_BDST_SET(F_BSRC)
      }
    }  
  }
 return dImg; 
}

/* shrinks the mask 2 times in both directions */
QImage shrink2XQImage(QImage *src)
{
 QImage dImg;

 dImg = QImage(src->width()/2, src->height()/2, src->depth(), src->numColors(), src->bitOrder());
 dImg.fill(0);
 int dwid=dImg.width();
 int dhei=dImg.height();
 int numOnes;
 unsigned char *sline0, *sline1;
 unsigned char *dline;

 for (int y = 0; y < dhei; y++)
  {
   sline0 = src->scanLine(y<<1);
   sline1 = src->scanLine((y<<1)+1);
   dline = dImg.scanLine(y);
   for (int x = 0; x < dwid; x++)
    {
     numOnes=0;
     if (sline0[(x<<1)>>3]     & (1 <<  ((x<<1)    & 7)))
      numOnes++;
     if (sline0[((x<<1)+1)>>3] & (1 << (((x<<1)+1) & 7)))
      numOnes++;
     if (sline1[(x<<1)>>3]     & (1 <<  ((x<<1)    & 7)))
      numOnes++;
     if (sline1[((x<<1)+1)>>3] & (1 << (((x<<1)+1) & 7)))
      numOnes++;
     if (numOnes>1)
       dline[x>>3] |= 1 << (x & 7);
    }
  }
 return dImg; 
}

/* shrinks the mask 3 times in both directions */
QImage shrink3XQImage(QImage *src)
{
 QImage dImg;

 dImg = QImage(src->width()/3, src->height()/3, src->depth(), src->numColors(), src->bitOrder());
 dImg.fill(0);
 int dwid=dImg.width();
 int dhei=dImg.height();
 int numOnes;
 unsigned char *sline0, *sline1, *sline2;
 unsigned char *dline;


 for (int y = 0; y < dhei; y++)
  {
   sline0 = src->scanLine(y*3);
   sline1 = src->scanLine(y*3+1);
   sline2 = src->scanLine(y*3+2);
   dline = dImg.scanLine(y);
   for (int x = 0; x < dwid; x++)
    {
     numOnes=0;
     if (sline0[(x*3)>>3]   & (1 << ((x*3)   & 7)))
      numOnes++;
     if (sline0[(x*3+1)>>3] & (1 << ((x*3+1) & 7)))
      numOnes++;
     if (sline0[(x*3+2)>>3] & (1 << ((x*3+2) & 7)))
      numOnes++;
     if (sline1[(x*3)>>3]   & (1 << ((x*3)   & 7)))
      numOnes++;
     if (sline1[(x*3+1)>>3] & (1 << ((x*3+1) & 7)))
      numOnes++;
     if (sline1[(x*3+2)>>3] & (1 << ((x*3+2) & 7)))
      numOnes++;
     if (sline2[(x*3)>>3]   & (1 << ((x*3)   & 7)))
      numOnes++;
     if (sline2[(x*3+1)>>3] & (1 << ((x*3+1) & 7)))
      numOnes++;
     if (sline2[(x*3+2)>>3] & (1 << ((x*3+2) & 7)))
      numOnes++;
     if (numOnes>4)
       dline[x>>3] |= 1 << (x & 7);
    }
  }
 return dImg; 
}

/* shrinks the mask 4 times in both directions */
QImage shrink4XQImage(QImage *src)
{
 QImage dImg;

 dImg = QImage(src->width()/4, src->height()/4, src->depth(), src->numColors(), src->bitOrder());
 dImg.fill(0);
 int dwid=dImg.width();
 int dhei=dImg.height();
 int numOnes;
 unsigned char *sline0, *sline1, *sline2, *sline3;
 unsigned char *dline;

 for (int y = 0; y < dhei; y++)
  {
   sline0 = src->scanLine(y*4);
   sline1 = src->scanLine(y*4+1);
   sline2 = src->scanLine(y*4+2);
   sline3 = src->scanLine(y*4+3);
   dline = dImg.scanLine(y);
   for (int x = 0; x < dwid; x++)
    {
     numOnes=0;
     if (sline0[(x<<2)    >>3] & (1 << ((x<<2)     & 7)))
      numOnes++;
     if (sline0[((x<<2)+1)>>3] & (1 << (((x<<2)+1) & 7)))
      numOnes++;
     if (sline0[((x<<2)+2)>>3] & (1 << (((x<<2)+2) & 7)))
      numOnes++;
     if (sline0[((x<<2)+3)>>3] & (1 << (((x<<2)+3) & 7)))
      numOnes++;
     if (sline1[(x<<2)    >>3] & (1 << ((x<<2)     & 7)))
      numOnes++;
     if (sline1[((x<<2)+1)>>3] & (1 << (((x<<2)+1) & 7)))
      numOnes++;
     if (sline1[((x<<2)+2)>>3] & (1 << (((x<<2)+2) & 7)))
      numOnes++;
     if (sline1[((x<<2)+3)>>3] & (1 << (((x<<2)+3) & 7)))
      numOnes++;
     if (sline2[(x<<2)    >>3] & (1 << ((x<<2)     & 7)))
      numOnes++;
     if (sline2[((x<<2)+1)>>3] & (1 << (((x<<2)+1) & 7)))
      numOnes++;
     if (sline2[((x<<2)+2)>>3] & (1 << (((x<<2)+2) & 7)))
      numOnes++;
     if (sline2[((x<<2)+3)>>3] & (1 << (((x<<2)+3) & 7)))
      numOnes++;
     if (sline3[(x<<2)    >>3] & (1 << ((x<<2)     & 7)))
      numOnes++;
     if (sline3[((x<<2)+1)>>3] & (1 << (((x<<2)+1) & 7)))
      numOnes++;
     if (sline3[((x<<2)+2)>>3] & (1 << (((x<<2)+2) & 7)))
      numOnes++;
     if (sline3[((x<<2)+3)>>3] & (1 << (((x<<2)+3) & 7)))
      numOnes++;

     if (numOnes>7)
       dline[x>>3] |= 1 << (x & 7);
    }
  }
 return dImg; 
}

/* halves the mask horizontally and vertically */
QImage halveQImage(int halveType, QImage *src)
{
 QImage dImg;
 int halveX = (halveType==HALVE_X) ? 2 : 1;
 int halveY = (halveType==HALVE_Y) ? 2 : 1;
 dImg = QImage(src->width()/halveX, src->height()/halveY, src->depth(), src->numColors(), src->bitOrder());
 dImg.fill(0);
 int dwid=dImg.width();
 int dhei=dImg.height();

 /* in all cases just copy the first part of original mask, horizontally or vertically */
 for (int sy = 0; sy < dhei; sy++)
  for (int sx = 0; sx < dwid; sx++)
    if (*(src->scanLine(sy)+(sx>>3)) & 1 << (sx & 7))
      *(dImg.scanLine(sy)+(sx>>3)) |= 1 << (sx & 7);

 return dImg; 
}

/* shrink the mask horizontally and vertically */
QImage shrinkQImage(int shrinkType, QImage *src)
{
 QImage dImg;
 int shrinkX = (shrinkType==SHRINK_X) ? 2 : 1;
 int shrinkY = (shrinkType==SHRINK_Y) ? 2 : 1;
 dImg = QImage(src->width()/shrinkX, src->height()/shrinkY, src->depth(), src->numColors(), src->bitOrder());
 dImg.fill(0);
 int dwid=dImg.width();
 int dhei=dImg.height();

 if (shrinkType==SHRINK_X)
  {
   for (int sy = 0; sy < dhei; sy++)
    for (int sx = 0; sx < dwid; sx++)
      if (*(src->scanLine(sy)+((sx<<1)>>3)) & 1 << ((sx<<1) & 7))
	*(dImg.scanLine(sy)+(sx>>3)) |= 1 << (sx & 7);
  }
 else  // SHRINK_Y
  {
   for (int sy = 0; sy < dhei; sy++)
    for (int sx = 0; sx < dwid; sx++)
      if (*(src->scanLine(sy<<1)+(sx>>3)) & 1 << (sx & 7))
	*(dImg.scanLine(sy)+(sx>>3)) |= 1 << (sx & 7);
  }

 return dImg; 
}

/* multiply the mask horizontally */
QImage multiplyXQImage(int times, QImage *src)
{
 QImage dImg;
 dImg = QImage(src->width()*times, src->height(), src->depth(), src->numColors(), src->bitOrder());
 dImg.fill(0);

 for (int sy = 0; sy < src->height(); sy++)
  for (int sx = 0; sx < src->width(); sx++)
    if (*(src->scanLine(sy)+(sx>>3)) & 1 << (sx & 7))
      for (int xx = 0; xx < times; xx++)
       *(dImg.scanLine(sy)+((sx*times+xx)>>3)) |= 1 << ((sx*times+xx) & 7);
 return dImg; 
}

/* multiply the mask vertically */
QImage multiplyYQImage(int times, QImage *src)
{
 QImage dImg;
 dImg = QImage(src->width(), src->height()*times, src->depth(), src->numColors(), src->bitOrder());
 dImg.fill(0);

 for (int sy = 0; sy < src->height(); sy++)
  for (int yy = 0; yy < times; yy++)
   memcpy(dImg.scanLine(sy*times+yy), src->scanLine(sy), (src->width()+7)/8);
 return dImg; 
}

QImage cutCorners(QImage& src, QRect &copyRect)
{
 int sx=0, wid=0, sy=0, hei=0, x, y;
 bool notFound;
 
 /*must be bitmap*/ 
 if (src.depth()!=1) return src;

 /*cut Y*/
 notFound = true;
 for (y=0; y<src.height() && notFound; y++)
  for (x=0; x<src.width() && notFound; x++)
   {
    if (PIX_ON(src, x, y))
     {
      notFound = false;
      sy = y;
     }
   }
  notFound = true;
  for (y=src.height()-1; y>=0 && notFound; y--)  
   for (x=src.width()-1; x>=0 && notFound; x--)
    {
     if (PIX_ON(src, x, y))
      {
       notFound = false;
       hei = y-sy+1;
      }  
    }

  /*cut X*/
  notFound = true;
  for (x=0; x<src.width() && notFound; x++)
   for (y=0; y<src.height() && notFound; y++)
    {
     if (PIX_ON(src, x, y))
      {
       notFound = false;
       sx = x;
      }
    }
  notFound = true;
  for (x=src.width()-1; x>=0 && notFound; x--)
   for (y=src.height()-1; y>=0 && notFound; y--)
    {
     if (PIX_ON(src, x, y))
      {
       notFound = false;
       wid = x-sx+1;
      }
    }

  copyRect = QRect(sx, sy, wid, hei);
  QImage dimg = QImage(wid, hei, 1, 2, QImage::LittleEndian);
  dimg.fill(0);
  // some Qt operations need color table
  dimg.setColor(1, qRgb(0, 0, 0));
  dimg.setColor(0, qRgb(255, 255, 255));
  
  copy1BitImage(dimg, 0, 0, src, sx, sy, wid, hei, false, 0, 0);
  return dimg;
}

// dimg must prevoiusly be set to zero
void copy1BitImage(QImage& dimg, int dx, int dy,
                   QImage& simg, int sx, int sy, int sw, int sh,
                   bool useTransImage, ApaintImage *timg, bool *transCol)
{
  int x, y;
  

  if (useTransImage && !timg) return;
  if (dimg.isNull() || simg.isNull()) return;
  if (dimg.depth()!=1 || simg.depth()!=1) return;

  if ( sw < 0 ) sw = simg.width();
  if ( sh < 0 ) sh = simg.height();
  if ( sx < 0 ) { dx -= sx; sw += sx; sx = 0; }
  if ( sy < 0 ) { dy -= sy; sh += sy; sy = 0; }
  if ( dx < 0 ) { sx -= dx; sw += dx; dx = 0; }
  if ( dy < 0 ) { sy -= dy; sh += dy; dy = 0; }
  if ( sx + sw > simg.width() ) sw = simg.width() - sx;
  if ( sy + sh > simg.height() ) sh = simg.height() - sy;
  if ( dx + sw > dimg.width() ) sw = dimg.width() - dx;
  if ( dy + sh > dimg.height() ) sh = dimg.height() - dy;
  if ( sw <= 0 || sh <= 0 ) return; // Nothing left to copy
  
  uchar *dln, *sln, *imgLn;

  if (timg && timg->type() == LOAD_CMAP && useTransImage)
  {
    for (y=0; y<sh; y++)
    {
     dln = dimg.scanLine(dy+y);
     sln = simg.scanLine(sy+y);
     imgLn = timg->scanLine(sy+y);
     for (x=0; x<sw; x++)
     {
       if (!transCol[imgLn[x+sx]])
         if (sln[(x+sx)>>3] & (1 << ((x+sx)&7)))
          dln[(x+dx)>>3] |= 1 << ((x+dx)&7);
     }
    }
  }
  else 
  {
    for (y=0; y<sh; y++)
    {
     dln = dimg.scanLine(dy+y);
     sln = simg.scanLine(sy+y);
     for (x=0; x<sw; x++)
     {
      //if (sln[(x+sx)>>3] & (1 << ((x+sx)&7)))
      if (PIX_ON(simg, x+sx, y+sy))
       dln[(x+dx)>>3] |= 1 << ((x+dx)&7);
     }
    }
  }
}

QImage setMaskWithColor(ApaintImage& simg, uint bgCol)
{
  if (simg.isNull()) return QImage();
    
  QImage dmask = QImage(simg.width(), simg.height(), 1, 2, QImage::LittleEndian);
  dmask.fill(0);
  
  if (simg.type() == LOAD_TRUE_COL) return dmask;
  
  uchar *dln, *sln;
  QRgb *ctab = simg.colorTable();
  int x, y;
  
  for (y=0; y<simg.height(); y++)
  {
   dln = dmask.scanLine(y);
   sln = simg.scanLine(y);
   for (x=0; x<simg.width(); x++)
   {
     if (ctab[sln[x]] != bgCol)
      dln[x>>3] |= 1 << (x&7);
   }
  }
  return dmask;
}

// JUST FOR DOWNSCALING!!!
/* this function should be obsolete once we correct drawing with different pen x and y size */
QImage scaleQImage(int sx, int sy, QImage &src, QRect totalSize, QRect finalSize, int *srcZoomX, int *srcZoomY, int &dx, int &dy, int &dstw, int &dsth)
{
 //scale data
 int x, y;
 uchar *dline=NULL, *sline, *prevLine, *bline;

 if (!dstw || !dsth) return QImage();
 if (!srcZoomX || !srcZoomY)
 {
   qDebug("NULL pointer ApaintDraw::drawToImage: srcZoom");
   return QImage();
 }

 int dxe = (sx+src.width()) < totalSize.width() ? (sx+src.width()) : totalSize.width();
 int dye = (sy+src.height()) < totalSize.height() ? (sy+src.height()) : totalSize.height();

 dx = srcZoomX[sx];
 dy = srcZoomY[sy];
 dstw = srcZoomX[dxe - 1] - dx;
 dsth = srcZoomY[dye - 1] - dy;

 if ((!dstw)||(!dsth))
   return QImage();

 QImage dst = QImage(dstw, dsth, 1, 2, QImage::LittleEndian);
 QImage bufLine = QImage(dstw, 1, 1, 2, QImage::LittleEndian); 
 bline = bufLine.scanLine(0);
 memset(bline, 0, bufLine.bytesPerLine());
 prevLine = dst.scanLine(srcZoomY[sy]-dy);  
 for (y=sy; (y < dye) && ((srcZoomY[y]-dy) < dsth); y++)
  {
   dline = dst.scanLine(srcZoomY[y]-dy);
   sline = src.scanLine(y-sy);
   if (dline != prevLine)
    {
     memcpy(prevLine, bline, bufLine.bytesPerLine());
     memset(bline, 0, bufLine.bytesPerLine());
     prevLine = dline;
    }
   if (src.bitOrder() == QImage::LittleEndian)
    {
     for (x=sx; x < dxe; x++)
       if (*(sline+((x-sx)>>3)) & 1 << ((x-sx) & 7))
         *(bline+((srcZoomX[x]-dx)>>3)) |= 1 << ((srcZoomX[x]-dx) & 7);
    }
   else
    {
     for (x=sx; x < dxe ; x++)
      if (*(sline+((x-sx)>>3)) & 1<<(7-((x-sx) & 7)))
        *(bline+((srcZoomX[x]-dx)>>3)) |= 1 << ((srcZoomX[x]-dx) & 7);
    }
  }
 if (dline)
  memcpy(dline, bline, bufLine.bytesPerLine());
 return(dst); 
}

QImage invertQImage(QImage &src)
{
 QImage dst = QImage(src.width(), src.height(), 1, 2, QImage::LittleEndian);
 uchar *sline = src.scanLine(0);
 uchar *dline = dst.scanLine(0);

 for (int i=0; i<src.numBytes(); i++)
  {
   *dline = ~(*sline);
   dline++; sline++;
  }
 return dst;
}

void flipImageX(uchar *data, int type, int wid, int hei, int bpl)
{
 int x, y;
 uchar *data1, *data2;
 uchar *line;

 if (!data) return;
 line = (uchar *) malloc(bpl);
 if (!line) return;

 switch (type)
  {
   case LOAD_BW:
    for (y=0; y<hei; y++)
     {
      data1 = &data[y*bpl];
      memcpy(line, data1, bpl);
      memset(data1, 0, bpl);
      for (x=0; x<wid; x++)
       if (line[x>>3] & (1 << (x&7)))
        data1[(wid-x-1)>>3] |= (1 << ((wid-x-1)&7));
     }
    break;
   case LOAD_CMAP:
    for (y=0; y<hei; y++)
     {
      data1 = &data[y*bpl];
      data2 = data1+wid-1;
      for (x=wid/2; x; x--)
       { SWAP(*data1, *data2); data1++; data2--; }
     }
    break;
   case LOAD_TRUE_COL:
    for (y=0; y<hei; y++)
     {
      data1 = &data[y*bpl];
      data2 = data1+(wid-1)*3;
      for (x=wid/2; x; x--)
       { SWAP(*data1, *data2); data1++; data2++;
         SWAP(*data1, *data2); data1++; data2++;
         SWAP(*data1, *data2); data1++; data2-=5; }
      }
    break;
  }
 free(line);
}

void flipImageY(uchar *data, int hei, int bpl)
{
 int y;
 uchar *line, *data1, *data2;


 if (!data) return;
 line = (uchar *) malloc(bpl);
 if (!line) return;

 for (y=0; y<hei/2; y++)
  {
   data1 = &data[y*bpl];
   data2 = &data[(hei-y-1)*bpl];
   memcpy(line, data1, bpl);
   memcpy(data1, data2, bpl);
   memcpy(data2, line, bpl);
  }
 free(line);
}

void flipImageXY(bool noRotate, uchar *srcData, uchar *dstData, int type, int swid, int shei, int sbpl)
{
 int x, y, newBPL=3;
 uchar *data1, *data2, setBit;

 if (!srcData || !dstData) return;
 switch (type)
  {
   case LOAD_BW:
    newBPL = (shei+7)/8;
    for (y=0; y<shei; y++)
     {
      if (noRotate)
       data1 = &srcData[y*sbpl];
      else
       data1 = &srcData[(shei-y-1)*sbpl];
      data2 = &dstData[y>>3];
      setBit = 1 << (y&7);
      for (x=0; x<swid; x++, data2+=newBPL)
       if (data1[x>>3] & (1 << (x&7)))
         *data2 |= setBit;
      }
     break;
    case LOAD_CMAP:
     newBPL = shei;
     for (y=0; y<shei; y++)
      {
       if (noRotate)
	data1 = &srcData[y*sbpl];
       else
	data1 = &srcData[(shei-y-1)*sbpl];
	data2 = &dstData[y];
	for (x=0; x<swid; x++, data2+=newBPL)
         *data2=*data1++;
      }
     break;
    case LOAD_TRUE_COL:
     newBPL = shei*3;        
     for (y=0; y<shei; y++)
      {
       if (noRotate)
	data1 = &srcData[y*sbpl];
       else
	data1 = &srcData[(shei-y-1)*sbpl];
       data2 = &dstData[y*3];
       for (x=0; x<swid; x++, data2+=newBPL-2)
	{ *data2++=*data1++; *data2++=*data1++; *data2=*data1++; }
      }
     break;
  }
}

/* move the image so that center of the image becomes lower left point */
/* make it so you don't allocate whole image, but just one line */
/* for simplicity, make it two pass, first move it horizontally, and second vertically */
void explodeImage(uchar *data, int type, int wid, int hei, int bpl)
{
 int x, y, half;
 uchar *data1, *data2;
 uchar *line;

 if (!data) return;
 line = (uchar *) malloc(bpl);
 if (!line) return;

 /* first move it horizontally */
 switch (type)
  {
   case LOAD_BW:
    for (y=0; y<hei; y++)
     {
      data1 = &data[y*bpl];
      memcpy(line, data1, bpl);
      memset(data1, 0, bpl);
      half = wid/2;
      for (x=0; x<half; x++)
       if (line[x>>3] & (1 << (x&7)))
        data1[(half+x)>>3] |= (1 << ((half+x)&7));
      for (x=half; x<wid; x++)
       if (line[x>>3] & (1 << (x&7)))
        data1[(x-half)>>3] |= (1 << ((x-half)&7));
     }
    break;
   case LOAD_CMAP:
    for (y=0; y<hei; y++)
     {
      memcpy(line, &data[y*bpl], bpl);
      half = wid/2;
      memcpy(&data[y*bpl+half], line, wid-half);
      memcpy(&data[y*bpl], &line[half], half);
     }
    break;
   case LOAD_TRUE_COL:
    for (y=0; y<hei; y++)
     {
      memcpy(line, &data[y*bpl], bpl);
      half = (wid/2)*3;
      memcpy(&data[y*bpl+half], line, wid*3-half);
      memcpy(&data[y*bpl], &line[half], half);
     }
    break;
  }
 half = hei/2;
 /* now move it vertically */
 for (y=0; y<half; y++)
  {
   data1 = &data[y*bpl];
   data2 = &data[(half+y)*bpl];
   memcpy(line, data1, bpl);
   memcpy(data1, data2, bpl);
   memcpy(data2, line, bpl);
  }
 if (hei&1) /* on sizes not divisible by 2, we need to move the other half by one pixel */
  {
   memcpy(line, &data[(hei-1)*bpl], bpl); /* remember last line */
   for (y=half; y; y--)
    memcpy(&data[(half+y)*bpl], &data[(half+y-1)*bpl], bpl);
   memcpy(&data[half*bpl], line, bpl); /* copy last line to final destination */
  }
 free(line);
}

/* dupliates the image horizontally and vertically with mirroring and halfdrop */
void duplicateImage(int duplicateType, uchar *srcData, uchar *dstData, int type, int swid, int shei, int sbpl)
{
 int x, xx, y, dbpl, copyOffset=0;
 uchar *srcPtr, *srcPtr3, *dstPtr;
 int doubleX = (duplicateType<DUPLICATE_Y1) ? 2 : 1;
 int minusX = (duplicateType==DUPLICATE_X3) ? 1 : 0;

 if (!srcData || !dstData)
  return;

 dbpl = sbpl*doubleX;
 if (type==LOAD_CMAP)
  dbpl -= minusX;
 else
  dbpl -= minusX*3;

 /* in all cases we first copy the original image */
 /* vertical duplicate must keep the original at the bottom */
 if ((duplicateType>DUPLICATE_Y1) && (duplicateType<DUPLICATE_Y10))
  copyOffset = shei;
 if (duplicateType==DUPLICATE_Y4)
  copyOffset--;
 for (y=0; y<shei; y++)
  memcpy(&dstData[(y+copyOffset)*dbpl], &srcData[y*sbpl], sbpl);

 switch (duplicateType)
  {
   case DUPLICATE_X1:
    for (y=0; y<shei; y++)
     memcpy(&dstData[y*dbpl + sbpl], &srcData[y*sbpl], sbpl);
    break;

   case DUPLICATE_X2:
    for (y=0; y<shei; y++)
     {
      srcPtr = &srcData[y*sbpl];
      dstPtr = &dstData[y*dbpl + sbpl];
      if (type==LOAD_CMAP)
       for (x=0; x<swid; x++)
        *dstPtr++ = srcPtr[swid-x-1];
      else
       for (x=0; x<swid; x++)
        {
	 srcPtr3 = &srcPtr[(swid-x-1)*3];
         *dstPtr++ = *srcPtr3++;
         *dstPtr++ = *srcPtr3++;
         *dstPtr++ = *srcPtr3;
	}
     }
    break;

   case DUPLICATE_X3:
    for (y=0; y<shei; y++)
     {
      srcPtr = &srcData[y*sbpl];
      dstPtr = &dstData[y*dbpl + sbpl];
      if (type==LOAD_CMAP)
       for (x=1; x<swid; x++)
        *dstPtr++ = srcPtr[swid-x-1];
      else
       for (x=1; x<swid; x++)
        {
	 srcPtr3 = &srcPtr[(swid-x-1)*3];
         *dstPtr++ = *srcPtr3++;
         *dstPtr++ = *srcPtr3++;
         *dstPtr++ = *srcPtr3;
	}
     }
    break;

   case DUPLICATE_X4:
    for (y=0; y<shei; y++)
     memcpy(&dstData[(shei - y -1)*dbpl+sbpl], &srcData[y*sbpl], sbpl);
    break;

   case DUPLICATE_X5:
    for (y=0; y<shei; y++)
     {
      srcPtr = &srcData[y*sbpl];
      dstPtr = &dstData[(shei - y -1)*dbpl+sbpl];
      if (type==LOAD_CMAP)
       for (x=0; x<swid; x++)
        *dstPtr++ = srcPtr[swid-x-1];
      else
       for (x=0; x<swid; x++)
        {
	 srcPtr3 = &srcPtr[(swid-x-1)*3];
         *dstPtr++ = *srcPtr3++;
         *dstPtr++ = *srcPtr3++;
         *dstPtr++ = *srcPtr3;
	}
     }
    break;

   case DUPLICATE_X6:
    for (y=0; y<shei; y++)
     memcpy(&dstData[((y + shei/2)%shei)*dbpl+sbpl], &srcData[y*sbpl], sbpl);
    break;

   case DUPLICATE_X7:
    for (y=0; y<shei; y++)
     {
      srcPtr = &srcData[y*sbpl];
      dstPtr = &dstData[((y + shei/2)%shei)*dbpl + sbpl];
      if (type==LOAD_CMAP)
       for (x=0; x<swid; x++)
        *dstPtr++ = srcPtr[swid-x-1];
      else
       for (x=0; x<swid; x++)
        {
	 srcPtr3 = &srcPtr[(swid-x-1)*3];
         *dstPtr++ = *srcPtr3++;
         *dstPtr++ = *srcPtr3++;
         *dstPtr++ = *srcPtr3;
	}
     }
    break;

   case DUPLICATE_X8:
    for (y=0; y<shei; y++)
     memcpy(&dstData[((shei*3 + shei/2 - y - 1)%shei)*dbpl+sbpl], &srcData[y*sbpl], sbpl);
    break;

   case DUPLICATE_X9:
    for (y=0; y<shei; y++)
     {
      srcPtr = &srcData[y*sbpl];
      dstPtr = &dstData[((shei*3 + shei/2 - y - 1)%shei)*dbpl + sbpl];
      if (type==LOAD_CMAP)
       for (x=0; x<swid; x++)
        *dstPtr++ = srcPtr[swid-x-1];
      else
       for (x=0; x<swid; x++)
        {
	 srcPtr3 = &srcPtr[(swid-x-1)*3];
         *dstPtr++ = *srcPtr3++;
         *dstPtr++ = *srcPtr3++;
         *dstPtr++ = *srcPtr3;
	}
     }
    break;

   case DUPLICATE_X10:
    {
     int offsetX;
     int posx, posy;
     int dwid = swid*2;

     /* incorporates in one step: tilt right original size, duplicate horizontally, tilt down resulting size */
     for (y = 0; y < shei; y++)
      {
       offsetX = (int)(((shei-y-1)*swid*1.0)/shei+0.5);
       srcPtr = &srcData[y*sbpl];
       if (type==LOAD_CMAP)
	for (x = 0; x < dwid; x++)
	 {
	  posy = y - (int)((x*shei*1.0)/dwid+0.5);
	  while (posy<0)
	   posy+=shei;

	  posx = x+offsetX;
	  while (posx>=swid)
	   posx-=swid;

          dstData[posy*dbpl+x] = srcPtr[posx];
	 }
       else /* type==LOAD_TRUE_COL */
	for (x = 0; x < dwid; x++)
	 {
	  posy = y - (int)((x*shei*1.0)/dwid+0.5);
	  while (posy<0)
	   posy+=shei;

	  posx = x+offsetX;
	  while (posx>swid)
	   posx-=swid;
	  posx*=3;

          dstPtr = &dstData[posy*dbpl+x*3];
          *dstPtr++ = srcPtr[posx++];
          *dstPtr++ = srcPtr[posx++];
          *dstPtr = srcPtr[posx];
	 }
      }
     }
    break;

   case DUPLICATE_Y1:
     memcpy(&dstData[sbpl*shei], srcData, sbpl*shei);
    break;

   case DUPLICATE_Y2:
    for (y=0; y<shei; y++)
     {
      srcPtr = &srcData[y*sbpl];
      dstPtr = &dstData[y*dbpl];
      if (type==LOAD_CMAP)
       for (x=0; x<swid; x++)
        *dstPtr++ = srcPtr[swid-x-1];
      else
       for (x=0; x<swid; x++)
        {
	 srcPtr3 = &srcPtr[(swid-x-1)*3];
         *dstPtr++ = *srcPtr3++;
         *dstPtr++ = *srcPtr3++;
         *dstPtr++ = *srcPtr3;
	}
     }
    break;

   case DUPLICATE_Y3:
    for (y=0; y<shei; y++)
     memcpy(&dstData[(shei - y - 1)*dbpl], &srcData[y*sbpl], sbpl);
    break;
 
   case DUPLICATE_Y4:
    for (y=0; y<shei-1; y++)
     memcpy(&dstData[(shei - y - 2)*dbpl], &srcData[(y+1)*sbpl], sbpl);
    break;

   case DUPLICATE_Y5:
    for (y=0; y<shei; y++)
     {
      srcPtr = &srcData[y*sbpl];
      dstPtr = &dstData[(shei - y - 1)*dbpl];
      if (type==LOAD_CMAP)
       for (x=0; x<swid; x++)
        *dstPtr++ = srcPtr[swid-x-1];
      else
       for (x=0; x<swid; x++)
        {
	 srcPtr3 = &srcPtr[(swid-x-1)*3];
         *dstPtr++ = *srcPtr3++;
         *dstPtr++ = *srcPtr3++;
         *dstPtr++ = *srcPtr3;
	}
     }
    break;

   case DUPLICATE_Y6:
    for (y=0; y<shei; y++)
     {
      srcPtr = &srcData[y*sbpl];
      dstPtr = &dstData[y*dbpl];
      if (type==LOAD_CMAP)
       for (x=0; x<swid; x++)
        {
	 xx = x+(swid>>1);
	 if (xx>=swid)
	  xx-=swid;
         *dstPtr++ = srcPtr[xx];
	}
      else
       for (x=0; x<swid; x++)
        {
	 xx = x+(swid>>1);
	 if (xx>=swid)
	  xx-=swid;
	 srcPtr3 = &srcPtr[xx*3];
         *dstPtr++ = *srcPtr3++;
         *dstPtr++ = *srcPtr3++;
         *dstPtr++ = *srcPtr3;
	}
     }
    break;

   case DUPLICATE_Y7:
    for (y=0; y<shei; y++)
     {
      srcPtr = &srcData[y*sbpl];
      dstPtr = &dstData[y*dbpl];
      if (type==LOAD_CMAP)
       for (x=0; x<swid; x++)
        {
	 xx = x+(swid>>1);
	 if (xx>=swid)
	  xx-=swid;
	 xx = swid-xx-1;
         *dstPtr++ = srcPtr[xx];
	}
      else
       for (x=0; x<swid; x++)
        {
	 xx = x+(swid>>1);
	 if (xx>=swid)
	  xx-=swid;
	 xx = swid-xx-1;
	 srcPtr3 = &srcPtr[xx*3];
         *dstPtr++ = *srcPtr3++;
         *dstPtr++ = *srcPtr3++;
         *dstPtr++ = *srcPtr3;
	}
     }
    break;

   case DUPLICATE_Y8:
    for (y=0; y<shei; y++)
     {
      srcPtr = &srcData[y*sbpl];
      dstPtr = &dstData[(shei - y - 1)*dbpl];
      if (type==LOAD_CMAP)
       for (x=0; x<swid; x++)
        {
	 xx = x+(swid>>1);
	 if (xx>=swid)
	  xx-=swid;
         *dstPtr++ = srcPtr[xx];
	}
      else
       for (x=0; x<swid; x++)
        {
	 xx = x+(swid>>1);
	 if (xx>=swid)
	  xx-=swid;
	 srcPtr3 = &srcPtr[xx*3];
         *dstPtr++ = *srcPtr3++;
         *dstPtr++ = *srcPtr3++;
         *dstPtr++ = *srcPtr3;
	}
     }
    break;

   case DUPLICATE_Y9:
    for (y=0; y<shei; y++)
     {
      srcPtr = &srcData[y*sbpl];
      dstPtr = &dstData[(shei - y - 1)*dbpl];
      if (type==LOAD_CMAP)
       for (x=0; x<swid; x++)
        {
	 xx = x+(swid>>1);
	 if (xx>=swid)
	  xx-=swid;
	 xx = swid-xx-1;
         *dstPtr++ = srcPtr[xx];
	}
      else
       for (x=0; x<swid; x++)
        {
	 xx = x+(swid>>1);
	 if (xx>=swid)
	  xx-=swid;
	 xx = swid-xx-1;
	 srcPtr3 = &srcPtr[xx*3];
         *dstPtr++ = *srcPtr3++;
         *dstPtr++ = *srcPtr3++;
         *dstPtr++ = *srcPtr3;
	}
     }
    break;

   case DUPLICATE_Y10:
    {
     int offsetY;
     int posx, posy;
     int dhei = shei*2;

     /* incorporates in one step: tilt down original size, duplicate vertically, tilt right resulting size */
     for (x = 0; x < swid; x++)
      {
       offsetY = (int)(((swid-x-1)*shei*1.0)/swid+0.5);
       if (type==LOAD_CMAP)
	for (y = 0; y < dhei; y++)
	 {
	  posx = x - (int)((y*swid*1.0)/dhei+0.5);
	  while (posx<0)
	   posx+=swid;

	  posy = y+offsetY;
	  while (posy>=shei)
	   posy-=shei;

          dstData[y*dbpl+posx] = srcData[posy*sbpl+x];
	 }
       else /* type==LOAD_TRUE_COL */
	for (y = 0; y < dhei; y++)
	 {
	  posx = x - (int)((y*swid*1.0)/dhei+0.5);
	  while (posx<0)
	   posx+=swid;

	  posy = y+offsetY;
	  while (posy>=shei)
	   posy-=shei;

          dstPtr = &dstData[y*dbpl+posx*3];
          srcPtr = &srcData[posy*sbpl+x*3];
          *dstPtr++ = *srcPtr++;
          *dstPtr++ = *srcPtr++;
          *dstPtr = *srcPtr;
	 }
      }
     }
    break;
  }
}

/* macros for access of colormap image */
#define A_SRC (sline0[x0])
#define B_SRC (sline0[x])
#define C_SRC (sline0[x1])
#define D_SRC (sline1[x0])
#define E_SRC (sline1[x])
#define F_SRC (sline1[x1])
#define G_SRC (sline2[x0])
#define H_SRC (sline2[x])
#define I_SRC (sline2[x1])

#define E1_DST dline0[x<<1]
#define E2_DST dline0[(x<<1)+1]
#define E3_DST dline1[x<<1]
#define E4_DST dline1[(x<<1)+1]

#define EE1_DST dline0[x*3]
#define EE2_DST dline0[x*3+1]
#define EE3_DST dline0[x*3+2]
#define EE4_DST dline1[x*3]
#define EE5_DST dline1[x*3+1]
#define EE6_DST dline1[x*3+2]
#define EE7_DST dline2[x*3]
#define EE8_DST dline2[x*3+1]
#define EE9_DST dline2[x*3+2]

/* macros for access of true color image image */
#define A_SRC3 ((sline0[x0*3]<<16) | (sline0[x0*3+1]<<8) | (sline0[x0*3+2]<<8))
#define B_SRC3 ((sline0[x*3 ]<<16) | (sline0[x*3 +1]<<8) | (sline0[x*3 +2]<<8))
#define C_SRC3 ((sline0[x1*3]<<16) | (sline0[x1*3+1]<<8) | (sline0[x1*3+2]<<8))
#define D_SRC3 ((sline1[x0*3]<<16) | (sline1[x0*3+1]<<8) | (sline1[x0*3+2]<<8))
#define E_SRC3 ((sline1[x*3 ]<<16) | (sline1[x*3 +1]<<8) | (sline1[x*3 +2]<<8))
#define F_SRC3 ((sline1[x1*3]<<16) | (sline1[x1*3+1]<<8) | (sline1[x1*3+2]<<8))
#define G_SRC3 ((sline2[x0*3]<<16) | (sline2[x0*3+1]<<8) | (sline2[x0*3+2]<<8))
#define H_SRC3 ((sline2[x*3 ]<<16) | (sline2[x*3 +1]<<8) | (sline2[x*3 +2]<<8))
#define I_SRC3 ((sline2[x1*3]<<16) | (sline2[x1*3+1]<<8) | (sline2[x1*3+2]<<8))

#define A_SRC3V(RGB) (sline0[x0*3+RGB])
#define B_SRC3V(RGB) (sline0[x*3 +RGB])
#define C_SRC3V(RGB) (sline0[x1*3+RGB])
#define D_SRC3V(RGB) (sline1[x0*3+RGB])
#define E_SRC3V(RGB) (sline1[x*3 +RGB])
#define F_SRC3V(RGB) (sline1[x1*3+RGB])
#define G_SRC3V(RGB) (sline2[x0*3+RGB])
#define H_SRC3V(RGB) (sline2[x*3 +RGB])
#define I_SRC3V(RGB) (sline2[x1*3+RGB])

#define E1_DST3(RGB) (dline0[(x<<1)*3    +RGB])
#define E2_DST3(RGB) (dline0[((x<<1)+1)*3+RGB])
#define E3_DST3(RGB) (dline1[(x<<1)*3    +RGB])
#define E4_DST3(RGB) (dline1[((x<<1)+1)*3+RGB])

#define EE1_DST3(RGB) (dline0[(x*3)*3   +RGB])
#define EE2_DST3(RGB) (dline0[(x*3+1)*3 +RGB])
#define EE3_DST3(RGB) (dline0[(x*3+2)*3 +RGB])
#define EE4_DST3(RGB) (dline1[(x*3)*3   +RGB])
#define EE5_DST3(RGB) (dline1[(x*3+1)*3 +RGB])
#define EE6_DST3(RGB) (dline1[(x*3+2)*3 +RGB])
#define EE7_DST3(RGB) (dline2[(x*3)*3   +RGB])
#define EE8_DST3(RGB) (dline2[(x*3+1)*3 +RGB])
#define EE9_DST3(RGB) (dline2[(x*3+2)*3 +RGB])


/* scales the image two times */
void scale2XImage(uchar *srcData, uchar *dstData, int type, int swid, int shei, int sbpl, int dbpl)
{
 if (!srcData || !dstData)
  return;

 unsigned char *sline0, *sline1, *sline2;
 unsigned char *dline0, *dline1;
 int x0, x1;

 for (int y=0; y<shei; y++)
  {
   sline0 = &srcData[(((y-1)<0) ? shei-1 : y-1)*sbpl];
   sline1 = &srcData[y*sbpl];
   sline2 = &srcData[(((y+1) < shei) ? y+1 : 0)*sbpl];

   dline0 = &dstData[(y<<1)*dbpl];
   dline1 = &dstData[((y<<1) + 1)*dbpl];

   if (type==LOAD_CMAP)
    for (int x=0; x<swid; x++)
     {
      if (x)
       x0=x-1;
      else
       x0 = swid-1;
      x1 = x+1;
      if (x1>=swid)
       x1=0;

      /* basic scale2x, thanks to http://scale2x.sourceforge.net */
      E1_DST = E2_DST = E3_DST = E4_DST = E_SRC;
      if ( (B_SRC!=H_SRC) && (D_SRC!=F_SRC) )
       {
        if (D_SRC==B_SRC)
         E1_DST = D_SRC;
	if (B_SRC==F_SRC)
         E2_DST = F_SRC;
	if (D_SRC==H_SRC)
         E3_DST = D_SRC;
	if (H_SRC==F_SRC)
         E4_DST = F_SRC;
       }
      // Modification for smooth diagonals, not 2x2 jagged
      // XX?
      // .XX
      // ..X
      if ((A_SRC==B_SRC) && (A_SRC==E_SRC) && (A_SRC==F_SRC) && (A_SRC==I_SRC) &&
	  (A_SRC!=D_SRC) && (A_SRC!=G_SRC) && (A_SRC!=H_SRC))
       E3_DST = E_SRC;
      // ?XX
      // XX.
      // X..
      if ((C_SRC==B_SRC) && (C_SRC==E_SRC) && (C_SRC==D_SRC) && (C_SRC==G_SRC) &&
	  (C_SRC!=F_SRC) && (C_SRC!=H_SRC) && (C_SRC!=I_SRC))
       E4_DST = E_SRC;

      // Motification to keep square angles, not eaten/rounded
      // ?..
      // .XX
      // .X?
      if ((E_SRC==F_SRC) && (E_SRC==H_SRC) &&
	  (E_SRC!=B_SRC) && (E_SRC!=C_SRC) && (E_SRC!=D_SRC) && (E_SRC!=G_SRC))
       E1_DST = E_SRC;
      // ..?
      // XX.
      // ?X.
      if ((E_SRC==D_SRC) && (E_SRC==H_SRC) &&
	  (E_SRC!=A_SRC) && (E_SRC!=B_SRC) && (E_SRC!=F_SRC) && (E_SRC!=I_SRC))
       E2_DST = E_SRC;
      // .X?
      // .XX
      // ?..
      if ((E_SRC==B_SRC) && (E_SRC==F_SRC) &&
	  (E_SRC!=A_SRC) && (E_SRC!=D_SRC) && (E_SRC!=H_SRC) && (E_SRC!=I_SRC))
       E3_DST = E_SRC;
      // ?X.
      // XX.
      // ..?
      if ((E_SRC==B_SRC) && (E_SRC==D_SRC) &&
	  (E_SRC!=C_SRC) && (E_SRC!=F_SRC) && (E_SRC!=G_SRC) && (E_SRC!=H_SRC))
       E4_DST = E_SRC;
     }
   else // LOAD_TRUE_COL
    for (int x=0; x<swid; x++)
     {
      if (x)
       x0=x-1;
      else
       x0 = swid-1;
      x1 = x+1;
      if (x1>=swid)
       x1=0;

      /* basic scale2x, thanks to http://scale2x.sourceforge.net */
      E1_DST3(0) = E2_DST3(0) = E3_DST3(0) = E4_DST3(0) = E_SRC3V(0);
      E1_DST3(1) = E2_DST3(1) = E3_DST3(1) = E4_DST3(1) = E_SRC3V(1);
      E1_DST3(2) = E2_DST3(2) = E3_DST3(2) = E4_DST3(2) = E_SRC3V(2);
      if ( (B_SRC3!=H_SRC3) && (D_SRC3!=F_SRC3) )
       {
        if (D_SRC3==B_SRC3)
         { E1_DST3(0) = D_SRC3V(0); E1_DST3(1) = D_SRC3V(1); E1_DST3(2) = D_SRC3V(2); }
	if (B_SRC3==F_SRC3)
         { E2_DST3(0) = F_SRC3V(0); E2_DST3(1) = F_SRC3V(1); E2_DST3(2) = F_SRC3V(2); }
	if (D_SRC3==H_SRC3)
         { E3_DST3(0) = D_SRC3V(0); E3_DST3(1) = D_SRC3V(1); E3_DST3(2) = D_SRC3V(2); }
	if (H_SRC3==F_SRC3)
         { E4_DST3(0) = F_SRC3V(0); E4_DST3(1) = F_SRC3V(1); E4_DST3(2) = F_SRC3V(2); }
       }
      // Modification for smooth diagonals, not 2x2 jagged
      // XX?
      // .XX
      // ..X
      if ((A_SRC3==B_SRC3) && (A_SRC3==E_SRC3) && (A_SRC3==F_SRC3) && (A_SRC3==I_SRC3) &&
	  (A_SRC3!=D_SRC3) && (A_SRC3!=G_SRC3) && (A_SRC3!=H_SRC3))
       { E3_DST3(0) = E_SRC3V(0); E3_DST3(1) = E_SRC3V(1); E3_DST3(2) = E_SRC3V(2); }
      // ?XX
      // XX.
      // X..
      if ((C_SRC3==B_SRC3) && (C_SRC3==E_SRC3) && (C_SRC3==D_SRC3) && (C_SRC3==G_SRC3) &&
	  (C_SRC3!=F_SRC3) && (C_SRC3!=H_SRC3) && (C_SRC3!=I_SRC3))
       { E4_DST3(0) = E_SRC3V(0); E4_DST3(1) = E_SRC3V(1); E4_DST3(2) = E_SRC3V(2); }

      // Motification to keep square angles, not eaten/rounded
      // ?..
      // .XX
      // .X?
      if ((E_SRC3==F_SRC3) && (E_SRC3==H_SRC3) &&
	  (E_SRC3!=B_SRC3) && (E_SRC3!=C_SRC3) && (E_SRC3!=D_SRC3) && (E_SRC3!=G_SRC3))
       { E1_DST3(0) = E_SRC3V(0); E1_DST3(1) = E_SRC3V(1); E1_DST3(2) = E_SRC3V(2); }
      // ..?
      // XX.
      // ?X.
      if ((E_SRC3==D_SRC3) && (E_SRC3==H_SRC3) &&
	  (E_SRC3!=A_SRC3) && (E_SRC3!=B_SRC3) && (E_SRC3!=F_SRC3) && (E_SRC3!=I_SRC3))
       { E2_DST3(0) = E_SRC3V(0); E2_DST3(1) = E_SRC3V(1); E2_DST3(2) = E_SRC3V(2); }
      // .X?
      // .XX
      // ?..
      if ((E_SRC3==B_SRC3) && (E_SRC3==F_SRC3) &&
	  (E_SRC3!=A_SRC3) && (E_SRC3!=D_SRC3) && (E_SRC3!=H_SRC3) && (E_SRC3!=I_SRC3))
       { E3_DST3(0) = E_SRC3V(0); E3_DST3(1) = E_SRC3V(1); E3_DST3(2) = E_SRC3V(2); }
      // ?X.
      // XX.
      // ..?
      if ((E_SRC3==B_SRC3) && (E_SRC3==D_SRC3) &&
	  (E_SRC3!=C_SRC3) && (E_SRC3!=F_SRC3) && (E_SRC3!=G_SRC3) && (E_SRC3!=H_SRC3))
       { E4_DST3(0) = E_SRC3V(0); E4_DST3(1) = E_SRC3V(1); E4_DST3(2) = E_SRC3V(2); }
     }
   }
}

/* scales the image three times */
void scale3XImage(uchar *srcData, uchar *dstData, int type, int swid, int shei, int sbpl, int dbpl)
{
 if (!srcData || !dstData)
  return;

 unsigned char *sline0, *sline1, *sline2;
 unsigned char *dline0, *dline1, *dline2;
 int x0, x1;

 for (int y=0; y<shei; y++)
  {
   sline0 = &srcData[(((y-1)<0) ? shei-1 : y-1)*sbpl];
   sline1 = &srcData[y*sbpl];
   sline2 = &srcData[(((y+1) < shei) ? y+1 : 0)*sbpl];

   dline0 = &dstData[(y*3)*dbpl];
   dline1 = &dstData[(y*3 + 1)*dbpl];
   dline2 = &dstData[(y*3 + 2)*dbpl];
   if (type==LOAD_CMAP)
    for (int x=0; x<swid; x++)
     {
      if (x)
       x0=x-1;
      else
       x0 = swid-1;
      x1 = x+1;
      if (x1>=swid)
       x1=0;

      EE1_DST = EE2_DST = EE3_DST = 
      EE4_DST = EE5_DST = EE6_DST = 
      EE7_DST = EE8_DST = EE9_DST = E_SRC;

      /* basic scale3x http://scale2x.sourceforge.net */
      if ( (B_SRC!=H_SRC) && (D_SRC!=F_SRC) )
       {
        if (D_SRC==B_SRC)
         EE1_DST = D_SRC;
	if ( ( (D_SRC==B_SRC) && (E_SRC!=C_SRC) ) || ( (B_SRC==F_SRC) && (E_SRC!=A_SRC) ) )
         EE2_DST = B_SRC;
        if (B_SRC==F_SRC)
         EE3_DST = F_SRC;
	if ( ( (D_SRC==B_SRC) && (E_SRC!=G_SRC) ) || ( (D_SRC==H_SRC) && (E_SRC!=A_SRC) ) )
         EE4_DST = D_SRC;
	if ( ( (B_SRC==F_SRC) && (E_SRC!=I_SRC) ) || ( (H_SRC==F_SRC) && (E_SRC!=C_SRC) ) )
         EE6_DST = F_SRC;
        if (D_SRC==H_SRC)
         EE7_DST = D_SRC;
	if ( ( (D_SRC==H_SRC) && (E_SRC!=I_SRC) ) || ( (H_SRC==F_SRC) && (E_SRC!=G_SRC) ) )
         EE8_DST = H_SRC;
        if (H_SRC==F_SRC)
         EE9_DST = F_SRC;
       }
     }
   else // LOAD_TRUE_COL
    for (int x=0; x<swid; x++)
     {
      if (x)
       x0=x-1;
      else
       x0 = swid-1;
      x1 = x+1;
      if (x1>=swid)
       x1=0;

      EE1_DST3(0) = EE2_DST3(0) = EE3_DST3(0) = EE4_DST3(0) = EE5_DST3(0) = EE6_DST3(0) = EE7_DST3(0) = EE8_DST3(0) = EE9_DST3(0) = E_SRC3V(0);
      EE1_DST3(1) = EE2_DST3(1) = EE3_DST3(1) = EE4_DST3(1) = EE5_DST3(1) = EE6_DST3(1) = EE7_DST3(1) = EE8_DST3(1) = EE9_DST3(1) = E_SRC3V(1);
      EE1_DST3(2) = EE2_DST3(2) = EE3_DST3(2) = EE4_DST3(2) = EE5_DST3(2) = EE6_DST3(2) = EE7_DST3(2) = EE8_DST3(2) = EE9_DST3(2) = E_SRC3V(2);

      /* basic scale3x http://scale2x.sourceforge.net */
      if ( (B_SRC3!=H_SRC3) && (D_SRC3!=F_SRC3) )
       {
        if (D_SRC3==B_SRC3)
         { EE1_DST3(0) = D_SRC3V(0); EE1_DST3(1) = D_SRC3V(1); EE1_DST3(2) = D_SRC3V(2); }
	if ( ( (D_SRC3==B_SRC3) && (E_SRC3!=C_SRC3) ) || ( (B_SRC3==F_SRC3) && (E_SRC3!=A_SRC3) ) )
         { EE2_DST3(0) = B_SRC3V(0); EE2_DST3(1) = B_SRC3V(1); EE2_DST3(2) = B_SRC3V(2); }
        if (B_SRC3==F_SRC3)
         { EE3_DST3(0) = F_SRC3V(0); EE3_DST3(1) = F_SRC3V(1); EE3_DST3(2) = F_SRC3V(2); }
	if ( ( (D_SRC3==B_SRC3) && (E_SRC3!=G_SRC3) ) || ( (D_SRC3==H_SRC3) && (E_SRC3!=A_SRC3) ) )
         { EE4_DST3(0) = D_SRC3V(0); EE4_DST3(1) = D_SRC3V(1); EE4_DST3(2) = D_SRC3V(2); }
	if ( ( (B_SRC3==F_SRC3) && (E_SRC3!=I_SRC3) ) || ( (H_SRC3==F_SRC3) && (E_SRC3!=C_SRC3) ) )
         { EE6_DST3(0) = F_SRC3V(0); EE6_DST3(1) = F_SRC3V(1); EE6_DST3(2) = F_SRC3V(2); }
        if (D_SRC3==H_SRC3)
         { EE7_DST3(0) = D_SRC3V(0); EE7_DST3(1) = D_SRC3V(1); EE7_DST3(2) = D_SRC3V(2); }
	if ( ( (D_SRC3==H_SRC3) && (E_SRC3!=I_SRC3) ) || ( (H_SRC3==F_SRC3) && (E_SRC3!=G_SRC3) ) )
         { EE8_DST3(0) = H_SRC3V(0); EE8_DST3(1) = H_SRC3V(1); EE8_DST3(2) = H_SRC3V(2); }
        if (H_SRC3==F_SRC3)
         { EE9_DST3(0) = F_SRC3V(0); EE9_DST3(1) = F_SRC3V(1); EE9_DST3(2) = F_SRC3V(2); }
       }
     }
  }
}

/* shrinks the image two times horizontally and vertically */
void shrink2XImage(uchar *srcData, uchar *dstData, int type, int swid, int shei, int sbpl, int dbpl)
{
 if (!srcData || !dstData)
  return;
 
 unsigned char *sline0, *sline1;
 unsigned char *dline;
 int dhei = shei/2;
 int dwid = swid/2;

 for (int y=0; y<dhei; y++)
  {
   sline0 = &srcData[(y<<1)*sbpl];
   sline1 = &srcData[((y<<1)+1)*sbpl];

   dline = &dstData[y*dbpl];

   if (type==LOAD_CMAP)
    for (int x=0; x<dwid; x++)
     {
      dline[x] = sline0[x<<1];
      /* some basic averaging: if two other points to the right or below are equal, presome set to those points, otherwise we already have the top left point */
      if (sline1[x<<1] == sline1[(x<<1)+1])
        dline[x] = sline1[x<<1];
      else if (sline0[(x<<1)+1] == sline1[(x<<1)+1])
        dline[x] = sline0[(x<<1)+1];
     }
   else
    for (int x=0; x<dwid; x++)
     {
      dline[x*3]   = ((int)sline0[(x<<1)*3]   + sline0[((x<<1)+1)*3]   + sline1[(x<<1)*3]   + sline1[((x<<1)+1)*3]  )>>2;
      dline[x*3+1] = ((int)sline0[(x<<1)*3+1] + sline0[((x<<1)+1)*3+1] + sline1[(x<<1)*3+1] + sline1[((x<<1)+1)*3+1])>>2;
      dline[x*3+2] = ((int)sline0[(x<<1)*3+2] + sline0[((x<<1)+1)*3+2] + sline1[(x<<1)*3+2] + sline1[((x<<1)+1)*3+2])>>2;
     }
  }
}

/* shrinks the image three times horizontally and vertically */
void shrink3XImage(uchar *srcData, uchar *dstData, int type, int swid, int shei, int sbpl, int dbpl)
{
 if (!srcData || !dstData)
  return;
 
 unsigned char *sline0, *sline1, *sline2;
 unsigned char *dline;
 int dhei = shei/3;
 int dwid = swid/3;
 unsigned char hist[256];
 int maxCol;

 for (int y=0; y<dhei; y++)
  {
   sline0 = &srcData[(y*3)*sbpl];
   sline1 = &srcData[(y*3+1)*sbpl];
   sline2 = &srcData[(y*3+2)*sbpl];

   dline = &dstData[y*dbpl];

   if (type==LOAD_CMAP)
    for (int x=0; x<dwid; x++)
     {
      /* calculate the most popular color to pick that one */
      memset(hist, 0, 256);
      hist[sline0[x*3]]++;
      hist[sline0[x*3+1]]++;
      hist[sline0[x*3+2]]++;
      hist[sline1[x*3]]++;
      hist[sline1[x*3+1]]++;
      hist[sline1[x*3+2]]++;
      hist[sline2[x*3]]++;
      hist[sline2[x*3+1]]++;
      hist[sline2[x*3+2]]++;
      maxCol = sline0[x*3];
      if (hist[maxCol]<5) /* if 5 or more, we already have the most popular color */
       {
        /* otherwise evaluate which points have higher histogram value */
	/* only check 8 pixels instead of full 256 palette */
        if (hist[sline0[x*3+1]]>hist[maxCol])
         maxCol = sline0[x*3+1];
        if (hist[sline0[x*3+2]]>hist[maxCol])
         maxCol = sline0[x*3+2];
        if (hist[sline1[x*3]]>hist[maxCol])
         maxCol = sline1[x*3];
        if (hist[sline1[x*3+1]]>hist[maxCol])
         maxCol = sline1[x*3+1];
        if (hist[sline1[x*3+2]]>hist[maxCol])
         maxCol = sline1[x*3+2];
        if (hist[sline2[x*3]]>hist[maxCol])
         maxCol = sline2[x*3];
        if (hist[sline2[x*3+1]]>hist[maxCol])
         maxCol = sline2[x*3+1];
        if (hist[sline2[x*3+2]]>hist[maxCol])
         maxCol = sline2[x*3+2];
       }
      dline[x] = maxCol;
     }
   else
    for (int x=0; x<dwid; x++)
     {
      dline[x*3]   = ((int)sline0[x*3*3]   + sline0[(x*3+1)*3]   + sline0[(x*3+2)*3]   + sline1[x*3*3]   + sline1[(x*3+1)*3]   + sline1[(x*3+2)*3]   + sline2[x*3*3]   + sline2[(x*3+1)*3]   + sline2[(x*3+2)*3])/9;
      dline[x*3+1] = ((int)sline0[x*3*3+1] + sline0[(x*3+1)*3+1] + sline0[(x*3+2)*3+1] + sline1[x*3*3+1] + sline1[(x*3+1)*3+1] + sline1[(x*3+2)*3+1] + sline2[x*3*3+1] + sline2[(x*3+1)*3+1] + sline2[(x*3+2)*3+1])/9;
      dline[x*3+2] = ((int)sline0[x*3*3+2] + sline0[(x*3+1)*3+2] + sline0[(x*3+2)*3+2] + sline1[x*3*3+2] + sline1[(x*3+1)*3+2] + sline1[(x*3+2)*3+2] + sline2[x*3*3+2] + sline2[(x*3+1)*3+2] + sline2[(x*3+2)*3+2])/9;
     }
  }
}

/* shrinks the image four times horizontally and vertically */
void shrink4XImage(uchar *srcData, uchar *dstData, int type, int swid, int shei, int sbpl, int dbpl)
{
 if (!srcData || !dstData)
  return;
 
 unsigned char *sline0, *sline1, *sline2, *sline3;
 unsigned char *dline;
 int dhei = shei/4;
 int dwid = swid/4;
 unsigned char hist[256];
 int maxCol, i;

 for (int y=0; y<dhei; y++)
  {
   sline0 = &srcData[ (y<<2)*sbpl];
   sline1 = &srcData[((y<<2)+1)*sbpl];
   sline2 = &srcData[((y<<2)+2)*sbpl];
   sline3 = &srcData[((y<<2)+2)*sbpl];

   dline = &dstData[y*dbpl];

   if (type==LOAD_CMAP)
    for (int x=0; x<dwid; x++)
     {
      /* calculate the most popular color to pick that one */
      memset(hist, 0, 256);
      hist[sline0[ x<<2]]++;
      hist[sline0[(x<<2)+1]]++;
      hist[sline0[(x<<2)+2]]++;
      hist[sline0[(x<<2)+3]]++;
      hist[sline1[ x<<2]]++;
      hist[sline1[(x<<2)+1]]++;
      hist[sline1[(x<<2)+2]]++;
      hist[sline1[(x<<2)+3]]++;
      hist[sline2[ x<<2]]++;
      hist[sline2[(x<<2)+1]]++;
      hist[sline2[(x<<2)+2]]++;
      hist[sline2[(x<<2)+3]]++;
      hist[sline3[ x<<2]]++;
      hist[sline3[(x<<2)+1]]++;
      hist[sline3[(x<<2)+2]]++;
      hist[sline3[(x<<2)+3]]++;

      maxCol = sline0[x<<2];
      if (hist[maxCol]<8) /* if 8 or more, we already have the most popular color */
       for (i=0; i<256; i++)
        if (hist[i] > hist[maxCol])
	 maxCol = i;
      dline[x] = maxCol;
     }
   else
    for (int x=0; x<dwid; x++)
     {
      dline[x*3] =     ((int)sline0[(x<<2)*3]   + sline0[((x<<2)+1)*3]   + sline0[((x<<2)+2)*3]   + sline0[((x<<2)+3)*3]   + sline1[(x<<2)*3]   + sline1[((x<<2)+1)*3]   + sline1[((x<<2)+2)*3]   + sline1[((x<<2)+3)*3]   + sline2[(x<<2)*3]   + sline2[((x<<2)+1)*3]   + sline2[((x<<2)+2)*3]   + sline2[((x<<2)+3)*3]   + sline3[(x<<2)*3]   + sline3[((x<<2)+1)*3]   + sline3[((x<<2)+2)*3]   + sline3[((x<<2)+3)*3]   )>>4;
      dline[(x*3)+1] = ((int)sline0[(x<<2)*3+1] + sline0[((x<<2)+1)*3+1] + sline0[((x<<2)+2)*3+1] + sline0[((x<<2)+3)*3+1] + sline1[(x<<2)*3+1] + sline1[((x<<2)+1)*3+1] + sline1[((x<<2)+2)*3+1] + sline1[((x<<2)+3)*3+1] + sline2[(x<<2)*3+1] + sline2[((x<<2)+1)*3+1] + sline2[((x<<2)+2)*3+1] + sline2[((x<<2)+3)*3+1] + sline3[(x<<2)*3+1] + sline3[((x<<2)+1)*3+1] + sline3[((x<<2)+2)*3+1] + sline3[((x<<2)+3)*3+1] )>>4;
      dline[(x*3)+2] = ((int)sline0[(x<<2)*3+2] + sline0[((x<<2)+1)*3+2] + sline0[((x<<2)+2)*3+2] + sline0[((x<<2)+3)*3+2] + sline1[(x<<2)*3+2] + sline1[((x<<2)+1)*3+2] + sline1[((x<<2)+2)*3+2] + sline1[((x<<2)+3)*3+2] + sline2[(x<<2)*3+2] + sline2[((x<<2)+1)*3+2] + sline2[((x<<2)+2)*3+2] + sline2[((x<<2)+3)*3+2] + sline3[(x<<2)*3+2] + sline3[((x<<2)+1)*3+2] + sline3[((x<<2)+2)*3+2] + sline3[((x<<2)+3)*3+2] )>>4;
     }
  }
}

/* halves the image horizontally or vertically */
void halveImage(int halveType, uchar *srcData, uchar *dstData, int shei, int sbpl, int dbpl)
{
 if (!srcData || !dstData)
  return;

 int dhei = shei/((halveType==HALVE_Y) ? 2 : 1);
 /* in all cases just copy half of the original image */
 for (int y=0; y<dhei; y++)
  memcpy(&dstData[y*dbpl], &srcData[y*sbpl], dbpl);
}

/* shrinks the image horizontally or vertically */
void shrinkImage(int shrinkType, uchar *srcData, uchar *dstData, int type, int swid, int shei, int sbpl, int dbpl)
{
 if (!srcData || !dstData)
  return;

 int dhei = shei/((shrinkType==SHRINK_Y) ? 2 : 1);
 int dwid = swid/((shrinkType==SHRINK_X) ? 2 : 1);
 unsigned char *srcPtr, *dstPtr;

 if (shrinkType==SHRINK_Y)
  for (int y=0; y<dhei; y++)
   memcpy(&dstData[y*dbpl], &srcData[(y<<1)*sbpl], dbpl);
 else
  for (int y=0; y<dhei; y++)
   {
    dstPtr = &dstData[y*dbpl];
    srcPtr = &srcData[y*sbpl];
    if (type==LOAD_CMAP)
     for (int x=0; x<dwid; x++)
      { *dstPtr++ = *srcPtr++; srcPtr++; }
    else  // type==LOAD_TRUE_COL
     for (int x=0; x<dwid; x++)
      {
       *dstPtr++ = *srcPtr++;
       *dstPtr++ = *srcPtr++;
       *dstPtr++ = *srcPtr++;
        srcPtr++;
	srcPtr++;
	srcPtr++;
      }
   }
}

/* multiplies the image horizontally */
void multiplyXImage(int times, uchar *srcData, uchar *dstData, int type, int swid, int shei, int sbpl, int dbpl)
{
 if (!srcData || !dstData)
  return;

 unsigned char *srcPtr, *dstPtr;

 for (int y=0; y<shei; y++)
  {
   dstPtr = &dstData[y*dbpl];
   srcPtr = &srcData[y*sbpl];
   if (type==LOAD_CMAP)
    for (int x=0; x<swid; x++)
     {
      for (int xx = 0; xx<times; xx++)
       *dstPtr++ = *srcPtr;
      srcPtr++;
     }
   else  // type==LOAD_TRUE_COL
    for (int x=0; x<swid; x++)
     {
      for (int xx = 0; xx<times; xx++)
       {
        *dstPtr++ = srcPtr[0];
        *dstPtr++ = srcPtr[1];
        *dstPtr++ = srcPtr[2];
       }
      srcPtr++;
      srcPtr++;
      srcPtr++;
     }
  }
}

/* multiplies the image vertically */
void multiplyYImage(int times, uchar *srcData, uchar *dstData, int shei, int sbpl, int dbpl)
{
 if (!srcData || !dstData)
  return;

 for (int y=0; y<shei; y++)
  for (int yy = 0; yy<times; yy++)
   memcpy(&dstData[(y*times + yy)*dbpl], &srcData[y*sbpl], sbpl);
}

/*
int depthToType(int depth)
{
 switch(depth)
  {
   case 1:
    return LOAD_BW;
   case 8:
    return LOAD_CMAP;
   case 24:
    return LOAD_TRUE_COL;
   case 32:
    return LOAD_RGBA;
  }
  return -1;
}
*/

void ApaintImage::sortColors(int sortOrder, int selColor, int sortUp)
{
 /* sort colors in the colormap according to given order */
 if (type()!=LOAD_CMAP)
  return;
 int remapOrder[256];
 int x,y,i,j, best=0;
 int numColors =  ApaintImage::numColors();
 unsigned char red[256], green[256], blue[256];
 int hue[256], saturation[256], value[256];
 unsigned char used[256], *srcData;
 int hist[256], bestVal, val=0;

 for (i=0; i<numColors; i++)
  {
   QRgb color = ApaintImage::color(i);
   red[i]=qRed(color);
   green[i]=qGreen(color);
   blue[i]=qBlue(color);
   QColor qc = QColor(color);
   qc.getHsv(&hue[i], &saturation[i], &value[i]);
   /* ony trolls know why getHsv returns negative hue */
   if (hue[i]<0)
    hue[i]=0;
   used[i]=FALSE;
   remapOrder[i]=i;
   hist[i]=0;
  }

 if (sortOrder==SORT_COLORS_POPULARITY)
  {
   for (y=0; y<height(); y++)
    {
     srcData = &data->imageData[y*data->imageBpl];
     for (x=width(); x; x--, srcData++)
      hist[*srcData]++;
    }
   for (j=0; j<numColors; j++)
    {
     bestVal = -1;
     for (i=0; i<numColors; i++)
      if (!used[i])
       if (hist[i]>bestVal)
	{ bestVal=hist[i]; best = i; }
      used[best]=TRUE;
      remapOrder[best]=j;
    }
  }
 else
  for (j=0; j<numColors; j++)
   {
    bestVal = INT_MAX;
    for (i=0; i<numColors; i++)
     if (!used[i])
      {
       switch(sortOrder)
	{
	 case SORT_COLORS_LIGHTNESS:
	  val = (305*red[i] + 510*green[i] + 205*blue[i] + 512)>>10;
	  break;
	 case SORT_COLORS_HUE:
	  val = hue[i];
	  break;
	 case SORT_COLORS_SATURATION:
	  val = saturation[i];
	  break;
	 case SORT_COLORS_SIMILARITY:
	  val = (red[i]-red[selColor])*(red[i]-red[selColor]) + (green[i]-green[selColor])*(green[i]-green[selColor]) + (blue[i]-blue[selColor])*(blue[i]-blue[selColor]);
	  break;
	}
       if (val<bestVal)
	{ bestVal=val; best = i; }
      }
    used[best]=TRUE;
    remapOrder[best]=j; 
   }
 /* mirror the normal sorting */
 if (!sortUp)
  for (i=0; i<(numColors/2); i++)
   SWAP(remapOrder[i], remapOrder[numColors-i-1]);

 for (i=0; i<numColors; i++)
  ApaintImage::setColor(remapOrder[i], qRgb(red[i], green[i], blue[i]));

 for (y=0; y<height(); y++)
  {
   srcData = &data->imageData[y*data->imageBpl];
   for (x=width(); x; x--, srcData++)
    *srcData = remapOrder[*srcData];
  }
}

/* next two functions have the same code as in the library, but we duplicate it here, since optimization does not work well */
/* if it is in a different library */

/* RGB to HSL transformation by Ken Fiskhin of Pixar, Graphics Gems I p. 763 */
/* r,g,b: 0..255, h: 0..359, s,l: 0..1000 */
static void RGBtoHSL (int *rgb, int *hsl)
{
 double v, m, vm, r1, g1, b1, h1, s1, l1, r2, g2, b2;

 r1 = (double) rgb[0]/255.0;
 g1 = (double) rgb[1]/255.0;
 b1 = (double) rgb[2]/255.0;

 v = max(r1, g1);
 v = max(v, b1);
 m = min(r1, g1);
 m = min(m, b1);

 l1 = (m+v)/2.0;
 if (l1<=0.0)
  {
   hsl[0] = 0;
   hsl[1] = 0;
   hsl[2] = (int) (l1*1000);
   return;
  }

 s1 = vm = v - m;
 if (s1 > 0.0)
  { s1 /= (l1 <= 0.5) ? (v + m) : (2.0 - v - m); }
 else
  {
   hsl[0] = 0;
   hsl[1] = (int) (s1*1000);
   hsl[2] = (int) (l1*1000);
   return;
  }

 r2 = (v - r1)/vm;
 g2 = (v - g1)/vm;
 b2 = (v - b1)/vm;

 if (r1==v)
  h1 = (g1 == m ? 5.0 + b2 : 1.0 - g2);
 else if (g1==v)
  h1 = (b1 == m ? 1.0 + r2 : 3.0 - b2);
 else
  h1 = (r1 == m ? 3.0 + g2 : 5.0 - r2);

 h1 /= 6.0;

 hsl[0] = (int) (h1*359 + 0.5);
 hsl[1] = (int) (s1*1000 + 0.5);
 hsl[2] = (int) (l1*1000 + 0.5);
}

/* HSL to RGB transformation by Ken Fiskhin of Pixar, Graphics Gems I p. 763 */
/* r,g,b: 0..255, h: 0..359, s,l: 0..1000 */
static void HSLtoRGB (int *hsl, int *rgb)
{
 double h1, s1, l1, r1, g1, b1, v;

 h1 = hsl[0]/360.0;
 s1 = hsl[1]/1000.0;
 l1 = hsl[2]/1000.0;

 v = (l1<=0.5) ? (l1*(1.0 + s1)) : (l1+s1 - l1*s1);
 r1 = g1 = b1 = 0.0;
 if (v>0)
  {
   double m, sv, fract, vsf, mid1, mid2;
   int sextant;

   m = l1 + l1 - v;
   sv = (v - m)/v;
   h1 *= 6.0;
   sextant = (int) h1;
   fract = h1 - sextant;
   vsf = v * sv * fract;
   mid1 = m + vsf;
   mid2 = v - vsf;
   switch (sextant)
    {
     case 0: r1=v; g1=mid1; b1=m; break;
     case 1: r1=mid2; g1=v; b1=m; break;
     case 2: r1=m; g1=v; b1=mid1; break;
     case 3: r1=m; g1=mid2; b1=v; break;
     case 4: r1=mid1; g1=m; b1=v; break;
     case 5: r1=v; g1=m; b1=mid2; break;
    }
  }

 rgb[0] = (int) (r1*255 + 0.5);
 rgb[1] = (int) (g1*255 + 0.5);
 rgb[2] = (int) (b1*255 + 0.5);
}

void ApaintImage::invertColors(int what, bool *protColors)
{
 int hsl[3], rgb[3];

 /* invert the color palette */
 if (type()==LOAD_CMAP)
  {
   int i, j, numColors =  ApaintImage::numColors();
   QRgb color; 

   for (i=0; i<numColors; i++)
    if (!protColors[i])
     {
      color = ApaintImage::color(i);
      switch (what)
       {
        case SWAP_COLORS_PALETTE:
	 j = rand()%numColors;
	 if ((i!=j) && (!protColors[j]))
	  {
	   ApaintImage::setColor(i, ApaintImage::color(j));
	   ApaintImage::setColor(j, color);
	  }
	 break;
	case INVERT_COLORS_ALL:
         ApaintImage::setColor(i, qRgb(~qRed(color), ~qGreen(color), ~qBlue(color)));
	 break;
	case INVERT_COLORS_RED:
         ApaintImage::setColor(i, qRgb(~qRed(color), qGreen(color), qBlue(color)));
	 break;
	case INVERT_COLORS_GREEN:
         ApaintImage::setColor(i, qRgb(qRed(color), ~qGreen(color), qBlue(color)));
	 break;
	case INVERT_COLORS_BLUE:
         ApaintImage::setColor(i, qRgb(qRed(color), qGreen(color), ~qBlue(color)));
	 break;
	case INVERT_COLORS_CYAN:
         ApaintImage::setColor(i, qRgb(qRed(color), ~qGreen(color), ~qBlue(color)));
	 break;
	case INVERT_COLORS_YELLOW:
         ApaintImage::setColor(i, qRgb(~qRed(color), ~qGreen(color), qBlue(color)));
	 break;
	case INVERT_COLORS_MAGENTA:
         ApaintImage::setColor(i, qRgb(~qRed(color), qGreen(color), ~qBlue(color)));
	 break;
	case INVERT_COLORS_GREY:
	 rgb[0]=qRed(color);
	 rgb[1]=qGreen(color);
	 rgb[2]=qBlue(color);
	 RGBtoHSL(rgb, hsl);
	 hsl[2]=1000-hsl[2];
	 HSLtoRGB(hsl, rgb);
         ApaintImage::setColor(i, qRgb(rgb[0], rgb[1], rgb[2]));
	 break;
	case INVERT_COLORS_SATURATION:
	 rgb[0]=qRed(color);
	 rgb[1]=qGreen(color);
	 rgb[2]=qBlue(color);
	 RGBtoHSL(rgb, hsl);
	 hsl[1]=1000-hsl[1];
	 HSLtoRGB(hsl, rgb);
         ApaintImage::setColor(i, qRgb(rgb[0], rgb[1], rgb[2]));
	 break;

	case SWAP_COLORS_RGB_GBR:
         ApaintImage::setColor(i, qRgb(qGreen(color), qBlue(color), qRed(color)));
	 break;
	case SWAP_COLORS_RGB_BRG:
         ApaintImage::setColor(i, qRgb(qBlue(color), qRed(color), qGreen(color)));
	 break;
	case SWAP_COLORS_RG:
         ApaintImage::setColor(i, qRgb(qGreen(color), qRed(color), qBlue(color)));
	 break;
	case SWAP_COLORS_RB:
         ApaintImage::setColor(i, qRgb(qBlue(color), qGreen(color), qRed(color)));
	 break;
	case SWAP_COLORS_GB:
         ApaintImage::setColor(i, qRgb(qRed(color), qBlue(color), qGreen(color)));
	 break;
	case SWAP_COLORS_SL:
	 rgb[0]=qRed(color);
	 rgb[1]=qGreen(color);
	 rgb[2]=qBlue(color);
	 RGBtoHSL(rgb, hsl);
	 SWAP(hsl[1], hsl[2]);
	 HSLtoRGB(hsl, rgb);
         ApaintImage::setColor(i, qRgb(rgb[0], rgb[1], rgb[2]));
	 break;
       }
     }
  }
 else /* true color, need to invert the image */
  {
   int x,y;
   unsigned char *srcData;

   for (y=0; y<height(); y++)
    {
     srcData = &data->imageData[y*data->imageBpl];
     switch (what)
      {
       case INVERT_COLORS_ALL:
	for (x=width()*3; x; x--)
         *srcData++ = ~*srcData;
	break;
       case INVERT_COLORS_RED:
	for (x=width(); x; x--)
         { *srcData++ = ~*srcData; srcData++; srcData++; }
	break;
       case INVERT_COLORS_GREEN:
	for (x=width(); x; x--)
         { srcData++; *srcData++ = ~*srcData; srcData++; }
	break;
       case INVERT_COLORS_BLUE:
	for (x=width(); x; x--)
         { srcData++; srcData++; *srcData++ = ~*srcData; }
	break;
       case INVERT_COLORS_CYAN:
	for (x=width(); x; x--)
         { srcData++; *srcData++ = ~*srcData; *srcData++ = ~*srcData; }
	break;
       case INVERT_COLORS_YELLOW:
	for (x=width(); x; x--)
         { *srcData++ = ~*srcData; *srcData++ = ~*srcData; srcData++; }
	break;
       case INVERT_COLORS_MAGENTA:
	for (x=width(); x; x--)
         { *srcData++ = ~*srcData; srcData++; *srcData++ = ~*srcData; }
	break;
       case INVERT_COLORS_GREY:
	for (x=width(); x; x--)
         { rgb[0]= srcData[0]; rgb[1]= srcData[1]; rgb[2]= srcData[2];
	   RGBtoHSL(rgb, hsl);
	   hsl[2]=1000-hsl[2];
	   HSLtoRGB(hsl, rgb);
	   *srcData++ = rgb[0];*srcData++ = rgb[1]; *srcData++ = rgb[2]; }
	break;

       case INVERT_COLORS_SATURATION:
	for (x=width(); x; x--)
         { rgb[0]= srcData[0]; rgb[1]= srcData[1]; rgb[2]= srcData[2];
	   RGBtoHSL(rgb, hsl);
	   hsl[1]=1000-hsl[1];
	   HSLtoRGB(hsl, rgb);
	   *srcData++ = rgb[0];*srcData++ = rgb[1]; *srcData++ = rgb[2]; }
	break;

       case SWAP_COLORS_RGB_GBR:
	for (x=width(); x; x--)
         { rgb[0]= srcData[0]; rgb[1]= srcData[1]; rgb[2]= srcData[2]; 
	   *srcData++ = rgb[1]; *srcData++ = rgb[2]; *srcData++ = rgb[0]; }
	break;
       case SWAP_COLORS_RGB_BRG:
	for (x=width(); x; x--)
         { rgb[0]= srcData[0]; rgb[1]= srcData[1]; rgb[2]= srcData[2]; 
	   *srcData++ = rgb[2]; *srcData++ = rgb[0]; *srcData++ = rgb[1]; }
	break;
       case SWAP_COLORS_RG:
	for (x=width(); x; x--)
         { rgb[0]= srcData[0]; rgb[1]= srcData[1]; rgb[2]= srcData[2]; 
	   *srcData++ = rgb[1]; *srcData++ = rgb[0]; *srcData++ = rgb[2]; }
	break;
       case SWAP_COLORS_RB:
	for (x=width(); x; x--)
         { rgb[0]= srcData[0]; rgb[1]= srcData[1]; rgb[2]= srcData[2]; 
	   *srcData++ = rgb[2]; *srcData++ = rgb[1]; *srcData++ = rgb[0]; }
	break;
       case SWAP_COLORS_GB:
	for (x=width(); x; x--)
         { rgb[0]= srcData[0]; rgb[1]= srcData[1]; rgb[2]= srcData[2]; 
	   *srcData++ = rgb[0]; *srcData++ = rgb[2]; *srcData++ = rgb[1]; }
	break;
       case SWAP_COLORS_SL:
	for (x=width(); x; x--)
         { rgb[0]= srcData[0]; rgb[1]= srcData[1]; rgb[2]= srcData[2];
	   RGBtoHSL(rgb, hsl);
	   SWAP(hsl[1], hsl[2]);
	   HSLtoRGB(hsl, rgb);
	   *srcData++ = rgb[0];*srcData++ = rgb[1]; *srcData++ = rgb[2]; }
	break;
      }
    }
  }
}

void ApaintImage::swapColors(int first, int second)
{
 QRgb tmpCol =  ApaintImage::color(first);
 int i;
 RemapArray transTable;

 ApaintImage::setColor(first, ApaintImage::color(second));
 ApaintImage::setColor(second, tmpCol);

 for (i=0; i<256; i++)
  {
   transTable.setVal(i, i);
   if (i==first)
    transTable.setVal(i, second);
   if (i==second)
    transTable.setVal(i, first);
  }
 remapData(transTable);
}

void ApaintImage::contrastStretchAP(void)
{
 /* invert the color palette */
 if (type()==LOAD_CMAP)
  {
   int i, numColors =  ApaintImage::numColors();
   QRgb color;
   unsigned char cmap[256*3];

   for (i=0; i<numColors; i++)
    {
     color = ApaintImage::color(i);
     cmap[i*3+0]=qRed(color);
     cmap[i*3+1]=qGreen(color);
     cmap[i*3+2]=qBlue(color);
    }
   contrastStretch(data->imageData, data->imageBpl, LOAD_CMAP, width(), height(), cmap);
   for (i=0; i<numColors; i++)
    ApaintImage::setColor(i, qRgb(cmap[i*3], cmap[i*3+1], cmap[i*3+2]));
  }
 else /* true color, need to invert the image */
  contrastStretch(data->imageData, data->imageBpl, LOAD_TRUE_COL, width(), height(), NULL);
}

/* make colors shades in colormap from foreground to background color */
void ApaintImage::makeColorShades(int fgIndex, int bgIndex)
{
 if (type()==LOAD_CMAP)
  {
   int shadeLenght = bgIndex - fgIndex;
   int i;
   QRgb firstColor, lastColor;
   int redDiff, greenDiff, blueDiff;

   firstColor = ApaintImage::color(fgIndex);
   lastColor = ApaintImage::color(bgIndex);
   redDiff = (int) qRed(lastColor) - qRed(firstColor);
   greenDiff = (int) qGreen(lastColor) - qGreen(firstColor);
   blueDiff = (int) qBlue(lastColor) - qBlue(firstColor);
   for (i=1; i<shadeLenght; i++)
     ApaintImage::setColor(fgIndex + i, qRgb(qRed(firstColor)+ (redDiff*i)/shadeLenght,
      qGreen(firstColor) + (greenDiff*i)/shadeLenght, qBlue(firstColor) + (blueDiff*i)/shadeLenght));
  }
}

